@@ -53,13 +53,11 @@ hv_to_job_desc_msg(HV* hv, job_desc_msg_t* job_desc_msg)
 		}
 	}
 	FETCH_FIELD(hv, job_desc_msg, features, charp, FALSE);
+	FETCH_FIELD(hv, job_desc_msg, reservation, charp, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, immediate, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, job_id, uint32_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, name, charp, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, job_min_procs, uint16_t, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, job_min_sockets, uint16_t, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, job_min_cores, uint16_t, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, job_min_threads, uint16_t, FALSE);
+	FETCH_FIELD(hv, job_desc_msg, job_min_cpus, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, job_min_memory, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, job_min_tmp_disk, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, partition, charp, FALSE);
@@ -72,11 +70,8 @@ hv_to_job_desc_msg(HV* hv, job_desc_msg_t* job_desc_msg)
 	FETCH_FIELD(hv, job_desc_msg, min_nodes, uint32_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, max_nodes, uint32_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, min_sockets, uint16_t, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, max_sockets, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, min_cores, uint16_t, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, max_cores, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, min_threads, uint16_t, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, max_threads, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, cpus_per_task, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, ntasks_per_node, uint16_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, ntasks_per_socket, uint16_t, FALSE);
@@ -103,9 +98,9 @@ hv_to_job_desc_msg(HV* hv, job_desc_msg_t* job_desc_msg)
 			Perl_warn(aTHX_ "`argv' of job descriptor is not an array reference, ignored");
 		}
 	}
-	FETCH_FIELD(hv, job_desc_msg, err, charp, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, in, charp, FALSE);
-	FETCH_FIELD(hv, job_desc_msg, out, charp, FALSE);
+	FETCH_FIELD(hv, job_desc_msg, std_err, charp, FALSE);
+	FETCH_FIELD(hv, job_desc_msg, std_in, charp, FALSE);
+	FETCH_FIELD(hv, job_desc_msg, std_out, charp, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, user_id, uint32_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, group_id, uint32_t, FALSE);
 	FETCH_FIELD(hv, job_desc_msg, work_dir, charp, FALSE);
@@ -139,7 +134,7 @@ hv_to_job_desc_msg(HV* hv, job_desc_msg_t* job_desc_msg)
 		avp = (AV*)SvRV(*svp);
 		for(i = 0; i < SYSTEM_DIMENSIONS; i ++) {
 			if(! (svp = av_fetch(avp, i, FALSE))) {
-				Perl_warn(aTHX_ "geometry of dimension %s missing in job descriptor", i);
+				Perl_warn(aTHX_ "geometry of dimension %d missing in job descriptor", i);
 				free_job_desc_msg_memory(job_desc_msg);
 				return -1;
 			}
