@@ -14,7 +14,16 @@ typedef char* charp;
  */
 inline static int av_store_uint16_t(AV* av, int index, uint16_t val)
 {
-	SV* sv = newSVuv(val);
+	SV* sv = NULL;
+	/* Perl has a hard time figuring out the an unsigned int is
+	   equal to INFINITE or NO_VAL since they are treated as
+	   signed ints so we will handle this here. */
+	if(val == (uint16_t)INFINITE)
+		sv = newSViv(INFINITE);
+	else if(val == (uint16_t)NO_VAL)
+		sv = newSViv(NO_VAL);
+	else
+		sv = newSViv(val);
 	
 	if (av_store(av, (I32)index, sv) == NULL) {
 		SvREFCNT_dec(sv);
@@ -28,7 +37,16 @@ inline static int av_store_uint16_t(AV* av, int index, uint16_t val)
  */
 inline static int av_store_uint32_t(AV* av, int index, uint32_t val)
 {
-	SV* sv = newSVuv(val);
+	SV* sv = NULL;
+	/* Perl has a hard time figuring out the an unsigned int is
+	   equal to INFINITE or NO_VAL since they are treated as
+	   signed ints so we will handle this here. */
+	if(val == (uint32_t)INFINITE)
+		sv = newSViv(INFINITE);
+	else if(val == (uint32_t)NO_VAL)
+		sv = newSViv(NO_VAL);
+	else
+		sv = newSViv(val);
 	
 	if (av_store(av, (I32)index, sv) == NULL) {
 		SvREFCNT_dec(sv);
@@ -37,7 +55,6 @@ inline static int av_store_uint32_t(AV* av, int index, uint32_t val)
 	return 0;
 }
 
-
 /*
  * store an int into AV 
  */
@@ -74,8 +91,17 @@ inline static int hv_store_charp(HV* hv, const char *key, charp val)
  */
 inline static int hv_store_uint32_t(HV* hv, const char *key, uint32_t val)
 {
-	SV* sv = newSVuv(val);
-	
+	SV* sv = NULL;
+	/* Perl has a hard time figuring out the an unsigned int is
+	   equal to INFINITE or NO_VAL since they are treated as
+	   signed ints so we will handle this here. */
+	if(val == (uint32_t)INFINITE)
+		sv = newSViv(INFINITE);
+	else if(val == (uint32_t)NO_VAL)
+		sv = newSViv(NO_VAL);
+	else
+		sv = newSVuv(val);
+
 	if (!key || hv_store(hv, key, (I32)strlen(key), sv, 0) == NULL) {
 		SvREFCNT_dec(sv);
 		return -1;
@@ -88,7 +114,16 @@ inline static int hv_store_uint32_t(HV* hv, const char *key, uint32_t val)
  */
 inline static int hv_store_uint16_t(HV* hv, const char *key, uint16_t val)
 {
-	SV* sv = newSVuv(val);
+	SV* sv = NULL;
+	/* Perl has a hard time figuring out the an unsigned int is
+	   equal to INFINITE or NO_VAL since they are treated as
+	   signed ints so we will handle this here. */
+	if(val == (uint16_t)INFINITE)
+		sv = newSViv(INFINITE);
+	else if(val == (uint16_t)NO_VAL)
+		sv = newSViv(NO_VAL);
+	else
+		sv = newSVuv(val);
 	
 	if (!key || hv_store(hv, key, (I32)strlen(key), sv, 0) == NULL) {
 		SvREFCNT_dec(sv);
@@ -102,7 +137,16 @@ inline static int hv_store_uint16_t(HV* hv, const char *key, uint16_t val)
  */
 inline static int hv_store_uint8_t(HV* hv, const char *key, uint8_t val)
 {
-	SV* sv = newSVuv(val);
+	SV* sv = NULL;
+	/* Perl has a hard time figuring out the an unsigned int is
+	   equal to INFINITE or NO_VAL since they are treated as
+	   signed ints so we will handle this here. */
+	if(val == (uint8_t)INFINITE)
+		sv = newSViv(INFINITE);
+	else if(val == (uint8_t)NO_VAL)
+		sv = newSViv(NO_VAL);
+	else
+		sv = newSVuv(val);
 	
 	if (!key || hv_store(hv, key, (I32)strlen(key), sv, 0) == NULL) {
 		SvREFCNT_dec(sv);
