@@ -8,36 +8,37 @@
 #  Copyright (C) 2007 The Regents of the University of California.
 #  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
 #  Written by Danny Auble <auble1@llnl.gov>.
-#  LLNL-CODE-402394.
-#  
+#  CODE-OCEC-09-009. All rights reserved.
+#
 #  This file is part of SLURM, a resource management program.
-#  For details, see <http://www.llnl.gov/linux/slurm/>.
-#  
+#  For details, see <https://computing.llnl.gov/linux/slurm/>.
+#  Please also read the included file: DISCLAIMER.
+#
 #  SLURM is free software; you can redistribute it and/or modify it under
 #  the terms of the GNU General Public License as published by the Free
 #  Software Foundation; either version 2 of the License, or (at your option)
 #  any later version.
 #
-#  In addition, as a special exception, the copyright holders give permission 
+#  In addition, as a special exception, the copyright holders give permission
 #  to link the code of portions of this program with the OpenSSL library under
-#  certain conditions as described in each individual source file, and 
-#  distribute linked combinations including the two. You must obey the GNU 
-#  General Public License in all respects for all of the code used other than 
-#  OpenSSL. If you modify file(s) with this exception, you may extend this 
-#  exception to your version of the file(s), but you are not obligated to do 
+#  certain conditions as described in each individual source file, and
+#  distribute linked combinations including the two. You must obey the GNU
+#  General Public License in all respects for all of the code used other than
+#  OpenSSL. If you modify file(s) with this exception, you may extend this
+#  exception to your version of the file(s), but you are not obligated to do
 #  so. If you do not wish to do so, delete this exception statement from your
-#  version.  If you delete this exception statement from all source files in 
+#  version.  If you delete this exception statement from all source files in
 #  the program, then also delete it here.
-#  
+#
 #  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 #  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 #  details.
-#  
+#
 #  You should have received a copy of the GNU General Public License along
 #  with SLURM; if not, write to the Free Software Foundation, Inc.,
 #  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
-#  
+#
 #  Based off code with permission copyright 2006, 2007 Cluster Resources, Inc.
 ###############################################################################
 
@@ -102,9 +103,13 @@ if ($man)
 
 # Use sole remaining argument as jobIds
 
-$hostname = `hostname -f`;
+if($^O eq 'aix') {
+	$hostname = `hostname`;
+} else {
+	$hostname = `hostname -f`;
+}
 chomp $hostname;
-		
+
 # Handle unsupported arguments
 #unsupportedOption("-e", DIE) if $executable;
 #unsupportedOption("-W", DIE) if $siteSpecific;
@@ -129,21 +134,47 @@ if(defined($queueList)) {
 	if(!$resp) {
 		die "Problem loading jobs.\n";
 	}
-	
+
 	my $line = 0;
 	foreach my $part (@{$resp->{partition_array}}) {
 		if (@queueIds) {
 			next unless grep /^$part->{'name'}/, @queueIds;
-		} 
-		
+		}
+
 		if ($full) { # Full
 			print_part_full($part);
 		} else { # Brief
 			print_part_brief($part, $line);
 			$line++;
-		}	
+		}
 		$rc = 0;
 	}
+} elsif($queueStatus) {
+	my $jresp = Slurm->load_jobs(1);
+	die "Problem loading jobs.\n" if(!$jresp);
+	my $resp = Slurm->load_partitions(1);
+	die "Problem loading partitions.\n" if(!$resp);
+	my $total_running = 0;
+	my $total_queued = 0;
+	my $line = 0;
+	foreach my $part (@{$resp->{partition_array}}) {
+		$part->{'running_jobs'} = 0;
+		$part->{'queued_jobs'} = 0;
+		foreach my $job (@{$jresp->{job_array}}) {
+			next if($job->{'partition'} ne $part->{'name'});
+			$part->{'running_jobs'}++
+				if($job->{'job_state'} = JOB_RUNNING);
+			$part->{'queued_jobs'}++
+				if($job->{'job_state'} = JOB_PENDING);
+		}
+		$total_running += $part->{'running_jobs'};
+		$total_queued += $part->{'queued_jobs'};
+		print_part_limits($part, $line);
+		$line++;
+        }
+	printf("                                               ----- -----\n");
+	printf("                                               %5d %5d\n",
+	       $total_running, $total_queued);
 } else {
 	my @jobIds = @ARGV;
 	my @userIds = split(/,/, $userList) if $userList;
@@ -159,15 +190,15 @@ if(defined($queueList)) {
 		my $state = $job->{'job_state'};
 		if($job->{'end_time'} && $job->{'end_time'} < $now_time) {
 			$use_time = $job->{'end_time'};
-		} 	
+		}
 		$job->{'statPSUtl'} = job_time_used($job);
 		$job->{'aWDuration'} = $job->{'statPSUtl'};
 
 		$job->{'allocNodeList'} = $job->{'nodes'} || "--";
 		$job->{'stateCode'} = stateCode($job->{'job_state'});
 		$job->{'user_name'} = getpwuid($job->{'user_id'});
-		$job->{'name'} = "Allocation" if !$job->{'name'};	       
-		
+		$job->{'name'} = "Allocation" if !$job->{'name'};
+
 		# Filter jobs according to options and arguments
 		if (@jobIds) {
 			next unless grep /^$job->{'job_id'}/, @jobIds;
@@ -180,11 +211,11 @@ if(defined($queueList)) {
 				next unless ($job->{'stateCode'} eq 'Q');
 			}
 			if (@userIds) {
-				next unless 
+				next unless
 					grep /^$job->{'user_name'}$/, @userIds;
 			}
 		}
-		
+
 		if ($all || $idle || $nodes || $running
 		    || $giga || $mega || $userList) { # Normal
 			print_job_select($job);
@@ -214,7 +245,7 @@ sub stateCode
 	}
 
 	switch($state) {
-		case [JOB_COMPLETE, 
+		case [JOB_COMPLETE,
 		      JOB_CANCELLED,
 		      JOB_TIMEOUT,
 		      JOB_FAILED]    { return 'C' }
@@ -234,11 +265,11 @@ sub stateCode
 sub hhmm
 {
 	my ($hhmmss) = @_;
-	
+
 	if ($hhmmss == INFINITE) {
 		return "Infinite";
 	}
-	
+
 	# Convert hhmmss to duration in minutes
 	$hhmmss = 0 unless $hhmmss;
 	$hhmmss =~ /(?:(\d+):)?(?:(\d+):)?([\d\.]+)/;
@@ -347,8 +378,8 @@ sub job_time_used
 	} else {
 		$end_time = $job->{'end_time'};
 	}
-	
-	return ($end_time - $job->{'suspend_time'} + $job->{'pre_sus_time'}) 
+
+	return ($end_time - $job->{'suspend_time'} + $job->{'pre_sus_time'})
 		if $job->{'suspend_time'};
 	return ($end_time - $job->{'start_time'});
 }
@@ -360,7 +391,21 @@ sub yes_no
 	return "no";
 }
 
-sub get_exec_host 
+sub en_dis
+{
+	my ($query) = @_;
+	return "E" if $query;
+	return "D";
+}
+
+sub running_stopped
+{
+	my ($query) = @_;
+	return "R" if $query;
+	return "S";
+}
+
+sub get_exec_host
 {
 	my ($job) = @_;
 
@@ -373,12 +418,12 @@ sub get_exec_host
 		while((my $host = Slurm::Hostlist::shift($hl))) {
 			push(@allocNodes,
 			     "$host/" . $job->{'cpus_per_node'}[$inx]);
-			
+
 			$cpu_cnt++;
 			if($cpu_cnt >= $job->{'cpu_count_reps'}[$inx]) {
 				$cpu_cnt = 0;
 				$inx++;
-			}			
+			}
 		}
 		$execHost = join '+', @allocNodes;
 	}
@@ -388,7 +433,7 @@ sub get_exec_host
 ###############################################################################
 # Job Functions
 ###############################################################################
-sub print_job_brief 
+sub print_job_brief
 {
 	my ($job, $line_num) = @_;
 
@@ -411,7 +456,7 @@ sub print_job_select
 
 	if (!defined $header_printed) {
 		print "\n${hostname}:\n";
-		
+
 		printf("%-20s %-8s %-8s %-20s %-6s %-5s %-3s %-6s %-5s %-1s %-5s\n",
 		       "", "", "", "", "", "", "", "Req'd", "Req'd", "", "Elap");
 		printf(
@@ -427,10 +472,10 @@ sub print_job_select
 		$header_printed = 1;
 	}
 	$execHost = get_exec_host($job) if $nodes;
-		
+
 	printf("%-20.20s %-8.8s %-8.8s %-20.20s " .
 	       "%-6.6s %5.5s %3.3s %6.6s %-5.5s %-1s %-5.5s",
-	       $job->{'job_id'}, 
+	       $job->{'job_id'},
 	       $job->{'user_name'},
 	       $job->{'partition'},
 	       $job->{'name'},
@@ -458,25 +503,25 @@ sub print_job_full
 	# Print the job attributes
 	printf("Job Id:\t%s\n", $job->{'job_id'});
 	printf("\tJob_Name = %s\n", $job->{'name'}) if $job->{'name'};
-	printf("\tJob_Owner = %s@%s\n", 
+	printf("\tJob_Owner = %s@%s\n",
 	       $job->{'user_name'}, $job->{'alloc_node'});
-	
+
 	printf "\tinteractive = True\n" if !$job->{'batch_flag'};
-	
+
 	printf("\tjob_state = %s\n", $job->{'stateCode'});
 	printf("\tqueue = %s\n", $job->{'partition'});
-	
+
 	printf("\tqtime = %s\n", hRTime($job->{'submit_time'}));
-	printf("\tmtime = %s\n", hRTime($job->{'start_time'})) 
+	printf("\tmtime = %s\n", hRTime($job->{'start_time'}))
 		if $job->{'start_time'};
 	printf("\tctime = %s\n", hRTime($job->{'end_time'}))
 		if $job->{'end_time'};
-	
+
 	printf("\tAccount_Name = %s\n", $job->{'account'}) if $job->{'account'};
-	
+
 	my $execHost = get_exec_host($job);
 	printf("\texec_host = %s\n", $execHost) if $execHost ne '--';
-		
+
 	printf("\tPriority = %u\n", $job->{'priority'});
 	printf("\teuser = %s(%d)\n", $job->{'user_name'}, $job->{'user_id'});
 
@@ -488,39 +533,40 @@ sub print_job_full
 	printf("\tResource_List.walltime = %s\n", hhmmss($job->{'aWDuration'}));
 	printf("\tResource_List.nodect = %d\n", $job->{'num_nodes'})
 		if $job->{'num_nodes'};
-	printf("\tResource_List.ncpus = %s\n", $job->{'num_procs'}) 
-		if $job->{'num_procs'}; 
-	
+	printf("\tResource_List.ncpus = %s\n", $job->{'num_procs'})
+		if $job->{'num_procs'};
+
 	if ($job->{'reqNodes'}) {
 		my $nodeExpr = $job->{'reqNodes'};
-		$nodeExpr .= ":ppn=" . $job->{'ntasks_per_node'} 
-		        if $job->{'ntasks_per_node'}; 
-		
+		$nodeExpr .= ":ppn=" . $job->{'ntasks_per_node'}
+		        if $job->{'ntasks_per_node'};
+
 		printf("\tResource_List.nodes = %s\n", $nodeExpr);
 	}
-		
+
 	print "\n";
 }
 
 ###############################################################################
 # Partition Functions
 ###############################################################################
-sub print_part_brief 
+sub print_part_brief
 {
 	my ($part, $line_num) = @_;
 
 	if(!$line_num) {
-		printf("%-16s %5s %5s %5s %5s %5s %5s %5s %5s %5s %5s %1s\n\n",
+		printf("%-16s %5s %5s %5s %5s %5s %5s %5s %5s %5s %5s %1s\n",
 		       "Queue", "Max", "Tot", "Ena",  "Str", "Que", "Run",
 		       "Hld", "Wat", "Trn", "Ext", "T");
-		printf("%-16s %5s %5s %5s %5s %5s %5s %5s %5s %5s %5s %1s\n\n",
-		       "----------------", "---", "---", "---",  "---", "---", 
+		printf("%-16s %5s %5s %5s %5s %5s %5s %5s %5s %5s %5s %1s\n",
+		       "----------------", "---", "---", "---",  "---", "---",
 		       "---", "---", "---", "---", "---", "-");
 	}
 	printf("%-16.16s %5.5s %5.5s %5.5s %5.5s %5.5s %5.5s %5.5s " .
 	       "%5.5s %5.5s %5.5s %1.1s\n",
 	       $part->{'name'}, '?', '?', yes_no($part->{'state_up'}),
-	       yes_no($part->{'state_up'}), '?', '?', '?', '?', '?', '?', 'E');
+	       yes_no($part->{'state_up'}), '?', '?', '?', '?', '?', '?',
+	       en_dis($part->{'state_up'}));
 }
 
 sub print_part_full
@@ -530,14 +576,48 @@ sub print_part_full
 	printf("Queue:\t%s\n", $part->{'name'});
 	printf("\tqueue_type = Execution\n");
 	printf("\tresources_default.nodes = %d\n", $part->{'total_nodes'});
-	
+
 	printf("\tenabled = %s\n", yes_no($part->{'state_up'}));
-	
+
 	printf("\tstarted = %s\n", yes_no($part->{'state_up'}));
-	
+
 	print "\n";
 }
 
+sub print_part_limits
+{
+	my ($part, $line_num) = @_;
+
+	if(!$line_num) {
+		printf("%-16s %6s %8s %8s %4s  %3s %3s %2s %5s\n",
+		       "Queue", "Memory", "CPU Time", "Walltime",
+		       "Node", "Run", "Que", "Lm", "State");
+		printf("%-16s %6s %8s %8s %4s  %3s %3s %2s %5s\n",
+		       "----------------", "------", "--------", "--------",
+		       "----", "---", "---", "--", "-----");
+	}
+
+
+	printf("%-16.16s   --      --    ", $part->{'name'});
+	if($part->{'max_time'} != INFINITE) {
+		printf("%8u ", $part->{'max_time'});
+	} else {
+		printf("   --    ");
+
+	}
+
+	if($part->{'max_nodes'} != INFINITE) {
+		printf("%4u  ", $part->{'max_nodes'});
+	} else {
+		printf("  --  ");
+	}
+
+	printf("%3u %3u --  %1.1s %1.1s \n", $part->{'running_jobs'},
+	       $part->{'queued_jobs'}, en_dis($part->{'state_up'}),
+	       running_stopped($part->{'state_up'}));
+}
+
+
 ##############################################################################
 
 __END__
@@ -550,6 +630,10 @@ B<qstat> - display job/partition information in a familiar pbs format
 
 B<qstat> [B<-f>] [B<-a>|B<-i>|B<-r>] [B<-n> [B<-1>]] [B<-G>|B<-M>] [B<-u> I<user_list>] [B<-? | --help>] [B<--man>] [I<job_id>...]
 
+B<qstat> -Q [-f]
+
+B<qstat> -q
+
 =head1 DESCRIPTION
 
 The B<qstat> command displays information about jobs.
