@@ -1,36 +1,38 @@
 /*****************************************************************************\
  *  checkpoint.c - Process checkpoint related functions.
- *  $Id: checkpoint.c 13672 2008-03-19 23:10:58Z jette $
+ *  $Id: checkpoint.c 19095 2009-12-01 22:59:18Z da $
  *****************************************************************************
- *  Copyright (C) 2004 The Regents of the University of California.
+ *  Copyright (C) 2004-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov> et. al.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -58,29 +60,33 @@
 
 static int _handle_rc_msg(slurm_msg_t *msg);
 static int _checkpoint_op (uint16_t op, uint16_t data,
-		uint32_t job_id, uint32_t step_id);
+			   uint32_t job_id, uint32_t step_id,
+			   char *image_dir);
 /*
  * _checkpoint_op - perform many checkpoint operation for some job step.
- * IN op      - operation to perform
- * IN data    - operation-specific data
- * IN job_id  - job on which to perform operation
- * IN step_id - job step on which to perform operation
+ * IN op        - operation to perform
+ * IN data      - operation-specific data
+ * IN job_id    - job on which to perform operation
+ * IN step_id   - job step on which to perform operation
+ * IN image_dir - directory used to get/put checkpoint images
  * RET 0 or a slurm error code
  */
 static int _checkpoint_op (uint16_t op, uint16_t data,
-		uint32_t job_id, uint32_t step_id)
+			   uint32_t job_id, uint32_t step_id,
+			   char *image_dir)
 {
 	int rc;
 	checkpoint_msg_t ckp_req;
 	slurm_msg_t req_msg;
 
 	slurm_msg_t_init(&req_msg);
-	ckp_req.op       = op;
-	ckp_req.data     = data;
-	ckp_req.job_id   = job_id;
-	ckp_req.step_id  = step_id;
-	req_msg.msg_type = REQUEST_CHECKPOINT;
-	req_msg.data     = &ckp_req;
+	ckp_req.op        = op;
+	ckp_req.data      = data;
+	ckp_req.job_id    = job_id;
+	ckp_req.step_id   = step_id;
+	ckp_req.image_dir = image_dir;
+	req_msg.msg_type  = REQUEST_CHECKPOINT;
+	req_msg.data      = &ckp_req;
 
 	if (slurm_send_recv_controller_rc_msg(&req_msg, &rc) < 0)
 		return SLURM_ERROR;
@@ -97,7 +103,7 @@ static int _checkpoint_op (uint16_t op, uint16_t data,
  * OUT start_time - time at which checkpoint request was issued
  * RET 0 (can be checkpoined) or a slurm error code
  */
-extern int slurm_checkpoint_able (uint32_t job_id, uint32_t step_id, 
+extern int slurm_checkpoint_able (uint32_t job_id, uint32_t step_id,
 		time_t *start_time)
 {
 	int rc;
@@ -105,13 +111,14 @@ extern int slurm_checkpoint_able (uint32_t job_id, uint32_t step_id,
 	checkpoint_msg_t ckp_req;
 	checkpoint_resp_msg_t *resp;
 
-	ckp_req.op       = CHECK_ABLE;
-	ckp_req.job_id   = job_id;
-	ckp_req.step_id  = step_id;
+	ckp_req.op        = CHECK_ABLE;
+	ckp_req.job_id    = job_id;
+	ckp_req.step_id   = step_id;
+	ckp_req.image_dir = NULL;
 	slurm_msg_t_init(&req_msg);
 	slurm_msg_t_init(&resp_msg);
-	req_msg.msg_type = REQUEST_CHECKPOINT;
-	req_msg.data     = &ckp_req;
+	req_msg.msg_type  = REQUEST_CHECKPOINT;
+	req_msg.data      = &ckp_req;
 
 	if (slurm_send_recv_controller_msg(&req_msg, &resp_msg) < 0)
 		return SLURM_ERROR;
@@ -141,7 +148,7 @@ extern int slurm_checkpoint_able (uint32_t job_id, uint32_t step_id,
  */
 extern int slurm_checkpoint_disable (uint32_t job_id, uint32_t step_id)
 {
-	return _checkpoint_op (CHECK_DISABLE, 0, job_id, step_id);
+	return _checkpoint_op (CHECK_DISABLE, 0, job_id, step_id, NULL);
 }
 
 
@@ -153,21 +160,23 @@ extern int slurm_checkpoint_disable (uint32_t job_id, uint32_t step_id)
  */
 extern int slurm_checkpoint_enable (uint32_t job_id, uint32_t step_id)
 {
-	return _checkpoint_op (CHECK_ENABLE, 0, job_id, step_id);
+	return _checkpoint_op (CHECK_ENABLE, 0, job_id, step_id, NULL);
 }
 
 /*
  * slurm_checkpoint_create - initiate a checkpoint requests for some job step.
  *	the job will continue execution after the checkpoint operation completes
- * IN job_id  - job on which to perform operation
- * IN step_id - job step on which to perform operation
- * IN max_wait - maximum wait for operation to complete, in seconds
+ * IN job_id   - job on which to perform operation
+ * IN step_id  - job step on which to perform operation
+ * IN max_wait  - maximum wait for operation to complete, in seconds
+ * IN image_dir - directory used to get/put checkpoint images
  * RET 0 or a slurm error code
  */
-extern int slurm_checkpoint_create (uint32_t job_id, uint32_t step_id, 
-		uint16_t max_wait)
+extern int slurm_checkpoint_create (uint32_t job_id, uint32_t step_id,
+		uint16_t max_wait, char *image_dir)
 {
-	return _checkpoint_op (CHECK_CREATE, max_wait, job_id, step_id);
+	return _checkpoint_op (CHECK_CREATE, max_wait, job_id, step_id,
+			       image_dir);
 }
 
 /*
@@ -176,12 +185,14 @@ extern int slurm_checkpoint_create (uint32_t job_id, uint32_t step_id,
  * IN job_id  - job on which to perform operation
  * IN step_id - job step on which to perform operation
  * IN max_wait - maximum wait for operation to complete, in seconds
+ * IN image_dir - directory used to get/put checkpoint images
  * RET 0 or a slurm error code
  */
-extern int slurm_checkpoint_vacate (uint32_t job_id, uint32_t step_id, 
-		uint16_t max_wait)
+extern int slurm_checkpoint_vacate (uint32_t job_id, uint32_t step_id,
+		uint16_t max_wait, char *image_dir)
 {
-	return _checkpoint_op (CHECK_VACATE, max_wait, job_id, step_id);
+	return _checkpoint_op (CHECK_VACATE, max_wait, job_id, step_id,
+			       image_dir);
 }
 
 /*
@@ -190,9 +201,10 @@ extern int slurm_checkpoint_vacate (uint32_t job_id, uint32_t step_id,
  * IN step_id - job step on which to perform operation
  * RET 0 or a slurm error code
  */
-extern int slurm_checkpoint_restart (uint32_t job_id, uint32_t step_id)
+extern int slurm_checkpoint_restart (uint32_t job_id, uint32_t step_id,
+				     uint16_t stick, char *image_dir)
 {
-	return _checkpoint_op (CHECK_RESTART, 0, job_id, step_id);
+	return _checkpoint_op (CHECK_RESTART, stick, job_id, step_id, image_dir);
 }
 
 /*
@@ -229,7 +241,7 @@ extern int slurm_checkpoint_complete (uint32_t job_id, uint32_t step_id,
 }
 
 /*
- * slurm_checkpoint_error - gather error information for the last checkpoint 
+ * slurm_checkpoint_error - gather error information for the last checkpoint
  *	operation for some job step
  * IN job_id  - job on which to perform operation
  * IN step_id - job step on which to perform operation
@@ -237,11 +249,11 @@ extern int slurm_checkpoint_complete (uint32_t job_id, uint32_t step_id,
  *	this value is dependent upon the checkpoint plugin used and may be
  *	completely unrelated to slurm error codes, the highest value for all
  *	complete calls is preserved
- * OUT error_msg - error message, preserved for highest error_code, value 
+ * OUT error_msg - error message, preserved for highest error_code, value
  *	must be freed by the caller to prevent memory leak
  * RET 0 or a slurm error code
  */
-extern int slurm_checkpoint_error ( uint32_t job_id, uint32_t step_id, 
+extern int slurm_checkpoint_error ( uint32_t job_id, uint32_t step_id,
 		uint32_t *error_code, char **error_msg)
 {
 	int rc;
@@ -256,17 +268,18 @@ extern int slurm_checkpoint_error ( uint32_t job_id, uint32_t step_id,
 	/*
 	 * Request message:
 	 */
-	req.op       = CHECK_ERROR;
-	req.job_id   = job_id;
-	req.step_id  = step_id;
+	req.op        = CHECK_ERROR;
+	req.job_id    = job_id;
+	req.step_id   = step_id;
+	req.image_dir = NULL;
 	slurm_msg_t_init(&msg);
 	slurm_msg_t_init(&resp_msg);
-	msg.msg_type = REQUEST_CHECKPOINT;
-	msg.data     = &req;
+	msg.msg_type  = REQUEST_CHECKPOINT;
+	msg.data      = &req;
 
 	rc = slurm_send_recv_controller_msg(&msg, &resp_msg);
 
-	if (rc == SLURM_SOCKET_ERROR) 
+	if (rc == SLURM_SOCKET_ERROR)
 		return rc;
 
 	switch (resp_msg.msg_type) {
@@ -293,7 +306,7 @@ extern int slurm_checkpoint_error ( uint32_t job_id, uint32_t step_id,
 }
 
 /*
- *  Handle a return code message type. 
+ *  Handle a return code message type.
  *    Sets errno to return code and returns it
  */
 static int
@@ -317,7 +330,9 @@ _handle_rc_msg(slurm_msg_t *msg)
  * RET 0 or a slurm error code
  */
 extern int slurm_checkpoint_task_complete (uint32_t job_id, uint32_t step_id,
-		uint32_t task_id, time_t begin_time, uint32_t error_code, char *error_msg)
+					   uint32_t task_id, time_t begin_time,
+					   uint32_t error_code,
+					   char *error_msg)
 {
 	int rc;
 	slurm_msg_t msg;
@@ -341,77 +356,19 @@ extern int slurm_checkpoint_task_complete (uint32_t job_id, uint32_t step_id,
 }
 
 /*
- * slurm_get_checkpoint_file_path - return the checkpoint file
- *      path of this process, creating the directory if needed.
- * IN len: length of the file path buffer
- * OUT buf: buffer to store the checkpoint file path
- * RET: 0 on success, -1 on failure with errno set
+ * slurm_checkpoint_tasks - send checkpoint request to tasks of
+ *     specified step
+ * IN job_id: job ID of step
+ * IN step_id: step ID of step
+ * IN image_dir: location to store ckpt images. parameter to plugin.
+ * IN max_wait: seconds to wait for the operation to complete
+ * IN nodelist: nodes to send the request
+ * RET: 0 on success, non-zero on failure with errno set
  */
 extern int
-slurm_get_checkpoint_file_path(size_t len, char *buf)
+slurm_checkpoint_tasks(uint32_t job_id, uint16_t step_id, time_t begin_time,
+		       char *image_dir, uint16_t max_wait, char *nodelist)
 {
-       char *ckpt_path, *job_id, *step_id, *proc_id;
-       struct stat mystat;
-       int idx;
-
-       len --;                 /* for a terminating 0 */
-
-       ckpt_path = getenv("SLURM_CHECKPOINT_PATH");
-       if (ckpt_path == NULL) { /* this should not happen since the program may chdir */
-               ckpt_path = getcwd(buf, len);
-               if (ckpt_path == NULL)  /* ERANGE: len is too short */
-                       return -1;
-       } else {
-               if (snprintf(buf, len, "%s", ckpt_path) >= len) { /* glibc >= 2.1 */
-                       errno = ERANGE;
-                       return -1;
-               }
-               ckpt_path = buf;
-       }
-       idx = strlen(ckpt_path) - 1;
-       while (idx > 0 && ckpt_path[idx] == '/')
-               ckpt_path[idx --] = 0;
-
-       if (stat(ckpt_path, &mystat) < 0)
-               return -1;
-       if (! S_ISDIR(mystat.st_mode)) {
-               errno = ENOTDIR;
-               return -1;
-       }
-
-       job_id = getenv("SLURM_JOBID");
-       step_id = getenv("SLURM_STEPID");
-       proc_id = getenv("SLURM_PROCID");
-       if (job_id == NULL || step_id == NULL || proc_id == NULL) {
-               errno = ENODATA;
-               return -1;
-       }
-       idx = strlen(buf);
-       if (snprintf(buf + idx, len - idx, "/%s.%s", job_id, step_id) >= len - idx) {
-               errno = ERANGE;
-               return -1;
-       }
-
-       if (stat(buf, &mystat) < 0) {
-               if (errno == ENOENT) { /* dir does not exists */
-                       if (mkdir(buf, 0750) < 0 && errno != EEXIST)
-                               return -1;
-                       if (stat(buf, &mystat) < 0)
-                               return -1;
-               }
-               else
-                       return -1;
-       }
-       if (! S_ISDIR(mystat.st_mode)) {
-               errno = ENOTDIR;
-               return -1;
-       }
-
-       idx = strlen(buf);
-       if (snprintf(buf + idx, len - idx, "/%s.%s.ckpt", __progname, proc_id) >= len - idx) {
-               errno = ERANGE;
-               return -1;
-       }
-
-       return 0;
+	return checkpoint_tasks(job_id, step_id, begin_time,
+				image_dir, max_wait, nodelist);
 }
