@@ -1,36 +1,37 @@
 /*****************************************************************************\
  *  daemonize.c - daemonization routine
- *  $Id: daemonize.c 13672 2008-03-19 23:10:58Z jette $
+ *  $Id: daemonize.c 19095 2009-12-01 22:59:18Z da $
  *****************************************************************************
  *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark A. Grondona <mgrondona@llnl.gov>.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -42,6 +43,7 @@
 
 #include <fcntl.h>
 #include <unistd.h>
+#include <sys/resource.h>
 
 #include "src/common/daemonize.h"
 #include "src/common/fd.h"
@@ -55,7 +57,7 @@ closeall(int fd)
 {
 	int fdlimit = sysconf(_SC_OPEN_MAX);
 
-	while (fd < fdlimit) 
+	while (fd < fdlimit)
 		close(fd++);
 }
 
@@ -97,7 +99,7 @@ daemon(int nochdir, int noclose)
 		dup2(0, STDOUT_FILENO);
 		dup2(0, STDERR_FILENO);
 	} else {
-		/* 
+		/*
 		 * Otherwise, dup stdin, stdout, and stderr onto /dev/null
 		 */
 		int devnull = open("/dev/null", O_RDWR);
@@ -131,10 +133,10 @@ read_pidfile(const char *pidfile, int *pidfd)
 	unsigned long pid;
 	pid_t         lpid;
 
-	if ((fd = open(pidfile, O_RDONLY)) < 0) 
+	if ((fd = open(pidfile, O_RDONLY)) < 0)
 		return ((pid_t) 0);
 
-	if (!(fp = fdopen(fd, "r")) && (errno != ENOENT)) 
+	if (!(fp = fdopen(fd, "r")) && (errno != ENOENT))
 		error ("Unable to access old pidfile at `%s': %m", pidfile);
 
 	if (fscanf(fp, "%lu", &pid) < 1) {
@@ -149,13 +151,13 @@ read_pidfile(const char *pidfile, int *pidfd)
 		return ((pid_t) 0);
 	}
 
-	if (lpid != (pid_t) pid) 
+	if (lpid != (pid_t) pid)
 		fatal ("pidfile locked by %lu but contains pid=%lu",
 		       (unsigned long) lpid, (unsigned long) pid);
 
 	if (pidfd != NULL)
 		*pidfd = fd;
-	else 
+	else
 		(void) close(fd);
 
 	return (lpid);
@@ -189,7 +191,7 @@ create_pidfile(const char *pidfile)
 	}
 
 	fflush(fp);
-	
+
 	return (fileno(fp));
 
   error:
@@ -198,3 +200,20 @@ create_pidfile(const char *pidfile)
 	return -1;
 }
 
+void
+test_core_limit(void)
+{
+#ifdef RLIMIT_CORE
+	struct rlimit rlim[1];
+	if (getrlimit(RLIMIT_CORE, rlim) < 0)
+		error("Unable to get core limit");
+	else if (rlim->rlim_cur != RLIM_INFINITY) {
+		rlim->rlim_cur /= 1024;	/* bytes to KB */
+		if (rlim->rlim_cur < 2048) {
+			verbose("Warning: Core limit is only %u KB",
+				rlim->rlim_cur);
+		}
+	}
+#endif
+	return;
+}
