@@ -4,39 +4,40 @@
  *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 #if HAVE_CONFIG_H
 #  include <config.h>
-#endif 
+#endif
 
 #include <sys/poll.h>
 #include <unistd.h>
@@ -48,6 +49,7 @@
 #include "src/common/list.h"
 #include "src/common/fd.h"
 #include "src/common/eio.h"
+#include "src/common/slurm_protocol_api.h"
 
 /*
  * outside threads can stick new objects on the new_objs List and
@@ -108,10 +110,88 @@ void eio_handle_destroy(eio_handle_t *eio)
 	xfree(eio);
 }
 
+bool eio_message_socket_readable(eio_obj_t *obj)
+{
+	debug3("Called eio_message_socket_readable");
+	xassert(obj);
+	if (obj->shutdown == true) {
+		if (obj->fd != -1) {
+			debug2("  false, shutdown");
+			close(obj->fd);
+			obj->fd = -1;
+		} else {
+			debug2("  false");
+		}
+		return false;
+	}
+	return true;
+}
+
+int eio_message_socket_accept(eio_obj_t *obj, List objs)
+{
+	int fd;
+	unsigned char *uc;
+	unsigned short port;
+	struct sockaddr_in addr;
+	slurm_msg_t *msg = NULL;
+	int len = sizeof(addr);
+
+	debug3("Called eio_msg_socket_accept");
+
+	xassert(obj);
+	xassert(obj->ops->handle_msg);
+
+	while ((fd = accept(obj->fd, (struct sockaddr *)&addr,
+			    (socklen_t *)&len)) < 0) {
+		if (errno == EINTR)
+			continue;
+		if (errno == EAGAIN       ||
+		    errno == ECONNABORTED ||
+		    errno == EWOULDBLOCK) {
+			return SLURM_SUCCESS;
+		}
+		error("Error on msg accept socket: %m");
+		obj->shutdown = true;
+		return SLURM_SUCCESS;
+	}
+
+	fd_set_close_on_exec(fd);
+	fd_set_blocking(fd);
+
+	/* Should not call slurm_get_addr() because the IP may not be
+	   in /etc/hosts. */
+	uc = (unsigned char *)&addr.sin_addr.s_addr;
+	port = addr.sin_port;
+	debug2("got message connection from %u.%u.%u.%u:%hu",
+	       uc[0], uc[1], uc[2], uc[3], ntohs(port));
+	fflush(stdout);
+
+	msg = xmalloc(sizeof(slurm_msg_t));
+	slurm_msg_t_init(msg);
+again:
+	if(slurm_receive_msg(fd, msg, obj->ops->timeout) != 0) {
+		if (errno == EINTR) {
+			goto again;
+		}
+		error("slurm_receive_msg[%u.%u.%u.%u]: %m",
+		      uc[0],uc[1],uc[2],uc[3]);
+		goto cleanup;
+	}
+
+	(*obj->ops->handle_msg)(obj->arg, msg); /* handle_msg should free
+					      * msg->data */
+cleanup:
+	if ((msg->conn_fd >= 0) && slurm_close_accepted_conn(msg->conn_fd) < 0)
+		error ("close(%d): %m", msg->conn_fd);
+	slurm_free_msg(msg);
+
+	return SLURM_SUCCESS;
+}
+
 int eio_signal_shutdown(eio_handle_t *eio)
 {
 	char c = 1;
-	if (write(eio->fds[1], &c, sizeof(char)) != 1) 
+	if (write(eio->fds[1], &c, sizeof(char)) != 1)
 		return error("eio_handle_signal_shutdown: write; %m");
 	return 0;
 }
@@ -119,7 +199,7 @@ int eio_signal_shutdown(eio_handle_t *eio)
 int eio_signal_wakeup(eio_handle_t *eio)
 {
 	char c = 0;
-	if (write(eio->fds[1], &c, sizeof(char)) != 1) 
+	if (write(eio->fds[1], &c, sizeof(char)) != 1)
 		return error("eio_handle_signal_wake: write; %m");
 	return 0;
 }
@@ -176,15 +256,15 @@ int eio_handle_mainloop(eio_handle_t *eio)
 			maxnfds = n;
 			xrealloc(pollfds, (maxnfds+1) * sizeof(struct pollfd));
 			xrealloc(map,     maxnfds     * sizeof(eio_obj_t *  ));
-			/* 
-			 * Note: xrealloc() also handles initial malloc 
+			/*
+			 * Note: xrealloc() also handles initial malloc
 			 */
 		}
 
-		debug4("eio: handling events for %d objects", 
+		debug4("eio: handling events for %d objects",
 		       list_count(eio->obj_list));
 		nfds = _poll_setup_pollfds(pollfds, map, eio->obj_list);
-		if (nfds <= 0) 
+		if (nfds <= 0)
 			goto done;
 
 		/*
@@ -199,7 +279,7 @@ int eio_handle_mainloop(eio_handle_t *eio)
 		if (_poll_internal(pollfds, nfds) < 0)
 			goto error;
 
-		if (pollfds[nfds-1].revents & POLLIN) 
+		if (pollfds[nfds-1].revents & POLLIN)
 			_eio_wakeup_handler(eio);
 
 		_poll_dispatch(pollfds, nfds-1, map, eio->obj_list);
@@ -208,14 +288,14 @@ int eio_handle_mainloop(eio_handle_t *eio)
 	retval = -1;
   done:
 	xfree(pollfds);
-	xfree(map); 
+	xfree(map);
 	return retval;
 }
 
 static int
 _poll_internal(struct pollfd *pfds, unsigned int nfds)
-{               
-	int n;          
+{
+	int n;
 	while ((n = poll(pfds, nfds, -1)) < 0) {
 		switch (errno) {
 		case EINTR : return 0;
@@ -273,7 +353,7 @@ _poll_setup_pollfds(struct pollfd *pfds, eio_obj_t *map[], List l)
 }
 
 static void
-_poll_dispatch(struct pollfd *pfds, unsigned int nfds, eio_obj_t *map[], 
+_poll_dispatch(struct pollfd *pfds, unsigned int nfds, eio_obj_t *map[],
 	       List objList)
 {
 	int i;
@@ -395,7 +475,7 @@ void eio_new_initial_obj(eio_handle_t *eio, eio_obj_t *obj)
 {
 	xassert(eio != NULL);
 	xassert(eio->magic == EIO_MAGIC);
-	
+
 	list_enqueue(eio->obj_list, obj);
 }
 
