@@ -3,32 +3,33 @@
  *****************************************************************************
  *  Copyright (C) 2005 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -65,10 +66,16 @@ struct spank_launcher_job_info {
 
 int spank_init (slurmd_job_t *job);
 
+int spank_init_allocator (void);
+
+int spank_init_post_opt (void);
+
 int spank_user (slurmd_job_t *job);
 
 int spank_local_user (struct spank_launcher_job_info *job);
 
+int spank_task_privileged (slurmd_job_t *job, int taskid);
+
 int spank_user_task (slurmd_job_t *job, int taskid);
 
 int spank_task_post_fork (slurmd_job_t *job, int taskid);
@@ -84,7 +91,7 @@ int spank_fini (slurmd_job_t *job);
 /*
  *  Create a struct option table (suitable for passing to getopt_long())
  *   from SPANK plugin provided options, optionally prepending an existing
- *   table of options `orig_options'  Result must be freed by 
+ *   table of options `orig_options'  Result must be freed by
  *   spank_option_table_destroy().
  *
  *  If any options in orig_options conflict with internal spank options,
@@ -94,7 +101,7 @@ int spank_fini (slurmd_job_t *job);
 struct option *spank_option_table_create (const struct option *orig_options);
 
 /*
- *  Free memory associated with an option table created by 
+ *  Free memory associated with an option table created by
  *   spank_p[tion_table_create.
  */
 void spank_option_table_destroy (struct option *opt_table);
@@ -108,7 +115,7 @@ void spank_option_table_destroy (struct option *opt_table);
  *   the option belongs to a *required* plugin, and the plugin's callback
  *   for that option fails.
  */
-int spank_process_option (int optval, const char *optarg); 
+int spank_process_option (int optval, const char *optarg);
 
 /*
  *  Generate --help style output on stream `fp' for all internal spank
@@ -119,11 +126,17 @@ int spank_process_option (int optval, const char *optarg);
  */
 int spank_print_options (FILE *fp, int width, int left_pad);
 
-/*  Set all registered remote options (i.e. those passed to 
+/*  Set all registered remote options (i.e. those passed to
  *   spank_process_option) in the job options `options'.
  */
 int spank_set_remote_options (job_options_t options);
 
+/*  Set all registered remote options (i.e. those passed to
+ *   spank_process_option) in the current environment for later
+ *   retreival by spank_get_remote_options_env().
+ */
+int spank_set_remote_options_env (void);
+
 /*  Register any remote spank options that exist in `options'
  *    to their respective spank plugins. This function ends up invoking
  *    all plugin option callbacks, and will fail (return < 0) if
@@ -134,4 +147,14 @@ int spank_set_remote_options (job_options_t options);
  */
 int spank_get_remote_options (job_options_t options);
 
+/*  Register any remote spank options that exist in the environment `env'
+ *    to their respective spank plugins. This function ends up invoking
+ *    all plugin option callbacks, and will fail (return < 0) if
+ *    a *required* plugin callback returns < 0.
+ *
+ *  A warning is printed if no plugin matches a remote option
+ *   in the job_options structure, but the funtion does not return failure.
+ */
+int spank_get_remote_options_env (char **env);
+
 #endif /* !_PLUGSTACK_H */
