@@ -1,36 +1,37 @@
 /*****************************************************************************\
- *  src/common/slurm_cred.h  - SLURM job credential operations
- *  $Id: slurm_cred.h 14499 2008-07-11 22:54:48Z jette $
+ *  src/common/slurm_cred.h - SLURM job and sbcast credential functions
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <grondona1@llnl.gov>.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -51,6 +52,7 @@
 #  include <sys/types.h>
 #endif
 
+#include "src/common/bitstring.h"
 #include "src/common/macros.h"
 #include "src/common/pack.h"
 
@@ -61,10 +63,20 @@
  */
 #ifndef __slurm_cred_t_defined
 #  define __slurm_cred_t_defined
-   typedef struct slurm_job_credential * slurm_cred_t;
+   typedef struct slurm_job_credential slurm_cred_t;
 #endif
 
-/* 
+/*
+ * The incomplete slurm_cred_t type is also defined in slurm_protocol_defs.h
+ * so check to ensure that this header has not been included after
+ * slurm_protocol_defs.h:
+ */
+#ifndef __sbcast_cred_t_defined
+#  define  __sbcast_cred_t_defined
+   typedef struct sbcast_cred *sbcast_cred_t;		/* opaque data type */
+#endif
+
+/*
  * The slurm_cred_ctx_t incomplete type
  */
 typedef struct slurm_cred_context   * slurm_cred_ctx_t;
@@ -74,7 +86,7 @@ typedef struct slurm_cred_context   * slurm_cred_ctx_t;
  * Initialize current process for slurm credential creation.
  *
  * `privkey' contains the absolute path to the slurmctld private
- * key, which needs to be readable by the current process. 
+ * key, which needs to be readable by the current process.
  *
  * Returns 0 for success, -1 on failure and sets errno to reason.
  *
@@ -107,16 +119,16 @@ int slurm_cred_ctx_get(slurm_cred_ctx_t ctx, slurm_cred_opt_t opt, ...);
 int slurm_cred_ctx_key_update(slurm_cred_ctx_t ctx, const char *keypath);
 
 
-/* 
+/*
  * Destroy a credential context, freeing associated memory.
  */
 void slurm_cred_ctx_destroy(slurm_cred_ctx_t ctx);
 
 /*
- * Pack and unpack slurm credential context. 
+ * Pack and unpack slurm credential context.
  *
  * On pack() ctx is packed in machine-independent format into the
- * buffer, on unpack() the contents of the buffer are used to 
+ * buffer, on unpack() the contents of the buffer are used to
  * intialize the state of the context ctx.
  */
 int  slurm_cred_ctx_pack(slurm_cred_ctx_t ctx, Buf buffer);
@@ -124,17 +136,28 @@ int  slurm_cred_ctx_unpack(slurm_cred_ctx_t ctx, Buf buffer);
 
 
 /*
- * Container for SLURM credential create and verify arguments:
+ * Container for SLURM credential create and verify arguments
+ *
+ * The core_bitmap, cores_per_socket, sockets_per_node, and
+ * sock_core_rep_count is based upon the nodes allocated to the
+ * JOB, but the bits set in core_bitmap are those cores allocated
+ * to this STEP
  */
 typedef struct {
 	uint32_t jobid;
 	uint32_t stepid;
-	uint32_t job_mem;	/* MB of memory reserved for job */
-	uint32_t task_mem;	/* MB of memory reserved per task */
+	uint32_t job_mem;	/* MB of memory reserved per node OR
+				 * real memory per CPU | MEM_PER_CPU,
+				 * default=0 (no limit) */
 	uid_t    uid;
 	char    *hostlist;
-	uint32_t alloc_lps_cnt;
-        uint32_t *alloc_lps;
+
+	bitstr_t *core_bitmap;
+	uint16_t *cores_per_socket;
+	uint16_t *sockets_per_node;
+	uint32_t *sock_core_rep_count;
+	uint32_t  job_nhosts;	/* count of nodes allocated to JOB */
+	char     *job_hostlist;	/* list of nodes allocated to JOB */
 } slurm_cred_arg_t;
 
 /* Terminate the plugin and release all memory. */
@@ -149,13 +172,13 @@ int slurm_crypto_fini(void);
  *
  * Returns NULL on failure.
  */
-slurm_cred_t slurm_cred_create(slurm_cred_ctx_t ctx, slurm_cred_arg_t *arg);
+slurm_cred_t *slurm_cred_create(slurm_cred_ctx_t ctx, slurm_cred_arg_t *arg);
 
 /*
  * Copy a slurm credential.
  * Returns NULL on failure.
  */
-slurm_cred_t slurm_cred_copy(slurm_cred_t cred);
+slurm_cred_t *slurm_cred_copy(slurm_cred_t *cred);
 
 /*
  * Create a "fake" credential with bogus data in the signature.
@@ -163,14 +186,14 @@ slurm_cred_t slurm_cred_copy(slurm_cred_t cred);
  * to talk to slurmd directly, bypassing the controller
  * (which normally signs creds)
  */
-slurm_cred_t slurm_cred_faker(slurm_cred_arg_t *arg);
+slurm_cred_t *slurm_cred_faker(slurm_cred_arg_t *arg);
 
 /* Free the credential arguments as loaded by either
  * slurm_cred_get_args() or slurm_cred_verify() */
 void slurm_cred_free_args(slurm_cred_arg_t *arg);
 
 /* Make a copy of the credential's arguements */
-int slurm_cred_get_args(slurm_cred_t cred, slurm_cred_arg_t *arg);
+int slurm_cred_get_args(slurm_cred_t *cred, slurm_cred_arg_t *arg);
 
 /*
  * Verify the signed credential `cred,' and return cred contents in
@@ -183,7 +206,7 @@ int slurm_cred_get_args(slurm_cred_t cred, slurm_cred_arg_t *arg);
  *   - Credential has not been revoked
  *   - Credential has not been replayed
  */
-int slurm_cred_verify(slurm_cred_ctx_t ctx, slurm_cred_t cred, 
+int slurm_cred_verify(slurm_cred_ctx_t ctx, slurm_cred_t *cred,
 		      slurm_cred_arg_t *arg);
 
 /*
@@ -191,7 +214,7 @@ int slurm_cred_verify(slurm_cred_ctx_t ctx, slurm_cred_t cred,
  *  be used again. Returns SLURM_FAILURE if no credential state is found
  *  to be rewound, SLURM_SUCCESS otherwise.
  */
-int slurm_cred_rewind(slurm_cred_ctx_t ctx, slurm_cred_t cred);
+int slurm_cred_rewind(slurm_cred_ctx_t ctx, slurm_cred_t *cred);
 
 /*
  * Check to see if this credential is a reissue of an existing credential
@@ -199,7 +222,7 @@ int slurm_cred_rewind(slurm_cred_ctx_t ctx, slurm_cred_t cred);
  * this credential is a reissue, then the old credential is cleared
  * from the cred context "ctx".
  */
-void slurm_cred_handle_reissue(slurm_cred_ctx_t ctx, slurm_cred_t cred);
+void slurm_cred_handle_reissue(slurm_cred_ctx_t ctx, slurm_cred_t *cred);
 
 /*
  * Revoke all credentials for job id jobid
@@ -209,14 +232,14 @@ void slurm_cred_handle_reissue(slurm_cred_ctx_t ctx, slurm_cred_t cred);
 int slurm_cred_revoke(slurm_cred_ctx_t ctx, uint32_t jobid, time_t time);
 
 /*
- * Report if a all credentials for a give job id have been 
+ * Report if a all credentials for a give job id have been
  * revoked (i.e. has the job been killed)
- * 
+ *
  * If we are re-running the job, the new job credential is newer
- * than the revoke time, see "scontrol requeue", purge the old 
+ * than the revoke time, see "scontrol requeue", purge the old
  * job record and make like it never existed
  */
-bool slurm_cred_revoked(slurm_cred_ctx_t ctx, slurm_cred_t cred);
+bool slurm_cred_revoked(slurm_cred_ctx_t ctx, slurm_cred_t *cred);
 
 /*
  * Begin expiration period for the revocation of credentials
@@ -229,7 +252,7 @@ bool slurm_cred_revoked(slurm_cred_ctx_t ctx, slurm_cred_t cred);
  *
  *  ESRCH  if jobid is not cached
  *  EEXIST if expiration period has already begun for jobid.
- * 
+ *
  */
 int slurm_cred_begin_expiration(slurm_cred_ctx_t ctx, uint32_t jobid);
 
@@ -250,35 +273,54 @@ int slurm_cred_insert_jobid(slurm_cred_ctx_t ctx, uint32_t jobid);
 
 /* Free memory associated with slurm credential `cred.'
  */
-void slurm_cred_destroy(slurm_cred_t cred);
+void slurm_cred_destroy(slurm_cred_t *cred);
 
 /*
  * Pack a slurm credential for network transmission
  */
-void slurm_cred_pack(slurm_cred_t cred, Buf buffer);
+void slurm_cred_pack(slurm_cred_t *cred, Buf buffer);
 
 /*
  * Unpack a slurm job credential
  */
-slurm_cred_t slurm_cred_unpack(Buf buffer);
+slurm_cred_t *slurm_cred_unpack(Buf buffer);
 
 /*
  * Get a pointer to the slurm credential signature
  * (used by slurm IO connections to verify connecting agent)
  */
-int slurm_cred_get_signature(slurm_cred_t cred, char **datap, int *len);
+int slurm_cred_get_signature(slurm_cred_t *cred, char **datap,
+			     uint32_t *len);
 
+/*
+ * Retrieve the set of cores that were allocated to the job and format them
+ * in the List Format (e.g., "0-2,7,12-14").
+ *
+ * NOTE: caller must xfree the returned string.
+ */
+char* format_core_allocs(slurm_cred_t *cred, char *node_name);
 
 /*
  * Print a slurm job credential using the info() call
  */
-void slurm_cred_print(slurm_cred_t cred);
+void slurm_cred_print(slurm_cred_t *cred);
 
 /*
- * Get count of allocated LPS (processors) by node
+ * Functions to create, delete, pack, and unpack an sbcast credential
+ * Caller of extract_sbcast_cred() must xfree returned node string
  */
-int slurm_cred_get_alloc_lps(slurm_cred_t cred, char **nodes, 
-			     uint32_t *alloc_lps_cnt, uint32_t **alloc_lps);
+sbcast_cred_t *create_sbcast_cred(slurm_cred_ctx_t ctx,
+				 uint32_t job_id, char *nodes);
+sbcast_cred_t *copy_sbcast_cred(sbcast_cred_t *sbcast_cred);
+void          delete_sbcast_cred(sbcast_cred_t *sbcast_cred);
+int           extract_sbcast_cred(slurm_cred_ctx_t ctx,
+				  sbcast_cred_t *sbcast_cred, uint16_t block_no,
+				  uint32_t *job_id, char **nodes);
+void          pack_sbcast_cred(sbcast_cred_t *sbcast_cred, Buf buffer);
+sbcast_cred_t *unpack_sbcast_cred(Buf buffer);
+void          print_sbcast_cred(sbcast_cred_t *sbcast_cred);
+
+
 #ifdef DISABLE_LOCALTIME
 extern char * timestr (const time_t *tp, char *buf, size_t n);
 #endif
