@@ -7,31 +7,32 @@
  *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
- *  
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -58,14 +59,14 @@
  * where <application> is a description of the intended application of
  * the plugin (e.g., "jobacct" for SLURM job completion logging) and <method>
  * is a description of how this plugin satisfies that application.  SLURM will
- * only load job completion logging plugins if the plugin_type string has a 
+ * only load job completion logging plugins if the plugin_type string has a
  * prefix of "jobacct/".
  *
  * plugin_version - an unsigned 32-bit integer giving the version number
  * of the plugin.  If major and minor revisions are desired, the major
  * version number may be multiplied by a suitable magnitude constant such
  * as 100 or 1000.  Various SLURM versions will likely require a certain
- * minimum versions for their plugins as the job accounting API 
+ * minimum versions for their plugins as the job accounting API
  * matures.
  */
 const char plugin_name[] = "Accounting storage NOT INVOKED plugin";
@@ -133,18 +134,24 @@ extern int acct_storage_p_add_associations(void *db_conn, uint32_t uid,
 	return SLURM_SUCCESS;
 }
 
-extern int acct_storage_p_add_qos(void *db_conn, uint32_t uid, 
+extern int acct_storage_p_add_qos(void *db_conn, uint32_t uid,
 				  List qos_list)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int acct_storage_p_add_wckeys(void *db_conn, uint32_t uid, 
+extern int acct_storage_p_add_wckeys(void *db_conn, uint32_t uid,
 				  List wckey_list)
 {
 	return SLURM_SUCCESS;
 }
 
+extern int acct_storage_p_add_reservation(void *db_conn,
+					  acct_reservation_rec_t *resv)
+{
+	return SLURM_SUCCESS;
+}
+
 extern List acct_storage_p_modify_users(void *db_conn, uint32_t uid,
 				       acct_user_cond_t *user_q,
 				       acct_user_rec_t *user)
@@ -187,6 +194,12 @@ extern List acct_storage_p_modify_wckeys(void *db_conn, uint32_t uid,
 	return SLURM_SUCCESS;
 }
 
+extern int acct_storage_p_modify_reservation(void *db_conn,
+					     acct_reservation_rec_t *resv)
+{
+	return SLURM_SUCCESS;
+}
+
 extern List acct_storage_p_remove_users(void *db_conn, uint32_t uid,
 				       acct_user_cond_t *user_q)
 {
@@ -194,7 +207,7 @@ extern List acct_storage_p_remove_users(void *db_conn, uint32_t uid,
 }
 
 extern List acct_storage_p_remove_coord(void *db_conn, uint32_t uid,
-					List acct_list, 
+					List acct_list,
 					acct_user_cond_t *user_q)
 {
 	return SLURM_SUCCESS;
@@ -218,18 +231,24 @@ extern List acct_storage_p_remove_associations(void *db_conn, uint32_t uid,
 	return SLURM_SUCCESS;
 }
 
-extern List acct_storage_p_remove_qos(void *db_conn, uint32_t uid, 
+extern List acct_storage_p_remove_qos(void *db_conn, uint32_t uid,
 				      acct_qos_cond_t *qos_cond)
 {
 	return NULL;
 }
 
-extern List acct_storage_p_remove_wckeys(void *db_conn, uint32_t uid, 
+extern List acct_storage_p_remove_wckeys(void *db_conn, uint32_t uid,
 				      acct_wckey_cond_t *wckey_cond)
 {
 	return NULL;
 }
 
+extern int acct_storage_p_remove_reservation(void *db_conn,
+					     acct_reservation_rec_t *resv)
+{
+	return SLURM_SUCCESS;
+}
+
 extern List acct_storage_p_get_users(void *db_conn, uid_t uid,
 				     acct_user_cond_t *user_q)
 {
@@ -248,12 +267,23 @@ extern List acct_storage_p_get_clusters(void *db_conn, uid_t uid,
 	return NULL;
 }
 
+extern List acct_storage_p_get_config(void *db_conn)
+{
+	return NULL;
+}
+
 extern List acct_storage_p_get_associations(void *db_conn, uid_t uid,
 					    acct_association_cond_t *assoc_q)
 {
 	return NULL;
 }
 
+extern List acct_storage_p_get_problems(void *db_conn, uid_t uid,
+					acct_association_cond_t *assoc_q)
+{
+	return NULL;
+}
+
 extern List acct_storage_p_get_qos(void *db_conn, uid_t uid,
 				   acct_qos_cond_t *qos_cond)
 {
@@ -266,6 +296,12 @@ extern List acct_storage_p_get_wckeys(void *db_conn, uid_t uid,
 	return NULL;
 }
 
+extern List acct_storage_p_get_reservations(void *mysql_conn, uid_t uid,
+					    acct_reservation_cond_t *resv_cond)
+{
+	return NULL;
+}
+
 extern List acct_storage_p_get_txn(void *db_conn, uid_t uid,
 				   acct_txn_cond_t *txn_cond)
 {
@@ -281,8 +317,9 @@ extern int acct_storage_p_get_usage(void *db_conn, uid_t uid,
 	return rc;
 }
 
-extern int acct_storage_p_roll_usage(void *db_conn, 
-				     time_t sent_start)
+extern int acct_storage_p_roll_usage(void *db_conn,
+				     time_t sent_start, time_t sent_end,
+				     uint16_t archive_data)
 {
 	int rc = SLURM_SUCCESS;
 
@@ -313,6 +350,7 @@ extern int clusteracct_storage_p_register_ctld(void *db_conn,
 
 extern int clusteracct_storage_p_cluster_procs(void *db_conn,
 					       char *cluster,
+					       char *cluster_nodes,
 					       uint32_t procs,
 					       time_t event_time)
 {
@@ -327,7 +365,7 @@ extern int clusteracct_storage_p_get_usage(
 	return SLURM_SUCCESS;
 }
 
-/* 
+/*
  * load into the storage the start of a job
  */
 extern int jobacct_storage_p_job_start(void *db_conn, char *cluster_name,
@@ -336,7 +374,7 @@ extern int jobacct_storage_p_job_start(void *db_conn, char *cluster_name,
 	return SLURM_SUCCESS;
 }
 
-/* 
+/*
  * load into the storage the end of a job
  */
 extern int jobacct_storage_p_job_complete(void *db_conn,
@@ -345,7 +383,7 @@ extern int jobacct_storage_p_job_complete(void *db_conn,
 	return SLURM_SUCCESS;
 }
 
-/* 
+/*
  * load into the storage the start of a job step
  */
 extern int jobacct_storage_p_step_start(void *db_conn,
@@ -354,7 +392,7 @@ extern int jobacct_storage_p_step_start(void *db_conn,
 	return SLURM_SUCCESS;
 }
 
-/* 
+/*
  * load into the storage the end of a job step
  */
 extern int jobacct_storage_p_step_complete(void *db_conn,
@@ -363,7 +401,7 @@ extern int jobacct_storage_p_step_complete(void *db_conn,
 	return SLURM_SUCCESS;
 }
 
-/* 
+/*
  * load into the storage a suspention of a job
  */
 extern int jobacct_storage_p_suspend(void *db_conn,
@@ -372,8 +410,8 @@ extern int jobacct_storage_p_suspend(void *db_conn,
 	return SLURM_SUCCESS;
 }
 
-/* 
- * get info from the storage 
+/*
+ * get info from the storage
  * returns List of jobacct_job_rec_t *
  * note List needs to be freed when called
  */
@@ -383,8 +421,8 @@ extern List jobacct_storage_p_get_jobs_cond(void *db_conn, uid_t uid,
 	return NULL;
 }
 
-/* 
- * expire old info from the storage 
+/*
+ * expire old info from the storage
  */
 extern int jobacct_storage_p_archive(void *db_conn,
 				     acct_archive_cond_t *arch_cond)
@@ -392,10 +430,10 @@ extern int jobacct_storage_p_archive(void *db_conn,
 	return SLURM_SUCCESS;
 }
 
-/* 
- * load old info into the storage 
+/*
+ * load old info into the storage
  */
-extern int jobacct_storage_p_archive_load(void *db_conn, 
+extern int jobacct_storage_p_archive_load(void *db_conn,
 					  acct_archive_rec_t *arch_rec)
 {
 	return SLURM_SUCCESS;
