@@ -1,35 +1,37 @@
 /*****************************************************************************\
  *  auth_authd - plugin for Brent Chun's authd
  *****************************************************************************
- *  Copyright (C) 2002 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Kevin Tew <tew1@llnl.gov> et. al.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -62,9 +64,36 @@
 #include <slurm/slurm_errno.h>
 #include "src/common/slurm_xlator.h"
 
-const char plugin_name[]	= "Brent Chun's authd authentication plugin";
-const char plugin_type[]	= "auth/authd";
-const uint32_t plugin_version = 90;
+/*
+ * These variables are required by the generic plugin interface.  If they
+ * are not found in the plugin, the plugin loader will ignore it.
+ *
+ * plugin_name - a string giving a human-readable description of the
+ * plugin.  There is no maximum length, but the symbol must refer to
+ * a valid string.
+ *
+ * plugin_type - a string suggesting the type of the plugin or its
+ * applicability to a particular form of data or method of data handling.
+ * If the low-level plugin API is used, the contents of this string are
+ * unimportant and may be anything.  SLURM uses the higher-level plugin
+ * interface which requires this string to be of the form
+ *
+ *	<application>/<method>
+ *
+ * where <application> is a description of the intended application of
+ * the plugin (e.g., "auth" for SLURM authentication) and <method> is a
+ * description of how this plugin satisfies that application.  SLURM will
+ * only load authentication plugins if the plugin_type string has a prefix
+ * of "auth/".
+ *
+ * plugin_version   - specifies the version number of the plugin.
+ * min_plug_version - specifies the minumum version number of incomming
+ *                    messages that this plugin can accept
+ */
+const char plugin_name[]        = "Brent Chun's authd authentication plugin";
+const char plugin_type[]        = "auth/authd";
+const uint32_t plugin_version   = 100;
+const uint32_t min_plug_version = 90;
 
 /*
  * Where to find the timeout in the argument vector.  This is set
@@ -97,7 +126,7 @@ enum {
 int init( void )
 {
 	const arg_desc_t *desc;
-	
+
 	verbose( "authd authentication module initializing" );
 
 	if ( ( desc = slurm_auth_get_arg_desc() ) == NULL ) {
@@ -109,7 +138,7 @@ int init( void )
 		error( "Required argument 'Timeout' not provided" );
 		return SLURM_ERROR;
 	}
-	
+
 	return SLURM_SUCCESS;
 }
 
@@ -121,7 +150,7 @@ extern int fini ( void )
 slurm_auth_credential_t *
 slurm_auth_create( void *argv[], char *auth_info )
 {
-	int ttl;	
+	int ttl;
 	int rc;
 	slurm_auth_credential_t *cred;
 
@@ -162,7 +191,7 @@ slurm_auth_create( void *argv[], char *auth_info )
 		xfree( cred );
 		return NULL;
 	}
-	
+
 	return cred;
 }
 
@@ -178,12 +207,12 @@ slurm_auth_destroy( slurm_auth_credential_t *cred )
 }
 
 int
-slurm_auth_verify( slurm_auth_credential_t *cred, void *argv[], char *auth_info )
+slurm_auth_verify( slurm_auth_credential_t *cred, char *auth_info )
 {
 	int rc;
 	time_t now;
 
-	if ( ( cred == NULL ) || ( argv == NULL ) ) {
+	if ( cred == NULL ) {
 		plugin_errno = SLURM_AUTH_BADARG;
 		return SLURM_ERROR;
 	}
@@ -200,7 +229,7 @@ slurm_auth_verify( slurm_auth_credential_t *cred, void *argv[], char *auth_info
 	}
 
 	/* XXX check to see if user is valid on the system. */
-	
+
 	return SLURM_SUCCESS;
 }
 
@@ -244,7 +273,7 @@ slurm_auth_pack( slurm_auth_credential_t *cred, Buf buf )
 	 */
 	packmem( (char *) plugin_type, strlen( plugin_type ) + 1, buf );
 	pack32( plugin_version, buf );
-	
+
 	pack32( (uint32_t) cred->cred.uid, buf );
 	pack32( (uint32_t) cred->cred.gid, buf );
 	pack_time( cred->cred.valid_from, buf );
@@ -260,7 +289,7 @@ slurm_auth_unpack( Buf buf )
 {
 	slurm_auth_credential_t *cred = NULL;
 	uint16_t sig_size; /* ignored */
-	uint32_t version, tmpint;	
+	uint32_t version, tmpint;
 	char *data;
 
 	if ( buf == NULL ) {
@@ -268,7 +297,7 @@ slurm_auth_unpack( Buf buf )
 		return NULL;
 	}
 
-	
+
 	/* Check the plugin type. */
 	safe_unpackmem_ptr( &data, &sig_size, buf );
 	if ( strcmp( data, plugin_type ) != 0 ) {
@@ -277,8 +306,8 @@ slurm_auth_unpack( Buf buf )
 	}
 
 	safe_unpack32( &version, buf );
-	if( version != plugin_version ) {
-		plugin_errno = SLURM_AUTH_MISMATCH;
+	if( version < min_plug_version ) {
+		plugin_errno = SLURM_AUTH_VERSION;
 		return NULL;
 	}
 
@@ -323,7 +352,7 @@ slurm_auth_print( slurm_auth_credential_t *cred, FILE *fp )
 			 cred->sig.data[ 2 ], cred->sig.data[ 3 ] );
 	verbose( "END AUTHD CREDENTIAL\n" );
 
-	return SLURM_SUCCESS;	
+	return SLURM_SUCCESS;
 }
 
 
