@@ -1,35 +1,37 @@
 /*****************************************************************************\
  *  auth_none.c - NO-OP slurm authentication plugin, validates all users.
  *****************************************************************************
- *  Copyright (C) 2002 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Kevin Tew <tew1@llnl.gov> et. al.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -87,16 +89,14 @@
  * only load authentication plugins if the plugin_type string has a prefix
  * of "auth/".
  *
- * plugin_version - an unsigned 32-bit integer giving the version number
- * of the plugin.  If major and minor revisions are desired, the major
- * version number may be multiplied by a suitable magnitude constant such
- * as 100 or 1000.  Various SLURM versions will likely require a certain
- * minimum versions for their plugins as the authentication API matures.
+ * plugin_version   - specifies the version number of the plugin.
+ * min_plug_version - specifies the minumum version number of incomming
+ *                    messages that this plugin can accept
  */
 const char plugin_name[]       	= "Null authentication plugin";
 const char plugin_type[]       	= "auth/none";
-const uint32_t plugin_version	= 90;
-
+const uint32_t plugin_version   = 100;
+const uint32_t min_plug_version = 90;
 
 /*
  * An opaque type representing authentication credentials.  This type can be
@@ -176,7 +176,7 @@ slurm_auth_create( void *argv[], char *auth_info )
 {
 	slurm_auth_credential_t *cred;
 
-	cred = ((slurm_auth_credential_t *) 
+	cred = ((slurm_auth_credential_t *)
 		xmalloc( sizeof( slurm_auth_credential_t ) ));
 	cred->cr_errno = SLURM_SUCCESS;
 	cred->uid = geteuid();
@@ -205,7 +205,7 @@ slurm_auth_destroy( slurm_auth_credential_t *cred )
  * Return SLURM_SUCCESS if the credential is in order and valid.
  */
 int
-slurm_auth_verify( slurm_auth_credential_t *cred, void *argv[], char *auth_info )
+slurm_auth_verify( slurm_auth_credential_t *cred, char *auth_info )
 {
 	return SLURM_SUCCESS;
 }
@@ -251,7 +251,7 @@ slurm_auth_pack( slurm_auth_credential_t *cred, Buf buf )
 		plugin_errno = SLURM_AUTH_BADARG;
 		return SLURM_ERROR;
 	}
-	
+
 	/*
 	 * Prefix the credential with a description of the credential
 	 * type so that it can be sanity-checked at the receiving end.
@@ -279,7 +279,7 @@ slurm_auth_unpack( Buf buf )
 	uint32_t tmpint;
 	uint32_t version;
 	uint32_t size;
-	
+
 	if ( buf == NULL ) {
 		plugin_errno = SLURM_AUTH_BADARG;
 		return NULL;
@@ -295,8 +295,8 @@ slurm_auth_unpack( Buf buf )
 		return NULL;
 	}
 	safe_unpack32( &version, buf );
-	if ( version != plugin_version ) {
-		plugin_errno = SLURM_AUTH_MISMATCH;
+	if ( version < min_plug_version ) {
+		plugin_errno = SLURM_AUTH_VERSION;
 		return NULL;
 	}
 
