@@ -1,36 +1,37 @@
 /*****************************************************************************\
- **  mpi_mpichgm.c - Library routines for initiating jobs on with gmpi type mpi 
+ **  mpi_mpichgm.c - Library routines for initiating jobs on with gmpi type mpi
  **  $Id$
  *****************************************************************************
  *  Copyright (C) 2004 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -66,7 +67,7 @@
  *      <application>/<method>
  *
  * where <application> is a description of the intended application of
- * the plugin (e.g., "switch" for SLURM switch) and <method> is a description 
+ * the plugin (e.g., "switch" for SLURM switch) and <method> is a description
  * of how this plugin satisfies that application.  SLURM will only load
  * a switch plugin if the plugin_type string has a prefix of "switch/".
  *
@@ -86,28 +87,28 @@ int p_mpi_hook_slurmstepd_task(const mpi_plugin_task_info_t *job,
 	char addrbuf[1024];
 	char *p;
 	char *addr = getenvp(*env, "SLURM_LAUNCH_NODE_IPADDR");
-	
+
 	debug("Using mpi/mpich-gm");
 	slurm_print_slurm_addr (job->self, addrbuf, sizeof(addrbuf));
-	
+
 	if ((p = strchr (addrbuf, ':')) != NULL)
 		*p = '\0';
-		
+
 	env_array_overwrite_fmt(env, "GMPI_MASTER", "%s", addr);
 	env_array_overwrite_fmt(env, "GMPI_SLAVE",  "%s", addrbuf);
 	env_array_overwrite_fmt(env, "GMPI_ID",  "%u", job->gtaskid);
 	if (!getenv("GMPI_RECV")) {
-		env_array_overwrite_fmt(env, "GMPI_RECV",  "%u", "hybrid");
+		env_array_overwrite_fmt(env, "GMPI_RECV",  "%s", "hybrid");
 	}
 
 	env_array_overwrite_fmt(env, "MXMPI_MASTER", "%s", addr);
 	env_array_overwrite_fmt(env, "MXMPI_ID", "%u", job->gtaskid);
 	env_array_overwrite_fmt(env, "MXMPI_SLAVE", "%s", addrbuf);
 	if (!getenv("MXMPI_RECV")) {
-		env_array_overwrite_fmt(env, "MXMPI_RECV",  "%u", "hybrid");
+		env_array_overwrite_fmt(env, "MXMPI_RECV",  "%s", "hybrid");
 	}
 	debug2("init for mpi rank %u\n", job->gtaskid);
-	
+
 	return SLURM_SUCCESS;
 }
 
