@@ -4,32 +4,33 @@
  *  Copyright (C) 2004 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Takao Hatazaki <takao.hatazaki@hp.com>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -100,7 +101,7 @@ static int _gmpi_parse_init_recv_msg(mpi_plugin_client_info_t *job, char *rbuf,
 		fatal("GMPI id is out of range");
 #if 0
 	/* Unlike GM ports, MX endpoints can be 0,
-	 * Pere Munt, BSC-CMS */ 
+	 * Pere Munt, BSC-CMS */
 	if (port_board_id == 0)
 		fatal("MPI id=%d was unable to open a GM port", id);
 #endif
@@ -137,7 +138,7 @@ static int _gmpi_establish_map(gmpi_state_t *st)
 	char *p, *rbuf = NULL, *gmap = NULL, *lmap = NULL, *map = NULL;
 	char tmp[128];
 	gm_slave_t *slave_data = NULL, *dp;
-	
+
 	/*
 	 * Collect info from slaves.
 	 * Will never finish unless slaves are GMPI processes.
@@ -151,7 +152,7 @@ static int _gmpi_establish_map(gmpi_state_t *st)
 		slave_data[i].defined = 0;
 	i = 0;
 	rbuf = (char *)xmalloc(GMPI_RECV_BUF_LEN);
-	
+
 	while (i < nprocs) {
 		newfd = accept(accfd, (struct sockaddr *)&addr, &addrlen);
 		if (newfd == -1) {
@@ -229,7 +230,7 @@ static int _gmpi_establish_map(gmpi_state_t *st)
 		if (setsockopt(newfd, SOL_SOCKET, SO_REUSEADDR,
 			       (void *)&j, sizeof(j)))
 			error("setsockopt in GMPI master: %m");
-		bzero(&addr, sizeof(addr));
+		memset(&addr, 0, sizeof(addr));
 		addr.sin_family = AF_INET;
 		addr.sin_addr.s_addr = htonl(iaddrs[i]);
 		addr.sin_port = htons(dp->remote_port);
@@ -306,7 +307,7 @@ static void *_gmpi_thr(void *arg)
 
 	debug3("GMPI master thread pid=%lu", (unsigned long) getpid());
 	_gmpi_establish_map(st);
-	
+
 	debug3("GMPI master thread is waiting for ABORT message.");
 	_gmpi_wait_abort(st);
 
@@ -371,7 +372,7 @@ gmpi_thr_create(const mpi_plugin_client_info_t *job, char ***env)
 
 	env_array_overwrite_fmt(env, "GMPI_PORT",  "%hu", port);
 	env_array_overwrite_fmt(env, "GMPI_MAGIC", "%u", job->jobid);
-	env_array_overwrite_fmt(env, "GMPI_NP",    "%d", 
+	env_array_overwrite_fmt(env, "GMPI_NP",    "%d",
 				job->step_layout->task_cnt);
 	env_array_overwrite_fmt(env, "GMPI_SHMEM", "1");
 	/* FIXME for multi-board config. */
@@ -385,7 +386,7 @@ gmpi_thr_create(const mpi_plugin_client_info_t *job, char ***env)
 				job->step_layout->task_cnt);
 	/* FIXME for multi-board config. */
 	env_array_overwrite_fmt(env, "MXMPI_BOARD", "-1");
- 
+
 
 	/* for MACOSX to override default malloc */
 	env_array_overwrite_fmt(env, "DYLD_FORCE_FLAT_NAMESPACE", "1");
@@ -396,6 +397,22 @@ gmpi_thr_create(const mpi_plugin_client_info_t *job, char ***env)
 	return st;
 }
 
+
+/*
+ * Warning: This pthread_cancel/pthread_join is a little unsafe.  The thread is
+ * not joinable, so on most systems the join will fail, then the thread's state
+ * will be destroyed, possibly before the thread has actually stopped.  In
+ * practice the thread will usually be waiting on an accept call when it gets
+ * cancelled.  If the mpi thread has a mutex locked when it is cancelled--while
+ * using the "info" or "error" functions for logging--the caller will deadlock.
+ * See mpich1_p4.c or mvapich.c for code that shuts down cleanly by letting
+ * the mpi thread wait on a poll call, and creating a pipe that the poll waits
+ * on, which can be written to by the main thread to tell the mpi thread to
+ * exit.  Also see rev 18654 of mpichmx.c, on
+ * branches/slurm-2.1.mpi.plugin.cleanup for an implementation.  There were no
+ * myrinet systems available for testing, which is why I couldn't complete the
+ * patch for this plugin.  -djb
+ */
 extern int gmpi_thr_destroy(gmpi_state_t *st)
 {
 	if (st != NULL) {
