@@ -5,32 +5,33 @@
  *  Copyright (C) 2004 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Takao Hatazaki <takao.hatazaki@hp.com>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -117,7 +118,7 @@ static void _push_to_hashtbl(pid_t ppid, pid_t pid,
 		ppids = ppids->next;
 	}
 	newppid = _alloc_ppid(ppid, pid, is_usercmd, cmd, hashtbl[idx]);
-	hashtbl[idx] = newppid;    
+	hashtbl[idx] = newppid;
 }
 
 static int get_myname(char *s)
@@ -150,7 +151,7 @@ static xppid_t **_build_hashtbl()
 	char path[PATH_MAX], *endptr, *num, rbuf[1024];
 	char myname[1024], cmd[1024];
 	int fd;
-	long pid, ppid;
+	long pid, ppid, ret_l;
 	xppid_t **hashtbl;
 
 	if ((dir = opendir("/proc")) == NULL) {
@@ -164,7 +165,11 @@ static xppid_t **_build_hashtbl()
 
 	while ((de = readdir(dir)) != NULL) {
 		num = de->d_name;
-		strtol(num, &endptr, 10);
+		ret_l = strtol(num, &endptr, 10);
+		if(errno == ERANGE)
+			error("couldn't do a strtol on str %s(%d): %m",
+			      num, ret_l);
+
 		if (endptr == NULL || *endptr != 0)
 			continue;
 		sprintf(path, "/proc/%s/stat", num);
@@ -182,7 +187,7 @@ static xppid_t **_build_hashtbl()
 		close(fd);
 
 		/* Record cmd for debugging purpose */
-		_push_to_hashtbl((pid_t)ppid, (pid_t)pid, 
+		_push_to_hashtbl((pid_t)ppid, (pid_t)pid,
 				 strcmp(myname, cmd), cmd, hashtbl);
 	}
 	closedir(dir);
@@ -285,7 +290,7 @@ extern int kill_proc_tree(pid_t top, int sig)
 
 	if ((hashtbl = _build_hashtbl()) == NULL)
 		return -1;
-	
+
 	list = _get_list(top, NULL, hashtbl);
 	rc = _kill_proclist(list, sig);
 	_destroy_hashtbl(hashtbl);
@@ -295,7 +300,7 @@ extern int kill_proc_tree(pid_t top, int sig)
 
 
 /*
- * Return the pid of the process named "process_name" 
+ * Return the pid of the process named "process_name"
  * which is the ancestor of "process".
  */
 extern pid_t find_ancestor(pid_t process, char *process_name)
