@@ -1,44 +1,47 @@
 /*****************************************************************************\
  *  msg.c - Message/communcation manager for Wiki plugin
  *****************************************************************************
- *  Copyright (C) 2006 The Regents of the University of California.
+ *  Copyright (C) 2006-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
-#include "./crypto.h"
-#include "./msg.h"
-#include "src/common/uid.h"
-#include "src/slurmctld/locks.h"
+#include "slurm/slurm.h"
+#include <src/common/uid.h>
+#include <src/slurmctld/locks.h>
+#include <src/plugins/sched/wiki/crypto.h>
+#include <src/plugins/sched/wiki/msg.h>
 
 #define _DEBUG 0
 
@@ -88,7 +91,7 @@ extern int spawn_msg_thread(void)
 
 	parse_wiki_config();
 	slurm_attr_init(&thread_attr_msg);
-	if (pthread_create(&msg_thread_id, &thread_attr_msg, 
+	if (pthread_create(&msg_thread_id, &thread_attr_msg,
 			_msg_thread, NULL))
 		fatal("pthread_create %m");
 
@@ -111,7 +114,7 @@ extern void term_msg_thread(void)
 		thread_shutdown = true;
 
                 /* Open and close a connection to the wiki listening port.
-                 * Allows slurm_accept_msg_conn() to return in 
+                 * Allows slurm_accept_msg_conn() to return in
                  * _msg_thread() so that it can check the thread_shutdown
                  * flag.
                  */
@@ -180,8 +183,8 @@ static void *_msg_thread(void *no_data)
                         close(new_fd);
                         break;
                 }
-		/* It would be nice to create a pthread for each new 
-		 * RPC, but that leaks memory on some systems when 
+		/* It would be nice to create a pthread for each new
+		 * RPC, but that leaks memory on some systems when
 		 * done from a plugin.
 		 * FIXME: Maintain a pool of and reuse them. */
 		err_code = 0;
@@ -229,7 +232,7 @@ static char * _get_wiki_conf_path(void)
 /*****************************************************************************\
  * parse_wiki_config - Results go into global variables
  * RET SLURM_SUCESS or error code
- * 
+ *
  * wiki_conf options
  * JobPriority=hold|run
  * AuthKey=number
@@ -244,7 +247,7 @@ extern int parse_wiki_config(void)
 		{"ExcludePartitions", S_P_STRING},
 		{"HidePartitionJobs", S_P_STRING},
 		{"JobAggregationTime", S_P_UINT16},
-		{"JobPriority", S_P_STRING}, 
+		{"JobPriority", S_P_STRING},
 		{NULL} };
 	s_p_hashtbl_t *tbl;
 	char *exclude_partitions, *hide_partitions;
@@ -263,7 +266,7 @@ extern int parse_wiki_config(void)
 	if (conf->backup_addr) {
 		strncpy(e_host_bu, conf->backup_addr,
 			sizeof(e_host));
-	} 
+	}
 	kill_wait = conf->kill_wait;
 	slurm_conf_unlock();
 
@@ -295,7 +298,7 @@ extern int parse_wiki_config(void)
 		xfree(key);
 	}
 	s_p_get_uint16(&e_port, "EPort", tbl);
-	s_p_get_uint16(&job_aggregation_time, "JobAggregationTime", tbl); 
+	s_p_get_uint16(&job_aggregation_time, "JobAggregationTime", tbl);
 
 	if (s_p_get_string(&exclude_partitions, "ExcludePartitions", tbl)) {
 		char *tok = NULL, *tok_p = NULL;
@@ -306,7 +309,7 @@ extern int parse_wiki_config(void)
 				error("ExcludePartitions has too many entries "
 				      "skipping %s and later entries");
 				break;
-			}	
+			}
 			exclude_part_ptr[i] = find_part_record(tok);
 			if (exclude_part_ptr[i])
 				i++;
@@ -325,7 +328,7 @@ extern int parse_wiki_config(void)
 				error("HidePartitionJobs has too many entries "
 				      "skipping %s and later entries");
 				break;
-			}	
+			}
 			hide_part_ptr[i] = find_part_record(tok);
 			if (hide_part_ptr[i])
 				i++;
@@ -341,7 +344,7 @@ extern int parse_wiki_config(void)
 		else if (strcasecmp(priority_mode, "run") == 0)
 			init_prio_mode = PRIO_DECREMENT;
 		else
-			error("Invalid value for JobPriority in wiki.conf");	
+			error("Invalid value for JobPriority in wiki.conf");
 		xfree(priority_mode);
 	}
 	s_p_hashtbl_destroy(tbl);
@@ -354,7 +357,7 @@ extern int parse_wiki_config(void)
 	info("EPort              = %u", e_port);
 	info("JobAggregationTime = %u sec", job_aggregation_time);
 	info("JobPriority        = %s", init_prio_mode ? "run" : "hold");
-	info("KillWait           = %u sec", kill_wait);      
+	info("KillWait           = %u sec", kill_wait);
 	for (i=0; i<EXC_PART_CNT; i++) {
 		if (!exclude_part_ptr[i])
 			continue;
@@ -418,7 +421,7 @@ static size_t	_read_bytes(int fd, char *buf, const size_t size)
 		bytes_remaining -= bytes_read;
 		ptr += bytes_read;
 	}
-	
+
 	return size;
 }
 
@@ -440,9 +443,9 @@ static size_t	_write_bytes(int fd, char *buf, const size_t size)
 }
 
 /*****************************************************************************\
- * Read a message (request) from specified file descriptor 
+ * Read a message (request) from specified file descriptor
  *
- * RET - The message which must be xfreed or 
+ * RET - The message which must be xfreed or
  *       NULL on error
 \*****************************************************************************/
 static char *	_recv_msg(slurm_fd new_fd)
@@ -474,12 +477,14 @@ static char *	_recv_msg(slurm_fd new_fd)
 		return NULL;
 	}
 
-	debug2("wiki msg recv:%s", buf);
+	if (slurm_get_debug_flags() && DEBUG_FLAG_WIKI)
+		info("wiki msg recv:%s", buf);
+
 	return buf;
 }
 
 /*****************************************************************************\
- * Send a message (response) to specified file descriptor 
+ * Send a message (response) to specified file descriptor
  *
  * RET - Number of data bytes written (excludes header)
 \*****************************************************************************/
@@ -488,7 +493,8 @@ static size_t	_send_msg(slurm_fd new_fd, char *buf, size_t size)
 	char header[10];
 	size_t data_sent;
 
-	debug2("wiki msg send:%s", buf);
+	if (slurm_get_debug_flags() && DEBUG_FLAG_WIKI)
+		info("wiki msg send:%s", buf);
 
 	(void) sprintf(header, "%08lu\n", (unsigned long) size);
 	if (_write_bytes((int) new_fd, header, 9) != 9) {
@@ -519,7 +525,7 @@ static int	_parse_msg(char *msg, char **req)
 	char *cmd_ptr = strstr(msg, "CMD=");
 	time_t ts, now = time(NULL);
 	uint32_t delta_t;
-	
+
 	if ((auth_key[0] == '\0') && cmd_ptr) {
 		/* No authentication required */
 		*req = cmd_ptr;
@@ -546,7 +552,7 @@ static int	_parse_msg(char *msg, char **req)
 		error("wiki: request lacks TS=");
 		return -1;
 	}
-	ts = strtoul((ts_ptr+3), NULL, 10); 
+	ts = strtoul((ts_ptr+3), NULL, 10);
 	if (ts < now)
 		delta_t = (uint32_t) difftime(now, ts);
 	else
@@ -554,14 +560,14 @@ static int	_parse_msg(char *msg, char **req)
 	if (delta_t > 300) {
 		err_code = -350;
 		err_msg = "TS value too far from NOW";
-		error("wiki: TimeStamp too far from NOW (%u secs)", 
+		error("wiki: TimeStamp too far from NOW (%u secs)",
 			delta_t);
 		return -1;
 	}
 
 #if 0
 	/* Old wiki interface does not require checksum
-	 * (actually a cryptographic signature) on incomming 
+	 * (actually a cryptographic signature) on incomming
 	 * messages.  NOTE: This is not secure! */
 	if (auth_key[0] != '\0') {
 		char sum[20];	/* format is "CK=%08x08x" */
@@ -606,7 +612,7 @@ static void	_proc_msg(slurm_fd new_fd, char *msg)
 	cmd_ptr = strstr(req, "CMD=");
 	if (cmd_ptr == NULL) {
 		err_code = -300;
-		err_msg = "request lacks CMD"; 
+		err_msg = "request lacks CMD";
 		error("wiki: request lacks CMD");
 		goto resp_msg;
 	}
@@ -673,7 +679,7 @@ static void	_send_reply(slurm_fd new_fd, char *response)
 		xfree(tmp);
 	}
 
-	snprintf(buf, i, "CK=dummy67890123456 TS=%u AUTH=%s DT=%s", 
+	snprintf(buf, i, "CK=dummy67890123456 TS=%u AUTH=%s DT=%s",
 		(uint32_t) time(NULL), uname, response);
 	checksum(sum, auth_key, (buf+20));   /* overwrite "CK=dummy..." above */
 	memcpy(buf, sum, 19);
