@@ -2,34 +2,36 @@
  *  get_jobs.c - Process Wiki get job info request
  *****************************************************************************
  *  Copyright (C) 2006-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -48,7 +50,7 @@
 
 static char *	_dump_all_jobs(int *job_cnt, time_t update_time);
 static char *	_dump_job(struct job_record *job_ptr, time_t update_time);
-static void	_get_job_comment(struct job_record *job_ptr, 
+static void	_get_job_comment(struct job_record *job_ptr,
 			char *buffer, int buf_size);
 static uint16_t _get_job_cpus_per_task(struct job_record *job_ptr);
 static uint32_t	_get_job_end_time(struct job_record *job_ptr);
@@ -89,7 +91,7 @@ reject_msg_t reject_msgs[REJECT_MSG_MAX];
  * ARG=<cnt>#<JOBID>;
  *	STATE=<state>;			Moab equivalent job state
  *	[EXITCODE=<number>;]		Job exit code, if completed
- *	[RFEATURES=<features>;]		required features, if any, 
+ *	[RFEATURES=<features>;]		required features, if any,
  *					NOTE: OR operator not supported
  *	[HOSTLIST=<node1:node2>;]	list of required nodes, if any
  *	[STARTDATE=<uts>;]		earliest start time, if any
@@ -126,7 +128,7 @@ extern int	get_jobs(char *cmd_ptr, int *err_code, char **err_msg)
 	int job_rec_cnt = 0, buf_size = 0;
 
 	if (cr_test == 0) {
-		select_g_get_info_from_plugin(SELECT_CR_PLUGIN,
+		select_g_get_info_from_plugin(SELECT_CR_PLUGIN, NULL,
 					      &cr_enabled);
 		cr_test = 1;
 	}
@@ -234,7 +236,7 @@ static char *	_dump_job(struct job_record *job_ptr, time_t update_time)
 {
 	char *buf = NULL, tmp[16384];
 	char *gname, *quote, *uname;
-	uint32_t end_time, suspend_time;
+	uint32_t end_time, suspend_time, min_mem;
 	int i, rej_sent = 0;
 
 	if (!job_ptr)
@@ -247,7 +249,7 @@ static char *	_dump_job(struct job_record *job_ptr, time_t update_time)
 	if (update_time > last_job_update)
 		return buf;
 
-	if (job_ptr->job_state == JOB_PENDING) {
+	if (IS_JOB_PENDING(job_ptr)) {
 		char *req_features = _get_job_features(job_ptr);
 		if (req_features) {
 			snprintf(tmp, sizeof(tmp),
@@ -255,9 +257,9 @@ static char *	_dump_job(struct job_record *job_ptr, time_t update_time)
 			xstrcat(buf, tmp);
 			xfree(req_features);
 		}
-		if ((job_ptr->details)
-		&&  (job_ptr->details->req_nodes)
-		&&  (job_ptr->details->req_nodes[0])) {
+		if ((job_ptr->details) &&
+		    (job_ptr->details->req_nodes) &&
+		    (job_ptr->details->req_nodes[0])) {
 			char *hosts = bitmap2wiki_node_name(
 				job_ptr->details->req_node_bitmap);
 			snprintf(tmp, sizeof(tmp),
@@ -265,8 +267,7 @@ static char *	_dump_job(struct job_record *job_ptr, time_t update_time)
 			xstrcat(buf, tmp);
 			xfree(hosts);
 		}
-		if ((job_ptr->details)
-		&&  (job_ptr->details->begin_time)) {
+		if ((job_ptr->details) && (job_ptr->details->begin_time)) {
 			snprintf(tmp, sizeof(tmp),
 				"STARTDATE=%u;", (uint32_t)
 				job_ptr->details->begin_time);
@@ -301,7 +302,7 @@ static char *	_dump_job(struct job_record *job_ptr, time_t update_time)
 			break;
 		}
 	}
-	if ((rej_sent == 0) && (job_ptr->job_state == JOB_FAILED)) {
+	if ((rej_sent == 0) && IS_JOB_FAILED(job_ptr)) {
 		snprintf(tmp, sizeof(tmp),
 			"REJMESSAGE=\"%s\";",
 			job_reason_string(job_ptr->state_reason));
@@ -311,7 +312,7 @@ static char *	_dump_job(struct job_record *job_ptr, time_t update_time)
 	if (job_ptr->batch_flag == 0)
 		xstrcat(buf, "FLAGS=INTERACTIVE;");
 
-	snprintf(tmp, sizeof(tmp), 
+	snprintf(tmp, sizeof(tmp),
 		"UPDATETIME=%u;WCLIMIT=%u;TASKS=%u;",
 		(uint32_t) job_ptr->time_last_active,
 		(uint32_t) _get_job_time_limit(job_ptr),
@@ -337,9 +338,13 @@ static char *	_dump_job(struct job_record *job_ptr, time_t update_time)
 		job_ptr->partition);
 	xstrcat(buf, tmp);
 
+	min_mem = _get_job_min_mem(job_ptr);
+	if (min_mem & MEM_PER_CPU) {
+		min_mem &= ~MEM_PER_CPU;
+	}
 	snprintf(tmp, sizeof(tmp),
 		"RMEM=%u;RDISK=%u;",
-		_get_job_min_mem(job_ptr),
+		 min_mem,
 		_get_job_min_disk(job_ptr));
 	xstrcat(buf, tmp);
 
@@ -393,7 +398,7 @@ static char *	_dump_job(struct job_record *job_ptr, time_t update_time)
 	return buf;
 }
 
-static void	_get_job_comment(struct job_record *job_ptr, 
+static void	_get_job_comment(struct job_record *job_ptr,
 			char *buffer, int buf_size)
 {
 	int size, sharing = 0;
@@ -413,7 +418,7 @@ static void	_get_job_comment(struct job_record *job_ptr,
 	/* SHARED NODES */
 	if (cr_enabled)	{			/* consumable resources */
 		if (job_ptr->part_ptr &&
-		    (job_ptr->part_ptr->max_share == 0))	/* Exclusive use */
+		    (job_ptr->part_ptr->max_share == 0))  /* Exclusive use */
 			sharing = 0;
 		else if (job_ptr->details && job_ptr->details->shared)
 			sharing = 1;
@@ -434,7 +439,7 @@ static void	_get_job_comment(struct job_record *job_ptr,
 	/* TPN = tasks per node */
 	if (job_ptr->details && (job_ptr->details->ntasks_per_node != 0)) {
 		size += snprintf((buffer + size), (buf_size - size),
-			"%sTPN:%u", field_sep, 
+			"%sTPN:%u", field_sep,
 			job_ptr->details->ntasks_per_node);
 		field_sep = "?";
 	}
@@ -473,7 +478,7 @@ static uint32_t _get_job_min_mem(struct job_record *job_ptr)
 }
 
 static uint32_t _get_job_min_disk(struct job_record *job_ptr)
-	
+
 {
 	if (job_ptr->details)
 		return job_ptr->details->job_min_tmp_disk;
@@ -484,7 +489,7 @@ static uint32_t	_get_job_max_nodes(struct job_record *job_ptr)
 {
 	uint32_t max_nodes = 0;
 
-	if (job_ptr->job_state > JOB_PENDING) {
+	if (IS_JOB_STARTED(job_ptr)) {
 		/* return actual count of currently allocated nodes.
 		 * NOTE: gets decremented to zero while job is completing */
 		return job_ptr->node_cnt;
@@ -496,7 +501,7 @@ static uint32_t	_get_job_max_nodes(struct job_record *job_ptr)
 	if (job_ptr->details->max_nodes) {
 		max_nodes = job_ptr->details->max_nodes;
 		if (job_ptr->part_ptr->max_nodes != INFINITE) {
-			max_nodes = MIN(max_nodes, 
+			max_nodes = MIN(max_nodes,
 					job_ptr->part_ptr->max_nodes);
 		}
 	} else if (job_ptr->part_ptr->max_nodes == INFINITE)
@@ -509,15 +514,27 @@ static uint32_t	_get_job_max_nodes(struct job_record *job_ptr)
 
 static uint32_t	_get_job_min_nodes(struct job_record *job_ptr)
 {
-	if (job_ptr->job_state > JOB_PENDING) {
+	uint32_t min_nodes = 1;
+
+	if (IS_JOB_STARTED(job_ptr)) {
 		/* return actual count of currently allocated nodes.
 		 * NOTE: gets decremented to zero while job is completing */
 		return job_ptr->node_cnt;
 	}
 
-	if (job_ptr->details)
-		return job_ptr->details->min_nodes;
-	return (uint32_t) 1;
+	if ((job_ptr->details == NULL) || (job_ptr->part_ptr == NULL))
+		return min_nodes;	/* should never reach here */
+
+	if (job_ptr->details->min_nodes) {
+		min_nodes = job_ptr->details->min_nodes;
+		if (job_ptr->part_ptr->min_nodes > 1) {
+			min_nodes = MAX(min_nodes,
+					job_ptr->part_ptr->min_nodes);
+		}
+	} else	/* use partition limit */
+		min_nodes = job_ptr->part_ptr->min_nodes;
+
+	return min_nodes;
 }
 
 static uint32_t _get_job_submit_time(struct job_record *job_ptr)
@@ -531,7 +548,7 @@ static uint32_t _get_job_tasks(struct job_record *job_ptr)
 {
 	uint32_t task_cnt;
 
-	if (job_ptr->job_state > JOB_PENDING) {
+	if (IS_JOB_STARTED(job_ptr)) {
 		task_cnt = job_ptr->total_procs;
 	} else {
 		if (job_ptr->num_procs)
@@ -540,7 +557,7 @@ static uint32_t _get_job_tasks(struct job_record *job_ptr)
 			task_cnt = 1;
 		if (job_ptr->details) {
 			task_cnt = MAX(task_cnt,
-				       (_get_job_min_nodes(job_ptr) * 
+				       (_get_job_min_nodes(job_ptr) *
 				        job_ptr->details->
 					ntasks_per_node));
 		}
@@ -555,7 +572,7 @@ static uint32_t	_get_job_time_limit(struct job_record *job_ptr)
 
 	if ((limit == NO_VAL) && (job_ptr->part_ptr)) {
 		/* Job will get partition's time limit when schedule.
-		 * The partition's limit can change between now and 
+		 * The partition's limit can change between now and
 		 * job initiation time. */
 		limit = job_ptr->part_ptr->max_time;
 	}
@@ -566,34 +583,32 @@ static uint32_t	_get_job_time_limit(struct job_record *job_ptr)
 		return (limit * 60);	/* seconds, not minutes */
 }
 
-/* NOTE: if job has already completed, we append "EXITCODE=#" to 
+/* NOTE: if job has already completed, we append "EXITCODE=#" to
  * the state name */
 static char *	_get_job_state(struct job_record *job_ptr)
 {
-	uint16_t state = job_ptr->job_state;
-	uint16_t base_state = state & (~JOB_COMPLETING);
 	char *state_str;
 	static char return_msg[128];
 
-	if (state & JOB_COMPLETING) {
+	if (IS_JOB_COMPLETING(job_ptr)) {
 		/* Give configured KillWait+10 for job
-		 * to clear out, then then consider job 
-		 * done. Moab will allocate jobs to 
-		 * nodes that are already Idle. */ 
-		int age = (int) difftime(time(NULL), 
+		 * to clear out, then then consider job
+		 * done. Moab will allocate jobs to
+		 * nodes that are already Idle. */
+		int age = (int) difftime(time(NULL),
 			job_ptr->end_time);
 		if (age < (kill_wait+10))
 			return "Running";
 	}
 
-	if (base_state == JOB_RUNNING)
+	if (IS_JOB_RUNNING(job_ptr))
 		return "Running";
-	if (base_state == JOB_SUSPENDED)
+	if (IS_JOB_SUSPENDED(job_ptr))
 		return "Suspended";
-	if (base_state == JOB_PENDING)
+	if (IS_JOB_PENDING(job_ptr))
 		return "Idle";
 
-	if ((base_state == JOB_COMPLETE) || (base_state == JOB_FAILED))
+	if (IS_JOB_COMPLETE(job_ptr) || IS_JOB_FAILED(job_ptr))
 		state_str = "Completed";
 	else /* JOB_CANCELLED, JOB_TIMEOUT, JOB_NODE_FAIL */
 		state_str = "Removed";
@@ -617,9 +632,9 @@ static char * _get_job_features(struct job_record *job_ptr)
 	int i;
 	char *rfeatures;
 
-	if ((job_ptr->details == NULL)
-	||  (job_ptr->details->features == NULL)
-	||  (job_ptr->details->features[0] == '\0'))
+	if ((job_ptr->details == NULL) ||
+	    (job_ptr->details->features == NULL) ||
+	    (job_ptr->details->features[0] == '\0'))
 		return NULL;
 
 	rfeatures = xstrdup(job_ptr->details->features);
@@ -641,9 +656,9 @@ static char * _get_job_features(struct job_record *job_ptr)
 /* returns how long job has been suspended, in seconds */
 static uint32_t	_get_job_suspend_time(struct job_record *job_ptr)
 {
-	if (job_ptr->job_state == JOB_SUSPENDED) {
+	if (IS_JOB_SUSPENDED(job_ptr)) {
 		time_t now = time(NULL);
-		return (uint32_t) difftime(now, 
+		return (uint32_t) difftime(now,
 				job_ptr->suspend_time);
 	}
 	return (uint32_t) 0;
