@@ -1,35 +1,36 @@
 /*****************************************************************************\
  *  block_allocator.h
- * 
+ *
  *****************************************************************************
  *  Copyright (C) 2004 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Dan Phung <phung4@llnl.gov>, Danny Auble <da@llnl.gov>
- *  
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -68,28 +69,28 @@ enum {X, Y, Z};
 
 /* */
 
-/* 
- * structure that holds switch path information for finding the wiring 
+/*
+ * structure that holds switch path information for finding the wiring
  * path without setting the configuration.
  *
  * - dim      - Which Axis it is on
  * - geometry - node location
  * - in       - ingress port.
  * - out      - egress port.
- * 
+ *
  */
 typedef struct {
 	int dim;
 	int geometry[BA_SYSTEM_DIMENSIONS];
-	int in; 
+	int in;
 	int out;
-} ba_path_switch_t; 
+} ba_path_switch_t;
 
-/* 
+/*
  * structure that holds the configuration settings for each request
  */
 typedef struct {
-	bitstr_t *avail_node_bitmap;   /* pointer to available nodes */	
+	bitstr_t *avail_node_bitmap;   /* pointer to available nodes */
 #ifdef HAVE_BGL
 	char *blrtsimage;              /* BlrtsImage for this block */
 #endif
@@ -135,7 +136,7 @@ typedef struct {
 					    block */
 	int start_req;                 /* state there was a start
 					  request */
-} ba_request_t; 
+} ba_request_t;
 
 /* structure filled in from reading bluegene.conf file for block
  * creation */
@@ -168,7 +169,7 @@ typedef struct {
 #endif
 } blockreq_t;
 
-/* structure filled in from reading bluegene.conf file for specifing
+/* structure filled in from reading bluegene.conf file for specifying
  * images */
 typedef struct {
 	bool def;                      /* Whether image is the default
@@ -184,9 +185,9 @@ typedef struct {
 	gid_t gid;
 } image_group_t;
 
-/* 
+/*
  * structure that holds the configuration settings for each connection
- * 
+ *
  * - node_tar - coords of where the next hop is externally
  *              interanlly - nothing.
  *              exteranlly - location of next hop.
@@ -194,25 +195,25 @@ typedef struct {
  *              interanlly - always going to something within the switch.
  *              exteranlly - always going to the next hop outside the switch.
  * - used     - weather or not the connection is used.
- * 
+ *
  */
-typedef struct 
+typedef struct
 {
 	/* target label */
 	int node_tar[BA_SYSTEM_DIMENSIONS];
-	/* target port */ 
+	/* target port */
 	int port_tar;
-	bool used;	
+	bool used;
 } ba_connection_t;
 
-/* 
+/*
  * structure that holds the configuration settings for each switch
- * which pretty much means the wiring information 
+ * which pretty much means the wiring information
  * - int_wire - keeps details of where the wires are attached
  *   interanlly.
  * - ext_wire - keeps details of where the wires are attached
  *   exteranlly.
- * 
+ *
  */
 typedef struct
 {
@@ -225,7 +226,7 @@ typedef struct
  */
 typedef struct {
 	/* a switch for each dimensions */
-	ba_switch_t axis_switch[BA_SYSTEM_DIMENSIONS]; 
+	ba_switch_t axis_switch[BA_SYSTEM_DIMENSIONS];
 	/* coordinates of midplane */
 	int coord[BA_SYSTEM_DIMENSIONS];
 	/* color of letter used in smap */
@@ -233,8 +234,8 @@ typedef struct {
 	/* midplane index used for easy look up of the miplane */
 	int index;
 	/* letter used in smap */
-	char letter;                    
-//	int phys_x;	// no longer needed 
+	char letter;
+//	int phys_x;	// no longer needed
 	int state;
 	/* set if using this midplane in a block */
 	uint16_t used;
@@ -258,7 +259,7 @@ typedef struct {
 */
 typedef struct {
 	char *bp_id;
-	int coord[BA_SYSTEM_DIMENSIONS];	
+	int coord[BA_SYSTEM_DIMENSIONS];
 } ba_bp_map_t;
 
 /* Global */
@@ -272,23 +273,21 @@ extern int DIM_SIZE[BA_SYSTEM_DIMENSIONS]; /* how many midplanes in
 extern s_p_options_t bg_conf_file_options[]; /* used to parse the
 					      * bluegene.conf file. */
 extern uint16_t ba_deny_pass;
-
-/* Translate a state enum to a readable string */
-extern char *bg_block_state_string(rm_partition_state_t state);
+extern ba_system_t *ba_system_ptr;
 
 /* must xfree return of this */
 extern char *ba_passthroughs_string(uint16_t passthrough);
 
 /* Parse a block request from the bluegene.conf file */
 extern int parse_blockreq(void **dest, slurm_parser_enum_t type,
-			  const char *key, const char *value, 
+			  const char *key, const char *value,
 			  const char *line, char **leftover);
 
 extern void destroy_blockreq(void *ptr);
 
 /* Parse imagine information from blugene.conf file */
 extern int parse_image(void **dest, slurm_parser_enum_t type,
-		       const char *key, const char *value, 
+		       const char *key, const char *value,
 		       const char *line, char **leftover);
 
 extern void destroy_image_group_list(void *ptr);
@@ -299,10 +298,10 @@ extern void destroy_ba_node(void *ptr);
  * create a block request.  Note that if the geometry is given,
  * then size is ignored.  If elongate is true, the algorithm will try
  * to fit that a block of cubic shape and then it will try other
- * elongated geometries.  (ie, 2x2x2 -> 4x2x1 -> 8x1x1). 
- * 
- * IN/OUT - ba_request: structure to allocate and fill in.  
- * 
+ * elongated geometries.  (ie, 2x2x2 -> 4x2x1 -> 8x1x1).
+ *
+ * IN/OUT - ba_request: structure to allocate and fill in.
+ *
  * ALL below IN's need to be set within the ba_request before the call
  * if you want them to be used.
  * ALL below OUT's are set and returned within the ba_request.
@@ -314,13 +313,9 @@ extern void destroy_ba_node(void *ptr);
  * IN/OUT - geometry: requested/returned geometry of block
  * IN - linuximage: LinuxImage for this block if not default
  * IN - mloaderimage: MLoaderImage for this block if not default
- * IN - nodecards: Number of nodecards in each block in request only
- *      used of small block allocations.
  * OUT - passthroughs: if there were passthroughs used in the
  *       generation of the block.
  * IN - procs: Number of real processors requested
- * IN - quarters: Number of midplane quarters in each block in request only
- *      used of small block allocations.
  * IN - RamDiskimage: RamDiskImage for this block if not default
  * IN - rotate: if true, allows rotation of block during fit
  * OUT - save_name: hostlist of midplanes used in block
@@ -332,7 +327,7 @@ extern void destroy_ba_node(void *ptr);
 extern int new_ba_request(ba_request_t* ba_request);
 
 /*
- * delete a block request 
+ * delete a block request
  */
 extern void delete_ba_request(void *arg);
 
@@ -343,20 +338,21 @@ extern void delete_ba_request(void *arg);
 extern int empty_null_destroy_list(void *arg, void *key);
 
 /*
- * print a block request 
+ * print a block request
  */
 extern void print_ba_request(ba_request_t* ba_request);
 
 /*
  * Initialize internal structures by either reading previous block
  * configurations from a file or by running the graph solver.
- * 
- * IN: node_info_msg_t * can be null, 
+ *
+ * IN: node_info_msg_t * can be null,
  *     should be from slurm_load_node().
- * 
+ * IN: load_bridge: whiether or not to get bridge information
+ *
  * return: void.
  */
-extern void ba_init(node_info_msg_t *node_info_ptr);
+extern void ba_init(node_info_msg_t *node_info_ptr, bool load_bridge);
 
 /* If emulating a system set up a known configuration for wires in a
  * system of the size given.
@@ -365,31 +361,31 @@ extern void ba_init(node_info_msg_t *node_info_ptr);
  */
 extern void init_wires();
 
-/* 
+/*
  * destroy all the internal (global) data structs.
  */
 extern void ba_fini();
 
-/* 
+/*
  * set the node in the internal configuration as in, or not in use,
  * along with the current state of the node.
- * 
+ *
  * IN ba_node: ba_node_t to update state
  * IN state: new state of ba_node_t
  */
 extern void ba_update_node_state(ba_node_t *ba_node, uint16_t state);
 
-/* 
+/*
  * copy info from a ba_node, a direct memcpy of the ba_node_t
- * 
+ *
  * IN ba_node: node to be copied
  * Returned ba_node_t *: copied info must be freed with destroy_ba_node
  */
 extern ba_node_t *ba_copy_node(ba_node_t *ba_node);
 
-/* 
+/*
  * copy the path of the nodes given
- * 
+ *
  * IN nodes List of ba_node_t *'s: nodes to be copied
  * OUT dest_nodes List of ba_node_t *'s: filled in list of nodes
  * wiring.
@@ -397,33 +393,33 @@ extern ba_node_t *ba_copy_node(ba_node_t *ba_node);
  */
 extern int copy_node_path(List nodes, List *dest_nodes);
 
-/* 
+/*
  * Try to allocate a block.
- * 
+ *
  * IN - ba_request: allocation request
  * OUT - results: List of results of the allocation request.  Each
  * list entry will be a coordinate.  allocate_block will create the
  * list, but the caller must destroy it.
- * 
+ *
  * return: success or error of request
  */
 extern int allocate_block(ba_request_t* ba_request, List results);
 
-/* 
+/*
  * Admin wants to remove a previous allocation.
  * will allow Admin to delete a previous allocation retrival by letter code.
  */
 extern int remove_block(List nodes, int new_count);
 
-/* 
- * Admin wants to change something about a previous allocation. 
- * will allow Admin to change previous allocation by giving the 
+/*
+ * Admin wants to change something about a previous allocation.
+ * will allow Admin to change previous allocation by giving the
  * letter code for the allocation and the variable to alter
  * (Not currently used in the system, update this if it is)
  */
 extern int alter_block(List nodes, int conn_type);
 
-/* 
+/*
  * After a block is deleted or altered following allocations must
  * be redone to make sure correct path will be used in the real system
  * (Not currently used in the system, update this if it is)
@@ -447,8 +443,8 @@ extern int check_and_set_node_list(List nodes);
 
 /*
  * Used to find, and set up midplanes and the wires in the virtual
- * system and return them in List results 
- * 
+ * system and return them in List results
+ *
  * IN/OUT results - a list with a NULL destroyer filled in with
  *        midplanes and wires set to create the block with the api. If
  *        only interested in the hostlist NULL can be excepted also.
@@ -458,7 +454,7 @@ extern int check_and_set_node_list(List nodes);
  * RET char * - hostlist of midplanes results represent must be
  *     xfreed.  NULL on failure
  */
-extern char *set_bg_block(List results, int *start, 
+extern char *set_bg_block(List results, int *start,
 			  int *geometry, int conn_type);
 
 /*
@@ -475,7 +471,7 @@ extern int reset_ba_system(bool track_down_nodes);
  * RET: SLURM_SUCCESS on success, or SLURM_ERROR on error
  *
  * Note: Need to call reset_all_removed_bps before starting another
- * allocation attempt after 
+ * allocation attempt after
  */
 extern int removable_set_bps(char *bps);
 
@@ -525,7 +521,7 @@ extern int *find_bp_loc(char* bp_id);
 extern char *find_bp_rack_mid(char* xyz);
 
 /*
- * set the used wires in the virtual system for a block from the real system 
+ * set the used wires in the virtual system for a block from the real system
  */
 extern int load_block_wiring(char *bg_block_id);
 
@@ -533,7 +529,8 @@ extern int load_block_wiring(char *bg_block_id);
  * get the used wires for a block out of the database and return the
  * node list
  */
-extern List get_and_set_block_wiring(char *bg_block_id);
+extern List get_and_set_block_wiring(char *bg_block_id,
+				     rm_partition_t *block_ptr);
 
 /* make sure a node is in the system return 1 if it is 0 if not */
 extern int validate_coord(int *coord);
