@@ -1,35 +1,37 @@
 /*****************************************************************************\
  *  switch_none.c - Library for managing a switch with no special handling.
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -62,7 +64,7 @@
  *      <application>/<method>
  *
  * where <application> is a description of the intended application of
- * the plugin (e.g., "switch" for SLURM switch) and <method> is a description 
+ * the plugin (e.g., "switch" for SLURM switch) and <method> is a description
  * of how this plugin satisfies that application.  SLURM will only load
  * a switch plugin if the plugin_type string has a prefix of "switch/".
  *
@@ -112,43 +114,43 @@ int switch_p_libstate_clear ( void )
 /*
  * switch functions for job step specific credential
  */
-int switch_p_alloc_jobinfo ( switch_jobinfo_t *switch_job )
+int switch_p_alloc_jobinfo ( switch_jobinfo_t **switch_job )
 {
 	return SLURM_SUCCESS;
 }
 
-int switch_p_build_jobinfo ( switch_jobinfo_t switch_job, char *nodelist, 
+int switch_p_build_jobinfo ( switch_jobinfo_t *switch_job, char *nodelist,
 		uint16_t *tasks_per_node, int cyclic_alloc, char *network)
 {
 	return SLURM_SUCCESS;
 }
 
-switch_jobinfo_t switch_p_copy_jobinfo  ( switch_jobinfo_t switch_job )
+switch_jobinfo_t *switch_p_copy_jobinfo  ( switch_jobinfo_t *switch_job )
 {
 	return NULL;
 }
 
-void switch_p_free_jobinfo ( switch_jobinfo_t switch_job )
+void switch_p_free_jobinfo ( switch_jobinfo_t *switch_job )
 {
 	return;
 }
 
-int switch_p_pack_jobinfo ( switch_jobinfo_t switch_job, Buf buffer )
+int switch_p_pack_jobinfo ( switch_jobinfo_t *switch_job, Buf buffer )
 {
 	return 0;
 }
 
-int switch_p_unpack_jobinfo ( switch_jobinfo_t switch_job, Buf buffer )
+int switch_p_unpack_jobinfo ( switch_jobinfo_t *switch_job, Buf buffer )
 {
 	return SLURM_SUCCESS;
 }
 
-void switch_p_print_jobinfo(FILE *fp, switch_jobinfo_t jobinfo)
+void switch_p_print_jobinfo(FILE *fp, switch_jobinfo_t *jobinfo)
 {
 	return;
 }
 
-char *switch_p_sprint_jobinfo(switch_jobinfo_t switch_jobinfo, char *buf,
+char *switch_p_sprint_jobinfo(switch_jobinfo_t *switch_jobinfo, char *buf,
 		size_t size)
 {
 	if ((buf != NULL) && size) {
@@ -172,46 +174,46 @@ int switch_p_node_fini ( void )
 	return SLURM_SUCCESS;
 }
 
-int switch_p_job_preinit ( switch_jobinfo_t jobinfo )
+int switch_p_job_preinit ( switch_jobinfo_t *jobinfo )
 {
 	return SLURM_SUCCESS;
 }
 
-int switch_p_job_init ( switch_jobinfo_t jobinfo, uid_t uid )
+int switch_p_job_init ( switch_jobinfo_t *jobinfo, uid_t uid )
 {
 	return SLURM_SUCCESS;
 }
 
-int switch_p_job_fini ( switch_jobinfo_t jobinfo )
+int switch_p_job_fini ( switch_jobinfo_t *jobinfo )
 {
 	return SLURM_SUCCESS;
 }
 
-int switch_p_job_postfini ( switch_jobinfo_t jobinfo, uid_t pgid, 
+int switch_p_job_postfini ( switch_jobinfo_t *jobinfo, uid_t pgid,
 				uint32_t job_id, uint32_t step_id )
 {
 	/*
 	 *  Kill all processes in the job's session
 	 */
 	if (pgid) {
-		debug2("Sending SIGKILL to pgid %lu", 
-			(unsigned long) pgid); 
+		debug2("Sending SIGKILL to pgid %lu",
+			(unsigned long) pgid);
 		kill(-pgid, SIGKILL);
 	} else
-		debug("Job %u.%u: Bad pid valud %lu", job_id, 
+		debug("Job %u.%u: Bad pid valud %lu", job_id,
 		      step_id, (unsigned long) pgid);
 
 	return SLURM_SUCCESS;
 }
 
-int switch_p_job_attach ( switch_jobinfo_t jobinfo, char ***env, 
-			uint32_t nodeid, uint32_t procid, uint32_t nnodes, 
+int switch_p_job_attach ( switch_jobinfo_t *jobinfo, char ***env,
+			uint32_t nodeid, uint32_t procid, uint32_t nnodes,
 			uint32_t nprocs, uint32_t rank )
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_get_jobinfo(switch_jobinfo_t switch_job,
+extern int switch_p_get_jobinfo(switch_jobinfo_t *switch_job,
 	int key, void *resulting_data)
 {
 	slurm_seterrno(EINVAL);
@@ -221,11 +223,6 @@ extern int switch_p_get_jobinfo(switch_jobinfo_t switch_job,
 /*
  * switch functions for other purposes
  */
-bool switch_p_no_frag ( void )
-{
-	return false;
-}
-
 extern int switch_p_get_errno(void)
 {
 	return SLURM_SUCCESS;
@@ -238,41 +235,41 @@ extern char *switch_p_strerror(int errnum)
 
 /*
  * node switch state monitoring functions
- * required for IBM Federation switch 
+ * required for IBM Federation switch
  */
 extern int switch_p_clear_node_state(void)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_alloc_node_info(switch_node_info_t *switch_node)
+extern int switch_p_alloc_node_info(switch_node_info_t **switch_node)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_build_node_info(switch_node_info_t switch_node)
+extern int switch_p_build_node_info(switch_node_info_t *switch_node)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_pack_node_info(switch_node_info_t switch_node, 
+extern int switch_p_pack_node_info(switch_node_info_t *switch_node,
 	Buf buffer)
 {
 	return 0;
 }
 
-extern int switch_p_unpack_node_info(switch_node_info_t switch_node,
+extern int switch_p_unpack_node_info(switch_node_info_t *switch_node,
 	Buf buffer)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_free_node_info(switch_node_info_t *switch_node)
+extern int switch_p_free_node_info(switch_node_info_t **switch_node)
 {
 	return SLURM_SUCCESS;
 }
 
-extern char*switch_p_sprintf_node_info(switch_node_info_t switch_node,
+extern char*switch_p_sprintf_node_info(switch_node_info_t *switch_node,
 	char *buf, size_t size)
 {
 	if ((buf != NULL) && size) {
@@ -283,13 +280,13 @@ extern char*switch_p_sprintf_node_info(switch_node_info_t switch_node,
 	return NULL;
 }
 
-extern int switch_p_job_step_complete(switch_jobinfo_t jobinfo,
+extern int switch_p_job_step_complete(switch_jobinfo_t *jobinfo,
 	char *nodelist)
 {
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_step_part_comp(switch_jobinfo_t jobinfo,
+extern int switch_p_job_step_part_comp(switch_jobinfo_t *jobinfo,
 	char *nodelist)
 {
 	return SLURM_SUCCESS;
@@ -300,7 +297,7 @@ extern bool switch_p_part_comp(void)
 	return false;
 }
 
-extern int switch_p_job_step_allocated(switch_jobinfo_t jobinfo,
+extern int switch_p_job_step_allocated(switch_jobinfo_t *jobinfo,
 	char *nodelist)
 {
 	return SLURM_SUCCESS;
