@@ -5,32 +5,33 @@
  *  Copyright (C) 2006 The Regents of the University of California and
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -150,7 +151,7 @@ void slurm_chk_memset(nodemask_t *mask, slurmd_job_t *job)
 
 int get_memset(nodemask_t *mask, slurmd_job_t *job)
 {
-	int nummasks, maskid, i;
+	int nummasks, maskid, i, threads;
 	char *curstr, *selstr;
 	char mstr[1 + NUMA_NUM_NODES / 4];
 	int local_id = job->envtp->localid;
@@ -167,7 +168,8 @@ int get_memset(nodemask_t *mask, slurmd_job_t *job)
 	}
 
 	if (job->mem_bind_type & MEM_BIND_RANK) {
-		nodemask_set(mask, job->envtp->localid % job->cpus);
+		threads = MAX(conf->threads, 1);
+		nodemask_set(mask, job->envtp->localid % (job->cpus*threads));
 		return true;
 	}
 
@@ -238,4 +240,81 @@ int get_memset(nodemask_t *mask, slurmd_job_t *job)
 	return false;
 }
 
+
+static uint16_t *numa_array = NULL;
+
+/* helper function */
+static void _add_numa_mask_to_array(unsigned long *cpu_mask, int size,
+					uint16_t maxcpus, uint16_t nnode_id)
+{
+	unsigned long count = 1;
+	int i, j, x = sizeof(unsigned long) * 8;
+	for (i = 0; i < size; i++) {
+		/* iterate over each bit of this unsigned long */
+		for (j = 0, count = 1; j < x; j++, count *= 2) {
+			if (count & cpu_mask[i]) {
+				/* this bit in the cpu_mask is set */
+				int cpu = i * sizeof(unsigned long) + j;
+				if (cpu < maxcpus) {
+					numa_array[cpu] = nnode_id;
+				}
+			}
+		}
+	}
+}
+
+/* return the numa node for the given cpuid */
+extern uint16_t slurm_get_numa_node(uint16_t cpuid)
+{
+	uint16_t maxcpus = 0, nnid = 0;
+	int size, retry, max_node;
+	unsigned long *cpu_mask;
+
+	maxcpus = conf->sockets * conf->cores * conf->threads;
+	if (cpuid >= maxcpus)
+		return 0;
+
+	if (numa_array) {
+		return numa_array[cpuid];
+	}
+
+	/* need to load the numa_array */
+	max_node = numa_max_node();
+
+	/* The required size of the mask buffer for numa_node_to_cpus()
+	 * is goofed up. The third argument is supposed to be the size
+	 * of the mask, which is an array of unsigned longs. The *unit*
+	 * of the third argument is unclear - should it be in bytes or
+	 * in unsigned longs??? Since I don't know, I'm using this retry
+	 * loop to try and determine an acceptable size. If anyone can
+	 * fix this interaction, please do!!
+	 */
+	size = 8;
+	cpu_mask = xmalloc(sizeof(unsigned long) * size);
+	retry = 0;
+	while (retry++ < 8 && numa_node_to_cpus(nnid, cpu_mask, size) < 0) {
+		size *= 2;
+		xrealloc(cpu_mask, sizeof(unsigned long) * size);
+	}
+	if (retry >= 8) {
+		xfree(cpu_mask);
+		error("NUMA problem with numa_node_to_cpus arguments");
+		return 0;
+	}
+	numa_array = xmalloc(sizeof(uint16_t) * maxcpus);
+	_add_numa_mask_to_array(cpu_mask, size, maxcpus, nnid);
+	while (nnid++ < max_node) {
+		if (numa_node_to_cpus(nnid, cpu_mask, size) < 0) {
+			error("NUMA problem - numa_node_to_cpus 2nd call fail");
+			xfree(cpu_mask);
+			xfree(numa_array);
+			numa_array = NULL;
+			return 0;
+		}
+		_add_numa_mask_to_array(cpu_mask, size, maxcpus, nnid);
+	}
+	xfree(cpu_mask);
+	return numa_array[cpuid];
+}
+
 #endif	/* HAVE_NUMA */
