@@ -5,39 +5,41 @@
  *  Copyright (C) 2002-2008 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
 #include "src/sacctmgr/sacctmgr.h"
-bool tree_display = 0;
+static bool tree_display = 0;
+static bool without_limits = 0;
 
 static int _set_cond(int *start, int argc, char *argv[],
 		     acct_association_cond_t *assoc_cond,
@@ -45,7 +47,6 @@ static int _set_cond(int *start, int argc, char *argv[],
 {
 	int i, end = 0;
 	int set = 0;
-	List qos_list = NULL;
 	int command_len = 0;
 	int option = 0;
 	for (i=(*start); i<argc; i++) {
@@ -60,39 +61,43 @@ static int _set_cond(int *start, int argc, char *argv[],
 			}
 		}
 
-		if (!end && !strncasecmp (argv[i], "Tree", 
+		if (!end && !strncasecmp (argv[i], "Tree",
 					  MAX(command_len, 4))) {
 			tree_display = 1;
 		} else if (!end && !strncasecmp (argv[i], "WithDeleted",
 						 MAX(command_len, 5))) {
 			assoc_cond->with_deleted = 1;
-		} else if (!end && 
+		} else if (!end &&
 			   !strncasecmp (argv[i], "WithRawQOSLevel",
 					 MAX(command_len, 5))) {
 			assoc_cond->with_raw_qos = 1;
-		} else if (!end && 
+		} else if (!end &&
 			   !strncasecmp (argv[i], "WithSubAccounts",
 					 MAX(command_len, 5))) {
 			assoc_cond->with_sub_accts = 1;
-		} else if (!end && !strncasecmp (argv[i], "WOPInfo", 
+		} else if (!end && !strncasecmp (argv[i], "WOPInfo",
 						 MAX(command_len, 4))) {
 			assoc_cond->without_parent_info = 1;
 		} else if (!end && !strncasecmp (argv[i], "WOPLimits",
 						 MAX(command_len, 4))) {
 			assoc_cond->without_parent_limits = 1;
-		} else if(!end && !strncasecmp(argv[i], "where", 
+		} else if (!end && !strncasecmp (argv[i], "WOLimits",
+						 MAX(command_len, 3))) {
+			without_limits = 1;
+			assoc_cond->without_parent_limits = 1;
+		} else if(!end && !strncasecmp(argv[i], "where",
 					       MAX(command_len, 5))) {
 			continue;
-		} else if(!end || !strncasecmp (argv[i], "Ids", 
+		} else if(!end || !strncasecmp (argv[i], "Ids",
 						MAX(command_len, 1))
-			  || !strncasecmp (argv[i], "Associations", 
+			  || !strncasecmp (argv[i], "Associations",
 					   MAX(command_len, 2))) {
 			ListIterator itr = NULL;
 			char *temp = NULL;
 			uint32_t id = 0;
 
 			if(!assoc_cond->id_list)
-				assoc_cond->id_list = 
+				assoc_cond->id_list =
 					list_create(slurm_destroy_char);
 			slurm_addto_char_list(assoc_cond->id_list,
 					      argv[i]+end);
@@ -112,7 +117,7 @@ static int _set_cond(int *start, int argc, char *argv[],
 			   || !strncasecmp (argv[i], "Acct",
 					    MAX(command_len, 4))) {
 			if(!assoc_cond->acct_list)
-				assoc_cond->acct_list = 
+				assoc_cond->acct_list =
 					list_create(slurm_destroy_char);
 			slurm_addto_char_list(assoc_cond->acct_list,
 					argv[i]+end);
@@ -120,7 +125,7 @@ static int _set_cond(int *start, int argc, char *argv[],
 		} else if (!strncasecmp (argv[i], "Clusters",
 					 MAX(command_len, 1))) {
 			if(!assoc_cond->cluster_list)
-				assoc_cond->cluster_list = 
+				assoc_cond->cluster_list =
 					list_create(slurm_destroy_char);
 			slurm_addto_char_list(assoc_cond->cluster_list,
 					argv[i]+end);
@@ -244,10 +249,10 @@ static int _set_cond(int *start, int argc, char *argv[],
 				   assoc_cond->max_wall_pj_list,
 				   argv[i]+end))
 				set = 1;
-		} else if (!strncasecmp (argv[i], "Partitions", 
+		} else if (!strncasecmp (argv[i], "Partitions",
 					 MAX(command_len, 4))) {
 			if(!assoc_cond->partition_list)
-				assoc_cond->partition_list = 
+				assoc_cond->partition_list =
 					list_create(slurm_destroy_char);
 			slurm_addto_char_list(assoc_cond->partition_list,
 					argv[i]+end);
@@ -255,7 +260,7 @@ static int _set_cond(int *start, int argc, char *argv[],
 		} else if (!strncasecmp (argv[i], "Parent",
 					 MAX(command_len, 4))) {
 			if(!assoc_cond->parent_acct_list) {
-				assoc_cond->parent_acct_list = 
+				assoc_cond->parent_acct_list =
 					list_create(slurm_destroy_char);
 			}
 			if(slurm_addto_char_list(assoc_cond->parent_acct_list,
@@ -264,16 +269,16 @@ static int _set_cond(int *start, int argc, char *argv[],
 		} else if (!strncasecmp (argv[i], "QosLevel",
 					 MAX(command_len, 1))) {
 			if(!assoc_cond->qos_list) {
-				assoc_cond->qos_list = 
+				assoc_cond->qos_list =
 					list_create(slurm_destroy_char);
 			}
-			
-			if(!qos_list) {
-				qos_list = acct_storage_g_get_qos(
+
+			if(!g_qos_list) {
+				g_qos_list = acct_storage_g_get_qos(
 					db_conn, my_uid, NULL);
 			}
-			
-			if(addto_qos_char_list(assoc_cond->qos_list, qos_list,
+
+			if(addto_qos_char_list(assoc_cond->qos_list, g_qos_list,
 					       argv[i]+end, option))
 				set = 1;
 			else
@@ -281,7 +286,7 @@ static int _set_cond(int *start, int argc, char *argv[],
 		} else if (!strncasecmp (argv[i], "Users",
 					 MAX(command_len, 1))) {
 			if(!assoc_cond->user_list)
-				assoc_cond->user_list = 
+				assoc_cond->user_list =
 					list_create(slurm_destroy_char);
 			slurm_addto_char_list(assoc_cond->user_list,
 					argv[i]+end);
@@ -291,8 +296,6 @@ static int _set_cond(int *start, int argc, char *argv[],
 			fprintf(stderr, " Unknown condition: %s\n", argv[i]);
 		}
 	}
-	if(qos_list)
-		list_destroy(qos_list);
 
 	(*start) = i;
 
@@ -313,7 +316,6 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 	char *object = NULL;
 	char *print_acct = NULL, *last_cluster = NULL;
 	List tree_list = NULL;
-	List qos_list = NULL;
 
 	int field_count = 0;
 
@@ -349,17 +351,26 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 		PRINT_USER
 	};
 
-	_set_cond(&i, argc, argv, assoc_cond, format_list);
+	for (i=0; i<argc; i++) {
+		int command_len = strlen(argv[i]);
+		if (!strncasecmp (argv[i], "Where", MAX(command_len, 5))
+		    || !strncasecmp (argv[i], "Set", MAX(command_len, 3)))
+			i++;
+		_set_cond(&i, argc, argv, assoc_cond, format_list);
+	}
 
 	if(exit_code) {
 		destroy_acct_association_cond(assoc_cond);
 		list_destroy(format_list);
 		return SLURM_ERROR;
-	} else if(!list_count(format_list)) 
-		slurm_addto_char_list(format_list,
-				      "C,A,U,Part,F,GrpJ,GrpN,GrpS,"
-				      "MaxJ,MaxN,MaxS,MaxW,QOS");
-
+	} else if(!list_count(format_list)) {
+		slurm_addto_char_list(format_list, "C,A,U,Part");
+		if(!without_limits)
+			slurm_addto_char_list(format_list,
+					      "F,GrpCPUMins,GrpJ,GrpN,"
+					      "GrpS,GrpWall,"
+					      "MaxJ,MaxN,MaxS,MaxW,QOS");
+	}
 	print_fields_list = list_create(destroy_print_field);
 
 	itr = list_iterator_create(format_list);
@@ -367,14 +378,14 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 		char *tmp_char = NULL;
 		int command_len = 0;
 		int newlen = 0;
-		
+
 		if((tmp_char = strstr(object, "\%"))) {
 			newlen = atoi(tmp_char+1);
 			tmp_char[0] = '\0';
-		} 
+		}
 
 		command_len = strlen(object);
-	
+
 		field = xmalloc(sizeof(print_field_t));
 
 		if(!strncasecmp("Account", object, MAX(command_len, 1))
@@ -382,7 +393,7 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			field->type = PRINT_ACCOUNT;
 			field->name = xstrdup("Account");
 			if(tree_display)
-				field->len = 20;
+				field->len = -20;
 			else
 				field->len = 10;
 			field->print_routine = print_fields_str;
@@ -404,13 +415,13 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			field->name = xstrdup("GrpCPUMins");
 			field->len = 11;
 			field->print_routine = print_fields_uint64;
-		} else if(!strncasecmp("GrpCPUs", object, 
+		} else if(!strncasecmp("GrpCPUs", object,
 				       MAX(command_len, 8))) {
 			field->type = PRINT_GRPC;
 			field->name = xstrdup("GrpCPUs");
 			field->len = 8;
 			field->print_routine = print_fields_uint;
-		} else if(!strncasecmp("GrpJobs", object, 
+		} else if(!strncasecmp("GrpJobs", object,
 				       MAX(command_len, 4))) {
 			field->type = PRINT_GRPJ;
 			field->name = xstrdup("GrpJobs");
@@ -434,12 +445,6 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			field->name = xstrdup("GrpWall");
 			field->len = 11;
 			field->print_routine = print_fields_time;
-		} else if(!strncasecmp("GrpWallRaw", object,
-				       MAX(command_len, 8))) {
-			field->type = PRINT_GRPW;
-			field->name = xstrdup("GrpWall");
-			field->len = 11;
-			field->print_routine = print_fields_uint;
 		} else if(!strncasecmp("ID", object, MAX(command_len, 1))) {
 			field->type = PRINT_ID;
 			field->name = xstrdup("ID");
@@ -468,7 +473,7 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			field->name = xstrdup("MaxJobs");
 			field->len = 7;
 			field->print_routine = print_fields_uint;
-		} else if(!strncasecmp("MaxNodesPerJob", object, 
+		} else if(!strncasecmp("MaxNodesPerJob", object,
 				       MAX(command_len, 4))) {
 			field->type = PRINT_MAXN;
 			field->name = xstrdup("MaxNodes");
@@ -486,12 +491,6 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			field->name = xstrdup("MaxWall");
 			field->len = 11;
 			field->print_routine = print_fields_time;
-		} else if(!strncasecmp("MaxWallRaw", object,
-				       MAX(command_len, 8))) {
-			field->type = PRINT_MAXW;
-			field->name = xstrdup("MaxWall");
-			field->len = 11;
-			field->print_routine = print_fields_uint;
 		} else if(!strncasecmp("QOSRAWLevel", object,
 				       MAX(command_len, 4))) {
 			field->type = PRINT_QOS_RAW;
@@ -546,10 +545,10 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			continue;
 		}
 
-		if(newlen > 0) 
+		if(newlen)
 			field->len = newlen;
-		
-		list_append(print_fields_list, field);		
+
+		list_append(print_fields_list, field);
 	}
 	list_iterator_destroy(itr);
 	list_destroy(format_list);
@@ -585,11 +584,11 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			if(tree_list) {
 				list_flush(tree_list);
 			} else {
-				tree_list = 
+				tree_list =
 					list_create(destroy_acct_print_tree);
 			}
 			last_cluster = assoc->cluster;
-		} 
+		}
 		while((field = list_next(itr2))) {
 			switch(field->type) {
 			case PRINT_ACCOUNT:
@@ -603,19 +602,18 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 					} else {
 						local_acct =
 							xstrdup(assoc->acct);
-						parent_acct = 
+						parent_acct =
 							assoc->parent_acct;
 					}
 					print_acct = get_tree_acct_name(
 						local_acct,
-						parent_acct,
-						assoc->cluster, tree_list);
+						parent_acct, tree_list);
 					xfree(local_acct);
 				} else {
 					print_acct = assoc->acct;
 				}
 				field->print_routine(
-					field, 
+					field,
 					print_acct,
 					(curr_inx == field_count));
 				break;
@@ -628,7 +626,7 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 			case PRINT_FAIRSHARE:
 				field->print_routine(
 					field,
-					assoc->fairshare,
+					assoc->shares_raw,
 					(curr_inx == field_count));
 				break;
 			case PRINT_GRPCM:
@@ -643,7 +641,7 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_GRPJ:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->grp_jobs,
 						     (curr_inx == field_count));
 				break;
@@ -653,7 +651,7 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_GRPS:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->grp_submit_jobs,
 						     (curr_inx == field_count));
 				break;
@@ -664,12 +662,12 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 					(curr_inx == field_count));
 				break;
 			case PRINT_ID:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->id,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_LFT:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->lft,
 						     (curr_inx == field_count));
 				break;
@@ -685,7 +683,7 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_MAXJ:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->max_jobs,
 						     (curr_inx == field_count));
 				break;
@@ -695,7 +693,7 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_MAXS:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->max_submit_jobs,
 						     (curr_inx == field_count));
 				break;
@@ -721,12 +719,12 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_QOS:
-				if(!qos_list) 
-					qos_list = acct_storage_g_get_qos(
+				if(!g_qos_list)
+					g_qos_list = acct_storage_g_get_qos(
 						db_conn, my_uid, NULL);
-				
+
 				field->print_routine(field,
-						     qos_list,
+						     g_qos_list,
 						     assoc->qos_list,
 						     (curr_inx == field_count));
 				break;
@@ -736,12 +734,12 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_RGT:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->rgt,
 						     (curr_inx == field_count));
 				break;
 			case PRINT_USER:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->user,
 						     (curr_inx == field_count));
 				break;
@@ -757,12 +755,9 @@ extern int sacctmgr_list_association(int argc, char *argv[])
 		printf("\n");
 	}
 
-	if(qos_list)
-		list_destroy(qos_list);
-
-	if(tree_list) 
+	if(tree_list)
 		list_destroy(tree_list);
-			
+
 	list_iterator_destroy(itr2);
 	list_iterator_destroy(itr);
 	list_destroy(first_list);
