@@ -6,32 +6,33 @@
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -39,10 +40,12 @@
 
 #include "src/sacctmgr/sacctmgr.h"
 #include "src/common/uid.h"
+static bool without_limits = 0;
 
 static int _set_cond(int *start, int argc, char *argv[],
 		     List cluster_list,
-		     List format_list)
+		     List format_list,
+		     uint16_t *classification)
 {
 	int i;
 	int set = 0;
@@ -68,15 +71,26 @@ static int _set_cond(int *start, int argc, char *argv[],
 		} else if(!end && !strncasecmp(argv[i], "where",
 					       MAX(command_len, 5))) {
 			continue;
+		} else if (!end && !strncasecmp (argv[i], "WOLimits",
+						 MAX(command_len, 3))) {
+			without_limits = 1;
 		} else if(!end || !strncasecmp (argv[i], "Names",
 						MAX(command_len, 1))
 			  || !strncasecmp (argv[i], "Clusters",
-					   MAX(command_len, 1))) {
+					   MAX(command_len, 3))) {
 			if(cluster_list) {
 				if(slurm_addto_char_list(cluster_list,
 							 argv[i]+end))
 					set = 1;
 			}
+		} else if (!strncasecmp (argv[i], "Classification",
+					 MAX(command_len, 3))) {
+			if(classification) {
+				*classification =
+					str_2_classification(argv[i]+end);
+				if(*classification)
+					set = 1;
+			}
 		} else if (!strncasecmp (argv[i], "Format",
 					 MAX(command_len, 1))) {
 			if(format_list)
@@ -95,12 +109,12 @@ static int _set_cond(int *start, int argc, char *argv[],
 
 static int _set_rec(int *start, int argc, char *argv[],
 		    List name_list,
-		    acct_association_rec_t *assoc)
+		    acct_association_rec_t *assoc,
+		    uint16_t *classification)
 {
 	int i, mins;
 	int set = 0;
 	int end = 0;
-	List qos_list = NULL;
 	int command_len = 0;
 	int option = 0;
 
@@ -115,39 +129,48 @@ static int _set_rec(int *start, int argc, char *argv[],
 				end++;
 			}
 		}
-		
+
 		if (!strncasecmp (argv[i], "Where", MAX(command_len, 5))) {
 			i--;
 			break;
-		} else if(!end && !strncasecmp(argv[i], "set", 
+		} else if(!end && !strncasecmp(argv[i], "set",
 					       MAX(command_len, 3))) {
 			continue;
 		} else if(!end
 			  || !strncasecmp (argv[i], "Names",
-					   MAX(command_len, 1)) 
-			  || !strncasecmp (argv[i], "Clusters", 
-					   MAX(command_len, 1))) {
+					   MAX(command_len, 1))
+			  || !strncasecmp (argv[i], "Clusters",
+					   MAX(command_len, 3))) {
 			if(name_list)
-				slurm_addto_char_list(name_list, argv[i]+end);
-		} else if (!strncasecmp (argv[i], "FairShare", 
+				slurm_addto_char_list(name_list,
+						      argv[i]+end);
+		} else if (!strncasecmp (argv[i], "Classification",
+					 MAX(command_len, 3))) {
+			if(classification) {
+				*classification =
+					str_2_classification(argv[i]+end);
+				if(*classification)
+					set = 1;
+			}
+		} else if (!strncasecmp (argv[i], "FairShare",
 					 MAX(command_len, 1))
 			   || !strncasecmp (argv[i], "Shares",
 					 MAX(command_len, 1))) {
-			if (get_uint(argv[i]+end, &assoc->fairshare, 
+			if (get_uint(argv[i]+end, &assoc->shares_raw,
 			    "FairShare") == SLURM_SUCCESS)
 				set = 1;
 		} else if (!strncasecmp (argv[i], "GrpCPUMins",
 					 MAX(command_len, 7))) {
-			if (get_uint64(argv[i]+end, 
-				       &assoc->grp_cpu_mins, 
-				       "GrpCPUMins") == SLURM_SUCCESS)
-				set = 1;
+			exit_code=1;
+			fprintf(stderr, "GrpCPUMins is not a valid option "
+				"for the root association of a cluster.\n");
+			break;
 		} else if (!strncasecmp (argv[i], "GrpCpus",
 					 MAX(command_len, 7))) {
 			if (get_uint(argv[i]+end, &assoc->grp_cpus,
 			    "GrpCpus") == SLURM_SUCCESS)
 				set = 1;
-		} else if (!strncasecmp (argv[i], "GrpJobs", 
+		} else if (!strncasecmp (argv[i], "GrpJobs",
 					 MAX(command_len, 4))) {
 			if (get_uint(argv[i]+end, &assoc->grp_jobs,
 			    "GrpJobs") == SLURM_SUCCESS)
@@ -164,23 +187,16 @@ static int _set_rec(int *start, int argc, char *argv[],
 				set = 1;
 		} else if (!strncasecmp (argv[i], "GrpWall",
 					 MAX(command_len, 4))) {
-			mins = time_str2mins(argv[i]+end);
-			if (mins != NO_VAL) {
-				assoc->grp_wall	= (uint32_t) mins;
-				set = 1;
-			} else {
-				exit_code=1;
-				fprintf(stderr, 
-					" Bad GrpWall time format: %s\n", 
-					argv[i]);
-			}
+			exit_code=1;
+			fprintf(stderr, "GrpWall is not a valid option "
+				"for the root association of a cluster.\n");
 		} else if (!strncasecmp (argv[i], "MaxCPUMinsPerJob",
 					 MAX(command_len, 7))) {
-			if (get_uint64(argv[i]+end, 
-				       &assoc->max_cpu_mins_pj, 
+			if (get_uint64(argv[i]+end,
+				       &assoc->max_cpu_mins_pj,
 				       "MaxCPUMins") == SLURM_SUCCESS)
 				set = 1;
-		} else if (!strncasecmp (argv[i], "MaxCpusPerJob", 
+		} else if (!strncasecmp (argv[i], "MaxCpusPerJob",
 					 MAX(command_len, 7))) {
 			if (get_uint(argv[i]+end, &assoc->max_cpus_pj,
 			    "MaxCpus") == SLURM_SUCCESS)
@@ -192,7 +208,7 @@ static int _set_rec(int *start, int argc, char *argv[],
 				set = 1;
 		} else if (!strncasecmp (argv[i], "MaxNodesPerJob",
 					 MAX(command_len, 4))) {
-			if (get_uint(argv[i]+end, 
+			if (get_uint(argv[i]+end,
 			    &assoc->max_nodes_pj,
 			    "MaxNodes") == SLURM_SUCCESS)
 				set = 1;
@@ -201,7 +217,7 @@ static int _set_rec(int *start, int argc, char *argv[],
 			if (get_uint(argv[i]+end, &assoc->max_submit_jobs,
 			    "MaxSubmitJobs") == SLURM_SUCCESS)
 				set = 1;
-		} else if (!strncasecmp (argv[i], "MaxWallDurationPerJob", 
+		} else if (!strncasecmp (argv[i], "MaxWallDurationPerJob",
 					 MAX(command_len, 4))) {
 			mins = time_str2mins(argv[i]+end);
 			if (mins != NO_VAL) {
@@ -209,22 +225,22 @@ static int _set_rec(int *start, int argc, char *argv[],
 				set = 1;
 			} else {
 				exit_code=1;
-				fprintf(stderr, 
-					" Bad MaxWall time format: %s\n", 
+				fprintf(stderr,
+					" Bad MaxWall time format: %s\n",
 					argv[i]);
 			}
-		} else if (!strncasecmp (argv[i], "QOSLevel", 
+		} else if (!strncasecmp (argv[i], "QOSLevel",
 					 MAX(command_len, 1))) {
-			if(!assoc->qos_list) 
-				assoc->qos_list = 
+			if(!assoc->qos_list)
+				assoc->qos_list =
 					list_create(slurm_destroy_char);
-						
-			if(!qos_list) 
-				qos_list = acct_storage_g_get_qos(
+
+			if(!g_qos_list)
+				g_qos_list = acct_storage_g_get_qos(
 					db_conn, my_uid, NULL);
-						
+
 			if(addto_qos_char_list(assoc->qos_list,
-					       qos_list, argv[i]+end, option))
+					       g_qos_list, argv[i]+end, option))
 				set = 1;
 			else
 				exit_code = 1;
@@ -237,9 +253,6 @@ static int _set_rec(int *start, int argc, char *argv[],
 	}
 	(*start) = i;
 
-	if(qos_list)
-		list_destroy(qos_list);
-
 	return set;
 
 }
@@ -257,19 +270,25 @@ extern int sacctmgr_add_cluster(int argc, char *argv[])
 	int limit_set = 0;
 	ListIterator itr = NULL, itr_c = NULL;
 	char *name = NULL;
+	uint16_t class = 0;
 
 	init_acct_association_rec(&start_assoc);
 
-	for (i=0; i<argc; i++) 
-		limit_set = _set_rec(&i, argc, argv, name_list, &start_assoc);
-
+	for (i=0; i<argc; i++) {
+		int command_len = strlen(argv[i]);
+		if (!strncasecmp (argv[i], "Where", MAX(command_len, 5))
+		    || !strncasecmp (argv[i], "Set", MAX(command_len, 3)))
+			i++;
+		limit_set += _set_rec(&i, argc, argv,
+				     name_list, &start_assoc, &class);
+	}
 	if(exit_code) {
 		list_destroy(name_list);
 		return SLURM_ERROR;
 	} else if(!list_count(name_list)) {
 		list_destroy(name_list);
 		exit_code=1;
-		fprintf(stderr, " Need name of cluster to add.\n"); 
+		fprintf(stderr, " Need name of cluster to add.\n");
 		return SLURM_ERROR;
 	} else {
 		List temp_list = NULL;
@@ -278,6 +297,7 @@ extern int sacctmgr_add_cluster(int argc, char *argv[])
 
 		memset(&cluster_cond, 0, sizeof(acct_cluster_cond_t));
 		cluster_cond.cluster_list = name_list;
+		cluster_cond.classification = class;
 
 		temp_list = acct_storage_g_get_clusters(db_conn, my_uid,
 							&cluster_cond);
@@ -319,24 +339,26 @@ extern int sacctmgr_add_cluster(int argc, char *argv[])
 	itr = list_iterator_create(name_list);
 	while((name = list_next(itr))) {
 		cluster = xmalloc(sizeof(acct_cluster_rec_t));
-		
+
 		list_append(cluster_list, cluster);
 		cluster->name = xstrdup(name);
+		cluster->classification = class;
 		cluster->root_assoc = xmalloc(sizeof(acct_association_rec_t));
 		init_acct_association_rec(cluster->root_assoc);
 		printf("  Name          = %s\n", cluster->name);
+		if(cluster->classification)
+			printf("  Classification= %s\n",
+			       get_classification_str(cluster->classification));
+
+		cluster->root_assoc->shares_raw = start_assoc.shares_raw;
 
-		cluster->root_assoc->fairshare = start_assoc.fairshare;		
-		
-		cluster->root_assoc->grp_cpu_mins = start_assoc.grp_cpu_mins;
 		cluster->root_assoc->grp_cpus = start_assoc.grp_cpus;
 		cluster->root_assoc->grp_jobs = start_assoc.grp_jobs;
 		cluster->root_assoc->grp_nodes = start_assoc.grp_nodes;
 		cluster->root_assoc->grp_submit_jobs =
 			start_assoc.grp_submit_jobs;
-		cluster->root_assoc->grp_wall = start_assoc.grp_wall;
 
-		cluster->root_assoc->max_cpu_mins_pj = 
+		cluster->root_assoc->max_cpu_mins_pj =
 			start_assoc.max_cpu_mins_pj;
 		cluster->root_assoc->max_cpus_pj = start_assoc.max_cpus_pj;
 		cluster->root_assoc->max_jobs = start_assoc.max_jobs;
@@ -345,7 +367,7 @@ extern int sacctmgr_add_cluster(int argc, char *argv[])
 			start_assoc.max_submit_jobs;
 		cluster->root_assoc->max_wall_pj = start_assoc.max_wall_pj;
 
-		cluster->root_assoc->qos_list = 
+		cluster->root_assoc->qos_list =
 			copy_char_list(start_assoc.qos_list);
 	}
 	list_iterator_destroy(itr);
@@ -379,7 +401,7 @@ extern int sacctmgr_add_cluster(int argc, char *argv[])
 	}
 end_it:
 	list_destroy(cluster_list);
-	
+
 	return rc;
 }
 
@@ -394,7 +416,6 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 	ListIterator itr2 = NULL;
 	acct_cluster_rec_t *cluster = NULL;
 	char *object;
-	List qos_list = NULL;
 
 	int field_count = 0;
 
@@ -407,6 +428,8 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 		PRINT_CLUSTER,
 		PRINT_CHOST,
 		PRINT_CPORT,
+		PRINT_CLASS,
+		PRINT_CPUS,
 		PRINT_FAIRSHARE,
 		PRINT_GRPCM,
 		PRINT_GRPC,
@@ -420,14 +443,25 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 		PRINT_MAXN,
 		PRINT_MAXS,
 		PRINT_MAXW,
+		PRINT_NODECNT,
+		PRINT_NODES,
 		PRINT_QOS,
 		PRINT_QOS_RAW,
-		PRINT_RPC_VERSION		
+		PRINT_RPC_VERSION
 	};
 
 
 	cluster_cond->cluster_list = list_create(slurm_destroy_char);
-	_set_cond(&i, argc, argv, cluster_cond->cluster_list, format_list);
+	for (i=0; i<argc; i++) {
+		int command_len = strlen(argv[i]);
+		if (!strncasecmp (argv[i], "Where", MAX(command_len, 5))
+		    || !strncasecmp (argv[i], "Set", MAX(command_len, 3)))
+			i++;
+		_set_cond(&i, argc, argv, cluster_cond->cluster_list,
+			  format_list,
+			  &cluster_cond->classification);
+	}
+
 	if(exit_code) {
 		destroy_acct_cluster_cond(cluster_cond);
 		list_destroy(format_list);
@@ -437,9 +471,12 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 	print_fields_list = list_create(destroy_print_field);
 
 	if(!list_count(format_list)) {
-		slurm_addto_char_list(format_list, 
-				      "Cl,Controlh,Controlp,RPC,F,"
-				      "GrpJ,GrpN,GrpS,MaxJ,MaxN,MaxS,MaxW,QOS");
+		slurm_addto_char_list(format_list,
+				      "Cl,Controlh,Controlp,RPC");
+		if(!without_limits)
+			slurm_addto_char_list(format_list,
+					      "F,GrpJ,GrpN,GrpS,MaxJ,MaxN,"
+					      "MaxS,MaxW,QOS");
 	}
 
 	itr = list_iterator_create(format_list);
@@ -447,11 +484,11 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 		char *tmp_char = NULL;
 		int command_len = 0;
 		int newlen = 0;
-		
+
 		if((tmp_char = strstr(object, "\%"))) {
 			newlen = atoi(tmp_char+1);
 			tmp_char[0] = '\0';
-		} 
+		}
 
 		command_len = strlen(object);
 
@@ -465,28 +502,34 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 		} else if(!strncasecmp("ControlHost", object,
 				       MAX(command_len, 8))) {
 			field->type = PRINT_CHOST;
-			field->name = xstrdup("Control Host");
+			field->name = xstrdup("ControlHost");
 			field->len = 15;
 			field->print_routine = print_fields_str;
 		} else if(!strncasecmp("ControlPort", object,
 				       MAX(command_len, 8))) {
 			field->type = PRINT_CPORT;
-			field->name = xstrdup("Control Port");
+			field->name = xstrdup("ControlPort");
 			field->len = 12;
 			field->print_routine = print_fields_uint;
-		} else if(!strncasecmp("FairShare", object, 
+		} else if(!strncasecmp("Classification", object,
+				       MAX(command_len, 2))) {
+			field->type = PRINT_CPUS;
+			field->name = xstrdup("Class");
+			field->len = 9;
+			field->print_routine = print_fields_str;
+		} else if(!strncasecmp("CPUCount", object,
+				       MAX(command_len, 2))) {
+			field->type = PRINT_CPUS;
+			field->name = xstrdup("CPUCount");
+			field->len = 9;
+			field->print_routine = print_fields_str;
+		} else if(!strncasecmp("FairShare", object,
 				       MAX(command_len, 1))) {
 			field->type = PRINT_FAIRSHARE;
 			field->name = xstrdup("FairShare");
 			field->len = 9;
 			field->print_routine = print_fields_uint;
-		} else if(!strncasecmp("GrpCPUMins", object, 
-				       MAX(command_len, 8))) {
-			field->type = PRINT_GRPCM;
-			field->name = xstrdup("GrpCPUMins");
-			field->len = 11;
-			field->print_routine = print_fields_uint64;
-		} else if(!strncasecmp("GrpCPUs", object, 
+		} else if(!strncasecmp("GrpCPUs", object,
 				       MAX(command_len, 8))) {
 			field->type = PRINT_GRPC;
 			field->name = xstrdup("GrpCPUs");
@@ -510,12 +553,6 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 			field->name = xstrdup("GrpSubmit");
 			field->len = 9;
 			field->print_routine = print_fields_uint;
-		} else if(!strncasecmp("GrpWall", object,
-				       MAX(command_len, 4))) {
-			field->type = PRINT_GRPW;
-			field->name = xstrdup("GrpWall");
-			field->len = 11;
-			field->print_routine = print_fields_time;
 		} else if(!strncasecmp("MaxCPUMinsPerJob", object,
 				       MAX(command_len, 7))) {
 			field->type = PRINT_MAXCM;
@@ -534,7 +571,7 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 			field->name = xstrdup("MaxJobs");
 			field->len = 7;
 			field->print_routine = print_fields_uint;
-		} else if(!strncasecmp("MaxNodesPerJob", object, 
+		} else if(!strncasecmp("MaxNodesPerJob", object,
 				       MAX(command_len, 4))) {
 			field->type = PRINT_MAXN;
 			field->name = xstrdup("MaxNodes");
@@ -546,13 +583,25 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 			field->name = xstrdup("MaxSubmit");
 			field->len = 9;
 			field->print_routine = print_fields_uint;
-		} else if(!strncasecmp("MaxWallDurationPerJob", object, 
+		} else if(!strncasecmp("MaxWallDurationPerJob", object,
 				       MAX(command_len, 4))) {
 			field->type = PRINT_MAXW;
 			field->name = xstrdup("MaxWall");
 			field->len = 11;
 			field->print_routine = print_fields_time;
-		} else if(!strncasecmp("QOSRAWLevel", object, 
+		} else if(!strncasecmp("NodeCount", object,
+				       MAX(command_len, 5))) {
+			field->type = PRINT_NODECNT;
+			field->name = xstrdup("NodeCount");
+			field->len = 9;
+			field->print_routine = print_fields_uint;
+		} else if(!strncasecmp("NodeNames", object,
+				       MAX(command_len, 5))) {
+			field->type = PRINT_NODES;
+			field->name = xstrdup("NodeNames");
+			field->len = 20;
+			field->print_routine = print_fields_str;
+		} else if(!strncasecmp("QOSRAWLevel", object,
 				       MAX(command_len, 4))) {
 			field->type = PRINT_QOS_RAW;
 			field->name = xstrdup("QOS_RAW");
@@ -569,7 +618,7 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 			field->name = xstrdup("RPC");
 			field->len = 3;
 			field->print_routine = print_fields_uint;
-		} else if(!strncasecmp("Shares", object, 
+		} else if(!strncasecmp("Shares", object,
 				       MAX(command_len, 1))) {
 			field->type = PRINT_FAIRSHARE;
 			field->name = xstrdup("Shares");
@@ -581,11 +630,11 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 			xfree(field);
 			continue;
 		}
-		
-		if(newlen > 0) 
+
+		if(newlen)
 			field->len = newlen;
-		
-		list_append(print_fields_list, field);		
+
+		list_append(print_fields_list, field);
 	}
 	list_iterator_destroy(itr);
 	list_destroy(format_list);
@@ -599,7 +648,7 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 	cluster_list = acct_storage_g_get_clusters(db_conn, my_uid,
 						   cluster_cond);
 	destroy_acct_cluster_cond(cluster_cond);
-	
+
 	if(!cluster_list) {
 		exit_code=1;
 		fprintf(stderr, " Problem with query.\n");
@@ -633,16 +682,28 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 						     cluster->control_port,
 						     (curr_inx == field_count));
 				break;
+			case PRINT_CLASS:
+				field->print_routine(field,
+						     get_classification_str(
+							     cluster->
+							     classification),
+						     (curr_inx == field_count));
+				break;
+			case PRINT_CPUS:
+			{
+				char tmp_char[9];
+				convert_num_unit((float)cluster->cpu_count,
+						 tmp_char, sizeof(tmp_char),
+						 UNIT_NONE);
+				field->print_routine(field,
+						     tmp_char,
+						     (curr_inx == field_count));
+				break;
+			}
 			case PRINT_FAIRSHARE:
 				field->print_routine(
 					field,
-					cluster->root_assoc->fairshare,
-					(curr_inx == field_count));
-				break;
-			case PRINT_GRPCM:
-				field->print_routine(
-					field,
-					assoc->grp_cpu_mins,
+					assoc->shares_raw,
 					(curr_inx == field_count));
 				break;
 			case PRINT_GRPC:
@@ -651,7 +712,7 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_GRPJ:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->grp_jobs,
 						     (curr_inx == field_count));
 				break;
@@ -661,16 +722,10 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_GRPS:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->grp_submit_jobs,
 						     (curr_inx == field_count));
 				break;
-			case PRINT_GRPW:
-				field->print_routine(
-					field,
-					assoc->grp_wall,
-					(curr_inx == field_count));
-				break;
 			case PRINT_MAXCM:
 				field->print_routine(
 					field,
@@ -683,7 +738,7 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_MAXJ:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->max_jobs,
 						     (curr_inx == field_count));
 				break;
@@ -693,7 +748,7 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_MAXS:
-				field->print_routine(field, 
+				field->print_routine(field,
 						     assoc->max_submit_jobs,
 						     (curr_inx == field_count));
 				break;
@@ -703,13 +758,35 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 					assoc->max_wall_pj,
 					(curr_inx == field_count));
 				break;
+
+			case PRINT_NODECNT:
+			{
+				hostlist_t hl = hostlist_create(cluster->nodes);
+				int cnt = 0;
+				if(hl) {
+					cnt = hostlist_count(hl);
+					hostlist_destroy(hl);
+				}
+				field->print_routine(
+					field,
+					cnt,
+					(curr_inx == field_count));
+				break;
+			}
+			case PRINT_NODES:
+				field->print_routine(
+					field,
+					cluster->nodes,
+					(curr_inx == field_count));
+				break;
+
 			case PRINT_QOS:
-				if(!qos_list) 
-					qos_list = acct_storage_g_get_qos(
+				if(!g_qos_list)
+					g_qos_list = acct_storage_g_get_qos(
 						db_conn, my_uid, NULL);
-				
+
 				field->print_routine(field,
-						     qos_list,
+						     g_qos_list,
 						     assoc->qos_list,
 						     (curr_inx == field_count));
 				break;
@@ -736,9 +813,6 @@ extern int sacctmgr_list_cluster(int argc, char *argv[])
 		printf("\n");
 	}
 
-	if(qos_list)
-		list_destroy(qos_list);
-
 	list_iterator_destroy(itr2);
 	list_iterator_destroy(itr);
 	list_destroy(cluster_list);
@@ -756,7 +830,7 @@ extern int sacctmgr_modify_cluster(int argc, char *argv[])
 		xmalloc(sizeof(acct_association_cond_t));
 	int cond_set = 0, rec_set = 0, set = 0;
 	List ret_list = NULL;
-
+	uint16_t class_cond = 0, class_rec = 0;
 
 	init_acct_association_rec(assoc);
 
@@ -768,15 +842,17 @@ extern int sacctmgr_modify_cluster(int argc, char *argv[])
 		if (!strncasecmp (argv[i], "Where", MAX(command_len, 5))) {
 			i++;
 			if(_set_cond(&i, argc, argv,
-				     assoc_cond->cluster_list, NULL))
+				     assoc_cond->cluster_list,
+				     NULL, &class_cond))
 				cond_set = 1;
 		} else if (!strncasecmp (argv[i], "Set", MAX(command_len, 3))) {
 			i++;
-			if(_set_rec(&i, argc, argv, NULL, assoc))
+			if(_set_rec(&i, argc, argv, NULL, assoc, &class_rec))
 				rec_set = 1;
 		} else {
 			if(_set_cond(&i, argc, argv,
-				     assoc_cond->cluster_list, NULL))
+				     assoc_cond->cluster_list,
+				     NULL, &class_cond))
 				cond_set = 1;
 		}
 	}
@@ -794,24 +870,63 @@ extern int sacctmgr_modify_cluster(int argc, char *argv[])
 			destroy_acct_association_rec(assoc);
 			destroy_acct_association_cond(assoc_cond);
 			return SLURM_SUCCESS;
-		}		
+		}
 	} else if(exit_code) {
 		destroy_acct_association_rec(assoc);
 		destroy_acct_association_cond(assoc_cond);
-		return SLURM_ERROR;		
+		return SLURM_ERROR;
+	}
+
+	if(class_cond) {
+		List temp_list = NULL;
+		acct_cluster_cond_t cluster_cond;
+
+		memset(&cluster_cond, 0, sizeof(acct_cluster_cond_t));
+		cluster_cond.cluster_list = assoc_cond->cluster_list;
+		cluster_cond.classification = class_cond;
+
+		temp_list = acct_storage_g_get_clusters(db_conn, my_uid,
+							&cluster_cond);
+		if(!temp_list) {
+			exit_code=1;
+			fprintf(stderr,
+				" Problem getting clusters from database.  "
+				"Contact your admin.\n");
+			destroy_acct_association_rec(assoc);
+			destroy_acct_association_cond(assoc_cond);
+			return SLURM_ERROR;
+		} else if(!list_count(temp_list)) {
+			fprintf(stderr,
+				" The class you gave %s didn't "
+				"return any clusters.\n",
+				get_classification_str(class_cond));
+			destroy_acct_association_rec(assoc);
+			destroy_acct_association_cond(assoc_cond);
+			list_destroy(temp_list);
+			return SLURM_ERROR;
+		}
+		/* we are only looking for the clusters returned from
+		   this query, so we free the cluster_list and replace
+		   it */
+		if(assoc_cond->cluster_list)
+			list_destroy(assoc_cond->cluster_list);
+		assoc_cond->cluster_list = temp_list;
 	}
 
 	printf(" Setting\n");
 	if(rec_set) {
 		printf(" Default Limits =\n");
 		sacctmgr_print_assoc_limits(assoc);
+		if(class_rec)
+			printf(" Cluster Classification = %s\n",
+			       get_classification_str(class_rec));
 	}
 
 	list_append(assoc_cond->acct_list, "root");
 	notice_thread_init();
 	ret_list = acct_storage_g_modify_associations(
 		db_conn, my_uid, assoc_cond, assoc);
-	
+
 	if(ret_list && list_count(ret_list)) {
 		char *object = NULL;
 		ListIterator itr = list_iterator_create(ret_list);
@@ -831,10 +946,47 @@ extern int sacctmgr_modify_cluster(int argc, char *argv[])
 
 	if(ret_list)
 		list_destroy(ret_list);
+
+	if(class_rec) {
+		acct_cluster_cond_t cluster_cond;
+		acct_cluster_rec_t cluster_rec;
+
+		memset(&cluster_cond, 0, sizeof(acct_cluster_cond_t));
+		memset(&cluster_rec, 0, sizeof(acct_cluster_rec_t));
+		/* the class has already returned these clusters so
+		   just go with it */
+		cluster_cond.cluster_list = assoc_cond->cluster_list;
+
+		cluster_rec.classification = class_rec;
+
+		ret_list = acct_storage_g_modify_clusters(
+			db_conn, my_uid, &cluster_cond, &cluster_rec);
+
+		if(ret_list && list_count(ret_list)) {
+			char *object = NULL;
+			ListIterator itr = list_iterator_create(ret_list);
+			printf(" Modified cluster classifications...\n");
+			while((object = list_next(itr))) {
+				printf("  %s\n", object);
+			}
+			list_iterator_destroy(itr);
+			set = 1;
+		} else if(ret_list) {
+			printf(" Nothing modified\n");
+		} else {
+			exit_code=1;
+			fprintf(stderr, " Error with request\n");
+			rc = SLURM_ERROR;
+		}
+
+		if(ret_list)
+			list_destroy(ret_list);
+	}
+
 	notice_thread_fini();
 
 	if(set) {
-		if(commit_check("Would you like to commit changes?")) 
+		if(commit_check("Would you like to commit changes?"))
 			acct_storage_g_commit(db_conn, 1);
 		else {
 			printf(" Changes Discarded\n");
@@ -854,18 +1006,38 @@ extern int sacctmgr_delete_cluster(int argc, char *argv[])
 		xmalloc(sizeof(acct_cluster_cond_t));
 	int i=0;
 	List ret_list = NULL;
+	int cond_set = 0;
 
 	cluster_cond->cluster_list = list_create(slurm_destroy_char);
-	
-	if(!_set_cond(&i, argc, argv, cluster_cond->cluster_list, NULL)) {
+
+	for (i=0; i<argc; i++) {
+		int command_len = strlen(argv[i]);
+		if (!strncasecmp (argv[i], "Where", MAX(command_len, 5))
+		    || !strncasecmp (argv[i], "Set", MAX(command_len, 3)))
+			i++;
+		cond_set += _set_cond(&i, argc, argv,
+				      cluster_cond->cluster_list,
+				      NULL,
+				      &cluster_cond->classification);
+	}
+
+	if(exit_code) {
+		destroy_acct_cluster_cond(cluster_cond);
+		return SLURM_ERROR;
+	} else if(!cond_set) {
 		exit_code=1;
-		fprintf(stderr, 
+		fprintf(stderr,
 			" No conditions given to remove, not executing.\n");
 		destroy_acct_cluster_cond(cluster_cond);
 		return SLURM_ERROR;
 	}
 
-	if(!list_count(cluster_cond->cluster_list)) {
+	if(!list_count(cluster_cond->cluster_list)
+	   && !cluster_cond->classification) {
+		exit_code=1;
+		fprintf(stderr,
+			"problem with delete request.  "
+			"Nothing given to delete.\n");
 		destroy_acct_cluster_cond(cluster_cond);
 		return SLURM_SUCCESS;
 	}
@@ -921,6 +1093,7 @@ extern int sacctmgr_dump_cluster (int argc, char *argv[])
 	char *line = NULL;
 	int i, command_len = 0;
 	FILE *fd = NULL;
+	char *class_str = NULL;
 
 	for (i=0; i<argc; i++) {
 		int end = parse_option_end(argv[i]);
@@ -939,7 +1112,7 @@ extern int sacctmgr_dump_cluster (int argc, char *argv[])
 					 MAX(command_len, 1))) {
 			if(cluster_name) {
 				exit_code=1;
-				fprintf(stderr, 
+				fprintf(stderr,
 					" Can only do one cluster at a time.  "
 				       "Already doing %s\n", cluster_name);
 				continue;
@@ -949,22 +1122,54 @@ extern int sacctmgr_dump_cluster (int argc, char *argv[])
 					 MAX(command_len, 1))) {
 			if(file_name) {
 				exit_code=1;
-				fprintf(stderr, 
+				fprintf(stderr,
 					" File name already set to %s\n",
 					file_name);
 				continue;
-			}		
+			}
 			file_name = xstrdup(argv[i]+end);
 		} else {
 			exit_code=1;
 			fprintf(stderr, " Unknown option: %s\n", argv[i]);
-		}		
+		}
 	}
 
 	if(!cluster_name) {
 		exit_code=1;
 		fprintf(stderr, " We need a cluster to dump.\n");
 		return SLURM_ERROR;
+	} else {
+		List temp_list = NULL;
+		acct_cluster_cond_t cluster_cond;
+		acct_cluster_rec_t *cluster_rec = NULL;
+
+		memset(&cluster_cond, 0, sizeof(acct_cluster_cond_t));
+		cluster_cond.cluster_list = list_create(NULL);
+		list_push(cluster_cond.cluster_list, cluster_name);
+
+		temp_list = acct_storage_g_get_clusters(db_conn, my_uid,
+							&cluster_cond);
+		list_destroy(cluster_cond.cluster_list);
+		if(!temp_list) {
+			exit_code=1;
+			fprintf(stderr,
+				" Problem getting clusters from database.  "
+				"Contact your admin.\n");
+			xfree(cluster_name);
+			return SLURM_ERROR;
+		}
+
+		cluster_rec = list_peek(temp_list);
+		if(!cluster_rec) {
+			exit_code=1;
+			fprintf(stderr, " Cluster %s doesn't exist.\n",
+				cluster_name);
+			xfree(cluster_name);
+			list_destroy(temp_list);
+			return SLURM_ERROR;
+		}
+		class_str = get_classification_str(cluster_rec->classification);
+		list_destroy(temp_list);
 	}
 
 	if(!file_name) {
@@ -991,18 +1196,20 @@ extern int sacctmgr_dump_cluster (int argc, char *argv[])
 		exit_code=1;
 		fprintf(stderr, " Your uid (%u) is not in the "
 			"accounting system, can't dump cluster.\n", my_uid);
+		xfree(cluster_name);
 		xfree(user_name);
 		if(user_list)
 			list_destroy(user_list);
 		list_destroy(assoc_cond.cluster_list);
 		return SLURM_ERROR;
-		
+
 	} else {
 		if(my_uid != slurm_get_slurm_user_id() && my_uid != 0
 		    && user->admin_level < ACCT_ADMIN_SUPER_USER) {
 			exit_code=1;
 			fprintf(stderr, " Your user does not have sufficient "
 				"privileges to dump clusters.\n");
+			xfree(cluster_name);
 			if(user_list)
 				list_destroy(user_list);
 			xfree(user_name);
@@ -1034,7 +1241,7 @@ extern int sacctmgr_dump_cluster (int argc, char *argv[])
 	acct_hierarchical_rec_list = get_acct_hierarchical_rec_list(assoc_list);
 
 	acct_list = acct_storage_g_get_accounts(db_conn, my_uid, NULL);
-	
+
 	fd = fopen(file_name, "w");
 	/* Add header */
 	if(fprintf(fd,
@@ -1064,19 +1271,23 @@ extern int sacctmgr_dump_cluster (int argc, char *argv[])
 		   "MaxWallDurationPerJob=1\n") < 0) {
 		exit_code=1;
 		fprintf(stderr, "Can't write to file");
+		xfree(cluster_name);
 		return SLURM_ERROR;
 	}
 
 	line = xstrdup_printf("Cluster - %s", cluster_name);
 
+	if(class_str)
+		xstrfmtcat(line, ":Classification=%s", class_str);
+
 	acct_hierarchical_rec = list_peek(acct_hierarchical_rec_list);
 	assoc = acct_hierarchical_rec->assoc;
-	if(strcmp(assoc->acct, "root")) 
+	if(strcmp(assoc->acct, "root"))
 		fprintf(stderr, "Root association not on the top it was %s\n",
 			assoc->acct);
-	else 
+	else
 		print_file_add_limits_to_line(&line, assoc);
-	
+
 	if(fprintf(fd, "%s\n", line) < 0) {
 		exit_code=1;
 		fprintf(stderr, " Can't write to file");
@@ -1086,7 +1297,7 @@ extern int sacctmgr_dump_cluster (int argc, char *argv[])
 
 	print_file_acct_hierarchical_rec_list(
 		fd, acct_hierarchical_rec_list, user_list, acct_list);
-	
+
 	xfree(cluster_name);
 	xfree(file_name);
 	list_destroy(acct_hierarchical_rec_list);
