@@ -5,25 +5,26 @@
  *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
- *  LLNL-CODE-402394.
+ *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
  *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
@@ -55,6 +56,8 @@
 #include <unistd.h>
 
 #include "src/common/xstring.h"
+#include "src/common/proc_args.h"
+
 #include "src/sbcast/sbcast.h"
 
 #define OPT_LONG_HELP   0x100
@@ -66,7 +69,6 @@
 static void     _help( void );
 static uint32_t _map_size( char *buf );
 static void     _print_options( void );
-static void     _print_version( void );
 static void     _usage( void );
 
 /*
@@ -109,7 +111,7 @@ extern void parse_command_line(int argc, char *argv[])
 			long_options, &option_index)) != -1) {
 		switch (opt_char) {
 		case (int)'?':
-			fprintf(stderr, 
+			fprintf(stderr,
 				"Try \"sbcast --help\" for more information\n");
 			exit(1);
 			break;
@@ -135,7 +137,7 @@ extern void parse_command_line(int argc, char *argv[])
 			params.verbose++;
 			break;
 		case (int) 'V':
-			_print_version();
+			print_slurm_version();
 			exit(0);
 		case (int) OPT_LONG_HELP:
 			_help();
@@ -147,12 +149,12 @@ extern void parse_command_line(int argc, char *argv[])
 	}
 
 	if ((argc - optind) != 2) {
-		fprintf(stderr, "Need two file names, have %d names\n", 
+		fprintf(stderr, "Need two file names, have %d names\n",
 			(argc - optind));
 		fprintf(stderr, "Try \"sbcast --help\" for more information\n");
 		exit(1);
 	}
-	
+
 	params.src_fname = xstrdup(argv[optind]);
 	params.dst_fname = xstrdup(argv[optind+1]);
 
@@ -206,11 +208,6 @@ static void _print_options( void )
 }
 
 
-static void _print_version(void)
-{
-	printf("%s %s\n", PACKAGE, SLURM_VERSION);
-}
-
 static void _usage( void )
 {
 	printf("Usage: sbcast [-CfFpvV] SOURCE DEST\n");
