@@ -2,35 +2,37 @@
  *  print.c - sinfo print job functions
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
- *  Written by Joey Ekstrom <ekstrom1@llnl.gov> and 
+ *  Written by Joey Ekstrom <ekstrom1@llnl.gov> and
  *  Morris Jette <jette1@llnl.gov>
- *  LLNL-CODE-402394.
- *   
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -52,9 +54,9 @@
 
 #define MIN_NODE_FIELD_SIZE 9
 
-static int   _build_min_max_16_string(char *buffer, int buf_size, 
+static int   _build_min_max_16_string(char *buffer, int buf_size,
 				uint16_t min, uint16_t max, bool range);
-static int   _build_min_max_32_string(char *buffer, int buf_size, 
+static int   _build_min_max_32_string(char *buffer, int buf_size,
 				uint32_t min, uint32_t max,
 				bool range, bool use_suffix);
 static int   _print_secs(long time, int width, bool right, bool cut_output);
@@ -97,7 +99,7 @@ int print_sinfo_entry(sinfo_data_t *sinfo_data)
 	sinfo_format_t *current;
 
 	while ((current = (sinfo_format_t *) list_next(i)) != NULL) {
-		if (current->function(sinfo_data, current->width, 
+		if (current->function(sinfo_data, current->width,
 				      current->right_justify, current->suffix)
 		    != SLURM_SUCCESS)
 			return SLURM_ERROR;
@@ -153,7 +155,7 @@ static int _print_secs(long time, int width, bool right, bool cut_output)
 	hours   = (time / 3600) % 24;
 	days    =  time / 86400;
 
-	if (days) 
+	if (days)
 		snprintf(str, FORMAT_STRING_SIZE,
 			 "%ld-%2.2ld:%2.2ld:%2.2ld",
 		         days, hours, minutes, seconds);
@@ -170,31 +172,31 @@ static int _print_secs(long time, int width, bool right, bool cut_output)
 	return SLURM_SUCCESS;
 }
 
-static int 
-_build_min_max_16_string(char *buffer, int buf_size, uint16_t min, uint16_t max, 
+static int
+_build_min_max_16_string(char *buffer, int buf_size, uint16_t min, uint16_t max,
 			 bool range)
 {
 	char tmp_min[8];
 	char tmp_max[8];
 	convert_num_unit((float)min, tmp_min, sizeof(tmp_min), UNIT_NONE);
 	convert_num_unit((float)max, tmp_max, sizeof(tmp_max), UNIT_NONE);
-	
+
 	if (max == min)
 		return snprintf(buffer, buf_size, "%s", tmp_max);
 	else if (range) {
 		if (max == (uint16_t) INFINITE)
-			return snprintf(buffer, buf_size, "%s-infinite", 
+			return snprintf(buffer, buf_size, "%s-infinite",
 					tmp_min);
 		else
-			return snprintf(buffer, buf_size, "%s-%s", 
+			return snprintf(buffer, buf_size, "%s-%s",
 					tmp_min, tmp_max);
 	} else
 		return snprintf(buffer, buf_size, "%s+", tmp_min);
 }
 
-static int 
+static int
 _build_min_max_32_string(char *buffer, int buf_size,
-			 uint32_t min, uint32_t max, 
+			 uint32_t min, uint32_t max,
 			 bool range, bool use_suffix)
 {
 	char tmp_min[8];
@@ -214,20 +216,20 @@ _build_min_max_32_string(char *buffer, int buf_size,
 		return snprintf(buffer, buf_size, "%s", tmp_max);
 	else if (range) {
 		if (max == INFINITE)
-			return snprintf(buffer, buf_size, "%s-infinite", 
+			return snprintf(buffer, buf_size, "%s-infinite",
 					tmp_min);
 		else
-			return snprintf(buffer, buf_size, "%s-%s", 
+			return snprintf(buffer, buf_size, "%s-%s",
 					tmp_min, tmp_max);
 	} else
 		return snprintf(buffer, buf_size, "%s+", tmp_min);
 }
 
 int
-format_add_function(List list, int width, bool right, char *suffix, 
+format_add_function(List list, int width, bool right, char *suffix,
 			int (*function) (sinfo_data_t *, int, bool, char*))
 {
-	sinfo_format_t *tmp = 
+	sinfo_format_t *tmp =
 		(sinfo_format_t *) xmalloc(sizeof(sinfo_format_t));
 	tmp->function = function;
 	tmp->width = width;
@@ -249,7 +251,7 @@ static void _set_node_field_size(List sinfo_list)
 	int max_width = MIN_NODE_FIELD_SIZE, this_width = 0;
 
 	while ((current = (sinfo_data_t *) list_next(i)) != NULL) {
-		this_width = hostlist_ranged_string(current->nodes, 
+		this_width = hostlist_ranged_string(current->nodes,
 					sizeof(tmp), tmp);
 		max_width = MAX(max_width, this_width);
 	}
@@ -261,7 +263,7 @@ static void _set_node_field_size(List sinfo_list)
  * _str_tolower - convert string to all lower case
  * upper_str IN - upper case input string
  * RET - lower case version of upper_str, caller must be xfree
- */ 
+ */
 static char *_str_tolower(char *upper_str)
 {
 	int i = strlen(upper_str) + 1;
@@ -300,9 +302,9 @@ int _print_cpus(sinfo_data_t * sinfo_data, int width,
 {
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
-		_build_min_max_32_string(id, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_cpus, 
-		                      sinfo_data->max_cpus, 
+		_build_min_max_32_string(id, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_cpus,
+		                      sinfo_data->max_cpus,
 				      false, true);
 		_print_str(id, width, right_justify, true);
 	} else
@@ -324,13 +326,13 @@ int _print_cpus_aiot(sinfo_data_t * sinfo_data, int width,
 	char tmpt[8];
 	if (sinfo_data) {
 #ifdef HAVE_BG
-		convert_num_unit((float)sinfo_data->cpus_alloc, 
+		convert_num_unit((float)sinfo_data->cpus_alloc,
 				 tmpa, sizeof(tmpa), UNIT_NONE);
-		convert_num_unit((float)sinfo_data->cpus_idle, 
+		convert_num_unit((float)sinfo_data->cpus_idle,
 				 tmpi, sizeof(tmpi), UNIT_NONE);
-		convert_num_unit((float)sinfo_data->cpus_other, 
+		convert_num_unit((float)sinfo_data->cpus_other,
 				 tmpo, sizeof(tmpo), UNIT_NONE);
-		convert_num_unit((float)sinfo_data->cpus_total, 
+		convert_num_unit((float)sinfo_data->cpus_total,
 				 tmpt, sizeof(tmpt), UNIT_NONE);
 #else
 		sprintf(tmpa, "%u", sinfo_data->cpus_alloc);
@@ -356,14 +358,14 @@ int _print_sct(sinfo_data_t * sinfo_data, int width,
 	char threads[FORMAT_STRING_SIZE];
 	char sct[(FORMAT_STRING_SIZE+1)*3];
 	if (sinfo_data) {
-		_build_min_max_16_string(sockets, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_sockets, 
+		_build_min_max_16_string(sockets, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_sockets,
 		                      sinfo_data->max_sockets, false);
-		_build_min_max_16_string(cores, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_cores, 
+		_build_min_max_16_string(cores, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_cores,
 		                      sinfo_data->max_cores, false);
-		_build_min_max_16_string(threads, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_threads, 
+		_build_min_max_16_string(threads, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_threads,
 		                      sinfo_data->max_threads, false);
 		sct[0] = '\0';
 		strcat(sct, sockets);
@@ -386,8 +388,8 @@ int _print_sockets(sinfo_data_t * sinfo_data, int width,
 {
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
-		_build_min_max_16_string(id, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_sockets, 
+		_build_min_max_16_string(id, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_sockets,
 		                      sinfo_data->max_sockets, false);
 		_print_str(id, width, right_justify, true);
 	} else {
@@ -404,8 +406,8 @@ int _print_cores(sinfo_data_t * sinfo_data, int width,
 {
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
-		_build_min_max_16_string(id, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_cores, 
+		_build_min_max_16_string(id, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_cores,
 		                      sinfo_data->max_cores, false);
 		_print_str(id, width, right_justify, true);
 	} else {
@@ -422,8 +424,8 @@ int _print_threads(sinfo_data_t * sinfo_data, int width,
 {
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
-		_build_min_max_16_string(id, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_threads, 
+		_build_min_max_16_string(id, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_threads,
 		                      sinfo_data->max_threads, false);
 		_print_str(id, width, right_justify, true);
 	} else {
@@ -440,8 +442,8 @@ int _print_disk(sinfo_data_t * sinfo_data, int width,
 {
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
-		_build_min_max_32_string(id, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_disk, 
+		_build_min_max_32_string(id, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_disk,
 		                      sinfo_data->max_disk,
 				      false, false);
 		_print_str(id, width, right_justify, true);
@@ -473,7 +475,7 @@ int _print_groups(sinfo_data_t * sinfo_data, int width,
 		if (sinfo_data->part_info == NULL)
 			_print_str("n/a", width, right_justify, true);
 		else if (sinfo_data->part_info->allow_groups)
-			_print_str(sinfo_data->part_info->allow_groups, 
+			_print_str(sinfo_data->part_info->allow_groups,
 					width, right_justify, true);
 		else
 			_print_str("all", width, right_justify, true);
@@ -485,13 +487,32 @@ int _print_groups(sinfo_data_t * sinfo_data, int width,
 	return SLURM_SUCCESS;
 }
 
+int _print_alloc_nodes(sinfo_data_t * sinfo_data, int width,
+		       bool right_justify, char *suffix)
+{
+	if (sinfo_data) {
+		if (sinfo_data->part_info == NULL)
+			_print_str("n/a", width, right_justify, true);
+		else if (sinfo_data->part_info->allow_alloc_nodes)
+			_print_str(sinfo_data->part_info->allow_alloc_nodes,
+				   width, right_justify, true);
+		else
+			_print_str("all", width, right_justify, true);
+	} else
+		_print_str("ALLOCNODES", width, right_justify, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_memory(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
-		_build_min_max_32_string(id, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_mem, 
+		_build_min_max_32_string(id, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_mem,
 		                      sinfo_data->max_mem,
 				      false, false);
 		_print_str(id, width, right_justify, true);
@@ -511,7 +532,7 @@ int _print_node_list(sinfo_data_t * sinfo_data, int width,
 
 	if (sinfo_data) {
 		char tmp[MAXHOSTRANGELEN];
-		hostlist_ranged_string(sinfo_data->nodes, 
+		hostlist_ranged_string(sinfo_data->nodes,
 					sizeof(tmp), tmp);
 		_print_str(tmp, width, right_justify, true);
 	} else {
@@ -533,8 +554,8 @@ int _print_nodes_t(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 	char tmp[8];
 	if (sinfo_data) {
-#ifdef HAVE_BG		
-		convert_num_unit((float)sinfo_data->nodes_total, tmp, 
+#ifdef HAVE_BG
+		convert_num_unit((float)sinfo_data->nodes_total, tmp,
 				 sizeof(tmp), UNIT_NONE);
 #else
 		snprintf(tmp, sizeof(tmp), "%d", sinfo_data->nodes_total);
@@ -556,16 +577,16 @@ int _print_nodes_ai(sinfo_data_t * sinfo_data, int width,
 	char tmpa[8];
 	char tmpi[8];
 	if (sinfo_data) {
-#ifdef HAVE_BG		
-		convert_num_unit((float)sinfo_data->nodes_alloc, 
+#ifdef HAVE_BG
+		convert_num_unit((float)sinfo_data->nodes_alloc,
 				 tmpa, sizeof(tmpa), UNIT_NONE);
-		convert_num_unit((float)sinfo_data->nodes_idle, 
+		convert_num_unit((float)sinfo_data->nodes_idle,
 				 tmpi, sizeof(tmpi), UNIT_NONE);
 #else
 		snprintf(tmpa, sizeof(tmpa), "%d", sinfo_data->nodes_alloc);
 		snprintf(tmpi, sizeof(tmpi), "%d", sinfo_data->nodes_idle);
 #endif
-		snprintf(id, FORMAT_STRING_SIZE, "%s/%s", 
+		snprintf(id, FORMAT_STRING_SIZE, "%s/%s",
 		         tmpa, tmpi);
 		_print_str(id, width, right_justify, true);
 	} else
@@ -585,12 +606,12 @@ int _print_nodes_aiot(sinfo_data_t * sinfo_data, int width,
 	char tmpo[8];
 	char tmpt[8];
 	if (sinfo_data) {
-#ifdef HAVE_BG		
-		convert_num_unit((float)sinfo_data->nodes_alloc, 
+#ifdef HAVE_BG
+		convert_num_unit((float)sinfo_data->nodes_alloc,
 				 tmpa, sizeof(tmpa), UNIT_NONE);
-		convert_num_unit((float)sinfo_data->nodes_idle, 
+		convert_num_unit((float)sinfo_data->nodes_idle,
 				 tmpi, sizeof(tmpi), UNIT_NONE);
-		convert_num_unit((float)sinfo_data->nodes_other, 
+		convert_num_unit((float)sinfo_data->nodes_other,
 				 tmpo, sizeof(tmpo), UNIT_NONE);
 		convert_num_unit((float)sinfo_data->nodes_total,
 				 tmpt, sizeof(tmpt), UNIT_NONE);
@@ -601,7 +622,7 @@ int _print_nodes_aiot(sinfo_data_t * sinfo_data, int width,
 		snprintf(tmpt, sizeof(tmpt), "%u", sinfo_data->nodes_total);
 #endif
 
-		snprintf(id, FORMAT_STRING_SIZE, "%s/%s/%s/%s", 
+		snprintf(id, FORMAT_STRING_SIZE, "%s/%s/%s/%s",
 		         tmpa, tmpi, tmpo, tmpt);
 		_print_str(id, width, right_justify, true);
 	} else
@@ -638,7 +659,7 @@ int _print_partition(sinfo_data_t * sinfo_data, int width,
 	return SLURM_SUCCESS;
 }
 
-int _print_prefix(sinfo_data_t * job, int width, bool right_justify, 
+int _print_prefix(sinfo_data_t * job, int width, bool right_justify,
 		char* suffix)
 {
 	if (suffix)
@@ -652,8 +673,8 @@ int _print_priority(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 
 	if (sinfo_data) {
-		_build_min_max_16_string(id, FORMAT_STRING_SIZE, 
-		                      sinfo_data->part_info->priority, 
+		_build_min_max_16_string(id, FORMAT_STRING_SIZE,
+		                      sinfo_data->part_info->priority,
 		                      sinfo_data->part_info->priority, true);
 		_print_str(id, width, right_justify, true);
 	} else
@@ -669,7 +690,7 @@ int _print_reason(sinfo_data_t * sinfo_data, int width,
 {
 	if (sinfo_data) {
 		char * reason = sinfo_data->reason ? sinfo_data->reason:"none";
-		if (strncmp(reason, "(null)", 6) == 0) 
+		if (strncmp(reason, "(null)", 6) == 0)
 			reason = "none";
 		_print_str(reason, width, right_justify, true);
 	} else
@@ -734,7 +755,7 @@ int _print_size(sinfo_data_t * sinfo_data, int width,
 			if ((sinfo_data->part_info->min_nodes < 1) &&
 			    (sinfo_data->part_info->max_nodes > 0))
 				sinfo_data->part_info->min_nodes = 1;
-			_build_min_max_32_string(id, FORMAT_STRING_SIZE, 
+			_build_min_max_32_string(id, FORMAT_STRING_SIZE,
 					      sinfo_data->part_info->min_nodes,
 					      sinfo_data->part_info->max_nodes,
 					      true, true);
@@ -805,13 +826,33 @@ int _print_time(sinfo_data_t * sinfo_data, int width,
 	return SLURM_SUCCESS;
 }
 
+int _print_default_time(sinfo_data_t * sinfo_data, int width,
+				bool right_justify, char *suffix)
+{
+	if (sinfo_data) {
+		if ((sinfo_data->part_info == NULL) ||
+		    (sinfo_data->part_info->default_time == NO_VAL))
+			_print_str("n/a", width, right_justify, true);
+		else if (sinfo_data->part_info->default_time == INFINITE)
+			_print_str("infinite", width, right_justify, true);
+		else
+			_print_secs((sinfo_data->part_info->default_time * 60L),
+					width, right_justify, true);
+	} else
+		_print_str("DEFAULTTIME", width, right_justify, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_weight(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
-		_build_min_max_32_string(id, FORMAT_STRING_SIZE, 
-		                      sinfo_data->min_weight, 
+		_build_min_max_32_string(id, FORMAT_STRING_SIZE,
+		                      sinfo_data->min_weight,
 		                      sinfo_data->max_weight,
 				      false, false);
 		_print_str(id, width, right_justify, true);
