@@ -1,62 +1,63 @@
 /*****************************************************************************\
- *  agent.c - parallel background communication functions. This is where  
+ *  agent.c - parallel background communication functions. This is where
  *	logic could be placed for broadcast communications.
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>, et. al.
  *  Derived from pdsh written by Jim Garlick <garlick1@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
  *****************************************************************************
  *  Theory of operation:
  *
- *  The functions below permit slurm to initiate parallel tasks as a 
- *  detached thread and let the functions below make sure the work happens. 
- *  For example, when a job's time limit is to be changed slurmctld needs 
- *  to notify the slurmd on every node to which the job was allocated.  
- *  We don't want to hang slurmctld's primary function (the job update RPC)  
- *  to perform this work, so it just initiates an agent to perform the work.  
- *  The agent is passed all details required to perform the work, so it will 
- *  be possible to execute the agent as an pthread, process, or even a daemon 
+ *  The functions below permit slurm to initiate parallel tasks as a
+ *  detached thread and let the functions below make sure the work happens.
+ *  For example, when a job's time limit is to be changed slurmctld needs
+ *  to notify the slurmd on every node to which the job was allocated.
+ *  We don't want to hang slurmctld's primary function (the job update RPC)
+ *  to perform this work, so it just initiates an agent to perform the work.
+ *  The agent is passed all details required to perform the work, so it will
+ *  be possible to execute the agent as an pthread, process, or even a daemon
  *  on some other computer.
  *
  *  The main agent thread creates a separate thread for each node to be
- *  communicated with up to AGENT_THREAD_COUNT. A special watchdog thread  
- *  sends SIGLARM to any threads that have been active (in DSH_ACTIVE state)  
- *  for more than COMMAND_TIMEOUT seconds. 
+ *  communicated with up to AGENT_THREAD_COUNT. A special watchdog thread
+ *  sends SIGLARM to any threads that have been active (in DSH_ACTIVE state)
+ *  for more than COMMAND_TIMEOUT seconds.
  *  The agent responds to slurmctld via a function call or an RPC as required.
  *  For example, informing slurmctld that some node is not responding.
  *
- *  All the state for each thread is maintained in thd_t struct, which is 
+ *  All the state for each thread is maintained in thd_t struct, which is
  *  used by the watchdog thread as well as the communication threads.
 \*****************************************************************************/
 
@@ -95,7 +96,7 @@
 #include "src/slurmctld/state_save.h"
 #include "src/slurmctld/srun_comm.h"
 
-#define MAX_RETRIES	10
+#define MAX_RETRIES	100
 
 typedef enum {
 	DSH_NEW,        /* Request not yet started */
@@ -119,7 +120,7 @@ typedef struct thd {
 	pthread_attr_t attr;		/* thread attributes */
 	state_t state;			/* thread state */
 	time_t start_time;		/* start time */
-	time_t end_time;		/* end time or delta time 
+	time_t end_time;		/* end time or delta time
 					 * upon termination */
 	slurm_addr *addr;	        /* specific addr to send to
 					 * will not do nodelist if set */
@@ -140,7 +141,7 @@ typedef struct agent_info {
 } agent_info_t;
 
 typedef struct task_info {
-	pthread_mutex_t *thread_mutex_ptr; /* pointer to agent specific 
+	pthread_mutex_t *thread_mutex_ptr; /* pointer to agent specific
 					    * mutex */
 	pthread_cond_t *thread_cond_ptr;/* pointer to agent specific
 					 * condition */
@@ -153,6 +154,8 @@ typedef struct task_info {
 
 typedef struct queued_request {
 	agent_arg_t* agent_arg_ptr;	/* The queued request */
+	time_t       first_attempt;	/* Time of first check for batch
+					 * launch RPC *only* */
 	time_t       last_attempt;	/* Time of last xmit attempt */
 } queued_request_t;
 
@@ -162,16 +165,17 @@ typedef struct mail_info {
 } mail_info_t;
 
 static void _sig_handler(int dummy);
+static int  _batch_launch_defer(queued_request_t *queued_req_ptr);
 static inline int _comm_err(char *node_name);
 static void _list_delete_retry(void *retry_entry);
 static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr);
 static task_info_t *_make_task_data(agent_info_t *agent_info_ptr, int inx);
 static void _notify_slurmctld_jobs(agent_info_t *agent_ptr);
-static void _notify_slurmctld_nodes(agent_info_t *agent_ptr, 
+static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 		int no_resp_cnt, int retry_cnt);
 static void _purge_agent_args(agent_arg_t *agent_arg_ptr);
 static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count);
-static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr, 
+static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr,
 			  int count, int *spot);
 static void _slurmctld_free_batch_job_launch_msg(batch_job_launch_msg_t * msg);
 static void _spawn_retry_agent(agent_arg_t * agent_arg_ptr);
@@ -196,10 +200,10 @@ static int agent_cnt = 0;
 static bool run_scheduler = false;
 
 /*
- * agent - party responsible for transmitting an common RPC in parallel 
- *	across a set of nodes. Use agent_queue_request() if immediate 
+ * agent - party responsible for transmitting an common RPC in parallel
+ *	across a set of nodes. Use agent_queue_request() if immediate
  *	execution is not essential.
- * IN pointer to agent_arg_t, which is xfree'd (including hostlist, 
+ * IN pointer to agent_arg_t, which is xfree'd (including hostlist,
  *	and msg_args) upon completion if AGENT_IS_THREAD is set
  * RET always NULL (function format just for use as pthread)
  */
@@ -219,8 +223,9 @@ void *agent(void *args)
 	     agent_cnt, MAX_AGENT_CNT, agent_arg_ptr->msg_type);
 #endif
 	slurm_mutex_lock(&agent_cnt_mutex);
-	while (slurmctld_config.shutdown_time == 0) {
-		if (agent_cnt < MAX_AGENT_CNT) {
+	while (1) {
+		if (slurmctld_config.shutdown_time ||
+		    (agent_cnt < MAX_AGENT_CNT)) {
 			agent_cnt++;
 			break;
 		} else {	/* wait for state change and retry */
@@ -230,7 +235,7 @@ void *agent(void *args)
 	slurm_mutex_unlock(&agent_cnt_mutex);
 	if (slurmctld_config.shutdown_time)
 		goto cleanup;
-	
+
 	/* basic argument value tests */
 	begin_time = time(NULL);
 	if (_valid_agent_arg(agent_arg_ptr))
@@ -268,7 +273,7 @@ void *agent(void *args)
 					  &agent_info_ptr->thread_mutex);
 		}
 
-		/* create thread specific data, NOTE: freed from 
+		/* create thread specific data, NOTE: freed from
 		 *      _thread_per_group_rpc() */
 		task_specific_ptr = _make_task_data(agent_info_ptr, i);
 
@@ -298,7 +303,7 @@ void *agent(void *args)
 		agent_info_ptr->threads_active++;
 		slurm_mutex_unlock(&agent_info_ptr->thread_mutex);
 	}
-		
+
 	/* wait for termination of remaining threads */
 	pthread_join(thread_wdog, NULL);
 	delay = (int) difftime(time(NULL), begin_time);
@@ -323,7 +328,7 @@ void *agent(void *args)
 		xfree(agent_info_ptr);
 	}
 	slurm_mutex_lock(&agent_cnt_mutex);
-	
+
 	if (agent_cnt > 0)
 		agent_cnt--;
 	else {
@@ -331,12 +336,11 @@ void *agent(void *args)
 		agent_cnt = 0;
 	}
 
-	if (agent_cnt && agent_cnt < MAX_AGENT_CNT) 
+	if (agent_cnt && agent_cnt < MAX_AGENT_CNT)
 		agent_retry(RPC_RETRY_INTERVAL, true);
-	
-	slurm_mutex_unlock(&agent_cnt_mutex);
-	
+
 	pthread_cond_broadcast(&agent_cnt_cond);
+	slurm_mutex_unlock(&agent_cnt_mutex);
 
 	return NULL;
 }
@@ -349,7 +353,7 @@ static int _valid_agent_arg(agent_arg_t *agent_arg_ptr)
 
 	if (agent_arg_ptr->node_count == 0)
 		return SLURM_FAILURE;	/* no messages to be sent */
-	if (agent_arg_ptr->node_count 
+	if (agent_arg_ptr->node_count
 	    != hostlist_count(agent_arg_ptr->hostlist)) {
 		error("you said you were going to send to %d "
 		     "hosts but I only have %d",
@@ -370,7 +374,7 @@ static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 	hostlist_t hl = NULL;
 	char buf[8192];
 	char *name = NULL;
-	
+
 	agent_info_ptr = xmalloc(sizeof(agent_info_t));
 	slurm_mutex_init(&agent_info_ptr->thread_mutex);
 	if (pthread_cond_init(&agent_info_ptr->thread_cond, NULL))
@@ -383,24 +387,25 @@ static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 	agent_info_ptr->thread_struct  = thread_ptr;
 	agent_info_ptr->msg_type       = agent_arg_ptr->msg_type;
 	agent_info_ptr->msg_args_pptr  = &agent_arg_ptr->msg_args;
-	
-	if ((agent_arg_ptr->msg_type != REQUEST_SHUTDOWN)
-	&&  (agent_arg_ptr->msg_type != REQUEST_RECONFIGURE)
-	&&  (agent_arg_ptr->msg_type != SRUN_EXEC)
-	&&  (agent_arg_ptr->msg_type != SRUN_TIMEOUT)
-	&&  (agent_arg_ptr->msg_type != SRUN_NODE_FAIL)
-	&&  (agent_arg_ptr->msg_type != SRUN_USER_MSG)
-	&&  (agent_arg_ptr->msg_type != SRUN_JOB_COMPLETE)) {
+
+	if ((agent_arg_ptr->msg_type != REQUEST_SHUTDOWN)	&&
+	    (agent_arg_ptr->msg_type != REQUEST_RECONFIGURE)	&&
+	    (agent_arg_ptr->msg_type != SRUN_EXEC)		&&
+	    (agent_arg_ptr->msg_type != SRUN_TIMEOUT)		&&
+	    (agent_arg_ptr->msg_type != SRUN_NODE_FAIL)		&&
+	    (agent_arg_ptr->msg_type != SRUN_USER_MSG)		&&
+	    (agent_arg_ptr->msg_type != SRUN_STEP_MISSING)	&&
+	    (agent_arg_ptr->msg_type != SRUN_JOB_COMPLETE)) {
 		/* Sending message to a possibly large number of slurmd.
-		 * Push all message forwarding to slurmd in order to 
+		 * Push all message forwarding to slurmd in order to
 		 * offload as much work from slurmctld as possible. */
 		agent_info_ptr->get_reply = true;
-		span = set_span(agent_arg_ptr->node_count, 1); 
+		span = set_span(agent_arg_ptr->node_count, 1);
 	} else {
-		/* Message is going to one node (for srun) or we want 
+		/* Message is going to one node (for srun) or we want
 		 * it to get processed ASAP (SHUTDOWN or RECONFIGURE).
 		 * Send the message directly to each node. */
-		span = set_span(agent_arg_ptr->node_count, 
+		span = set_span(agent_arg_ptr->node_count,
 				agent_arg_ptr->node_count);
 	}
 	i = 0;
@@ -435,7 +440,7 @@ static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 #if 0
 		info("sending to nodes %s", thread_ptr[thr_count].nodelist);
 #endif
-		thr_count++;		       
+		thr_count++;
 	}
 	xfree(span);
 	agent_info_ptr->thread_count = thr_count;
@@ -458,16 +463,16 @@ static task_info_t *_make_task_data(agent_info_t *agent_info_ptr, int inx)
 	return task_info_ptr;
 }
 
-static void _update_wdog_state(thd_t *thread_ptr, 
-			       state_t *state, 
+static void _update_wdog_state(thd_t *thread_ptr,
+			       state_t *state,
 			       thd_complete_t *thd_comp)
 {
 	switch(*state) {
 	case DSH_ACTIVE:
 		thd_comp->work_done = false;
 		if (thread_ptr->end_time <= thd_comp->now) {
-			debug3("agent thread %lu timed out\n", 
-			       (unsigned long) 
+			debug3("agent thread %lu timed out\n",
+			       (unsigned long)
 			       thread_ptr->thread);
 			if (pthread_kill(thread_ptr->thread, SIGUSR1) == ESRCH)
 				*state = DSH_NO_RESP;
@@ -492,11 +497,11 @@ static void _update_wdog_state(thd_t *thread_ptr,
 	}
 }
 
-/* 
- * _wdog - Watchdog thread. Send SIGUSR1 to threads which have been active 
- *	for too long. 
+/*
+ * _wdog - Watchdog thread. Send SIGUSR1 to threads which have been active
+ *	for too long.
  * IN args - pointer to agent_info_t with info on threads to watch
- * Sleep between polls with exponential times (from 0.125 to 1.0 second) 
+ * Sleep between polls with exponential times (from 0.125 to 1.0 second)
  */
 static void *_wdog(void *args)
 {
@@ -509,26 +514,27 @@ static void *_wdog(void *args)
 	thd_complete_t thd_comp;
 	ret_data_info_t *ret_data_info = NULL;
 
-	if ( (agent_ptr->msg_type == SRUN_JOB_COMPLETE)
-	||   (agent_ptr->msg_type == SRUN_EXEC)
-	||   (agent_ptr->msg_type == SRUN_PING)
-	||   (agent_ptr->msg_type == SRUN_TIMEOUT)
-	||   (agent_ptr->msg_type == SRUN_USER_MSG)
-	||   (agent_ptr->msg_type == RESPONSE_RESOURCE_ALLOCATION)
-	||   (agent_ptr->msg_type == SRUN_NODE_FAIL) )
+	if ( (agent_ptr->msg_type == SRUN_JOB_COMPLETE)			||
+	     (agent_ptr->msg_type == SRUN_STEP_MISSING)			||
+	     (agent_ptr->msg_type == SRUN_EXEC)				||
+	     (agent_ptr->msg_type == SRUN_PING)				||
+	     (agent_ptr->msg_type == SRUN_TIMEOUT)			||
+	     (agent_ptr->msg_type == SRUN_USER_MSG)			||
+	     (agent_ptr->msg_type == RESPONSE_RESOURCE_ALLOCATION)	||
+	     (agent_ptr->msg_type == SRUN_NODE_FAIL) )
 		srun_agent = true;
 
 	thd_comp.max_delay = 0;
-		
+
 	while (1) {
 		thd_comp.work_done   = true;/* assume all threads complete */
 		thd_comp.fail_cnt    = 0;   /* assume no threads failures */
 		thd_comp.no_resp_cnt = 0;   /* assume all threads respond */
 		thd_comp.retry_cnt   = 0;   /* assume no required retries */
 		thd_comp.now         = time(NULL);
-		
+
 		usleep(usec);
-		usec = MIN((usec * 2), 1000000);		
+		usec = MIN((usec * 2), 1000000);
 
 		slurm_mutex_lock(&agent_ptr->thread_mutex);
 		for (i = 0; i < agent_ptr->thread_count; i++) {
@@ -550,15 +556,15 @@ static void *_wdog(void *args)
 		}
 		if (thd_comp.work_done)
 			break;
-		
+
 		slurm_mutex_unlock(&agent_ptr->thread_mutex);
 	}
 
 	if (srun_agent) {
 	        _notify_slurmctld_jobs(agent_ptr);
 	} else {
-		_notify_slurmctld_nodes(agent_ptr, 
-					thd_comp.no_resp_cnt, 
+		_notify_slurmctld_nodes(agent_ptr,
+					thd_comp.no_resp_cnt,
 					thd_comp.retry_cnt);
 	}
 
@@ -567,10 +573,10 @@ static void *_wdog(void *args)
 			list_destroy(thread_ptr[i].ret_list);
 		xfree(thread_ptr[i].nodelist);
 	}
-	
+
 	if (thd_comp.max_delay)
 		debug2("agent maximum delay %d seconds", thd_comp.max_delay);
-	
+
 	slurm_mutex_unlock(&agent_ptr->thread_mutex);
 	return (void *) NULL;
 }
@@ -597,9 +603,10 @@ static void _notify_slurmctld_jobs(agent_info_t *agent_ptr)
 			*agent_ptr->msg_args_pptr;
 		job_id  = msg->job_id;
 		step_id = NO_VAL;
-	} else if ((agent_ptr->msg_type == SRUN_JOB_COMPLETE)
-	||         (agent_ptr->msg_type == SRUN_EXEC)
-	||         (agent_ptr->msg_type == SRUN_USER_MSG)) {
+	} else if ((agent_ptr->msg_type == SRUN_JOB_COMPLETE)		||
+		   (agent_ptr->msg_type == SRUN_STEP_MISSING)		||
+	           (agent_ptr->msg_type == SRUN_EXEC)			||
+	           (agent_ptr->msg_type == SRUN_USER_MSG)) {
 		return;		/* no need to note srun response */
 	} else if (agent_ptr->msg_type == SRUN_NODE_FAIL) {
 		return;		/* no need to note srun response */
@@ -619,7 +626,7 @@ static void _notify_slurmctld_jobs(agent_info_t *agent_ptr)
 #endif
 }
 
-static void _notify_slurmctld_nodes(agent_info_t *agent_ptr, 
+static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 				    int no_resp_cnt, int retry_cnt)
 {
 	ListIterator itr = NULL;
@@ -642,7 +649,7 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 		lock_slurmctld(node_write_lock);
 		if (agent_ptr->msg_type == REQUEST_BATCH_JOB_LAUNCH) {
 			/* Requeue the request */
-			batch_job_launch_msg_t *launch_msg_ptr = 
+			batch_job_launch_msg_t *launch_msg_ptr =
 					*agent_ptr->msg_args_pptr;
 			uint32_t job_id = launch_msg_ptr->job_id;
 			job_complete(job_id, 0, true, 0);
@@ -663,9 +670,9 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 			state = thread_ptr[i].state;
 			is_ret_list = 0;
 			goto switch_on_state;
-		} 
+		}
 		is_ret_list = 1;
-		
+
 		itr = list_iterator_create(thread_ptr[i].ret_list);
 		while((ret_data_info = list_next(itr))) {
 			state = ret_data_info->err;
@@ -687,7 +694,7 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 				error("Prolog/epilog failure");
 #else
 				if(!is_ret_list) {
-					set_node_down(thread_ptr[i].nodelist, 
+					set_node_down(thread_ptr[i].nodelist,
 						      "Prolog/epilog failure");
 					break;
 				}
@@ -744,7 +751,7 @@ static inline int _comm_err(char *node_name)
 #if AGENT_IS_THREAD
 	if ((rc = is_node_resp (node_name)))
 #endif
-		verbose("agent/send_recv_msg: %s: %m", node_name);
+		verbose("agent/is_node_resp: %s: %m", node_name);
 	return rc;
 }
 
@@ -779,9 +786,9 @@ static void *_thread_per_group_rpc(void *args)
 	int rc = SLURM_SUCCESS;
 	slurm_msg_t msg;
 	task_info_t *task_ptr = (task_info_t *) args;
-	/* we cache some pointers from task_info_t because we need 
-	 * to xfree args before being finished with their use. xfree 
-	 * is required for timely termination of this pthread because 
+	/* we cache some pointers from task_info_t because we need
+	 * to xfree args before being finished with their use. xfree
+	 * is required for timely termination of this pthread because
 	 * xfree could lock it at the end, preventing a timely
 	 * thread_exit */
 	pthread_mutex_t *thread_mutex_ptr   = task_ptr->thread_mutex_ptr;
@@ -799,19 +806,20 @@ static void *_thread_per_group_rpc(void *args)
 
 #if AGENT_IS_THREAD
 	/* Locks: Write job, write node */
-	slurmctld_lock_t job_write_lock = { 
+	slurmctld_lock_t job_write_lock = {
 		NO_LOCK, WRITE_LOCK, WRITE_LOCK, NO_LOCK };
 #endif
 	xassert(args != NULL);
 	xsignal(SIGUSR1, _sig_handler);
 	xsignal_unblock(sig_array);
-	is_kill_msg = (	(msg_type == REQUEST_KILL_TIMELIMIT) ||
+	is_kill_msg = (	(msg_type == REQUEST_KILL_TIMELIMIT)	||
 			(msg_type == REQUEST_TERMINATE_JOB) );
-	srun_agent = (	(msg_type == SRUN_PING)    ||
-			(msg_type == SRUN_EXEC)    ||
-			(msg_type == SRUN_JOB_COMPLETE) ||
-			(msg_type == SRUN_TIMEOUT) ||
-			(msg_type == SRUN_USER_MSG) ||
+	srun_agent = (	(msg_type == SRUN_PING)			||
+			(msg_type == SRUN_EXEC)			||
+			(msg_type == SRUN_JOB_COMPLETE)		||
+			(msg_type == SRUN_STEP_MISSING)		||
+			(msg_type == SRUN_TIMEOUT)		||
+			(msg_type == SRUN_USER_MSG)		||
 			(msg_type == RESPONSE_RESOURCE_ALLOCATION) ||
 			(msg_type == SRUN_NODE_FAIL) );
 
@@ -832,15 +840,15 @@ static void *_thread_per_group_rpc(void *args)
 	if (task_ptr->get_reply) {
 		if(thread_ptr->addr) {
 			msg.address = *thread_ptr->addr;
-			
+
 			if(!(ret_list = slurm_send_addr_recv_msgs(
 				     &msg, thread_ptr->nodelist, 0))) {
 				error("_thread_per_group_rpc: "
 				      "no ret_list given");
 				goto cleanup;
 			}
-			
-			
+
+
 		} else {
 			if(!(ret_list = slurm_send_recv_msgs(
 				     thread_ptr->nodelist,
@@ -860,7 +868,7 @@ static void *_thread_per_group_rpc(void *args)
 					       &msg.address) == SLURM_ERROR) {
 				error("_thread_per_group_rpc: "
 				      "can't find address for host %s, "
-				      "check slurm.conf", 
+				      "check slurm.conf",
 				      thread_ptr->nodelist);
 				goto cleanup;
 			}
@@ -874,39 +882,39 @@ static void *_thread_per_group_rpc(void *args)
 		}
 		goto cleanup;
 	}
-		
+
 	//info("got %d messages back", list_count(ret_list));
 	found = 0;
-	itr = list_iterator_create(ret_list);		
+	itr = list_iterator_create(ret_list);
 	while((ret_data_info = list_next(itr)) != NULL) {
-		rc = slurm_get_return_code(ret_data_info->type, 
+		rc = slurm_get_return_code(ret_data_info->type,
 					   ret_data_info->data);
 #if AGENT_IS_THREAD
-		/* SPECIAL CASE: Mark node as IDLE if job already 
+		/* SPECIAL CASE: Mark node as IDLE if job already
 		   complete */
-		if (is_kill_msg && 
+		if (is_kill_msg &&
 		    (rc == ESLURMD_KILL_JOB_ALREADY_COMPLETE)) {
 			kill_job_msg_t *kill_job;
-			kill_job = (kill_job_msg_t *) 
+			kill_job = (kill_job_msg_t *)
 				task_ptr->msg_args_ptr;
 			rc = SLURM_SUCCESS;
 			lock_slurmctld(job_write_lock);
-			if (job_epilog_complete(kill_job->job_id, 
+			if (job_epilog_complete(kill_job->job_id,
 						ret_data_info->
-						node_name, 
+						node_name,
 						rc))
 				run_scheduler = true;
 			unlock_slurmctld(job_write_lock);
 		}
-		/* SPECIAL CASE: Kill non-startable batch job, 
+		/* SPECIAL CASE: Kill non-startable batch job,
 		 * Requeue the job on ESLURMD_PROLOG_FAILED */
-		if ((msg_type == REQUEST_BATCH_JOB_LAUNCH) && 
+		if ((msg_type == REQUEST_BATCH_JOB_LAUNCH) &&
 		    (rc != SLURM_SUCCESS) && (rc != ESLURMD_PROLOG_FAILED) &&
 		    (ret_data_info->type != RESPONSE_FORWARD_FAILED)) {
-			batch_job_launch_msg_t *launch_msg_ptr = 
+			batch_job_launch_msg_t *launch_msg_ptr =
 				task_ptr->msg_args_ptr;
 			uint32_t job_id = launch_msg_ptr->job_id;
-			info("Killing non-startable batch job %u: %s", 
+			info("Killing non-startable batch job %u: %s",
 			     job_id, slurm_strerror(rc));
 			thread_state = DSH_DONE;
 			ret_data_info->err = thread_state;
@@ -916,42 +924,49 @@ static void *_thread_per_group_rpc(void *args)
 			continue;
 		}
 #endif
-		
-		
-		if (((msg_type == REQUEST_SIGNAL_TASKS) || 
-		     (msg_type == REQUEST_TERMINATE_TASKS)) && 
+
+
+		if (((msg_type == REQUEST_SIGNAL_TASKS) ||
+		     (msg_type == REQUEST_TERMINATE_TASKS)) &&
 		     (rc == ESRCH)) {
 			/* process is already dead, not a real error */
 			rc = SLURM_SUCCESS;
 		}
-		
+
 		switch (rc) {
 		case SLURM_SUCCESS:
-			/*debug3("agent processed RPC to node %s", 
+			/*debug3("agent processed RPC to node %s",
 			  ret_data_info->node_name); */
 			thread_state = DSH_DONE;
 			break;
+		case SLURM_UNKNOWN_FORWARD_ADDR:
+			error("We were unable to forward message to '%s'.  "
+			      "Make sure the slurm.conf for each slurmd "
+			      "contain all other nodes in your system.",
+			      ret_data_info->node_name);
+			thread_state = DSH_NO_RESP;
+			break;
 		case ESLURMD_EPILOG_FAILED:
 			error("Epilog failure on host %s, "
-			      "setting DOWN", 
+			      "setting DOWN",
 			      ret_data_info->node_name);
-				
+
 			thread_state = DSH_FAILED;
 			break;
 		case ESLURMD_PROLOG_FAILED:
 			error("Prolog failure on host %s, "
-			      "setting DOWN", 
+			      "setting DOWN",
 			      ret_data_info->node_name);
 			thread_state = DSH_FAILED;
 			break;
-		case ESLURM_INVALID_JOB_ID:  
+		case ESLURM_INVALID_JOB_ID:
 			/* Not indicative of a real error */
-		case ESLURMD_JOB_NOTRUNNING: 
+		case ESLURMD_JOB_NOTRUNNING:
 			/* Not indicative of a real error */
 			debug2("agent processed RPC to node %s: %s",
 			       ret_data_info->node_name,
 			       slurm_strerror(rc));
-			
+
 			thread_state = DSH_DONE;
 			break;
 		default:
@@ -970,20 +985,20 @@ static void *_thread_per_group_rpc(void *args)
 				 * cases */
 				thread_state = DSH_DONE;
 			}
-		}	
+		}
 		ret_data_info->err = thread_state;
 	}
 	list_iterator_destroy(itr);
 
 cleanup:
 	xfree(args);
-	
+
 	/* handled at end of thread just in case resend is needed */
 	destroy_forward(&msg.forward);
 	slurm_mutex_lock(thread_mutex_ptr);
 	thread_ptr->ret_list = ret_list;
 	thread_ptr->state = thread_state;
-	thread_ptr->end_time = (time_t) difftime(time(NULL), 
+	thread_ptr->end_time = (time_t) difftime(time(NULL),
 						 thread_ptr->start_time);
 	/* Signal completion so another thread can replace us */
 	(*threads_active_ptr)--;
@@ -1000,7 +1015,7 @@ static void _sig_handler(int dummy)
 {
 }
 
-static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr, 
+static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr,
 			  int count, int *spot)
 {
 	ret_data_info_t *ret_data_info = NULL;
@@ -1010,13 +1025,13 @@ static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr,
 		if (ret_data_info->err != DSH_NO_RESP)
 			continue;
 
-		debug("got the name %s to resend out of %d", 
+		debug("got the name %s to resend out of %d",
 		      ret_data_info->node_name, count);
-				
+
 		if(agent_arg_ptr) {
-			hostlist_push(agent_arg_ptr->hostlist, 
+			hostlist_push(agent_arg_ptr->hostlist,
 				      ret_data_info->node_name);
-			
+
 			if ((++(*spot)) == count) {
 				list_iterator_destroy(itr);
 				return 1;
@@ -1038,7 +1053,7 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 	queued_request_t *queued_req_ptr = NULL;
 	thd_t *thread_ptr = agent_info_ptr->thread_struct;
 	int i, j;
-	
+
 	if (count == 0)
 		return;
 
@@ -1057,25 +1072,25 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 			if (thread_ptr[i].state != DSH_NO_RESP)
 				continue;
 
-			debug("got the name %s to resend", 
+			debug("got the name %s to resend",
 			      thread_ptr[i].nodelist);
-			hostlist_push(agent_arg_ptr->hostlist, 
+			hostlist_push(agent_arg_ptr->hostlist,
 				      thread_ptr[i].nodelist);
 
 			if ((++j) == count)
 				break;
 		} else {
-			if(_setup_requeue(agent_arg_ptr, &thread_ptr[i], 
+			if(_setup_requeue(agent_arg_ptr, &thread_ptr[i],
 					  count, &j))
 				break;
 		}
 	}
 	if (count != j) {
-		error("agent: Retry count (%d) != actual count (%d)", 
+		error("agent: Retry count (%d) != actual count (%d)",
 			count, j);
 		agent_arg_ptr->node_count = j;
 	}
-	debug2("Queue RPC msg_type=%u, nodes=%d for retry", 
+	debug2("Queue RPC msg_type=%u, nodes=%d for retry",
 	       agent_arg_ptr->msg_type, j);
 
 	/* add the requeust to a list */
@@ -1094,7 +1109,7 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 }
 
 /*
- * _list_delete_retry - delete an entry from the retry list, 
+ * _list_delete_retry - delete an entry from the retry list,
  *	see common/list.h for documentation
  */
 static void _list_delete_retry(void *retry_entry)
@@ -1111,17 +1126,17 @@ static void _list_delete_retry(void *retry_entry)
 
 
 /*
- * agent_retry - Agent for retrying pending RPCs. One pending request is 
+ * agent_retry - Agent for retrying pending RPCs. One pending request is
  *	issued if it has been pending for at least min_wait seconds
  * IN min_wait - Minimum wait time between re-issue of a pending RPC
- * IN mai_too - Send pending email too, note this performed using a 
- *	fork/waitpid, so it can take longer than just creating  a pthread 
+ * IN mai_too - Send pending email too, note this performed using a
+ *	fork/waitpid, so it can take longer than just creating  a pthread
  *	to send RPCs
  * RET count of queued requests remaining
  */
 extern int agent_retry (int min_wait, bool mail_too)
 {
-	int list_size = 0;
+	int list_size = 0, rc;
 	time_t now = time(NULL);
 	queued_request_t *queued_req_ptr = NULL;
 	agent_arg_t *agent_arg_ptr = NULL;
@@ -1132,13 +1147,13 @@ extern int agent_retry (int min_wait, bool mail_too)
 		static time_t last_msg_time = (time_t) 0;
 		uint32_t msg_type[5], i = 0;
 		list_size = list_count(retry_list);
-		if ((list_size > MAX_AGENT_CNT) 
+		if ((list_size > MAX_AGENT_CNT)
 		&&  (difftime(now, last_msg_time) > 300)) {
 			/* Note sizable backlog of work */
-			info("WARNING: agent retry_list size is %d", 
+			info("WARNING: agent retry_list size is %d",
 				list_size);
 			retry_iter = list_iterator_create(retry_list);
-			while ((queued_req_ptr = (queued_request_t *) 
+			while ((queued_req_ptr = (queued_request_t *)
 					list_next(retry_iter))) {
 				agent_arg_ptr = queued_req_ptr->agent_arg_ptr;
 				msg_type[i++] = agent_arg_ptr->msg_type;
@@ -1146,7 +1161,7 @@ extern int agent_retry (int min_wait, bool mail_too)
 					break;
 			}
 			list_iterator_destroy(retry_iter);
-			info("   retry_list msg_type=%u,%u,%u,%u,%u", 
+			info("   retry_list msg_type=%u,%u,%u,%u,%u",
 				msg_type[0], msg_type[1], msg_type[2],
 				msg_type[3], msg_type[4]);
 			last_msg_time = now;
@@ -1163,6 +1178,17 @@ extern int agent_retry (int min_wait, bool mail_too)
 		retry_iter = list_iterator_create(retry_list);
 		while ((queued_req_ptr = (queued_request_t *)
 				list_next(retry_iter))) {
+			rc = _batch_launch_defer(queued_req_ptr);
+			if (rc == -1) {		/* abort request */
+				_purge_agent_args(queued_req_ptr->
+						  agent_arg_ptr);
+				xfree(queued_req_ptr);
+				list_remove(retry_iter);
+				list_size--;
+				continue;
+			}
+			if (rc > 0)
+				continue;
  			if (queued_req_ptr->last_attempt == 0) {
 				list_remove(retry_iter);
 				list_size--;
@@ -1173,14 +1199,25 @@ extern int agent_retry (int min_wait, bool mail_too)
 	}
 
 	if (retry_list && (queued_req_ptr == NULL)) {
-		/* now try to find a requeue request that is 
+		/* now try to find a requeue request that is
 		 * relatively old */
 		double age = 0;
 
 		retry_iter = list_iterator_create(retry_list);
 		/* next try to find an older record to retry */
-		while ((queued_req_ptr = (queued_request_t *) 
+		while ((queued_req_ptr = (queued_request_t *)
 				list_next(retry_iter))) {
+			rc = _batch_launch_defer(queued_req_ptr);
+			if (rc == -1) { 	/* abort request */
+				_purge_agent_args(queued_req_ptr->
+						  agent_arg_ptr);
+				xfree(queued_req_ptr);
+				list_remove(retry_iter);
+				list_size--;
+				continue;
+			}
+			if (rc > 0)
+				continue;
 			age = difftime(now, queued_req_ptr->last_attempt);
 			if (age > min_wait) {
 				list_remove(retry_iter);
@@ -1208,7 +1245,7 @@ extern int agent_retry (int min_wait, bool mail_too)
 		if (mi)
 			_mail_proc(mi);
 	}
-	
+
 	return list_size;
 }
 
@@ -1255,7 +1292,7 @@ void agent_queue_request(agent_arg_t *agent_arg_ptr)
 	list_append(retry_list, (void *)queued_req_ptr);
 	slurm_mutex_unlock(&retry_mutex);
 
-	/* now process the request in a separate pthread 
+	/* now process the request in a separate pthread
 	 * (if we can create another pthread to do so) */
 	agent_retry(999, false);
 }
@@ -1270,7 +1307,7 @@ static void _spawn_retry_agent(agent_arg_t * agent_arg_ptr)
 	if (agent_arg_ptr == NULL)
 		return;
 
-	debug2("Spawning RPC agent for msg_type %u", 
+	debug2("Spawning RPC agent for msg_type %u",
 	       agent_arg_ptr->msg_type);
 	slurm_attr_init(&attr_agent);
 	if (pthread_attr_setdetachstate(&attr_agent,
@@ -1286,7 +1323,7 @@ static void _spawn_retry_agent(agent_arg_t * agent_arg_ptr)
 	slurm_attr_destroy(&attr_agent);
 }
 
-/* _slurmctld_free_batch_job_launch_msg is a variant of 
+/* _slurmctld_free_batch_job_launch_msg is a variant of
  *	slurm_free_job_launch_msg because all environment variables currently
  *	loaded in one xmalloc buffer (see get_job_env()), which is different
  *	from how slurmd assembles the data from a message
@@ -1334,7 +1371,7 @@ static void _purge_agent_args(agent_arg_t *agent_arg_ptr)
 		if (agent_arg_ptr->msg_type == REQUEST_BATCH_JOB_LAUNCH)
 			_slurmctld_free_batch_job_launch_msg(
 				agent_arg_ptr->msg_args);
-		else if (agent_arg_ptr->msg_type == 
+		else if (agent_arg_ptr->msg_type ==
 				RESPONSE_RESOURCE_ALLOCATION)
 			slurm_free_resource_allocation_response_msg(
 					agent_arg_ptr->msg_args);
@@ -1383,13 +1420,15 @@ static void _mail_proc(mail_info_t *mi)
 		(void) close(0);
 		(void) close(1);
 		(void) close(2);
-		fd = open("/dev/null", O_RDWR);
-		dup(fd);
-		dup(fd);
-		execle(slurmctld_conf.mail_prog, "mail", 
+		fd = open("/dev/null", O_RDWR); // 0
+		if(dup(fd) == -1) // 1
+			error("Couldn't do a dup for 1: %m");
+		if(dup(fd) == -1) // 2
+			error("Couldn't do a dup for 2 %m");
+		execle(slurmctld_conf.mail_prog, "mail",
 			"-s", mi->message, mi->user_name,
 			NULL, NULL);
-		error("Failed to exec %s: %m", 
+		error("Failed to exec %s: %m",
 			slurmctld_conf.mail_prog);
 		exit(1);
 	} else {		/* parent */
@@ -1426,7 +1465,7 @@ extern void mail_job_info (struct job_record *job_ptr, uint16_t mail_type)
 
 	mi->message = xmalloc(sizeof(char)*128);
 	sprintf(mi->message, "SLURM Job_id=%u Name=%.24s %s",
-		job_ptr->job_id, job_ptr->name, 
+		job_ptr->job_id, job_ptr->name,
 		_mail_type_str(mail_type));
 
 	info ("msg to %s: %s", mi->user_name, mi->message);
@@ -1443,3 +1482,71 @@ extern void mail_job_info (struct job_record *job_ptr, uint16_t mail_type)
 	return;
 }
 
+/* Test if a batch launch request should be defered
+ * RET -1: abort the request, pending job cancelled
+ *      0: execute the request now
+ *      1: defer the request
+ */
+static int _batch_launch_defer(queued_request_t *queued_req_ptr)
+{
+	char hostname[512];
+	agent_arg_t *agent_arg_ptr;
+	batch_job_launch_msg_t *launch_msg_ptr;
+	struct node_record *node_ptr;
+	time_t now = time(NULL);
+	struct job_record  *job_ptr;
+
+	agent_arg_ptr = queued_req_ptr->agent_arg_ptr;
+	if (agent_arg_ptr->msg_type != REQUEST_BATCH_JOB_LAUNCH)
+		return 0;
+
+	if (difftime(now, queued_req_ptr->last_attempt) < 10) {
+		/* Reduce overhead by only testing once every 10 secs */
+		return 1;
+	}
+
+	launch_msg_ptr = (batch_job_launch_msg_t *)agent_arg_ptr->msg_args;
+	job_ptr = find_job_record(launch_msg_ptr->job_id);
+	if ((job_ptr == NULL) ||
+	    (!IS_JOB_RUNNING(job_ptr) && !IS_JOB_SUSPENDED(job_ptr))) {
+		info("agent(batch_launch): removed pending request for "
+		     "cancelled job %u",
+		     launch_msg_ptr->job_id);
+		return -1;	/* job cancelled while waiting */
+	}
+
+	hostlist_deranged_string(agent_arg_ptr->hostlist,
+				 sizeof(hostname), hostname);
+	node_ptr = find_node_record(hostname);
+	if (node_ptr == NULL) {
+		error("agent(batch_launch) removed pending request for job "
+		      "%u, missing node %s",
+		      launch_msg_ptr->job_id, agent_arg_ptr->hostlist);
+		return -1;	/* invalid request?? */
+	}
+
+	if (!IS_NODE_POWER_SAVE(node_ptr) && !IS_NODE_NO_RESPOND(node_ptr)) {
+		/* ready to launch, adjust time limit for boot time */
+		if (job_ptr->time_limit != INFINITE)
+			job_ptr->end_time = now + (job_ptr->time_limit * 60);
+		queued_req_ptr->last_attempt = (time_t) 0;
+		return 0;
+	}
+
+	if (queued_req_ptr->last_attempt == 0) {
+		queued_req_ptr->first_attempt = now;
+		queued_req_ptr->last_attempt  = now;
+	} else if (difftime(now, queued_req_ptr->first_attempt) >=
+				 slurm_get_resume_timeout()) {
+		error("agent waited too long for node %s to respond, "
+		      "sending batch request anyway...",
+		      node_ptr->name);
+		if (job_ptr->time_limit != INFINITE)
+			job_ptr->end_time = now + (job_ptr->time_limit * 60);
+		queued_req_ptr->last_attempt = (time_t) 0;
+		return 0;
+	}
+
+	queued_req_ptr->last_attempt  = now;
+	return 1;
+}
