@@ -2,35 +2,36 @@
  *  trigger_mgr.c - Event trigger management
  *****************************************************************************
  *  Copyright (C) 2007 The Regents of the University of California.
- *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov> et. al.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -61,17 +62,11 @@
 #include "src/slurmctld/state_save.h"
 #include "src/slurmctld/trigger_mgr.h"
 
-#define _DEBUG 0
 #define MAX_PROG_TIME 300	/* maximum run time for program */
 
 /* Change TRIGGER_STATE_VERSION value when changing the state save format */
 #define TRIGGER_STATE_VERSION      "VER002"
 
-/* TRIG_IS_JOB_FINI differs from IS_JOB_FINISHED by considering 
- * completing jobs as not really finished */
-#define TRIG_IS_JOB_FINI(_X)             \
-        (IS_JOB_FINISHED(_X) && ((_X->job_state & JOB_COMPLETING) == 0))
-
 List trigger_list;
 uint32_t next_trigger_id = 1;
 static pthread_mutex_t trigger_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -140,7 +135,6 @@ static char *_trig_type(uint16_t trig_type)
 		return "unknown";
 }
 
-#if _DEBUG
 static int _trig_offset(uint16_t offset)
 {
 	static int rc;
@@ -153,6 +147,9 @@ static void _dump_trigger_msg(char *header, trigger_info_msg_t *msg)
 {
 	int i;
 
+	if ((slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) == 0)
+		return;
+
 	info(header);
 	if ((msg == NULL) || (msg->record_count == 0)) {
 		info("Trigger has no entries");
@@ -171,11 +168,6 @@ static void _dump_trigger_msg(char *header, trigger_info_msg_t *msg)
 			msg->trigger_array[i].program);
 	}
 }
-#else
-static void _dump_trigger_msg(char *header, trigger_info_msg_t *msg)
-{
-}
-#endif
 
 /* Validate trigger program */
 static bool _validate_trigger(trig_mgr_info_t *trig_in)
@@ -317,9 +309,9 @@ extern int trigger_set(uid_t uid, gid_t gid, trigger_info_msg_t *msg)
 
 	if ((slurmctld_conf.slurm_user_id != 0)
 	&&  (slurmctld_conf.slurm_user_id != uid)) {
-		/* If SlurmUser is not root, then it is unable to set the 
-		 * appropriate user id and group id for the program to be 
-		 * launched. To prevent the launched program for an arbitrary 
+		/* If SlurmUser is not root, then it is unable to set the
+		 * appropriate user id and group id for the program to be
+		 * launched. To prevent the launched program for an arbitrary
 		 * user being executed as user SlurmUser, disable all other
 		 * users from setting triggers. */
 		info("Attempt to set trigger by uid %u != SlurmUser", uid);
@@ -495,16 +487,16 @@ static int _load_trigger_state(Buf buffer)
 	if (trig_ptr->res_type == TRIGGER_RES_TYPE_JOB) {
 		trig_ptr->job_id = (uint32_t) atol(trig_ptr->res_id);
 		trig_ptr->job_ptr = find_job_record(trig_ptr->job_id);
-		if ((trig_ptr->job_id == 0)
-		||  (trig_ptr->job_ptr == NULL)
-		||  (TRIG_IS_JOB_FINI(trig_ptr->job_ptr)))
+		if ((trig_ptr->job_id == 0)     ||
+		    (trig_ptr->job_ptr == NULL) ||
+		    (IS_JOB_COMPLETED(trig_ptr->job_ptr)))
 			goto unpack_error;
 	} else {
 		trig_ptr->job_id = 0;
 		trig_ptr->job_ptr = NULL;
-		if ((trig_ptr->res_id != NULL)
-		&&  (trig_ptr->res_id[0] != '*')
-		&&  (node_name2bitmap(trig_ptr->res_id, false,
+		if ((trig_ptr->res_id != NULL)   &&
+		    (trig_ptr->res_id[0] != '*') &&
+		    (node_name2bitmap(trig_ptr->res_id, false,
 				&trig_ptr->nodes_bitmap) != 0))
 			goto unpack_error;
 	}
@@ -528,6 +520,7 @@ unpack_error:
 }
 extern int trigger_state_save(void)
 {
+	/* Save high-water mark to avoid buffer growth with copies */
 	static int high_buffer_size = (1024 * 1024);
 	int error_code = 0, log_fd;
 	char *old_file, *new_file, *reg_file;
@@ -565,12 +558,12 @@ extern int trigger_state_save(void)
 
 	lock_state_files();
 	log_fd = creat(new_file, 0600);
-	if (log_fd == 0) {
+	if (log_fd < 0) {
 		error("Can't save state, create file %s error %m",
 		      new_file);
 		error_code = errno;
 	} else {
-		int pos = 0, nwrite = get_buf_offset(buffer), amount;
+		int pos = 0, nwrite = get_buf_offset(buffer), amount, rc;
 		char *data = (char *)get_buf_data(buffer);
 		high_buffer_size = MAX(nwrite, high_buffer_size);
 		while (nwrite > 0) {
@@ -583,16 +576,22 @@ extern int trigger_state_save(void)
 			nwrite -= amount;
 			pos    += amount;
 		}
-		fsync(log_fd);
-		close(log_fd);
+
+		rc = fsync_and_close(log_fd, "trigger");
+		if (rc && !error_code)
+			error_code = rc;
 	}
 	if (error_code)
 		(void) unlink(new_file);
 	else {			/* file shuffle */
 		(void) unlink(old_file);
-		(void) link(reg_file, old_file);
+		if(link(reg_file, old_file))
+			debug4("unable to create link for %s -> %s: %m",
+			       reg_file, old_file);
 		(void) unlink(reg_file);
-		(void) link(new_file, reg_file);
+		if(link(new_file, reg_file))
+			debug4("unable to create link for %s -> %s: %m",
+			       new_file, reg_file);
 		(void) unlink(new_file);
 	}
 	xfree(old_file);
@@ -604,6 +603,37 @@ extern int trigger_state_save(void)
 	return error_code;
 }
 
+/* Open the trigger state save file, or backup if necessary.
+ * state_file IN - the name of the state save file used
+ * RET the file description to read from or error code
+ */
+static int _open_resv_state_file(char **state_file)
+{
+	int state_fd;
+	struct stat stat_buf;
+
+	*state_file = xstrdup(slurmctld_conf.state_save_location);
+	xstrcat(*state_file, "/trigger_state");
+	state_fd = open(*state_file, O_RDONLY);
+	if (state_fd < 0) {
+		error("Could not open trigger state file %s: %m",
+		      *state_file);
+	} else if (fstat(state_fd, &stat_buf) < 0) {
+		error("Could not stat trigger state file %s: %m",
+		      *state_file);
+		(void) close(state_fd);
+	} else if (stat_buf.st_size < 10) {
+		error("Trigger state file %s too small", *state_file);
+		(void) close(state_fd);
+	} else 	/* Success */
+		return state_fd;
+
+	error("NOTE: Trying backup state save file. Triggers may be lost!");
+	xstrcat(*state_file, ".old");
+	state_fd = open(*state_file, O_RDONLY);
+	return state_fd;
+}
+
 extern int trigger_state_restore(void)
 {
 	int data_allocated, data_read = 0, error_code = 0;
@@ -616,10 +646,8 @@ extern int trigger_state_restore(void)
 	uint32_t ver_str_len;
 
 	/* read the file */
-	state_file = xstrdup(slurmctld_conf.state_save_location);
-	xstrcat(state_file, "/trigger_state");
 	lock_state_files();
-	state_fd = open(state_file, O_RDONLY);
+	state_fd = _open_resv_state_file(&state_file);
 	if (state_fd < 0) {
 		info("No trigger state file (%s) to recover", state_file);
 		error_code = ENOENT;
@@ -633,7 +661,7 @@ extern int trigger_state_restore(void)
 				if (errno == EINTR)
 					continue;
 				else {
-					error("Read error on %s: %m", 
+					error("Read error on %s: %m",
 					      state_file);
 					break;
 				}
@@ -651,7 +679,7 @@ extern int trigger_state_restore(void)
 	buffer = create_buf(data, data_size);
 	safe_unpackstr_xmalloc(&ver_str, &ver_str_len, buffer);
 	if (strcmp(ver_str, TRIGGER_STATE_VERSION) != 0) {
-		error("Can't recover trigger state, data version incompatable");
+		error("Can't recover trigger state, data version incompatible");
 		xfree(ver_str);
 		free_buf(buffer);
 		return EFAULT;
@@ -684,21 +712,21 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 
 	if ((trig_in->trig_type & TRIGGER_TYPE_FINI)
 	&&  ((trig_in->job_ptr == NULL) ||
-	     (TRIG_IS_JOB_FINI(trig_in->job_ptr)))) {
+	     (IS_JOB_COMPLETED(trig_in->job_ptr)))) {
 		trig_in->state = 1;
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
-#if _DEBUG
-		info("trigger[%u] event for job %u fini",
-			trig_in->trig_id, trig_in->job_id);
-#endif
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] event for job %u fini",
+				trig_in->trig_id, trig_in->job_id);
+		}
 		return;
 	}
 
 	if (trig_in->job_ptr == NULL) {
-#if _DEBUG
-		info("trigger[%u] for defunct job %u",
-			trig_in->trig_id, trig_in->job_id);
-#endif
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for defunct job %u",
+				trig_in->trig_id, trig_in->job_id);
+		}
 		trig_in->state = 2;
 		trig_in->trig_time = now;
 		return;
@@ -709,24 +737,24 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 		if (rem_time <= (0x8000 - trig_in->trig_time)) {
 			trig_in->state = 1;
 			trig_in->trig_time = now;
-#if _DEBUG
-			info("trigger[%u] for job %u time",
-				trig_in->trig_id, trig_in->job_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for job %u time",
+					trig_in->trig_id, trig_in->job_id);
+			}
 			return;
 		}
 	}
 
 	if (trig_in->trig_type & TRIGGER_TYPE_DOWN) {
 		if (trigger_down_nodes_bitmap
-		&&  bit_overlap(trig_in->job_ptr->node_bitmap, 
+		&&  bit_overlap(trig_in->job_ptr->node_bitmap,
 				trigger_down_nodes_bitmap)) {
-#if _DEBUG
-			info("trigger[%u] for job %u down",
-				trig_in->trig_id, trig_in->job_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for job %u down",
+					trig_in->trig_id, trig_in->job_id);
+			}
 			trig_in->state = 1;
-			trig_in->trig_time = now + 
+			trig_in->trig_time = now +
 					(trig_in->trig_time - 0x8000);
 			return;
 		}
@@ -734,14 +762,14 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 
 	if (trig_in->trig_type & TRIGGER_TYPE_FAIL) {
 		if (trigger_fail_nodes_bitmap
-		&&  bit_overlap(trig_in->job_ptr->node_bitmap, 
+		&&  bit_overlap(trig_in->job_ptr->node_bitmap,
 				trigger_fail_nodes_bitmap)) {
-#if _DEBUG
-			info("trigger[%u] for job %u node fail",
-				trig_in->trig_id, trig_in->job_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for job %u node fail",
+					trig_in->trig_id, trig_in->job_id);
+			}
 			trig_in->state = 1;
-			trig_in->trig_time = now + 
+			trig_in->trig_time = now +
 					(trig_in->trig_time - 0x8000);
 			return;
 		}
@@ -749,15 +777,15 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 
 	if (trig_in->trig_type & TRIGGER_TYPE_UP) {
 		if (trigger_up_nodes_bitmap
-		&&  bit_overlap(trig_in->job_ptr->node_bitmap, 
+		&&  bit_overlap(trig_in->job_ptr->node_bitmap,
 				trigger_up_nodes_bitmap)) {
 			trig_in->state = 1;
-			trig_in->trig_time = now + 
+			trig_in->trig_time = now +
 					(0x8000 - trig_in->trig_time);
-#if _DEBUG
-			info("trigger[%u] for job %u up",
-				trig_in->trig_id, trig_in->job_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for job %u up",
+					trig_in->trig_id, trig_in->job_id);
+			}
 			return;
 		}
 	}
@@ -769,9 +797,8 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 	&&   trigger_block_err) {
 		trig_in->state = 1;
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
-#if _DEBUG
-		info("trigger[%u] for block_err", trig_in->trig_id);
-#endif
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS)
+			info("trigger[%u] for block_err", trig_in->trig_id);
 		return;
 	}
 
@@ -783,9 +810,9 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->res_id = bitmap2node_name(
 					trigger_down_nodes_bitmap);
 			trig_in->state = 1;
-		} else if (bit_overlap(trig_in->nodes_bitmap, 
+		} else if (bit_overlap(trig_in->nodes_bitmap,
 					trigger_down_nodes_bitmap)) {
-			bit_and(trig_in->nodes_bitmap, 
+			bit_and(trig_in->nodes_bitmap,
 					trigger_down_nodes_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
@@ -793,12 +820,12 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->state = 1;
 		}
 		if (trig_in->state == 1) {
-			trig_in->trig_time = now + 
+			trig_in->trig_time = now +
 					(trig_in->trig_time - 0x8000);
-#if _DEBUG
-			info("trigger[%u] for node %s down",
-				trig_in->trig_id, trig_in->res_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for node %s down",
+					trig_in->trig_id, trig_in->res_id);
+			}
 			return;
 		}
 	}
@@ -811,9 +838,9 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->res_id = bitmap2node_name(
 					trigger_drained_nodes_bitmap);
 			trig_in->state = 1;
-		} else if (bit_overlap(trig_in->nodes_bitmap, 
+		} else if (bit_overlap(trig_in->nodes_bitmap,
 					trigger_drained_nodes_bitmap)) {
-			bit_and(trig_in->nodes_bitmap, 
+			bit_and(trig_in->nodes_bitmap,
 					trigger_drained_nodes_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
@@ -821,12 +848,12 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->state = 1;
 		}
 		if (trig_in->state == 1) {
-			trig_in->trig_time = now + 
+			trig_in->trig_time = now +
 					(trig_in->trig_time - 0x8000);
-#if _DEBUG
-			info("trigger[%u] for node %s drained",
-				trig_in->trig_id, trig_in->res_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for node %s drained",
+					trig_in->trig_id, trig_in->res_id);
+			}
 			return;
 		}
 	}
@@ -839,9 +866,9 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->res_id = bitmap2node_name(
 					trigger_fail_nodes_bitmap);
 			trig_in->state = 1;
-		} else if (bit_overlap(trig_in->nodes_bitmap, 
+		} else if (bit_overlap(trig_in->nodes_bitmap,
 					trigger_fail_nodes_bitmap)) {
-			bit_and(trig_in->nodes_bitmap, 
+			bit_and(trig_in->nodes_bitmap,
 					trigger_fail_nodes_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
@@ -849,30 +876,28 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->state = 1;
 		}
 		if (trig_in->state == 1) {
-			trig_in->trig_time = now + 
+			trig_in->trig_time = now +
 					(trig_in->trig_time - 0x8000);
-#if _DEBUG
-			info("trigger[%u] for node %s fail",
-				trig_in->trig_id, trig_in->res_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for node %s fail",
+					trig_in->trig_id, trig_in->res_id);
+			}
 			return;
 		}
 	}
 
 	if (trig_in->trig_type & TRIGGER_TYPE_IDLE) {
-		/* We need to determine which (if any) of these 
+		/* We need to determine which (if any) of these
 		 * nodes have been idle for at least the offset time */
 		time_t min_idle = now - (trig_in->trig_time - 0x8000);
 		int i;
-		uint16_t base_state;
 		struct node_record *node_ptr = node_record_table_ptr;
 		bitstr_t *trigger_idle_node_bitmap;
 
 		trigger_idle_node_bitmap = bit_alloc(node_record_count);
 		for (i = 0; i < node_record_count; i++, node_ptr++) {
-			base_state = node_ptr->node_state & NODE_STATE_BASE;
-			if ((base_state != NODE_STATE_IDLE)
-			||  (node_ptr->last_idle > min_idle))
+			if (!IS_NODE_IDLE(node_ptr) ||
+			    (node_ptr->last_idle > min_idle))
 				continue;
 			bit_set(trigger_idle_node_bitmap, i);
 		}
@@ -893,10 +918,10 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		bit_free(trigger_idle_node_bitmap);
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now;
-#if _DEBUG
-			info("trigger[%u] for node %s idle",
-				trig_in->trig_id, trig_in->res_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for node %s idle",
+					trig_in->trig_id, trig_in->res_id);
+			}
 			return;
 		}
 	}
@@ -909,9 +934,9 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->res_id = bitmap2node_name(
 					trigger_up_nodes_bitmap);
 			trig_in->state = 1;
-		} else if (bit_overlap(trig_in->nodes_bitmap, 
+		} else if (bit_overlap(trig_in->nodes_bitmap,
 					trigger_up_nodes_bitmap)) {
-			bit_and(trig_in->nodes_bitmap, 
+			bit_and(trig_in->nodes_bitmap,
 					trigger_up_nodes_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
@@ -919,12 +944,12 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->state = 1;
 		}
 		if (trig_in->state == 1) {
-			trig_in->trig_time = now + 
+			trig_in->trig_time = now +
 					(trig_in->trig_time - 0x8000);
-#if _DEBUG
-			info("trigger[%u] for node %s up",
-				trig_in->trig_id, trig_in->res_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("trigger[%u] for node %s up",
+					trig_in->trig_id, trig_in->res_id);
+			}
 			return;
 		}
 	}
@@ -935,18 +960,17 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("reconfig");
-#if _DEBUG
-		info("trigger[%u] for reconfig", trig_in->trig_id);
-#endif
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS)
+			info("trigger[%u] for reconfig", trig_in->trig_id);
 		return;
 	}
 }
 
-/* Ideally we would use the existing proctrack plugin to prevent any 
+/* Ideally we would use the existing proctrack plugin to prevent any
  * processes from escaping our control, but that plugin is tied
  * to various slurmd data structures. We just the process group ID
- * to kill the spawned program after MAX_PROG_TIME. Since triggers are 
- * meant primarily for system administrators rather than users, this 
+ * to kill the spawned program after MAX_PROG_TIME. Since triggers are
+ * meant primarily for system administrators rather than users, this
  * may be sufficient. */
 static void _trigger_run_program(trig_mgr_info_t *trig_in)
 {
@@ -1008,7 +1032,7 @@ static void _trigger_run_program(trig_mgr_info_t *trig_in)
 static void _clear_event_triggers(void)
 {
 	if (trigger_down_nodes_bitmap) {
-		bit_nclear(trigger_down_nodes_bitmap, 
+		bit_nclear(trigger_down_nodes_bitmap,
 			   0, (bit_size(trigger_down_nodes_bitmap) - 1));
 	}
 	if (trigger_drained_nodes_bitmap) {
@@ -1016,7 +1040,7 @@ static void _clear_event_triggers(void)
 			   0, (bit_size(trigger_drained_nodes_bitmap) - 1));
 	}
 	if (trigger_up_nodes_bitmap) {
-		bit_nclear(trigger_up_nodes_bitmap,   
+		bit_nclear(trigger_up_nodes_bitmap,
 			   0, (bit_size(trigger_up_nodes_bitmap) - 1));
 	}
 	trigger_node_reconfig = false;
@@ -1049,23 +1073,23 @@ extern void trigger_process(void)
 		}
 		if ((trig_in->state == 1) &&
 		    (trig_in->trig_time <= now)) {
-#if _DEBUG
-			info("launching program for trigger[%u]",
-				trig_in->trig_id);
-			info("  uid=%u gid=%u program=%s arg=%s", 
-				trig_in->user_id, trig_in->group_id,
-				trig_in->program, trig_in->res_id);
-#endif
+			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+				info("launching program for trigger[%u]",
+					trig_in->trig_id);
+				info("  uid=%u gid=%u program=%s arg=%s",
+					trig_in->user_id, trig_in->group_id,
+					trig_in->program, trig_in->res_id);
+			}
 			trig_in->state = 2;
 			trig_in->trig_time = now;
 			state_change = true;
 			_trigger_run_program(trig_in);
-		} else if ((trig_in->state == 2) && 
-			   (difftime(now, trig_in->trig_time) > 
+		} else if ((trig_in->state == 2) &&
+			   (difftime(now, trig_in->trig_time) >
 					MAX_PROG_TIME)) {
 			if (trig_in->group_id != 0) {
 				killpg(trig_in->group_id, SIGKILL);
-				rc = waitpid(trig_in->group_id, &prog_stat, 
+				rc = waitpid(trig_in->group_id, &prog_stat,
 					     WNOHANG);
 				if ((rc > 0) && prog_stat) {
 					info("trigger uid=%u type=%s:%s "
@@ -1073,7 +1097,7 @@ extern void trigger_process(void)
 					     trig_in->user_id,
 					     _res_type(trig_in->res_type),
 					     _trig_type(trig_in->trig_type),
-					     WIFEXITED(prog_stat), 
+					     WIFEXITED(prog_stat),
 					     WTERMSIG(prog_stat));
 				}
 				if ((rc == trig_in->group_id) ||
@@ -1082,9 +1106,11 @@ extern void trigger_process(void)
 			}
 
 			if (trig_in->group_id == 0) {
-#if _DEBUG
-				info("purging trigger[%u]", trig_in->trig_id);
-#endif
+				if (slurm_get_debug_flags() &
+				    DEBUG_FLAG_TRIGGERS) {
+					info("purging trigger[%u]",
+					     trig_in->trig_id);
+				}
 				list_delete_item(trig_iter);
 				state_change = true;
 			}
