@@ -1,14 +1,15 @@
 /*****************************************************************************\
  * src/slurmd/slurmstepd/io.c - Standard I/O handling routines for slurmstepd
- * $Id: io.c 13672 2008-03-19 23:10:58Z jette $
+ * $Id: io.c 18467 2009-08-20 21:34:27Z dbremer $
  *****************************************************************************
  *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>.
- *  LLNL-CODE-402394.
+ *  CODE-OCEC-09-009. All rights reserved.
  *  
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
  *  
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -79,6 +80,7 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
+#include "src/common/write_labelled_message.h"
 
 #include "src/slurmd/slurmd/slurmd.h"
 #include "src/slurmd/slurmstepd/io.h"
@@ -118,8 +120,27 @@ struct client_io_info {
 	struct io_buf *out_msg;
 	int32_t out_remaining;
 	bool out_eof;
+
+	/* For clients that only write stdout or stderr, and/or only
+	   write for one task. -1 means accept output from any task. */
+	int  ltaskid_stdout, ltaskid_stderr;
+	bool labelio;
+	int  label_width;
+
+	/* true if writing to a file, false if writing to a socket */
+	bool is_local_file;
 };
 
+
+static bool _local_file_writable(eio_obj_t *);
+static int  _local_file_write(eio_obj_t *, List);
+
+struct io_operations local_file_ops = {
+	writable:	&_local_file_writable,
+	handle_write:	&_local_file_write,
+};
+
+
 /**********************************************************************
  * Task write declarations
  **********************************************************************/
@@ -198,6 +219,8 @@ static void _free_incoming_msg(struct io_buf *msg, slurmd_job_t *job);
 static void _free_all_outgoing_msgs(List msg_queue, slurmd_job_t *job);
 static bool _incoming_buf_free(slurmd_job_t *job);
 static bool _outgoing_buf_free(slurmd_job_t *job);
+static int  _send_connection_okay_response(slurmd_job_t *job);
+static struct io_buf *_build_connection_okay_message(slurmd_job_t *job);
 
 /**********************************************************************
  * IO client socket functions
@@ -258,6 +281,9 @@ _client_writable(eio_obj_t *obj)
 		struct io_buf *msg;
 		client->msg_queue = list_create(NULL); /* need destructor */
 		msgs = list_iterator_create(client->job->outgoing_cache);
+		if (!msgs)
+			fatal("Could not allocate iterator");
+
 		while ((msg = list_next(msgs))) {
 			msg->ref_count++;
 			list_enqueue(client->msg_queue, msg);
@@ -321,7 +347,27 @@ _client_read(eio_obj_t *obj, List objs)
 	/*
 	 * Read the body
 	 */
-	if (client->header.length == 0) { /* zero length is an eof message */
+	if (client->header.type == SLURM_IO_CONNECTION_TEST) {
+		if (client->header.length != 0) {
+			debug5("  error in _client_read: bad connection test");
+			list_enqueue(client->job->free_incoming, client->in_msg);
+			client->in_msg = NULL;
+			return SLURM_ERROR;
+		}
+		if (_send_connection_okay_response(client->job)) {
+			/* 
+			 * If we get here because of a failed 
+			 * _send_connection_okay_response, it's because of a
+			 * lack of buffer space in the output queue.  Just 
+			 * keep the current input message client->in_msg in 
+			 * place, and resend on the next call.
+			 */
+			return SLURM_SUCCESS;
+		}
+		list_enqueue(client->job->free_incoming, client->in_msg);
+		client->in_msg = NULL;
+		return SLURM_SUCCESS;
+	} else if (client->header.length == 0) { /* zero length is an eof message */
 		debug5("  got stdin eof message!");
 	} else {
 		buf = client->in_msg->data + 
@@ -452,6 +498,92 @@ again:
 	return SLURM_SUCCESS;
 }
 
+
+static bool 
+_local_file_writable(eio_obj_t *obj)
+{
+	struct client_io_info *client = (struct client_io_info *) obj->arg;
+
+	xassert(client->magic == CLIENT_IO_MAGIC);
+
+	if (client->out_eof == true)
+		return false;
+
+	if (client->out_msg != NULL || !list_is_empty(client->msg_queue))
+		return true;
+
+	return false;
+}
+
+
+/*
+ * The slurmstepd writes I/O to a file, possibly adding a label.
+ */
+static int
+_local_file_write(eio_obj_t *obj, List objs)
+{
+	struct client_io_info *client = (struct client_io_info *) obj->arg;
+	void *buf;
+	int n;
+	struct slurm_io_header header;
+	Buf header_tmp_buf;
+
+	xassert(client->magic == CLIENT_IO_MAGIC);
+	/*
+	 * If we aren't already in the middle of sending a message, get the
+	 * next message from the queue.
+	 */
+	if (client->out_msg == NULL) {
+		client->out_msg = list_dequeue(client->msg_queue);
+		if (client->out_msg == NULL) {
+			return SLURM_SUCCESS;
+		}
+		client->out_remaining = client->out_msg->length - 
+					io_hdr_packed_size();
+	}
+
+	/* This code to make a buffer, fill it, unpack its contents, and free
+	   it is just used to read the header to get the global task id. */
+	header_tmp_buf = create_buf(client->out_msg->data, 
+				    client->out_msg->length);
+	if (!header_tmp_buf)
+		fatal("Failure to allocate memory for a message header");
+	io_hdr_unpack(&header, header_tmp_buf);
+	header_tmp_buf->head = NULL;
+	free_buf(header_tmp_buf);
+
+	/* A zero-length message indicates the end of a stream from one
+	   of the tasks.  Just free the message and return. */
+	if (header.length == 0) {
+		_free_outgoing_msg(client->out_msg, client->job);
+		client->out_msg = NULL;
+		return SLURM_SUCCESS;
+	}
+
+	/* Write the message to the file. */
+	buf = client->out_msg->data + 
+		(client->out_msg->length - client->out_remaining);
+
+	n = write_labelled_message(obj->fd, buf, client->out_remaining, 
+				   header.gtaskid, client->labelio, 
+				   client->label_width);
+	if (n < 0) {
+		client->out_eof = true;
+		_free_all_outgoing_msgs(client->msg_queue, client->job);
+		return SLURM_ERROR;
+	}
+
+	client->out_remaining -= n;
+	if (client->out_remaining == 0) {
+		_free_outgoing_msg(client->out_msg, client->job);
+		client->out_msg = NULL;
+	}
+	return SLURM_SUCCESS;
+}
+
+
+
+
 /**********************************************************************
  * Task write functions
  **********************************************************************/
@@ -541,7 +673,7 @@ _task_write(eio_obj_t *obj, List objs)
 	}
 
 	/*
-	 * Write message to socket.
+	 * Write message to pipe.
 	 */
 	buf = in->msg->data + (in->msg->length - in->remaining);
 again:
@@ -798,22 +930,7 @@ _spawn_window_manager(slurmd_task_info_t *task, slurmd_job_t *job)
 static int
 _init_task_stdio_fds(slurmd_task_info_t *task, slurmd_job_t *job)
 {
-	slurm_ctl_conf_t *conf;
-	int file_flags;
-
-	/* set files for opening stdout/err */
-	if (job->open_mode == OPEN_MODE_APPEND)
-		file_flags = O_CREAT|O_WRONLY|O_APPEND;
-	else if (job->open_mode == OPEN_MODE_TRUNCATE)
-		file_flags = O_CREAT|O_WRONLY|O_APPEND|O_TRUNC;
-	else {
-		conf = slurm_conf_lock();
-		if (conf->job_file_append)
-			file_flags = O_CREAT|O_WRONLY|O_APPEND;
-		else
-			file_flags = O_CREAT|O_WRONLY|O_APPEND|O_TRUNC;
-		slurm_conf_unlock();
-	}
+	int file_flags = io_get_file_flags(job);
 
 	/*
 	 *  Initialize stdin
@@ -855,7 +972,7 @@ _init_task_stdio_fds(slurmd_task_info_t *task, slurmd_job_t *job)
 		/* open file on task's stdin */
 		debug5("  stdin file name = %s", task->ifname);
 		if ((task->stdin_fd = open(task->ifname, O_RDONLY)) == -1) {
-			error("Could not open stdin file: %m");
+			error("Could not open stdin file %s: %m", task->ifname);
 			return SLURM_ERROR;
 		}
 		fd_set_close_on_exec(task->stdin_fd);
@@ -899,20 +1016,19 @@ _init_task_stdio_fds(slurmd_task_info_t *task, slurmd_job_t *job)
 			fd_set_close_on_exec(task->stdout_fd);
 			task->from_stdout = -1;  /* not used */
 		}
-	} else if (task->ofname != NULL) {
+	} else if (task->ofname != NULL && 
+		   (!job->labelio || strcmp(task->ofname, "/dev/null")==0)) {
 #else
-	if (task->ofname != NULL) {
+	if (task->ofname != NULL && 
+	    (!job->labelio || strcmp(task->ofname, "/dev/null")==0) ) {
 #endif
 		/* open file on task's stdout */
 		debug5("  stdout file name = %s", task->ofname);
 		task->stdout_fd = open(task->ofname, file_flags, 0666);
 		if (task->stdout_fd == -1) {
-			error("Could not open stdout file: %m");
-			xfree(task->ofname);
-			task->ofname = fname_create(job, "slurm-%J.out", 0);
-			task->stdout_fd = open(task->ofname, file_flags, 0666);
-			if (task->stdout_fd == -1)
-				return SLURM_ERROR;
+			error("Could not open stdout file %s: %m",
+			      task->ofname);
+			return SLURM_ERROR;
 		}
 		fd_set_close_on_exec(task->stdout_fd);
 		task->from_stdout = -1; /* not used */
@@ -941,15 +1057,16 @@ _init_task_stdio_fds(slurmd_task_info_t *task, slurmd_job_t *job)
 #ifdef HAVE_PTY_H
 	if (job->pty) {
 		if (task->gtid == 0) {
+			/* Make a file descriptor for the task to write to, but
+			   don't make a separate one read from, because in pty 
+			   mode we can't distinguish between stdout and stderr
+			   coming from the remote shell.  Both streams from the
+			   shell will go to task->stdout_fd, which is okay in 
+			   pty mode because any output routed through the stepd
+			   will be displayed. */
 			task->stderr_fd = dup(task->stdin_fd);
 			fd_set_close_on_exec(task->stderr_fd);
-			task->from_stderr = dup(task->to_stdin);
-			fd_set_close_on_exec(task->from_stderr);
-			fd_set_nonblocking(task->from_stderr);
-			task->err = _create_task_out_eio(task->from_stderr,
-						 SLURM_IO_STDERR, job, task);
-			list_append(job->stderr_eio_objs, (void *)task->err);
-			eio_new_initial_obj(job->eio, (void *)task->err);
+			task->from_stderr = -1;
 		} else {
 			xfree(task->efname);
 			task->efname = xstrdup("/dev/null");
@@ -957,20 +1074,19 @@ _init_task_stdio_fds(slurmd_task_info_t *task, slurmd_job_t *job)
 			fd_set_close_on_exec(task->stderr_fd);
 			task->from_stderr = -1;  /* not used */
 		}
-	} else if (task->efname != NULL) {
+	} else if (task->efname != NULL && 
+		   (!job->labelio || strcmp(task->efname, "/dev/null")==0)) {
 #else
-	if (task->efname != NULL) {
+	if (task->efname != NULL && 
+	    (!job->labelio || strcmp(task->efname, "/dev/null")==0) ) {
 #endif
 		/* open file on task's stdout */
 		debug5("  stderr file name = %s", task->efname);
 		task->stderr_fd = open(task->efname, file_flags, 0666);
 		if (task->stderr_fd == -1) {
-			error("Could not open stderr file: %m");
-			xfree(task->efname);
-			task->efname = fname_create(job, "slurm-%J.err", 0);
-			task->stderr_fd = open(task->efname, file_flags, 0666);
-			if (task->stderr_fd == -1)
-				return SLURM_ERROR;
+			error("Could not open stderr file %s: %m",
+			      task->efname);
+			return SLURM_ERROR;
 		}
 		fd_set_close_on_exec(task->stderr_fd);
 		task->from_stderr = -1; /* not used */
@@ -999,13 +1115,15 @@ _init_task_stdio_fds(slurmd_task_info_t *task, slurmd_job_t *job)
 int
 io_init_tasks_stdio(slurmd_job_t *job)
 {
-	int i;
+	int i, rc = SLURM_SUCCESS, tmprc;
 
 	for (i = 0; i < job->ntasks; i++) {
-		_init_task_stdio_fds(job->task[i], job);
+		tmprc = _init_task_stdio_fds(job->task[i], job);
+		if (tmprc != SLURM_SUCCESS)
+			rc = tmprc;
 	}
 
-	return 0;
+	return rc;
 }
 
 int
@@ -1053,6 +1171,78 @@ _shrink_msg_cache(List cache, slurmd_job_t *job)
 	}
 }
 
+
+
+static int
+_send_connection_okay_response(slurmd_job_t *job)
+{
+	eio_obj_t *eio;
+	ListIterator clients;
+	struct io_buf *msg;
+	struct client_io_info *client;
+
+	msg = _build_connection_okay_message(job);
+	if (!msg) {
+		error(  "Could not send connection okay message because of "
+			"lack of buffer space.");
+		return SLURM_ERROR;
+	}
+
+	clients = list_iterator_create(job->clients);
+	if (!clients)
+		fatal("Could not allocate memory");
+
+	while((eio = list_next(clients))) {
+		client = (struct client_io_info *)eio->arg;
+		if (client->out_eof || client->is_local_file)
+			continue;
+
+		debug5("Sent connection okay message");
+		xassert(client->magic == CLIENT_IO_MAGIC);
+		if (list_enqueue(client->msg_queue, msg))
+			msg->ref_count++;
+	}
+	list_iterator_destroy(clients);
+
+	return SLURM_SUCCESS;
+}
+
+
+
+static struct io_buf *
+_build_connection_okay_message(slurmd_job_t *job)
+{
+	struct io_buf *msg;
+	Buf packbuf;
+	struct slurm_io_header header;
+
+	if (_outgoing_buf_free(job)) {
+		msg = list_dequeue(job->free_outgoing);
+	} else {
+		return NULL;
+	}
+
+	header.type = SLURM_IO_CONNECTION_TEST;
+	header.ltaskid = 0;  /* Unused */
+	header.gtaskid = 0;  /* Unused */
+	header.length = 0;
+
+	packbuf = create_buf(msg->data, io_hdr_packed_size());
+	if (!packbuf)
+		fatal("Failure to allocate memory for a message header");
+	io_hdr_pack(&header, packbuf);
+	msg->length = io_hdr_packed_size();
+	msg->ref_count = 0; /* make certain it is initialized */
+
+	/* free the Buf packbuf, but not the memory to which it points */
+	packbuf->head = NULL;
+	free_buf(packbuf);
+	
+	return msg;
+}
+
+
+
 static void
 _route_msg_task_to_client(eio_obj_t *obj)
 {
@@ -1070,14 +1260,28 @@ _route_msg_task_to_client(eio_obj_t *obj)
 		if (msg == NULL)
 			return;
 
-/* 		debug5("\"%s\"", msg->data + io_hdr_packed_size()); */
-
 		/* Add message to the msg_queue of all clients */
 		clients = list_iterator_create(out->job->clients);
+		if (!clients)
+			fatal("Could not allocate iterator");
+
 		while((eio = list_next(clients))) {
 			client = (struct client_io_info *)eio->arg;
 			if (client->out_eof == true)
 				continue;
+
+			/* Some clients only take certain I/O streams */
+			if (out->type==SLURM_IO_STDOUT) {
+				if (client->ltaskid_stdout != -1 && 
+				    client->ltaskid_stdout != out->ltaskid)
+					continue;
+			}
+			if (out->type==SLURM_IO_STDERR) {
+				if (client->ltaskid_stderr != -1 && 
+				    client->ltaskid_stderr != out->ltaskid)
+					continue;
+			}
+
 			debug5("======================== Enqueued message");
 			xassert(client->magic == CLIENT_IO_MAGIC);
 			if (list_enqueue(client->msg_queue, msg))
@@ -1143,6 +1347,8 @@ _free_all_outgoing_msgs(List msg_queue, slurmd_job_t *job)
 	struct io_buf *msg;
 
 	msgs = list_iterator_create(msg_queue);
+	if (!msgs)
+		fatal("Could not allocate iterator");
 	while((msg = list_next(msgs))) {
 		_free_outgoing_msg(msg, job);
 	}
@@ -1180,7 +1386,7 @@ io_close_all(slurmd_job_t *job)
 	 *  and log facility may still try to write to stderr.
 	 */
 	if ((devnull = open("/dev/null", O_RDWR)) < 0) {
-		error("Unable to open /dev/null: %m");
+		error("Could not open /dev/null: %m");
 	} else {
 		if (dup2(devnull, STDERR_FILENO) < 0)
 			error("Unable to dup /dev/null onto stderr\n");
@@ -1192,11 +1398,41 @@ io_close_all(slurmd_job_t *job)
 	eio_signal_shutdown(job->eio);
 }
 
+void 
+io_close_local_fds(slurmd_job_t *job)
+{
+	ListIterator clients;
+	eio_obj_t *eio;
+	int rc;
+	struct client_io_info *client;
+
+	if (job == NULL || job->clients == NULL)
+		return;
+
+	clients = list_iterator_create(job->clients);
+	if (!clients)
+		fatal("Could not allocate iterator");
+	while((eio = list_next(clients))) {
+		client = (struct client_io_info *)eio->arg;
+		if (client->is_local_file) {
+			if (eio->fd >= 0) {
+				do {
+					rc = close(eio->fd);
+				} while (rc == -1 && errno == EINTR);
+				eio->fd = -1;
+			}
+		}
+	}
+}
+
+
+
 static void *
 _io_thr(void *arg)
 {
 	slurmd_job_t *job = (slurmd_job_t *) arg;
 	sigset_t set;
+	int rc;
 
 	/* A SIGHUP signal signals a reattach to the mgr thread.  We need
 	 * to block SIGHUP from being delivered to this thread so the mgr
@@ -1208,11 +1444,60 @@ _io_thr(void *arg)
 	pthread_sigmask(SIG_BLOCK, &set, NULL);
 
 	debug("IO handler started pid=%lu", (unsigned long) getpid());
-	eio_handle_mainloop(job->eio);
-	debug("IO handler exited");
+	rc = eio_handle_mainloop(job->eio);
+	debug("IO handler exited, rc=%d", rc);
 	return (void *)1;
 }
 
+/*
+ *  Add a client to the job's client list that will write stdout and/or
+ *  stderr from the slurmstepd.  The slurmstepd handles the write when
+ *  a file is created per node or per task, and the output needs to be
+ *  modified in some way, like labelling lines with the task number.
+ */
+int
+io_create_local_client(const char *filename, int file_flags, 
+		       slurmd_job_t *job, bool labelio,
+		       int stdout_tasks, int stderr_tasks)
+{
+	int fd = -1;
+	struct client_io_info *client;
+	eio_obj_t *obj;
+	int tmp;
+
+	fd = open(filename, file_flags, 0666);
+	if (fd == -1) {
+		return ESLURMD_IO_ERROR;
+	}
+	fd_set_close_on_exec(fd);
+
+	/* Now set up the eio object */
+	client = xmalloc(sizeof(struct client_io_info));
+#ifndef NDEBUG
+	client->magic = CLIENT_IO_MAGIC;
+#endif
+	client->job = job;
+	client->msg_queue = list_create(NULL); /* FIXME - destructor */
+
+	client->ltaskid_stdout = stdout_tasks;
+	client->ltaskid_stderr = stderr_tasks;
+	client->labelio = labelio;
+	client->is_local_file = true;
+
+	client->label_width = 1;
+	tmp = job->ntasks-1;
+	while ((tmp /= 10) > 0)
+		client->label_width++;
+
+
+	obj = eio_obj_create(fd, &local_file_ops, (void *)client);
+	list_append(job->clients, (void *)obj);
+	eio_new_initial_obj(job->eio, (void *)obj);
+	debug5("Now handling %d IO Client object(s)", list_count(job->clients));
+
+	return SLURM_SUCCESS;
+}
+
 /* 
  * Create the initial TCP connection back to a waiting client (e.g. srun).
  *
@@ -1224,7 +1509,8 @@ _io_thr(void *arg)
  * an IO stream.
  */
 int
-io_initial_client_connect(srun_info_t *srun, slurmd_job_t *job)
+io_initial_client_connect(srun_info_t *srun, slurmd_job_t *job, 
+			  int stdout_tasks, int stderr_tasks)
 {
 	int sock = -1;
 	struct client_io_info *client;
@@ -1267,6 +1553,12 @@ io_initial_client_connect(srun_info_t *srun, slurmd_job_t *job)
 	client->job = job;
 	client->msg_queue = list_create(NULL); /* FIXME - destructor */
 
+	client->ltaskid_stdout = stdout_tasks;
+	client->ltaskid_stderr = stderr_tasks;
+	client->labelio = false;
+	client->label_width = 0;
+	client->is_local_file = false;
+
 	obj = eio_obj_create(sock, &client_ops, (void *)client);
 	list_append(job->clients, (void *)obj);
 	eio_new_initial_obj(job->eio, (void *)obj);
@@ -1320,6 +1612,13 @@ io_client_connect(srun_info_t *srun, slurmd_job_t *job)
 #endif
 	client->job = job;
 	client->msg_queue = NULL; /* initialized in _client_writable */
+
+	client->ltaskid_stdout = -1;     /* accept from all tasks */
+	client->ltaskid_stderr = -1;     /* accept from all tasks */
+	client->labelio = false;
+	client->label_width = 0;
+	client->is_local_file = false;
+
 	/* client object adds itself to job->clients in _client_writable */
 
 	obj = eio_obj_create(sock, &client_ops, (void *)client);
@@ -1416,16 +1715,34 @@ _send_eof_msg(struct task_read_info *out)
 	header.length = 0; /* eof */
 
 	packbuf = create_buf(msg->data, io_hdr_packed_size());
+	if (!packbuf)
+		fatal("Failure to allocate memory for a message header");
+
 	io_hdr_pack(&header, packbuf);
 	msg->length = io_hdr_packed_size() + header.length;
 	msg->ref_count = 0; /* make certain it is initialized */
 
 	/* Add eof message to the msg_queue of all clients */
 	clients = list_iterator_create(out->job->clients);
+	if (!clients)
+		fatal("Could not allocate iterator");
 	while((eio = list_next(clients))) {
 		client = (struct client_io_info *)eio->arg;
 		debug5("======================== Enqueued eof message");
 		xassert(client->magic == CLIENT_IO_MAGIC);
+
+		/* Some clients only take certain I/O streams */
+		if (out->type==SLURM_IO_STDOUT) {
+			if (client->ltaskid_stdout != -1 && 
+			    client->ltaskid_stdout != out->ltaskid)
+				continue;
+		}
+		if (out->type==SLURM_IO_STDERR) {
+			if (client->ltaskid_stderr != -1 && 
+			    client->ltaskid_stderr != out->ltaskid)
+				continue;
+		}
+
 		if (list_enqueue(client->msg_queue, msg))
 			msg->ref_count++;
 	}
@@ -1435,6 +1752,7 @@ _send_eof_msg(struct task_read_info *out)
 }
 
 
+
 static struct io_buf *
 _task_build_message(struct task_read_info *out, slurmd_job_t *job, cbuf_t cbuf)
 {
@@ -1489,6 +1807,8 @@ _task_build_message(struct task_read_info *out, slurmd_job_t *job, cbuf_t cbuf)
 
 	debug5("  header.length = %d", n);
 	packbuf = create_buf(msg->data, io_hdr_packed_size());
+	if (!packbuf)
+		fatal("Failure to allocate memory for a message header");
 	io_hdr_pack(&header, packbuf);
 	msg->length = io_hdr_packed_size() + header.length;
 	msg->ref_count = 0; /* make certain it is initialized */
@@ -1532,6 +1852,7 @@ free_io_buf(struct io_buf *buf)
 	}
 }
 
+/* This just determines if there's space to hold more of the stdin stream */
 static bool
 _incoming_buf_free(slurmd_job_t *job)
 {
@@ -1624,3 +1945,141 @@ user_managed_io_client_connect(int ntasks, srun_info_t *srun,
 	return SLURM_SUCCESS;
 }
 
+
+void
+io_find_filename_pattern( slurmd_job_t *job, 
+			  slurmd_filename_pattern_t *outpattern, 
+			  slurmd_filename_pattern_t *errpattern,
+			  bool *same_out_err_files )
+{
+	int ii, jj;
+	int of_num_null = 0, ef_num_null = 0;
+	int of_num_devnull = 0, ef_num_devnull = 0;
+	int of_lastnull = -1, ef_lastnull = -1;
+	bool of_all_same = true, ef_all_same = true;
+	bool of_all_unique = true, ef_all_unique = true;
+
+	*outpattern = SLURMD_UNKNOWN;
+	*errpattern = SLURMD_UNKNOWN;
+	*same_out_err_files = false;
+
+	for (ii = 0; ii < job->ntasks; ii++) {
+		if (job->task[ii]->ofname == NULL) {
+			of_num_null++;
+			of_lastnull = ii;
+		} else if (strcmp(job->task[ii]->ofname, "/dev/null")==0) {
+			of_num_devnull++;
+		}
+
+		if (job->task[ii]->efname == NULL) {
+			ef_num_null++;
+			ef_lastnull = ii;
+		} else if (strcmp(job->task[ii]->efname, "/dev/null")==0) {
+			ef_num_devnull++;
+		}
+	}
+	if (of_num_null == job->ntasks)
+		*outpattern = SLURMD_ALL_NULL;
+
+	if (ef_num_null == job->ntasks)
+		*errpattern = SLURMD_ALL_NULL;
+
+	if (of_num_null == 1 && of_num_devnull == job->ntasks-1)
+		*outpattern = SLURMD_ONE_NULL;
+
+	if (ef_num_null == 1 && ef_num_devnull == job->ntasks-1)
+		*errpattern = SLURMD_ONE_NULL;
+
+	if (*outpattern == SLURMD_ALL_NULL && *errpattern == SLURMD_ALL_NULL)
+		*same_out_err_files = true;
+
+	if (*outpattern == SLURMD_ONE_NULL && *errpattern == SLURMD_ONE_NULL &&
+	    of_lastnull == ef_lastnull)
+		*same_out_err_files = true;
+
+	if (*outpattern != SLURMD_UNKNOWN && *errpattern != SLURMD_UNKNOWN)
+		return;
+
+	for (ii = 1; ii < job->ntasks; ii++) {
+		if (!job->task[ii]->ofname || !job->task[0]->ofname ||
+		    strcmp(job->task[ii]->ofname, job->task[0]->ofname) != 0)
+			of_all_same = false;
+
+		if (!job->task[ii]->efname || !job->task[0]->efname ||
+		    strcmp(job->task[ii]->efname, job->task[0]->efname) != 0)
+			ef_all_same = false;
+	}
+
+	if (of_all_same && *outpattern == SLURMD_UNKNOWN)
+		*outpattern = SLURMD_ALL_SAME;
+
+	if (ef_all_same && *errpattern == SLURMD_UNKNOWN)
+		*errpattern = SLURMD_ALL_SAME;
+
+	if (job->task[0]->ofname && job->task[0]->efname &&
+	    strcmp(job->task[0]->ofname, job->task[0]->efname)==0)
+		*same_out_err_files = true;
+
+	if (*outpattern != SLURMD_UNKNOWN && *errpattern != SLURMD_UNKNOWN)
+		return;
+
+	for (ii = 0; ii < job->ntasks-1; ii++) {
+		for (jj = ii+1; jj < job->ntasks; jj++) {
+
+			if (!job->task[ii]->ofname || !job->task[jj]->ofname ||
+			    strcmp(job->task[ii]->ofname, 
+				   job->task[jj]->ofname) == 0)
+				of_all_unique = false;
+
+			if (!job->task[ii]->efname || !job->task[jj]->efname ||
+			    strcmp(job->task[ii]->efname, 
+				   job->task[jj]->efname) == 0)
+				ef_all_unique = false;
+		}
+	}
+
+	if (of_all_unique)
+		*outpattern = SLURMD_ALL_UNIQUE;
+
+	if (ef_all_unique)
+		*errpattern = SLURMD_ALL_UNIQUE;
+
+	if (of_all_unique && ef_all_unique) {
+		*same_out_err_files = true;
+		for (ii = 0; ii < job->ntasks; ii++) {
+			if (job->task[ii]->ofname && 
+			    job->task[ii]->efname &&
+			    strcmp(job->task[ii]->ofname,
+				   job->task[ii]->efname) != 0) {
+				*same_out_err_files = false;
+				break;
+			}
+		}
+	}
+}
+
+
+int
+io_get_file_flags(slurmd_job_t *job)
+{
+	slurm_ctl_conf_t *conf;
+	int file_flags;
+
+	/* set files for opening stdout/err */
+	if (job->open_mode == OPEN_MODE_APPEND)
+		file_flags = O_CREAT|O_WRONLY|O_APPEND;
+	else if (job->open_mode == OPEN_MODE_TRUNCATE)
+		file_flags = O_CREAT|O_WRONLY|O_APPEND|O_TRUNC;
+	else {
+		conf = slurm_conf_lock();
+		if (conf->job_file_append)
+			file_flags = O_CREAT|O_WRONLY|O_APPEND;
+		else
+			file_flags = O_CREAT|O_WRONLY|O_APPEND|O_TRUNC;
+		slurm_conf_unlock();
+	}
+	return file_flags;
+}
+
+
+
