@@ -1,37 +1,38 @@
 /*****************************************************************************\
  *  src/slurmd/slurmstepd/mgr.c - job manager functions for slurmstepd
- *  $Id: mgr.c 17040 2009-03-26 15:03:18Z jette $
+ *  $Id: mgr.c 19152 2009-12-10 22:29:52Z da $
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -76,22 +77,23 @@
 
 #include <slurm/slurm_errno.h>
 
+#include "src/common/basil_resv_conf.h"
 #include "src/common/cbuf.h"
 #include "src/common/env.h"
+#include "src/common/fd.h"
+#include "src/common/forward.h"
 #include "src/common/hostlist.h"
 #include "src/common/log.h"
+#include "src/common/mpi.h"
 #include "src/common/node_select.h"
-#include "src/common/fd.h"
+#include "src/common/plugstack.h"
 #include "src/common/safeopen.h"
 #include "src/common/slurm_jobacct_gather.h"
 #include "src/common/switch.h"
+#include "src/common/util-net.h"
+#include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
-#include "src/common/xmalloc.h"
-#include "src/common/util-net.h"
-#include "src/common/forward.h"
-#include "src/common/plugstack.h"
-#include "src/common/mpi.h"
 
 #include "src/slurmd/slurmd/slurmd.h"
 
@@ -100,6 +102,7 @@
 #include "src/slurmd/common/task_plugin.h"
 #include "src/slurmd/common/run_script.h"
 #include "src/slurmd/common/reverse_tree.h"
+#include "src/slurmd/common/set_oomadj.h"
 
 #include "src/slurmd/slurmstepd/slurmstepd.h"
 #include "src/slurmd/slurmstepd/mgr.h"
@@ -124,11 +127,11 @@ static int mgr_sigarray[] = {
 };
 
 struct priv_state {
-	uid_t           saved_uid;
-	gid_t           saved_gid;
-	gid_t *         gid_list;
-	int             ngids;
-	char            saved_cwd [4096];
+	uid_t	saved_uid;
+	gid_t	saved_gid;
+	gid_t *	gid_list;
+	int	ngids;
+	char	saved_cwd [4096];
 };
 
 step_complete_t step_complete = {
@@ -143,7 +146,7 @@ step_complete_t step_complete = {
 	true,
 	(bitstr_t *)NULL,
 	0,
-        NULL
+	NULL
 };
 
 typedef struct kill_thread {
@@ -152,29 +155,29 @@ typedef struct kill_thread {
 } kill_thread_t;
 
 
-/* 
+/*
  * Prototypes
  */
 
-/* 
+/*
  * Job manager related prototypes
  */
 static int  _access(const char *path, int modes, uid_t uid, gid_t gid);
-static void _send_launch_failure(launch_tasks_request_msg_t *, 
-                                 slurm_addr *, int);
+static void _send_launch_failure(launch_tasks_request_msg_t *,
+				 slurm_addr *, int);
 static int  _fork_all_tasks(slurmd_job_t *job);
 static int  _become_user(slurmd_job_t *job, struct priv_state *ps);
 static void  _set_prio_process (slurmd_job_t *job);
 static void _set_job_log_prefix(slurmd_job_t *job);
 static int  _setup_normal_io(slurmd_job_t *job);
 static int  _drop_privileges(slurmd_job_t *job, bool do_setuid,
-				struct priv_state *state);
+			     struct priv_state *state);
 static int  _reclaim_privileges(struct priv_state *state);
 static void _send_launch_resp(slurmd_job_t *job, int rc);
-static void _slurmd_job_log_init(slurmd_job_t *job);
+static int  _slurmd_job_log_init(slurmd_job_t *job);
 static void _wait_for_io(slurmd_job_t *job);
-static int  _send_exit_msg(slurmd_job_t *job, uint32_t *tid, int n, 
-		int status);
+static int  _send_exit_msg(slurmd_job_t *job, uint32_t *tid, int n,
+			   int status);
 static void _wait_for_children_slurmstepd(slurmd_job_t *job);
 static int  _send_pending_exit_msgs(slurmd_job_t *job);
 static void _send_step_complete_msgs(slurmd_job_t *job);
@@ -192,7 +195,7 @@ static int  _run_script_as_user(const char *name, const char *path,
  */
 static char * _make_batch_dir(slurmd_job_t *job);
 static char * _make_batch_script(batch_job_launch_msg_t *msg, char *path);
-static int    _send_complete_batch_script_msg(slurmd_job_t *job, 
+static int    _send_complete_batch_script_msg(slurmd_job_t *job,
 					      int err, int status);
 
 /*
@@ -221,10 +224,10 @@ mgr_launch_tasks_setup(launch_tasks_request_msg_t *msg, slurm_addr *cli,
 	_set_job_log_prefix(job);
 
 	_setargs(job);
-	
+
 	job->envtp->cli = cli;
 	job->envtp->self = self;
-	
+
 	return job;
 }
 
@@ -237,7 +240,7 @@ batch_finish(slurmd_job_t *job, int rc)
 	int i;
 	for (i = 0; i < job->ntasks; i++) {
 		/* If signalled we only need to check one and then
-		 * break out of the loop */ 
+		 * break out of the loop */
 		if(WIFSIGNALED(job->task[i]->estatus)) {
 			switch(WTERMSIG(job->task[i]->estatus)) {
 			case SIGTERM:
@@ -263,7 +266,7 @@ batch_finish(slurmd_job_t *job, int rc)
 	if (job->aborted) {
 		if ((job->stepid == NO_VAL) ||
 		    (job->stepid == SLURM_BATCH_SCRIPT)) {
-			info("step %u.%u abort completed", 
+			info("step %u.%u abort completed",
 			     job->jobid, job->stepid);
 		} else
 			info("job %u abort completed", job->jobid);
@@ -287,7 +290,7 @@ slurmd_job_t *
 mgr_launch_batch_job_setup(batch_job_launch_msg_t *msg, slurm_addr *cli)
 {
 	slurmd_job_t *job = NULL;
-	
+
 	if (!(job = job_batch_job_create(msg))) {
 		error("job_batch_job_create() failed: %m");
 		return NULL;
@@ -306,7 +309,7 @@ mgr_launch_batch_job_setup(batch_job_launch_msg_t *msg, slurm_addr *cli)
 	if ((job->argv[0] = _make_batch_script(msg, job->batchdir)) == NULL) {
 		goto cleanup2;
 	}
-	
+
 	/* this is the new way of setting environment variables */
 	env_array_for_batch_job(&job->env, msg, conf->node_name);
 
@@ -314,7 +317,7 @@ mgr_launch_batch_job_setup(batch_job_launch_msg_t *msg, slurm_addr *cli)
 	 * needed) */
 	job->envtp->overcommit = msg->overcommit;
 	job->envtp->select_jobinfo = msg->select_jobinfo;
-	
+
 	return job;
 
 cleanup2:
@@ -329,8 +332,8 @@ cleanup1:
 	if (job->aborted)
 		verbose("job %u abort complete", job->jobid);
 	else if (msg->step_id == SLURM_BATCH_SCRIPT) {
-		_send_complete_batch_script_msg(job, 
-			ESLURMD_CREATE_BATCH_DIR_ERROR, -1);
+		_send_complete_batch_script_msg(
+			job, ESLURMD_CREATE_BATCH_DIR_ERROR, -1);
 	} else
 		_send_step_complete_msgs(job);
 
@@ -342,10 +345,10 @@ _set_job_log_prefix(slurmd_job_t *job)
 {
 	char buf[256];
 
-	if (job->jobid > MAX_NOALLOC_JOBID) 
+	if (job->jobid > MAX_NOALLOC_JOBID)
 		return;
 
-	if ((job->jobid >= MIN_NOALLOC_JOBID) || (job->stepid == NO_VAL)) 
+	if ((job->jobid >= MIN_NOALLOC_JOBID) || (job->stepid == NO_VAL))
 		snprintf(buf, sizeof(buf), "[%u]", job->jobid);
 	else
 		snprintf(buf, sizeof(buf), "[%u.%u]", job->jobid, job->stepid);
@@ -356,7 +359,7 @@ _set_job_log_prefix(slurmd_job_t *job)
 static int
 _setup_normal_io(slurmd_job_t *job)
 {
-	int            rc   = 0;
+	int rc = 0, ii = 0;
 	struct priv_state sprivs;
 
 	debug2("Entering _setup_normal_io");
@@ -369,12 +372,10 @@ _setup_normal_io(slurmd_job_t *job)
 	if (_drop_privileges(job, true, &sprivs) < 0)
 		return ESLURMD_SET_UID_OR_GID_ERROR;
 
-	/* FIXME - need to check a return code for failures */
-	io_init_tasks_stdio(job);
-
-	if (_reclaim_privileges(&sprivs) < 0)
-		error("sete{u/g}id(%lu/%lu): %m", 
-		      (u_long) sprivs.saved_uid, (u_long) sprivs.saved_gid);
+	if (io_init_tasks_stdio(job) != SLURM_SUCCESS) {
+		rc = ESLURMD_IO_ERROR;
+		goto claim;
+	}
 
 	/*
 	 * MUST create the initial client object before starting
@@ -382,17 +383,126 @@ _setup_normal_io(slurmd_job_t *job)
 	 */
 	if (!job->batch) {
 		srun_info_t *srun = list_peek(job->sruns);
+
+		/* local id of task that sends to srun, -1 for all tasks,
+		   any other value for no tasks */
+		int srun_stdout_tasks = -1;
+		int srun_stderr_tasks = -1;
+
 		xassert(srun != NULL);
-		rc = io_initial_client_connect(srun, job);
-		if (rc < 0) 
-			return ESLURMD_IO_ERROR;
+
+		/* If I/O is labelled with task num, and if a separate file is
+		   written per node or per task, the I/O needs to be sent
+		   back to the stepd, get a label appended, and written from
+		   the stepd rather than sent back to srun or written directly
+		   from the node.  When a task has ofname or efname == NULL, it
+		   means data gets sent back to the client. */
+
+		if (job->labelio) {
+			slurmd_filename_pattern_t outpattern, errpattern;
+			bool same = false;
+			int file_flags;
+
+			io_find_filename_pattern(job, &outpattern, &errpattern,
+						 &same);
+			file_flags = io_get_file_flags(job);
+
+			/* Make eio objects to write from the slurmstepd */
+			if (outpattern == SLURMD_ALL_UNIQUE) {
+				/* Open a separate file per task */
+				for (ii = 0; ii < job->ntasks; ii++) {
+					rc = io_create_local_client(
+						job->task[ii]->ofname,
+						file_flags, job, job->labelio,
+						job->task[ii]->id,
+						same ? job->task[ii]->id : -2);
+					if (rc != SLURM_SUCCESS) {
+						error("Could not open output "
+						      "file %s: %m",
+						      job->task[ii]->ofname);
+						rc = ESLURMD_IO_ERROR;
+						goto claim;
+					}
+				}
+				srun_stdout_tasks = -2;
+				if (same)
+					srun_stderr_tasks = -2;
+			} else if (outpattern == SLURMD_ALL_SAME) {
+				/* Open a file for all tasks */
+				rc = io_create_local_client(
+					job->task[0]->ofname,
+					file_flags, job, job->labelio,
+					-1, same ? -1 : -2);
+				if (rc != SLURM_SUCCESS) {
+					error("Could not open output "
+					      "file %s: %m",
+					      job->task[0]->ofname);
+					rc = ESLURMD_IO_ERROR;
+					goto claim;
+				}
+				srun_stdout_tasks = -2;
+				if (same)
+					srun_stderr_tasks = -2;
+			}
+
+			if (!same) {
+				if (errpattern == SLURMD_ALL_UNIQUE) {
+					/* Open a separate file per task */
+					for (ii = 0; ii < job->ntasks; ii++) {
+						rc = io_create_local_client(
+							job->task[ii]->efname,
+							file_flags, job,
+							job->labelio,
+							-2, job->task[ii]->id);
+						if (rc != SLURM_SUCCESS) {
+							error("Could not "
+							      "open error "
+							      "file %s: %m",
+							      job->task[ii]->
+							      efname);
+							rc = ESLURMD_IO_ERROR;
+							goto claim;
+						}
+					}
+					srun_stderr_tasks = -2;
+				} else if (errpattern == SLURMD_ALL_SAME) {
+					/* Open a file for all tasks */
+					rc = io_create_local_client(
+						job->task[0]->efname,
+						file_flags, job, job->labelio,
+						-2, -1);
+					if (rc != SLURM_SUCCESS) {
+						error("Could not open error "
+						      "file %s: %m",
+						      job->task[0]->efname);
+						rc = ESLURMD_IO_ERROR;
+						goto claim;
+					}
+					srun_stderr_tasks = -2;
+				}
+			}
+		}
+
+		if(io_initial_client_connect(srun, job, srun_stdout_tasks,
+					     srun_stderr_tasks) < 0) {
+			rc = ESLURMD_IO_ERROR;
+			goto claim;
+		}
 	}
 
-	if (!job->batch)
+claim:
+	if (_reclaim_privileges(&sprivs) < 0) {
+		error("sete{u/g}id(%lu/%lu): %m",
+		      (u_long) sprivs.saved_uid, (u_long) sprivs.saved_gid);
+	}
+
+	if (!rc && !job->batch) {
 		if (io_thread_start(job) < 0)
-			return ESLURMD_IO_ERROR;
+			rc = ESLURMD_IO_ERROR;
+	}
+
 	debug2("Leaving  _setup_normal_io");
-	return SLURM_SUCCESS;
+	return rc;
 }
 
 static int
@@ -404,7 +514,7 @@ _setup_user_managed_io(slurmd_job_t *job)
 		error("_setup_user_managed_io: no clients!");
 		return SLURM_ERROR;
 	}
-	
+
 	return user_managed_io_client_connect(job->ntasks, srun, job->task);
 }
 
@@ -412,7 +522,7 @@ static void
 _random_sleep(slurmd_job_t *job)
 {
 	long int delay = 0;
-	long int max   = (3 * job->nnodes); 
+	long int max   = (3 * job->nnodes);
 
 	srand48((long int) (job->jobid + job->nodeid));
 
@@ -436,20 +546,20 @@ _send_exit_msg(slurmd_job_t *job, uint32_t *tid, int n, int status)
 
 	debug3("sending task exit msg for %d tasks", n);
 
-	msg.task_id_list = tid;
-	msg.num_tasks    = n;
-	msg.return_code  = status;
-	msg.job_id       = job->jobid;
-	msg.step_id      = job->stepid;
+	msg.task_id_list	= tid;
+	msg.num_tasks		= n;
+	msg.return_code		= status;
+	msg.job_id		= job->jobid;
+	msg.step_id		= job->stepid;
 	slurm_msg_t_init(&resp);
-	resp.data        = &msg;
-	resp.msg_type    = MESSAGE_TASK_EXIT;
-	
+	resp.data		= &msg;
+	resp.msg_type		= MESSAGE_TASK_EXIT;
+
 	/*
 	 *  XXX Hack for TCP timeouts on exit of large, synchronized
 	 *  jobs. Delay a random amount if job->nnodes > 100
 	 */
-	if (job->nnodes > 100) 
+	if (job->nnodes > 100)
 		_random_sleep(job);
 
 	/*
@@ -505,7 +615,7 @@ _wait_for_children_slurmstepd(slurmd_job_t *job)
 	/* Find the maximum task return code */
 	for (i = 0; i < job->ntasks; i++) {
 		/* If signalled we only need to check one and then
-		   break out of the loop */ 
+		   break out of the loop */
 		if(WIFSIGNALED(job->task[i]->estatus)) {
 			switch(WTERMSIG(job->task[i]->estatus)) {
 			case SIGTERM:
@@ -544,6 +654,8 @@ _one_step_complete_msg(slurmd_job_t *job, int first, int last)
 	static bool acct_sent = false;
 
 	debug2("_one_step_complete_msg: first=%d, last=%d", first, last);
+
+	memset(&msg, 0, sizeof(step_complete_msg_t));
 	msg.job_id = job->jobid;
 	msg.job_step_id = job->stepid;
 	msg.range_first = first;
@@ -553,11 +665,11 @@ _one_step_complete_msg(slurmd_job_t *job, int first, int last)
 	/************* acct stuff ********************/
 	if(!acct_sent) {
 		jobacct_gather_g_aggregate(step_complete.jobacct, job->jobacct);
-		jobacct_gather_g_getinfo(step_complete.jobacct, 
+		jobacct_gather_g_getinfo(step_complete.jobacct,
 					 JOBACCT_DATA_TOTAL, msg.jobacct);
 		acct_sent = true;
 	}
-	/*********************************************/	
+	/*********************************************/
 	slurm_msg_t_init(&req);
 	req.msg_type = REQUEST_STEP_COMPLETE;
 	req.data = &msg;
@@ -684,7 +796,16 @@ _send_step_complete_msgs(slurmd_job_t *job)
 	pthread_mutex_unlock(&step_complete.lock);
 }
 
-/* 
+/* This dummy function is provided so that the checkpoint functions can
+ * 	resolve this symbol name (as needed for some of the checkpoint
+ *	functions used by slurmctld). */
+extern void agent_queue_request(void *dummy)
+{
+	fatal("Invalid agent_queue_request function call, likely from "
+	      "checkpoint plugin");
+}
+
+/*
  * Executes the functions of the slurmd job manager process,
  * which runs as root and performs shared memory and interconnect
  * initialization, etc.
@@ -692,21 +813,23 @@ _send_step_complete_msgs(slurmd_job_t *job)
  * Returns 0 if job ran and completed successfully.
  * Returns errno if job startup failed. NOTE: This will DRAIN the node.
  */
-int 
+int
 job_manager(slurmd_job_t *job)
 {
 	int  rc = 0;
 	bool io_initialized = false;
+	char *ckpt_type = slurm_get_checkpoint_type();
 
 	debug3("Entered job_manager for %u.%u pid=%lu",
 	       job->jobid, job->stepid, (unsigned long) job->jmgr_pid);
 	/*
 	 * Preload plugins.
 	 */
-	if (switch_init() != SLURM_SUCCESS
-	    || slurmd_task_init() != SLURM_SUCCESS
-	    || slurm_proctrack_init() != SLURM_SUCCESS
-	    || slurm_jobacct_gather_init() != SLURM_SUCCESS) {
+	if ((switch_init() != SLURM_SUCCESS)			||
+	    (slurmd_task_init() != SLURM_SUCCESS)		||
+	    (slurm_proctrack_init() != SLURM_SUCCESS)		||
+	    (checkpoint_init(ckpt_type) != SLURM_SUCCESS)	||
+	    (slurm_jobacct_gather_init() != SLURM_SUCCESS)) {
 		rc = SLURM_PLUGIN_NAME_INVALID;
 		goto fail1;
 	}
@@ -714,13 +837,18 @@ job_manager(slurmd_job_t *job)
 		rc = SLURM_MPI_PLUGIN_NAME_INVALID;
 		goto fail1;
 	}
-	
+
 	if (!job->batch &&
 	    (interconnect_preinit(job->switch_job) < 0)) {
 		rc = ESLURM_INTERCONNECT_FAILURE;
 		goto fail1;
 	}
-	
+
+#ifdef PR_SET_DUMPABLE
+	if (prctl(PR_SET_DUMPABLE, 1) < 0)
+		debug ("Unable to set dumpable to 1");
+#endif /* PR_SET_DUMPABLE */
+
 	set_umask(job);		/* set umask for stdout/err files */
 	if (job->user_managed_io)
 		rc = _setup_user_managed_io(job);
@@ -729,24 +857,19 @@ job_manager(slurmd_job_t *job)
 	/*
 	 * Initialize log facility to copy errors back to srun
 	 */
-	_slurmd_job_log_init(job);
-	
-#ifndef NDEBUG
-#  ifdef PR_SET_DUMPABLE
-	if (prctl(PR_SET_DUMPABLE, 1) < 0)
-		debug ("Unable to set dumpable to 1");
-#  endif /* PR_SET_DUMPABLE */
-#endif   /* !NDEBUG         */
+	if(!rc)
+		rc = _slurmd_job_log_init(job);
 
 	if (rc) {
 		error("IO setup failed: %m");
+		rc = SLURM_SUCCESS;	/* drains node otherwise */
 		goto fail2;
 	} else {
 		io_initialized = true;
 	}
-		
+
 	/* Call interconnect_init() before becoming user */
-	if (!job->batch && 
+	if (!job->batch &&
 	    (interconnect_init(job->switch_job, job->uid) < 0)) {
 		/* error("interconnect_init: %m"); already logged */
 		rc = ESLURM_INTERCONNECT_FAILURE;
@@ -754,6 +877,14 @@ job_manager(slurmd_job_t *job)
 		goto fail2;
 	}
 
+	/* fork necessary threads for checkpoint */
+	if (checkpoint_stepd_prefork(job) != SLURM_SUCCESS) {
+		error("Failed checkpoint_stepd_prefork");
+		rc = SLURM_FAILURE;
+		io_close_task_fds(job);
+		goto fail2;
+	}
+
 	/* calls pam_setup() and requires pam_finish() if successful */
 	if (_fork_all_tasks(job) < 0) {
 		debug("_fork_all_tasks failed");
@@ -768,7 +899,7 @@ job_manager(slurmd_job_t *job)
 	reattach_job = job;
 
 	job->state = SLURMSTEPD_STEP_RUNNING;
-	
+
 	/* if we are not polling then we need to make sure we get some
 	 * information here
 	 */
@@ -779,16 +910,16 @@ job_manager(slurmd_job_t *job)
 
 	_wait_for_all_tasks(job);
 	jobacct_gather_g_endpoll();
-		
+
 	job->state = SLURMSTEPD_STEP_ENDING;
 
-	/* 
+	/*
 	 * This just cleans up all of the PAM state and errors are logged
 	 * below, so there's no need for error handling.
 	 */
 	pam_finish();
 
-	if (!job->batch && 
+	if (!job->batch &&
 	    (interconnect_fini(job->switch_job) < 0)) {
 		error("interconnect_fini: %m");
 	}
@@ -800,8 +931,8 @@ job_manager(slurmd_job_t *job)
 	 * is moved behind wait_for_io(), we may block waiting for IO
 	 * on a hung process.
 	 *
-	 * Make sure all processes in session are dead. On systems 
-	 * with an IBM Federation switch, all processes must be 
+	 * Make sure all processes in session are dead. On systems
+	 * with an IBM Federation switch, all processes must be
 	 * terminated before the switch window can be released by
 	 * interconnect_postfini().
 	 */
@@ -820,10 +951,8 @@ job_manager(slurmd_job_t *job)
 	/*
 	 * Wait for io thread to complete (if there is one)
 	 */
-	if (!job->batch && !job->user_managed_io && io_initialized) {
-		eio_signal_shutdown(job->eio);
+	if (!job->batch && !job->user_managed_io && io_initialized)
 		_wait_for_io(job);
-	}
 
 	debug2("Before call to spank_fini()");
 	if (spank_fini (job)  < 0) {
@@ -832,7 +961,7 @@ job_manager(slurmd_job_t *job)
 	debug2("After call to spank_fini()");
 
     fail1:
-	/* If interactive job startup was abnormal, 
+	/* If interactive job startup was abnormal,
 	 * be sure to notify client.
 	 */
 	if (rc != 0) {
@@ -847,12 +976,25 @@ job_manager(slurmd_job_t *job)
 		_send_step_complete_msgs(job);
 	}
 
+	xfree(ckpt_type);
 	return(rc);
 }
 
+static int
+_spank_task_privileged(slurmd_job_t *job, int taskid, struct priv_state *sp)
+{
+	if (_reclaim_privileges(sp) < 0)
+		return SLURM_ERROR;
+
+	if (spank_task_privileged (job, taskid) < 0)
+		return error("spank_task_init_privileged failed");
+
+	return(_drop_privileges (job, true, sp));
+}
+
 
 /* fork and exec N tasks
- */ 
+ */
 static int
 _fork_all_tasks(slurmd_job_t *job)
 {
@@ -863,6 +1005,7 @@ _fork_all_tasks(slurmd_job_t *job)
 	int fdpair[2];
 	struct priv_state sprivs;
 	jobacct_id_t jobacct_id;
+	char *oom_value;
 
 	xassert(job != NULL);
 
@@ -871,6 +1014,13 @@ _fork_all_tasks(slurmd_job_t *job)
 		return SLURM_ERROR;
 	}
 
+#ifdef HAVE_CRAY_XT
+	if (basil_resv_conf(job->resv_id, job->jobid)) {
+		error("could not confirm reservation");
+		return SLURM_ERROR;
+	}
+#endif
+
 	debug2("Before call to spank_init()");
 	if (spank_init (job) < 0) {
 		error ("Plugin stack initialization failed.\n");
@@ -900,7 +1050,7 @@ _fork_all_tasks(slurmd_job_t *job)
 			error ("exec_all_tasks: pipe: %m");
 			return SLURM_ERROR;
 		}
-		debug3("New fdpair[0] = %d, fdpair[1] = %d", 
+		debug3("New fdpair[0] = %d, fdpair[1] = %d",
 		       fdpair[0], fdpair[1]);
 		fd_set_close_on_exec(fdpair[0]);
 		fd_set_close_on_exec(fdpair[1]);
@@ -908,6 +1058,12 @@ _fork_all_tasks(slurmd_job_t *job)
 		writefds[i] = fdpair[1];
 	}
 
+	set_oom_adj(0);	/* the tasks may be killed by OOM */
+	if (pre_setuid(job)) {
+		error("Failed task affinity setup");
+		return SLURM_ERROR;
+	}
+
 	/* Temporarily drop effective privileges, except for the euid.
 	 * We need to wait until after pam_setup() to drop euid.
 	 */
@@ -959,13 +1115,19 @@ _fork_all_tasks(slurmd_job_t *job)
 				if (j > i)
 					close(readfds[j]);
 			}
-			/* jobacct_gather_g_endpoll();	
+			/* jobacct_gather_g_endpoll();
 			 * closing jobacct files here causes deadlock */
 
 			if (conf->propagate_prio == 1)
 				_set_prio_process(job);
 
-			(void) pre_setuid(job);
+			/*
+			 *  Reclaim privileges and call any plugin hooks
+			 *   that may require elevated privs
+			 */
+			if (_spank_task_privileged(job, i, &sprivs) < 0)
+				exit(1);
+
  			if (_become_user(job, &sprivs) < 0) {
  				error("_become_user failed: %m");
 				/* child process, should not return */
@@ -980,13 +1142,13 @@ _fork_all_tasks(slurmd_job_t *job)
 		}
 
 		/*
-		 * Parent continues: 
+		 * Parent continues:
 		 */
 
 		close(readfds[i]);
-		verbose ("task %lu (%lu) started %M", 
-			(unsigned long) job->task[i]->gtid, 
-			(unsigned long) pid); 
+		verbose ("task %lu (%lu) started %M",
+			(unsigned long) job->task[i]->gtid,
+			(unsigned long) pid);
 
 		job->task[i]->pid = pid;
 		if (i == 0)
@@ -1006,6 +1168,12 @@ _fork_all_tasks(slurmd_job_t *job)
 		/* Don't bother erroring out here */
 	}
 
+	if ((oom_value = getenv("SLURMSTEPD_OOM_ADJ"))) {
+		int i = atoi(oom_value);
+		debug("Setting slurmstepd oom_adj to %d", i);
+		set_oom_adj(i);
+	}
+
 	if (chdir (sprivs.saved_cwd) < 0) {
 		error ("Unable to return to working directory");
 	}
@@ -1024,14 +1192,14 @@ _fork_all_tasks(slurmd_job_t *job)
 				i, job->task[i]->pid, job->pgid);
 		}
 
-                if (slurm_container_add(job, job->task[i]->pid)
+		if (slurm_container_add(job, job->task[i]->pid)
 		    == SLURM_ERROR) {
-                        error("slurm_container_add: %m");
+			error("slurm_container_add: %m");
 			goto fail1;
-                }
+		}
 		jobacct_id.nodeid = job->nodeid;
 		jobacct_id.taskid = job->task[i]->gtid;
-		jobacct_gather_g_add_task(job->task[i]->pid, 
+		jobacct_gather_g_add_task(job->task[i]->pid,
 				   &jobacct_id);
 
 		if (spank_task_post_fork (job, i) < 0) {
@@ -1040,22 +1208,22 @@ _fork_all_tasks(slurmd_job_t *job)
 		}
 	}
 	jobacct_gather_g_set_proctrack_container_id(job->cont_id);
-	
+
 	/*
 	 * Now it's ok to unblock the tasks, so they may call exec.
 	 */
 	for (i = 0; i < job->ntasks; i++) {
 		char c = '\0';
-		
+
 		debug3("Unblocking %u.%u task %d, writefd = %d",
 		       job->jobid, job->stepid, i, writefds[i]);
 		if (write (writefds[i], &c, sizeof (c)) != 1)
-			error ("write to unblock task %d failed", i); 
+			error ("write to unblock task %d failed", i);
 
 		close(writefds[i]);
 
 		/*
-		 * Prepare process for attach by parallel debugger 
+		 * Prepare process for attach by parallel debugger
 		 * (if specified and able)
 		 */
 		if (pdebug_trace_process(job, job->task[i]->pid)
@@ -1083,8 +1251,8 @@ fail1:
  * the same exit status (and whose statuses have not been sent back to
  * the client) Aggregate these tasks into a single task exit message.
  *
- */ 
-static int 
+ */
+static int
 _send_pending_exit_msgs(slurmd_job_t *job)
 {
 	int  i;
@@ -1093,17 +1261,17 @@ _send_pending_exit_msgs(slurmd_job_t *job)
 	bool set    = false;
 	uint32_t  tid[job->ntasks];
 
-	/* 
-	 * Collect all exit codes with the same status into a 
-	 * single message. 
+	/*
+	 * Collect all exit codes with the same status into a
+	 * single message.
 	 */
 	for (i = 0; i < job->ntasks; i++) {
 		slurmd_task_info_t *t = job->task[i];
 
-		if (!t->exited || t->esent) 
+		if (!t->exited || t->esent)
 			continue;
-		
-		if (!set) { 
+
+		if (!set) {
 			status = t->estatus;
 			set    = true;
 		} else if (status != t->estatus)
@@ -1112,7 +1280,7 @@ _send_pending_exit_msgs(slurmd_job_t *job)
 		tid[nsent++] = t->gtid;
 		t->esent = true;
 	}
-	
+
 	if (nsent) {
 		debug2("Aggregated %d task exit messages", nsent);
 		_send_exit_msg(job, tid, nsent, status);
@@ -1121,6 +1289,35 @@ _send_pending_exit_msgs(slurmd_job_t *job)
 	return nsent;
 }
 
+static inline void
+_log_task_exit(unsigned long taskid, unsigned long pid, int status)
+{
+	/*
+	 *  Print a nice message to the log describing the task exit status.
+	 *
+	 *  The final else is there just in case there is ever an exit status
+	 *   that isn't WIFEXITED || WIFSIGNALED. We'll probably never reach
+	 *   that code, but it is better than dropping a potentially useful
+	 *   exit status.
+	 */
+	if (WIFEXITED(status))
+		verbose("task %lu (%lu) exited with exit code %d.",
+			taskid, pid, WEXITSTATUS(status));
+	else if (WIFSIGNALED(status))
+		/* WCOREDUMP isn't available on AIX */
+		verbose("task %lu (%lu) exited. Killed by signal %d%s.",
+			taskid, pid, WTERMSIG(status),
+#ifdef WCOREDUMP
+			WCOREDUMP(status) ? " (core dumped)" : ""
+#else
+			""
+#endif
+			);
+	else
+		verbose("task %lu (%lu) exited with status 0x%04x.",
+			taskid, pid, status);
+}
+
 /*
  * If waitflag is true, perform a blocking wait for a single process
  * and then return.
@@ -1128,14 +1325,13 @@ _send_pending_exit_msgs(slurmd_job_t *job)
  * If waitflag is false, do repeated non-blocking waits until
  * there are no more processes to reap (waitpid returns 0).
  *
- * Returns the number of tasks for which a wait3() was succesfully
+ * Returns the number of tasks for which a wait3() was successfully
  * performed, or -1 if there are no child tasks.
  */
 static int
 _wait_for_any_task(slurmd_job_t *job, bool waitflag)
 {
 	slurmd_task_info_t *t = NULL;
-	int i;
 	int status;
 	pid_t pid;
 	int completed = 0;
@@ -1163,33 +1359,26 @@ _wait_for_any_task(slurmd_job_t *job, bool waitflag)
 		/************* acct stuff ********************/
 		jobacct = jobacct_gather_g_remove_task(pid);
 		if(jobacct) {
-			jobacct_gather_g_setinfo(jobacct, 
+			jobacct_gather_g_setinfo(jobacct,
 						 JOBACCT_DATA_RUSAGE, &rusage);
 			jobacct_gather_g_aggregate(job->jobacct, jobacct);
 			jobacct_gather_g_destroy(jobacct);
-		} 		
-		/*********************************************/	
-	
-		/* See if the pid matches that of one of the tasks */
-		for (i = 0; i < job->ntasks; i++) {
-			if (job->task[i]->pid == pid) {
-				t = job->task[i];
-				completed++;
-				break;
-			}
 		}
-		if (t != NULL) {
-			verbose("task %lu (%lu) exited status 0x%04x %M",
-				(unsigned long)job->task[i]->gtid,
-				(unsigned long)pid, status);
+		/*********************************************/
+
+		if ((t = job_task_info_by_pid(job, pid))) {
+			completed++;
+			_log_task_exit(t->gtid, pid, status);
+
 			t->exited  = true;
 			t->estatus = status;
 			job->envtp->env = job->env;
-			job->envtp->procid = job->task[i]->gtid;
-			job->envtp->localid = job->task[i]->id;
-			
+			job->envtp->procid = t->gtid;
+			job->envtp->localid = t->id;
+
 			job->envtp->distribution = -1;
-			setup_env(job->envtp);
+			job->envtp->batch_flag = job->batch;
+			setup_env(job->envtp, false);
 			job->env = job->envtp->env;
 			if (job->task_epilog) {
 				_run_script_as_user("user task_epilog",
@@ -1206,10 +1395,11 @@ _wait_for_any_task(slurmd_job_t *job, bool waitflag)
 						    job, -1, job->env);
 				xfree(my_epilog);
 			}
-			job->envtp->procid = i;
+			job->envtp->procid = t->id;
 
-			if (spank_task_exit (job, i) < 0)
-				error ("Unable to spank task %d at exit", i);
+			if (spank_task_exit (job, t->id) < 0)
+				error ("Unable to spank task %d at exit",
+				       t->id);
 
 			post_term(job);
 		}
@@ -1241,7 +1431,7 @@ _wait_for_all_tasks(slurmd_job_t *job)
 		if (rc != -1) {
 			i += rc;
 			if (i < tasks_left) {
-				/* To limit the amount of traffic back 
+				/* To limit the amount of traffic back
 				 * we will sleep a bit to make sure we
 				 * have most if not all the tasks
 				 * completed before we return */
@@ -1305,19 +1495,22 @@ _wait_for_io(slurmd_job_t *job)
 	} else
 		info("_wait_for_io: ioid==0");
 
+	/* Close any files for stdout/stderr opened by the stepd */
+	io_close_local_fds(job);
+
 	return;
 }
 
-	
+
 static char *
 _make_batch_dir(slurmd_job_t *job)
 {
-	char path[MAXPATHLEN]; 
+	char path[MAXPATHLEN];
 
 	if (job->stepid == NO_VAL)
 		snprintf(path, 1024, "%s/job%05u", conf->spooldir, job->jobid);
 	else {
-		snprintf(path, 1024, "%s/job%05u.%05u", 
+		snprintf(path, 1024, "%s/job%05u.%05u",
 			 conf->spooldir, job->jobid, job->stepid);
 	}
 
@@ -1348,7 +1541,7 @@ _make_batch_script(batch_job_launch_msg_t *msg, char *path)
 	FILE *fp = NULL;
 	char  script[MAXPATHLEN];
 
-	snprintf(script, 1024, "%s/%s", path, "slurm_script"); 
+	snprintf(script, 1024, "%s/%s", path, "slurm_script");
 
   again:
 	if ((fp = safeopen(script, "w", SAFEOPEN_CREATE_ONLY)) == NULL) {
@@ -1367,7 +1560,7 @@ _make_batch_script(batch_job_launch_msg_t *msg, char *path)
 	if (fclose(fp) < 0) {
 		error("fclose: %m");
 	}
-	
+
 	if (chown(script, (uid_t) msg->uid, (gid_t) -1) < 0) {
 		error("chown(%s): %m", path);
 		goto error;
@@ -1396,18 +1589,18 @@ _send_launch_failure (launch_tasks_request_msg_t *msg, slurm_addr *cli, int rc)
 	name = xstrdup(conf->node_name);
 #else
 	name = xstrdup(msg->complete_nodelist);
-	
+
 #endif
 	debug ("sending launch failure message: %s", slurm_strerror (rc));
 
 	slurm_msg_t_init(&resp_msg);
 	memcpy(&resp_msg.address, cli, sizeof(slurm_addr));
-	slurm_set_addr(&resp_msg.address, 
+	slurm_set_addr(&resp_msg.address,
 		       msg->resp_port[nodeid % msg->num_resp_port],
-		       NULL); 
+		       NULL);
 	resp_msg.data = &resp;
 	resp_msg.msg_type = RESPONSE_LAUNCH_TASKS;
-		
+
 	resp.node_name     = name;
 	resp.return_code   = rc ? rc : -1;
 	resp.count_of_pids = 0;
@@ -1419,7 +1612,7 @@ _send_launch_failure (launch_tasks_request_msg_t *msg, slurm_addr *cli, int rc)
 
 static void
 _send_launch_resp(slurmd_job_t *job, int rc)
-{	
+{
 	int i;
 	slurm_msg_t resp_msg;
 	launch_tasks_response_msg_t resp;
@@ -1431,13 +1624,13 @@ _send_launch_resp(slurmd_job_t *job, int rc)
 	debug("Sending launch resp rc=%d", rc);
 
 	slurm_msg_t_init(&resp_msg);
-        resp_msg.address      = srun->resp_addr;
-	resp_msg.data         = &resp;
-	resp_msg.msg_type     = RESPONSE_LAUNCH_TASKS;
-	
-	resp.node_name        = xstrdup(job->node_name);
-	resp.return_code      = rc;
-	resp.count_of_pids    = job->ntasks;
+	resp_msg.address	= srun->resp_addr;
+	resp_msg.data		= &resp;
+	resp_msg.msg_type	= RESPONSE_LAUNCH_TASKS;
+
+	resp.node_name		= xstrdup(job->node_name);
+	resp.return_code	= rc;
+	resp.count_of_pids	= job->ntasks;
 
 	resp.local_pids = xmalloc(job->ntasks * sizeof(*resp.local_pids));
 	resp.task_ids = xmalloc(job->ntasks * sizeof(*resp.task_ids));
@@ -1457,18 +1650,18 @@ _send_launch_resp(slurmd_job_t *job, int rc)
 static int
 _send_complete_batch_script_msg(slurmd_job_t *job, int err, int status)
 {
-	int                      rc, i;
-	slurm_msg_t              req_msg;
-	complete_batch_script_msg_t  req;
+	int		rc, i;
+	slurm_msg_t	req_msg;
+	complete_batch_script_msg_t req;
 
 	req.job_id	= job->jobid;
 	req.job_rc      = status;
-	req.slurm_rc	= err; 
-		
+	req.slurm_rc	= err;
+
 	slurm_msg_t_init(&req_msg);
 	req.node_name	= job->node_name;
 	req_msg.msg_type= REQUEST_COMPLETE_BATCH_SCRIPT;
-	req_msg.data	= &req;	
+	req_msg.data	= &req;
 
 	info("sending REQUEST_COMPLETE_BATCH_SCRIPT");
 
@@ -1509,7 +1702,12 @@ _drop_privileges(slurmd_job_t *job, bool do_setuid, struct priv_state *ps)
 
 	ps->gid_list = (gid_t *) xmalloc(ps->ngids * sizeof(gid_t));
 
-	getgroups(ps->ngids, ps->gid_list);
+	if(getgroups(ps->ngids, ps->gid_list) == -1) {
+		error("_drop_privileges: couldn't get %d groups: %m",
+		      ps->ngids);
+		xfree(ps->gid_list);
+		return -1;
+	}
 
 	/*
 	 * No need to drop privileges if we're not running as root
@@ -1523,7 +1721,7 @@ _drop_privileges(slurmd_job_t *job, bool do_setuid, struct priv_state *ps)
 	}
 
 	if (_initgroups(job) < 0) {
-		error("_initgroups: %m"); 
+		error("_initgroups: %m");
 	}
 
 	if (do_setuid && seteuid(job->pwd->pw_uid) < 0) {
@@ -1537,7 +1735,7 @@ _drop_privileges(slurmd_job_t *job, bool do_setuid, struct priv_state *ps)
 static int
 _reclaim_privileges(struct priv_state *ps)
 {
-	/* 
+	/*
 	 * No need to reclaim privileges if our uid == pwd->pw_uid
 	 */
 	if (geteuid() == ps->saved_uid)
@@ -1561,8 +1759,8 @@ _reclaim_privileges(struct priv_state *ps)
 }
 
 
-static void
-_slurmd_job_log_init(slurmd_job_t *job) 
+static int
+_slurmd_job_log_init(slurmd_job_t *job)
 {
 	char argv0[64];
 
@@ -1582,21 +1780,22 @@ _slurmd_job_log_init(slurmd_job_t *job)
 		conf->log_opts.stderr_level = LOG_LEVEL_DEBUG3;
 
 	snprintf(argv0, sizeof(argv0), "slurmd[%s]", conf->node_name);
-	/* 
-	 * reinitialize log 
+	/*
+	 * reinitialize log
 	 */
-	
+
 	log_alter(conf->log_opts, 0, NULL);
 	log_set_argv0(argv0);
-	
+
 	/* Connect slurmd stderr to job's stderr */
 	if (!job->user_managed_io && job->task != NULL) {
 		if (dup2(job->task[0]->stderr_fd, STDERR_FILENO) < 0) {
 			error("job_log_init: dup2(stderr): %m");
-			return;
+			return ESLURMD_IO_ERROR;
 		}
 	}
 	verbose("debug level = %d", conf->log_opts.stderr_level);
+	return SLURM_SUCCESS;
 }
 
 
@@ -1609,7 +1808,7 @@ _setargs(slurmd_job_t *job)
 	if ((job->jobid >= MIN_NOALLOC_JOBID) || (job->stepid == NO_VAL))
 		setproctitle("[%u]",    job->jobid);
 	else
-		setproctitle("[%u.%u]", job->jobid, job->stepid); 
+		setproctitle("[%u.%u]", job->jobid, job->stepid);
 
 	return;
 }
@@ -1677,7 +1876,7 @@ _become_user(slurmd_job_t *job, struct priv_state *ps)
 	}
 
 	return SLURM_SUCCESS;
-}	
+}
 
 
 static int
@@ -1738,17 +1937,17 @@ static int _access(const char *path, int modes, uid_t uid, gid_t gid)
 }
 
 /*
- * Run a script as a specific user, with the specified uid, gid, and 
+ * Run a script as a specific user, with the specified uid, gid, and
  * extended groups.
  *
- * name IN: class of program (task prolog, task epilog, etc.), 
+ * name IN: class of program (task prolog, task epilog, etc.),
  * path IN: pathname of program to run
  * job IN: slurd job structue, used to get uid, gid, and groups
  * max_wait IN: maximum time to wait in seconds, -1 for no limit
  * env IN: environment variables to use on exec, sets minimal environment
  *	if NULL
  *
- * RET 0 on success, -1 on failure. 
+ * RET 0 on success, -1 on failure.
  */
 int
 _run_script_as_user(const char *name, const char *path, slurmd_job_t *job,
@@ -1794,7 +1993,9 @@ _run_script_as_user(const char *name, const char *path, slurmd_job_t *job,
 			exit(127);
 		}
 
-		chdir(job->cwd);
+		if(chdir(job->cwd) == -1)
+			error("run_script_as_user: couldn't "
+			      "change working dir to %s: %m", job->cwd);
 #ifdef SETPGRP_TWO_ARGS
 		setpgrp(0, 0);
 #else
@@ -1834,6 +2035,6 @@ _run_script_as_user(const char *name, const char *path, slurmd_job_t *job,
 	/* Insure that all child processes get killed, one last time */
 	killpg(cpid, SIGKILL);
 	slurm_container_signal(job->cont_id, SIGKILL);
-	
+
 	return status;
 }
