@@ -4,10 +4,11 @@
  *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>.
- *  LLNL-CODE-402394.
+ *  CODE-OCEC-09-009. All rights reserved.
  *  
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
  *  
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -37,8 +38,14 @@
 #ifndef _PDEBUG_H
 #define _PDEBUG_H
 
+#if HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <unistd.h>
-#include <sys/ptrace.h>
+#ifdef HAVE_SYS_PTRACE_H
+#  include <sys/ptrace.h>
+#endif
 #include <sys/wait.h>
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
