@@ -1,36 +1,38 @@
 /*****************************************************************************\
- * src/slurmd/slurmstepd/slurmstepd_job.c - slurmd_job_t routines
- * $Id: slurmstepd_job.c 15043 2008-09-09 23:45:19Z jette $
+ *  src/slurmd/slurmstepd/slurmstepd_job.c - slurmd_job_t routines
+ *  $Id: slurmstepd_job.c 19187 2009-12-23 20:11:00Z da $
  *****************************************************************************
- *  Copyright (C) 2002 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -44,24 +46,25 @@
 #  include <string.h>
 #endif
 
+#include <grp.h>
 #include <signal.h>
 #include <sys/types.h>
-#include <grp.h>
 
-#include "src/common/xmalloc.h"
-#include "src/common/xassert.h"
-#include "src/common/xstring.h"
+#include "src/common/eio.h"
 #include "src/common/fd.h"
 #include "src/common/log.h"
-#include "src/common/eio.h"
+#include "src/common/node_select.h"
 #include "src/common/slurm_jobacct_gather.h"
 #include "src/common/slurm_protocol_api.h"
+#include "src/common/xassert.h"
+#include "src/common/xmalloc.h"
+#include "src/common/xstring.h"
 
 #include "src/slurmd/slurmd/slurmd.h"
-#include "src/slurmd/slurmstepd/slurmstepd_job.h"
 #include "src/slurmd/slurmstepd/io.h"
 #include "src/slurmd/slurmstepd/fname.h"
 #include "src/slurmd/slurmstepd/multi_prog.h"
+#include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
 static char ** _array_copy(int n, char **src);
 static void _array_free(char ***array);
@@ -111,7 +114,7 @@ _valid_gid(struct passwd *pwd, gid_t *gid)
 {
 	struct group *grp;
 	int i;
-	
+
 	if (!pwd)
 		return 0;
 	if (pwd->pw_gid == *gid)
@@ -134,8 +137,8 @@ _valid_gid(struct passwd *pwd, gid_t *gid)
 		       	return 1;
 	       	}
 	}
-	
-	/* root user may have launched this job for this user, but 
+
+	/* root user may have launched this job for this user, but
 	 * root did not explicitly set the gid. This would set the
 	 * gid to 0. In this case we should set the appropriate
 	 * default gid for the user (from the passwd struct).
@@ -150,7 +153,7 @@ _valid_gid(struct passwd *pwd, gid_t *gid)
 }
 
 /* create a slurmd job structure from a launch tasks message */
-slurmd_job_t * 
+slurmd_job_t *
 job_create(launch_tasks_request_msg_t *msg)
 {
 	struct passwd *pwd = NULL;
@@ -159,7 +162,7 @@ job_create(launch_tasks_request_msg_t *msg)
 	slurm_addr     resp_addr;
 	slurm_addr     io_addr;
 	int            nodeid = NO_VAL;
-	
+
 	xassert(msg != NULL);
 	xassert(msg->complete_nodelist != NULL);
 	debug3("entering job_create");
@@ -173,6 +176,18 @@ job_create(launch_tasks_request_msg_t *msg)
 		_pwd_destroy(pwd);
 		return NULL;
 	}
+
+	if(msg->job_mem && (msg->acctg_freq != (uint16_t) NO_VAL)
+	   && (msg->acctg_freq > conf->job_acct_gather_freq)) {
+		error("Can't set frequency to %u, it is higher than %u.  "
+		      "We need it to be at least at this level to "
+		      "monitor memory usage.",
+		      msg->acctg_freq, conf->job_acct_gather_freq);
+		slurm_seterrno (ESLURMD_INVALID_ACCT_FREQ);
+		_pwd_destroy(pwd);
+		return NULL;
+	}
+
 	job = xmalloc(sizeof(slurmd_job_t));
 #ifndef HAVE_FRONT_END
 	nodeid = nodelist_find(msg->complete_nodelist, conf->node_name);
@@ -182,12 +197,12 @@ job_create(launch_tasks_request_msg_t *msg)
 	job->node_name = xstrdup(msg->complete_nodelist);
 #endif
 	if(nodeid < 0) {
-		error("couldn't find node %s in %s", 
+		error("couldn't find node %s in %s",
 		      job->node_name, msg->complete_nodelist);
 		job_destroy(job);
 		return NULL;
 	}
-	
+
 	job->state	= SLURMSTEPD_STEP_STARTING;
 	job->pwd	= pwd;
 	job->ntasks	= msg->tasks_to_launch[nodeid];
@@ -196,25 +211,20 @@ job_create(launch_tasks_request_msg_t *msg)
 	job->stepid	= msg->job_step_id;
 
 	job->job_mem	= msg->job_mem;
-	job->task_mem	= msg->task_mem;
 	if (job->job_mem)
 		jobacct_common_set_mem_limit(job->jobid, job->job_mem);
-	else if (job->task_mem && job->ntasks) {
-		jobacct_common_set_mem_limit(job->jobid, 
-					     (job->task_mem * job->ntasks));
-	}
-	
+
 	job->uid	= (uid_t) msg->uid;
 	job->gid	= (gid_t) msg->gid;
 	job->cwd	= xstrdup(msg->cwd);
 	job->task_dist	= msg->task_dist;
-	job->plane_size	= msg->plane_size;
-	
+
 	job->cpu_bind_type = msg->cpu_bind_type;
 	job->cpu_bind = xstrdup(msg->cpu_bind);
 	job->mem_bind_type = msg->mem_bind_type;
 	job->mem_bind = xstrdup(msg->mem_bind);
-	job->ckpt_path = xstrdup(msg->ckpt_path);
+	job->ckpt_dir = xstrdup(msg->ckpt_dir);
+	job->restart_dir = xstrdup(msg->restart_dir);
 	job->cpus_per_task = msg->cpus_per_task;
 
 	job->env     = _array_copy(msg->envc, msg->env);
@@ -237,14 +247,12 @@ job_create(launch_tasks_request_msg_t *msg)
 	job->envtp->nodeid = -1;
 
 	job->envtp->distribution = 0;
-	job->envtp->plane_size = 0;
-
 	job->envtp->cpu_bind_type = 0;
 	job->envtp->cpu_bind = NULL;
 	job->envtp->mem_bind_type = 0;
 	job->envtp->mem_bind = NULL;
-	job->envtp->ckpt_path = NULL;
-	
+	job->envtp->ckpt_dir = NULL;
+
 	memcpy(&resp_addr, &msg->orig_addr, sizeof(slurm_addr));
 	slurm_set_addr(&resp_addr,
 		       msg->resp_port[nodeid % msg->num_resp_port],
@@ -256,10 +264,11 @@ job_create(launch_tasks_request_msg_t *msg)
 			       msg->io_port[nodeid % msg->num_io_port],
 			       NULL);
 	}
-	
+
 	srun = srun_info_create(msg->cred, &resp_addr, &io_addr);
 
 	job->buffered_stdio = msg->buffered_stdio;
+	job->labelio = msg->labelio;
 
 	job->task_prolog = xstrdup(msg->task_prolog);
 	job->task_epilog = xstrdup(msg->task_epilog);
@@ -280,7 +289,8 @@ job_create(launch_tasks_request_msg_t *msg)
 	job->pty         = msg->pty;
 	job->open_mode   = msg->open_mode;
 	job->options     = msg->options;
-	
+	job->alloc_cores = format_core_allocs(msg->cred, conf->node_name);
+
 	list_append(job->sruns, (void *) srun);
 
 	_job_init_task_info(job, msg->global_task_ids[nodeid],
@@ -295,13 +305,13 @@ job_create(launch_tasks_request_msg_t *msg)
 static char *
 _batchfilename(slurmd_job_t *job, const char *name)
 {
-	if (name == NULL) 
+	if (name == NULL)
 		return fname_create(job, "slurm-%J.out", 0);
 	else
 		return fname_create(job, name, 0);
 }
 
-slurmd_job_t * 
+slurmd_job_t *
 job_batch_job_create(batch_job_launch_msg_t *msg)
 {
 	struct passwd *pwd;
@@ -310,7 +320,7 @@ job_batch_job_create(batch_job_launch_msg_t *msg)
 	char *in_name;
 
 	xassert(msg != NULL);
-	
+
 	debug3("entering batch_job_create");
 
 	if ((pwd = _pwd_create((uid_t)msg->uid)) == NULL) {
@@ -323,13 +333,23 @@ job_batch_job_create(batch_job_launch_msg_t *msg)
 		_pwd_destroy(pwd);
 		return NULL;
 	}
-	
+	if(msg->job_mem && (msg->acctg_freq != (uint16_t) NO_VAL)
+	   && (msg->acctg_freq > conf->job_acct_gather_freq)) {
+		error("Can't set frequency to %u, it is higher than %u.  "
+		      "We need it to be at least at this level to "
+		      "monitor memory usage.",
+		      msg->acctg_freq, conf->job_acct_gather_freq);
+		slurm_seterrno (ESLURMD_INVALID_ACCT_FREQ);
+		_pwd_destroy(pwd);
+		return NULL;
+	}
+
 	job = xmalloc(sizeof(slurmd_job_t));
-	
+
 	job->state   = SLURMSTEPD_STEP_STARTING;
 	job->pwd     = pwd;
 	job->cpus    = msg->cpus_per_node[0];
-	job->ntasks  = 1; 
+	job->ntasks  = 1;
 	job->nprocs  = msg->nprocs;
 	job->jobid   = msg->job_id;
 	job->stepid  = msg->step_id;
@@ -350,6 +370,9 @@ job_batch_job_create(batch_job_launch_msg_t *msg)
 	job->gid     = (gid_t) msg->gid;
 	job->cwd     = xstrdup(msg->work_dir);
 
+	job->ckpt_dir = xstrdup(msg->ckpt_dir);
+	job->restart_dir = xstrdup(msg->restart_dir);
+
 	job->env     = _array_copy(msg->envc, msg->environment);
 	job->eio     = eio_handle_create();
 	job->sruns   = list_create((ListDelF) _srun_info_destructor);
@@ -361,14 +384,15 @@ job_batch_job_create(batch_job_launch_msg_t *msg)
 	job->envtp->nodeid = -1;
 
 	job->envtp->distribution = 0;
-	job->envtp->plane_size = 0;
-
-	job->envtp->cpu_bind_type = 0;
-	job->envtp->cpu_bind = NULL;
+	job->cpu_bind_type = msg->cpu_bind_type;
+	job->cpu_bind = xstrdup(msg->cpu_bind);
 	job->envtp->mem_bind_type = 0;
 	job->envtp->mem_bind = NULL;
-	job->envtp->ckpt_path = NULL;
+	job->envtp->ckpt_dir = NULL;
+	job->envtp->restart_cnt = msg->restart_cnt;
+
 	job->cpus_per_task = msg->cpus_per_node[0];
+	job->alloc_cores = format_core_allocs(msg->cred, conf->node_name);
 
 	srun = srun_info_create(NULL, NULL, NULL);
 
@@ -382,26 +406,31 @@ job_batch_job_create(batch_job_launch_msg_t *msg)
 		/* job script has not yet been written out to disk --
 		 * argv will be filled in later by _make_batch_script()
 		 */
-		job->argv    = (char **) xmalloc(sizeof(char *));
+		job->argv    = (char **) xmalloc(2 * sizeof(char *));
 	}
 
 	job->task = (slurmd_task_info_t **)
 		xmalloc(sizeof(slurmd_task_info_t *));
-	if (msg->err == NULL)
-		msg->err = xstrdup(msg->out);
+	if (msg->std_err == NULL)
+		msg->std_err = xstrdup(msg->std_out);
 
-	if (msg->in == NULL)
+	if (msg->std_in == NULL)
 		in_name = xstrdup("/dev/null");
-	else 
-		in_name = fname_create(job, msg->in, 0);
+	else
+		in_name = fname_create(job, msg->std_in, 0);
 
 	job->task[0] = task_info_create(0, 0,
 					in_name,
-					_batchfilename(job, msg->out),
-					_batchfilename(job, msg->err));
+					_batchfilename(job, msg->std_out),
+					_batchfilename(job, msg->std_err));
 	job->task[0]->argc = job->argc;
 	job->task[0]->argv = job->argv;
 
+#ifdef HAVE_CRAY_XT
+	select_g_get_jobinfo(msg->select_jobinfo, SELECT_DATA_RESV_ID,
+			     &job->resv_id);
+#endif
+
 	return job;
 }
 
@@ -452,7 +481,7 @@ _job_init_task_info(slurmd_job_t *job, uint32_t *gtid,
 		return;
 	}
 
-	job->task = (slurmd_task_info_t **) 
+	job->task = (slurmd_task_info_t **)
 		xmalloc(job->ntasks * sizeof(slurmd_task_info_t *));
 
 	for (i = 0; i < job->ntasks; i++){
@@ -481,14 +510,14 @@ job_signal_tasks(slurmd_job_t *job, int signal)
 		if ((job->task[n]->pid > (pid_t) 0)
 		&&  (kill(job->task[n]->pid, signal) < 0)) {
 			if (errno != ESRCH) {
-				error("job %d.%d: kill task %d: %m", 
+				error("job %d.%d: kill task %d: %m",
 				      job->jobid, job->stepid, n);
 			}
 		}
 	}
 }
 
-void 
+void
 job_destroy(slurmd_job_t *job)
 {
 	int i;
@@ -505,6 +534,7 @@ job_destroy(slurmd_job_t *job)
 	xfree(job->node_name);
 	xfree(job->task_prolog);
 	xfree(job->task_epilog);
+	xfree(job->alloc_cores);
 	xfree(job);
 }
 
@@ -526,7 +556,7 @@ static void
 _array_free(char ***array)
 {
 	int i = 0;
-	while ((*array)[i] != NULL) 
+	while ((*array)[i] != NULL)
 		xfree((*array)[i++]);
 	xfree(*array);
 	*array = NULL;
@@ -534,10 +564,10 @@ _array_free(char ***array)
 
 
 struct srun_info *
-srun_info_create(slurm_cred_t cred, slurm_addr *resp_addr, slurm_addr *ioaddr)
+srun_info_create(slurm_cred_t *cred, slurm_addr *resp_addr, slurm_addr *ioaddr)
 {
 	char             *data = NULL;
-	int               len  = 0;
+	uint32_t          len  = 0;
 	struct srun_info *srun = xmalloc(sizeof(struct srun_info));
 	srun_key_t       *key  = xmalloc(sizeof(srun_key_t));
 
@@ -558,8 +588,8 @@ srun_info_create(slurm_cred_t cred, slurm_addr *resp_addr, slurm_addr *ioaddr)
 		memcpy((void *) key->data, data, len);
 
 		if (len < SLURM_IO_KEY_SIZE)
-			memset( (void *) (key->data + len), 0, 
-			        SLURM_IO_KEY_SIZE - len);
+			memset( (void *) (key->data + len), 0,
+				SLURM_IO_KEY_SIZE - len);
 	}
 
 	if (ioaddr != NULL)
@@ -619,7 +649,7 @@ task_info_create(int taskid, int gtaskid,
 }
 
 
-static void 
+static void
 _task_info_destroy(slurmd_task_info_t *t, uint16_t multi_prog)
 {
 	slurm_mutex_lock(&t->mutex);
@@ -630,3 +660,4 @@ _task_info_destroy(slurmd_task_info_t *t, uint16_t multi_prog)
 	} /* otherwise, t->argv is a pointer to job->argv */
 	xfree(t);
 }
+
