@@ -2,36 +2,37 @@
  *  job_functions.c - Functions related to job display mode of smap.
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
  *
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -48,14 +49,15 @@ static int  _nodes_in_list(char *node_list);
 static void _print_header_job(void);
 static int  _print_text_job(job_info_t * job_ptr);
 
-extern void get_job()
+extern void get_job(void)
 {
 	int error_code = -1, i, recs;
 	static int printed_jobs = 0;
 	static int count = 0;
 	static job_info_msg_t *job_info_ptr = NULL, *new_job_ptr = NULL;
-	job_info_t job;
+	job_info_t *job_ptr = NULL;
 	uint16_t show_flags = 0;
+	bitstr_t *nodes_req = NULL;
 
 	show_flags |= SHOW_ALL;
 	if (job_info_ptr) {
@@ -68,7 +70,7 @@ extern void get_job()
 			new_job_ptr = job_info_ptr;
 		}
 	} else
-		error_code = slurm_load_jobs((time_t) NULL, &new_job_ptr, 
+		error_code = slurm_load_jobs((time_t) NULL, &new_job_ptr,
 					     show_flags);
 
 	if (error_code) {
@@ -76,7 +78,7 @@ extern void get_job()
 			if(!params.commandline) {
 				mvwprintw(text_win,
 					  main_ycord, 1,
-					  "slurm_load_job: %s", 
+					  "slurm_load_job: %s",
 					  slurm_strerror(slurm_get_errno()));
 				main_ycord++;
 			} else {
@@ -93,86 +95,95 @@ extern void get_job()
 		recs = new_job_ptr->record_count;
 	else
 		recs = 0;
-	
+
 	if(!params.commandline)
-		if((text_line_cnt+printed_jobs) > count) 
+		if((text_line_cnt+printed_jobs) > count)
 			text_line_cnt--;
 	printed_jobs = 0;
 	count = 0;
-	for (i = 0; i < recs; i++) {
-		job = new_job_ptr->job_array[i];
-		
-		if ((job.job_state != JOB_PENDING)
-		    &&  (job.job_state != JOB_RUNNING)
-		    &&  (job.job_state != JOB_SUSPENDED)
-		    &&  ((job.job_state & JOB_COMPLETING) == 0))
-			continue;	/* job has completed */
 
-		if (job.node_inx[0] != -1) {
-#ifdef HAVE_SUN_CONST
-			set_grid_name(job.nodes, count);
-#else
+	if(params.hl)
+		nodes_req = get_requested_node_bitmap();
+	for (i = 0; i < recs; i++) {
+		job_ptr = &(new_job_ptr->job_array[i]);
+		if(!IS_JOB_PENDING(job_ptr) && !IS_JOB_RUNNING(job_ptr)
+		   && !IS_JOB_SUSPENDED(job_ptr)
+		   && !IS_JOB_COMPLETING(job_ptr))
+			continue;	/* job has completed */
+		if(nodes_req) {
+			int overlap = 0;
+			bitstr_t *loc_bitmap = bit_alloc(bit_size(nodes_req));
+			inx2bitstr(loc_bitmap, job_ptr->node_inx);
+			overlap = bit_overlap(loc_bitmap, nodes_req);
+			FREE_NULL_BITMAP(loc_bitmap);
+			if(!overlap)
+				continue;
+		}
+
+		if (job_ptr->node_inx[0] != -1) {
 			int j = 0;
-			job.num_nodes = 0;
-			while (job.node_inx[j] >= 0) {
-				job.num_nodes +=
-				    (job.node_inx[j + 1] + 1) -
-				    job.node_inx[j];
-				set_grid_inx(job.node_inx[j],
-					     job.node_inx[j + 1], count);
+			job_ptr->num_nodes = 0;
+			while (job_ptr->node_inx[j] >= 0) {
+				job_ptr->num_nodes +=
+				    (job_ptr->node_inx[j + 1] + 1) -
+				    job_ptr->node_inx[j];
+				set_grid_inx(job_ptr->node_inx[j],
+					     job_ptr->node_inx[j + 1], count);
 				j += 2;
 			}
-#endif
 
 			if(!params.commandline) {
 				if((count>=text_line_cnt)
-				   && (printed_jobs 
+				   && (printed_jobs
 				       < (text_win->_maxy-3))) {
-					job.num_procs = (int)letters[count%62];
+					job_ptr->num_procs =
+						(int)letters[count%62];
 					wattron(text_win,
 						COLOR_PAIR(colors[count%6]));
-					_print_text_job(&job);
+					_print_text_job(job_ptr);
 					wattroff(text_win,
 						 COLOR_PAIR(colors[count%6]));
 					printed_jobs++;
-				} 
+				}
 			} else {
-				job.num_procs = (int)letters[count%62];
-				_print_text_job(&job);
+				job_ptr->num_procs = (int)letters[count%62];
+				_print_text_job(job_ptr);
 			}
-			count++;			
+			count++;
 		}
 		if(count==128)
 			count=0;
 	}
-		
+
 	for (i = 0; i < recs; i++) {
-		job = new_job_ptr->job_array[i];
-		
-		if (job.job_state != JOB_PENDING)
+		job_ptr = &(new_job_ptr->job_array[i]);
+
+		if (!IS_JOB_PENDING(job_ptr))
 			continue;	/* job has completed */
 
 		if(!params.commandline) {
 			if((count>=text_line_cnt)
-			   && (printed_jobs 
+			   && (printed_jobs
 			       < (text_win->_maxy-3))) {
-				job.nodes = "waiting...";
-				job.num_procs = (int) letters[count%62];
+				xfree(job_ptr->nodes);
+				job_ptr->nodes = xstrdup("waiting...");
+				job_ptr->num_procs = (int) letters[count%62];
 				wattron(text_win,
 					COLOR_PAIR(colors[count%6]));
-				_print_text_job(&job);
+				_print_text_job(job_ptr);
 				wattroff(text_win,
 					 COLOR_PAIR(colors[count%6]));
 				printed_jobs++;
-			} 
+			}
 		} else {
-			job.nodes = "waiting...";
-			job.num_procs = (int) letters[count%62];
-			_print_text_job(&job);
+			xfree(job_ptr->nodes);
+			job_ptr->nodes = xstrdup("waiting...");
+			job_ptr->num_procs = (int) letters[count%62];
+			_print_text_job(job_ptr);
 			printed_jobs++;
 		}
-		count++;			
-		
+		count++;
+
 		if(count==128)
 			count=0;
 	}
@@ -182,7 +193,7 @@ extern void get_job()
 
 	if(!params.commandline)
 		main_ycord++;
-	
+
 	job_info_ptr = new_job_ptr;
 	return;
 }
@@ -203,6 +214,11 @@ static void _print_header_job(void)
 		mvwprintw(text_win, main_ycord,
 			  main_xcord, "BG_BLOCK");
 		main_xcord += 18;
+#endif
+#ifdef HAVE_CRAY_XT
+		mvwprintw(text_win, main_ycord,
+			  main_xcord, "RESV_ID");
+		main_xcord += 18;
 #endif
 		mvwprintw(text_win, main_ycord,
 			  main_xcord, "USER");
@@ -229,7 +245,7 @@ static void _print_header_job(void)
 		main_xcord = 1;
 		main_ycord++;
 	} else {
-		printf("JOBID ");
+		printf("   JOBID ");
 		printf("PARTITION ");
 #ifdef HAVE_BG
 		printf("        BG_BLOCK ");
@@ -260,26 +276,15 @@ static int _print_text_job(job_info_t * job_ptr)
 	uint32_t node_cnt = 0;
 	char *ionodes = NULL, *uname;
 	time_t now_time = time(NULL);
-	char *temp = NULL;
-	/* first set the jname to the job_ptr->name */
-	char *jname = NULL;
-
-	if(job_ptr->name) {
-		jname = xstrdup(job_ptr->name);
-		/* then grep for " since that is the delimiter for
-		   the wckey */
-		if((temp = strchr(jname, '\"')))
-			temp[0] = '\0';
-	}
 
 #ifdef HAVE_BG
-	select_g_get_jobinfo(job_ptr->select_jobinfo, 
-			     SELECT_DATA_IONODES, 
-			     &ionodes);
-	select_g_get_jobinfo(job_ptr->select_jobinfo, 
-			     SELECT_DATA_NODE_CNT, 
-			     &node_cnt);
-	if(!strcasecmp(job_ptr->nodes,"waiting...")) 
+	select_g_select_jobinfo_get(job_ptr->select_jobinfo,
+				    SELECT_JOBDATA_IONODES,
+				    &ionodes);
+	select_g_select_jobinfo_get(job_ptr->select_jobinfo,
+				    SELECT_JOBDATA_NODE_CNT,
+				    &node_cnt);
+	if(!strcasecmp(job_ptr->nodes,"waiting..."))
 		xfree(ionodes);
 #else
 	node_cnt = job_ptr->num_nodes;
@@ -303,12 +308,21 @@ static int _print_text_job(job_info_t * job_ptr)
 		main_xcord += 10;
 #ifdef HAVE_BG
 		mvwprintw(text_win, main_ycord,
-			  main_xcord, "%.16s", 
-			  select_g_sprint_jobinfo(job_ptr->select_jobinfo, 
-						  time_buf, 
-						  sizeof(time_buf), 
+			  main_xcord, "%.16s",
+			  select_g_select_jobinfo_sprint(job_ptr->select_jobinfo,
+						  time_buf,
+						  sizeof(time_buf),
 						  SELECT_PRINT_BG_ID));
 		main_xcord += 18;
+#endif
+#ifdef HAVE_CRAY_XT
+		mvwprintw(text_win, main_ycord,
+			  main_xcord, "%.16s",
+			  select_g_select_jobinfo_sprint(job_ptr->select_jobinfo,
+						  time_buf,
+						  sizeof(time_buf),
+						  SELECT_PRINT_RESV_ID));
+		main_xcord += 18;
 #endif
 		uname = uid_to_string((uid_t) job_ptr->user_id);
 		mvwprintw(text_win, main_ycord,
@@ -316,7 +330,7 @@ static int _print_text_job(job_info_t * job_ptr)
 		xfree(uname);
 		main_xcord += 9;
 		mvwprintw(text_win, main_ycord,
-			  main_xcord, "%.9s", jname);
+			  main_xcord, "%.9s", job_ptr->name);
 		main_xcord += 10;
 		mvwprintw(text_win, main_ycord,
 			  main_xcord, "%.2s",
@@ -334,29 +348,29 @@ static int _print_text_job(job_info_t * job_ptr)
 			  time_buf);
 		main_xcord += 11;
 
-		mvwprintw(text_win, 
+		mvwprintw(text_win,
 			  main_ycord,
 			  main_xcord, "%5s", tmp_cnt);
-		
+
 		main_xcord += 6;
 
 		tempxcord = main_xcord;
-		
+
 		i=0;
 		while (job_ptr->nodes[i] != '\0') {
 			if ((printed = mvwaddch(text_win,
-						main_ycord, 
+						main_ycord,
 						main_xcord,
 						job_ptr->nodes[i])) < 0) {
 				xfree(ionodes);
 				return printed;
 			}
 			main_xcord++;
-			width = text_win->_maxx 
+			width = text_win->_maxx
 				- main_xcord;
 			if (job_ptr->nodes[i] == '[')
 				prefixlen = i + 1;
-			else if (job_ptr->nodes[i] == ',' 
+			else if (job_ptr->nodes[i] == ','
 				 && (width - 9) <= 0) {
 				main_ycord++;
 				main_xcord = tempxcord + prefixlen;
@@ -364,9 +378,9 @@ static int _print_text_job(job_info_t * job_ptr)
 			i++;
 		}
 		if(ionodes) {
-			mvwprintw(text_win, 
+			mvwprintw(text_win,
 				  main_ycord,
-				  main_xcord, "[%s]", 
+				  main_xcord, "[%s]",
 				  ionodes);
 			main_xcord += strlen(ionodes)+2;
 			xfree(ionodes);
@@ -378,16 +392,23 @@ static int _print_text_job(job_info_t * job_ptr)
 		printf("%8d ", job_ptr->job_id);
 		printf("%9.9s ", job_ptr->partition);
 #ifdef HAVE_BG
-		printf("%16.16s ", 
-		       select_g_sprint_jobinfo(job_ptr->select_jobinfo, 
-					       time_buf, 
-					       sizeof(time_buf), 
+		printf("%16.16s ",
+		       select_g_select_jobinfo_sprint(job_ptr->select_jobinfo,
+					       time_buf,
+					       sizeof(time_buf),
 					       SELECT_PRINT_BG_ID));
+#endif
+#ifdef HAVE_CRAY_XT
+		printf("%16.16s ",
+		       select_g_select_jobinfo_sprint(job_ptr->select_jobinfo,
+					       time_buf,
+					       sizeof(time_buf),
+					       SELECT_PRINT_RESV_ID));
 #endif
 		uname = uid_to_string((uid_t) job_ptr->user_id);
 		printf("%8.8s ", uname);
 		xfree(uname);
-		printf("%6.6s ", jname);
+		printf("%6.6s ", job_ptr->name);
 		printf("%2.2s ",
 		       job_state_string_compact(job_ptr->job_state));
 		if(!strcasecmp(job_ptr->nodes,"waiting...")) {
@@ -396,11 +417,11 @@ static int _print_text_job(job_info_t * job_ptr)
 			time_diff = now_time - job_ptr->start_time;
 			secs2time_str(time_diff, time_buf, sizeof(time_buf));
 		}
-		
+
 		printf("%10.10s ", time_buf);
 
 		printf("%5s ", tmp_cnt);
-		
+
 		printf("%s", job_ptr->nodes);
 		if(ionodes) {
 			printf("[%s]", ionodes);
@@ -408,9 +429,8 @@ static int _print_text_job(job_info_t * job_ptr)
 		}
 
 		printf("\n");
-		
+
 	}
-	xfree(jname);
 
 	return printed;
 }
