@@ -1,30 +1,32 @@
 /****************************************************************************\
  *  opts.c - srun command line option parsing
  *
- *  $Id: opts.c 16350 2009-01-29 18:16:08Z jette $
+ *  $Id: opts.c 19153 2009-12-10 22:47:09Z da $
  *****************************************************************************
- *  Copyright (C) 2002-2006 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Joey Ekstrom <ekstrom1@llnl.gov>, Morris Jette <jette1@llnl.gov>
- *  LLNL-CODE-402394.
+ *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
- *  to link the code of portions of this program with the OpenSSL library under 
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
  *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
@@ -59,12 +61,15 @@
 
 #include "src/common/read_config.h"
 #include "src/common/xstring.h"
+#include "src/common/proc_args.h"
+
 #include "src/squeue/squeue.h"
 
 /* getopt_long options, integers but not characters */
 #define OPT_LONG_HELP  0x100
 #define OPT_LONG_USAGE 0x101
 #define OPT_LONG_HIDE  0x102
+#define OPT_LONG_START 0x103
 
 /* FUNCTIONS */
 static List  _build_job_list( char* str );
@@ -77,10 +82,9 @@ static char *_get_prefix(char *token);
 static void  _help( void );
 static int   _max_procs_per_node(void);
 static int   _parse_state( char* str, enum job_states* states );
-static void  _parse_token( char *token, char *field, int *field_size, 
-                           bool *right_justify, char **suffix);
+static void  _parse_token( char *token, char *field, int *field_size,
+			   bool *right_justify, char **suffix);
 static void  _print_options( void );
-static void  _print_version( void );
 static void  _usage( void );
 
 /*
@@ -93,43 +97,53 @@ parse_command_line( int argc, char* argv[] )
 	int opt_char;
 	int option_index;
 	static struct option long_options[] = {
+		{"accounts",   required_argument, 0, 'A'},
 		{"all",        no_argument,       0, 'a'},
-		{"noheader",   no_argument,       0, 'h'},
+		{"format",     required_argument, 0, 'o'},
+		{"help",       no_argument,       0, OPT_LONG_HELP},
+		{"hide",       no_argument,       0, OPT_LONG_HIDE},
 		{"iterate",    required_argument, 0, 'i'},
 		{"jobs",       optional_argument, 0, 'j'},
 		{"long",       no_argument,       0, 'l'},
 		{"node",       required_argument, 0, 'n'},
 		{"nodes",      required_argument, 0, 'n'},
-		{"format",     required_argument, 0, 'o'},
+		{"noheader",   no_argument,       0, 'h'},
 		{"partitions", required_argument, 0, 'p'},
-		{"steps",      optional_argument, 0, 's'},
+		{"qos",        required_argument, 0, 'q'},
 		{"sort",       required_argument, 0, 'S'},
+		{"start",      no_argument,       0, OPT_LONG_START},
+		{"steps",      optional_argument, 0, 's'},
 		{"states",     required_argument, 0, 't'},
+		{"usage",      no_argument,       0, OPT_LONG_USAGE},
 		{"user",       required_argument, 0, 'u'},
 		{"users",      required_argument, 0, 'u'},
-		{"account",    required_argument, 0, 'U'},
 		{"verbose",    no_argument,       0, 'v'},
 		{"version",    no_argument,       0, 'V'},
-		{"help",       no_argument,       0, OPT_LONG_HELP},
-		{"usage",      no_argument,       0, OPT_LONG_USAGE},
-		{"hide",       no_argument,       0, OPT_LONG_HIDE},
 		{NULL,         0,                 0, 0}
 	};
 
 	if (getenv("SQUEUE_ALL"))
 		params.all_flag = true;
-	if ( ( env_val = getenv("SQUEUE_FORMAT") ) ) 
+	if ( ( env_val = getenv("SQUEUE_FORMAT") ) )
 		params.format = xstrdup(env_val);
 	if ( ( env_val = getenv("SQUEUE_SORT") ) )
 		params.sort = xstrdup(env_val);
 
-	while((opt_char = getopt_long(argc, argv, "ahi:j::ln:o:p:s::S:t:u:U:vV",
-			long_options, &option_index)) != -1) {
+	while((opt_char = getopt_long(argc, argv,
+				      "A:ahi:j::ln:o:p:q:s::S:t:u:U:vV",
+				      long_options, &option_index)) != -1) {
 		switch (opt_char) {
 			case (int)'?':
 				fprintf(stderr, "Try \"squeue --help\" "
 						"for more information\n");
 				exit(1);
+			case (int) 'A':
+			case (int) 'U':	/* backwards compatibility */
+				xfree(params.accounts);
+				params.accounts = xstrdup(optarg);
+				params.account_list =
+					_build_str_list( params.accounts );
+				break;
 			case (int)'a':
 				params.all_flag = true;
 				break;
@@ -146,11 +160,11 @@ parse_command_line( int argc, char* argv[] )
 			case (int) 'j':
 				if (optarg) {
 					params.jobs = xstrdup(optarg);
-					params.job_list = 
+					params.job_list =
 						_build_job_list(params.jobs);
 				}
 				params.job_flag = true;
-				break;	
+				break;
 			case (int) 'l':
 				params.long_list = true;
 				break;
@@ -172,20 +186,21 @@ parse_command_line( int argc, char* argv[] )
 			case (int) 'p':
 				xfree(params.partitions);
 				params.partitions = xstrdup(optarg);
-				params.part_list = 
+				params.part_list =
 					_build_str_list( params.partitions );
+				params.all_flag = true;
 				break;
-			case (int) 'U':
-				xfree(params.accounts);
-				params.accounts = xstrdup(optarg);
-				params.account_list = 
-					_build_str_list( params.accounts );
+			case (int) 'q':
+				xfree(params.qoss);
+				params.qoss = xstrdup(optarg);
+				params.qos_list =
+					_build_str_list( params.qoss );
 				break;
 			case (int) 's':
 				if (optarg) {
 					params.steps = xstrdup(optarg);
-					params.step_list = 
-						_build_step_list( params.steps );
+					params.step_list =
+						_build_step_list(params.steps);
 				}
 				params.step_flag = true;
 				break;
@@ -196,33 +211,36 @@ parse_command_line( int argc, char* argv[] )
 			case (int) 't':
 				xfree(params.states);
 				params.states = xstrdup(optarg);
-				params.state_list = 
+				params.state_list =
 					_build_state_list( params.states );
 				break;
 			case (int) 'u':
 				xfree(params.users);
 				params.users = xstrdup(optarg);
-				params.user_list = 
+				params.user_list =
 					_build_user_list( params.users );
 				break;
 			case (int) 'v':
 				params.verbose++;
 				break;
 			case (int) 'V':
-				_print_version();
+				print_slurm_version();
 				exit(0);
 			case OPT_LONG_HELP:
 				_help();
 				exit(0);
-			case OPT_LONG_USAGE:
-				_usage();
-				exit(0);
 			case OPT_LONG_HIDE:
 				params.all_flag = false;
 				break;
+			case OPT_LONG_START:
+				params.start_flag = true;
+				break;
+			case OPT_LONG_USAGE:
+				_usage();
+				exit(0);
 		}
 	}
-	
+
 	if (optind < argc) {
 		if (params.job_flag) {
 			params.jobs = xstrdup(argv[optind++]);
@@ -239,8 +257,13 @@ parse_command_line( int argc, char* argv[] )
 	}
 
 	if ( params.job_flag && params.step_flag) {
-		error("Incompatable options --jobs and --steps\n");
-		exit(1);
+		if (params.job_list) {
+			verbose("Printing job steps with job filter");
+			params.job_flag = false;
+		} else {
+			error("Incompatible options --jobs and --steps");
+			exit(1);
+		}
 	}
 
 	if ( params.nodes ) {
@@ -252,7 +275,7 @@ parse_command_line( int argc, char* argv[] )
 
 		while ( hostset_count(params.nodes) > 0 ) {
 			name1 = hostset_pop(params.nodes);
-			
+
 			/* localhost = use current host name */
 			if ( strcasecmp("localhost", name1) == 0 ) {
 				name2 = xmalloc(128);
@@ -269,36 +292,55 @@ parse_command_line( int argc, char* argv[] )
 			free(name1);
 			xfree(name2);
 		}
-		
+
 		/* Replace params.nodename with the new one */
 		hostset_destroy(params.nodes);
 		params.nodes = nodenames;
 	}
 
-	if ( ( params.partitions == NULL ) && 
-	     ( env_val = getenv("SQUEUE_PARTITION") ) ) {
-		params.partitions = xstrdup(env_val);
-		params.part_list = _build_str_list( params.partitions );
-	}
-
-	if ( ( params.accounts == NULL ) && 
+	if ( ( params.accounts == NULL ) &&
 	     ( env_val = getenv("SQUEUE_ACCOUNT") ) ) {
 		params.accounts = xstrdup(env_val);
 		params.account_list = _build_str_list( params.accounts );
 	}
 
-	if ( ( params.states == NULL ) && 
+	if ( ( params.partitions == NULL ) &&
+	     ( env_val = getenv("SQUEUE_PARTITION") ) ) {
+		params.partitions = xstrdup(env_val);
+		params.part_list = _build_str_list( params.partitions );
+		params.all_flag = true;
+	}
+
+	if ( ( params.qoss == NULL ) &&
+	     ( env_val = getenv("SQUEUE_QOS") ) ) {
+		params.qoss = xstrdup(env_val);
+		params.qos_list = _build_str_list( params.qoss );
+	}
+
+	if ( ( params.states == NULL ) &&
 	     ( env_val = getenv("SQUEUE_STATES") ) ) {
 		params.states = xstrdup(env_val);
 		params.state_list = _build_state_list( params.states );
 	}
 
-	if ( ( params.users == NULL ) && 
+	if ( ( params.users == NULL ) &&
 	     ( env_val = getenv("SQUEUE_USERS") ) ) {
 		params.users = xstrdup(env_val);
 		params.user_list = _build_user_list( params.users );
 	}
 
+	if ( params.start_flag && !params.step_flag ) {
+		/* Set more defaults */
+		if (params.format == NULL)
+			params.format = xstrdup("%.7i %.9P %.8j %.8u  %.2t  %.19S %.6D %R");
+		if (params.sort == NULL)
+			params.sort = xstrdup("S");
+		if (params.states == NULL) {
+			params.states = xstrdup("PD");
+			params.state_list = _build_state_list( params.states );
+		}
+	}
+
 	params.max_procs = _max_procs_per_node();
 
 	if ( params.verbose )
@@ -333,7 +375,7 @@ static int   _max_procs_per_node(void)
  */
 static int
 _parse_state( char* str, enum job_states* states )
-{	
+{
 	int i;
 	char *state_names;
 
@@ -341,17 +383,22 @@ _parse_state( char* str, enum job_states* states )
 		if (strcasecmp (job_state_string(i), str) == 0) {
 			*states = i;
 			return SLURM_SUCCESS;
-		}	
+		}
 		if (strcasecmp (job_state_string_compact(i), str) == 0) {
 			*states = i;
 			return SLURM_SUCCESS;
-		}	
+		}
 	}
 	if ((strcasecmp(job_state_string(JOB_COMPLETING), str) == 0) ||
 	    (strcasecmp(job_state_string_compact(JOB_COMPLETING),str) == 0)) {
 		*states = JOB_COMPLETING;
 		return SLURM_SUCCESS;
-	}	
+	}
+	if ((strcasecmp(job_state_string(JOB_CONFIGURING), str) == 0) ||
+	    (strcasecmp(job_state_string_compact(JOB_CONFIGURING),str) == 0)) {
+		*states = JOB_CONFIGURING;
+		return SLURM_SUCCESS;
+	}
 
 	error ("Invalid job state specified: %s", str);
 	state_names = xstrdup(job_state_string(0));
@@ -361,15 +408,17 @@ _parse_state( char* str, enum job_states* states )
 	}
 	xstrcat(state_names, ",");
 	xstrcat(state_names, job_state_string(JOB_COMPLETING));
+	xstrcat(state_names, ",");
+	xstrcat(state_names, job_state_string(JOB_CONFIGURING));
 	error ("Valid job states include: %s\n", state_names);
 	xfree (state_names);
 	return SLURM_ERROR;
 }
 
-/* 
- * parse_format - Take the user's format specification and use it to build 
- *	build the format specifications (internalize it to print.c data 
- *	structures) 
+/*
+ * parse_format - Take the user's format specification and use it to build
+ *	build the format specifications (internalize it to print.c data
+ *	structures)
  * IN format - user's format specification
  * RET zero or error code
  */
@@ -389,10 +438,10 @@ extern int parse_format( char* format )
 	params.format_list = list_create( NULL );
 	if ((prefix = _get_prefix(format))) {
 		if (params.step_flag)
-			step_format_add_prefix( params.format_list, 0, 0, 
-						prefix); 
+			step_format_add_prefix( params.format_list, 0, 0,
+						prefix);
 		else
-			job_format_add_prefix( params.format_list, 0, 0, 
+			job_format_add_prefix( params.format_list, 0, 0,
 					       prefix);
 	}
 
@@ -404,103 +453,103 @@ extern int parse_format( char* format )
 	if (token && (format[0] != '%'))	/* toss header */
 		token = strtok_r( NULL, "%", &tmp_char );
 	while (token) {
-		_parse_token( token, field, &field_size, &right_justify, 
+		_parse_token( token, field, &field_size, &right_justify,
 			      &suffix);
 		if (params.step_flag) {
 			if      (field[0] == 'A')
-				step_format_add_num_tasks( params.format_list, 
-							   field_size, 
-							   right_justify, 
+				step_format_add_num_tasks( params.format_list,
+							   field_size,
+							   right_justify,
 							   suffix );
 			else if (field[0] == 'i')
-				step_format_add_id( params.format_list, 
-				                    field_size, 
+				step_format_add_id( params.format_list,
+						    field_size,
 						    right_justify, suffix );
 			else if (field[0] == 'j')
 				step_format_add_name( params.format_list,
 							field_size,
 							right_justify,
 							suffix );
-
+			else if (field[0] == 'l')
+				step_format_add_time_limit( params.format_list,
+							    field_size,
+							    right_justify,
+							    suffix );
 			else if (field[0] == 'M')
-				step_format_add_time_used( params.format_list, 
-				                            field_size, 
-				                            right_justify, 
-				                            suffix );
+				step_format_add_time_used( params.format_list,
+							    field_size,
+							    right_justify,
+							    suffix );
 			else if (field[0] == 'N')
-				step_format_add_nodes( params.format_list, 
-				                       field_size, 
-				                       right_justify, suffix );
+				step_format_add_nodes( params.format_list,
+						       field_size,
+						       right_justify, suffix );
 			else if (field[0] == 'P')
-				step_format_add_partition( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				step_format_add_partition( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
 			else if (field[0] == 'S')
-				step_format_add_time_start( params.format_list, 
-				                            field_size, 
-				                            right_justify, 
-				                            suffix );
+				step_format_add_time_start( params.format_list,
+							    field_size,
+							    right_justify,
+							    suffix );
 			else if (field[0] == 'U')
-				step_format_add_user_id( params.format_list, 
-				                         field_size, 
-				                         right_justify, 
-				                         suffix );
+				step_format_add_user_id( params.format_list,
+							 field_size,
+							 right_justify,
+							 suffix );
 			else if (field[0] == 'u')
-				step_format_add_user_name( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				step_format_add_user_name( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
 			else
-				error ("Invalid job step format specification: %c",
-				       field[0] );
+				error ( "Invalid job step format "
+					"specification: %c",
+					field[0] );
 		} else {
 			if (field[0] == 'a')
 				job_format_add_account( params.format_list,
 							field_size,
 							right_justify,
 							suffix  );
-			else if (field[0] == 'b')
-				job_format_add_time_start( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix  );
 			else if (field[0] == 'c')
-				job_format_add_min_procs( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix  );
+				job_format_add_min_procs( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix  );
 			else if (field[0] == 'C')
-				job_format_add_num_procs( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix  );
+				job_format_add_num_procs( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix  );
 			else if (field[0] == 'd')
-				job_format_add_min_tmp_disk( 
-				                          params.format_list,
-				                          field_size, 
-				                          right_justify, 
-				                          suffix  );
+				job_format_add_min_tmp_disk(
+							  params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix  );
 			else if (field[0] == 'D')
 				job_format_add_num_nodes( params.format_list,
-				                          field_size, 
-				                          right_justify, 
-				                          suffix  );
+							  field_size,
+							  right_justify,
+							  suffix  );
 			else if (field[0] == 'e')
-				job_format_add_time_end( params.format_list, 
-				                         field_size, 
-				                         right_justify, 
-				                         suffix );
+				job_format_add_time_end( params.format_list,
+							 field_size,
+							 right_justify,
+							 suffix );
 			else if (field[0] == 'E')
 				job_format_add_dependency( params.format_list,
 							field_size,
 							right_justify,
 							suffix );
 			else if (field[0] == 'f')
-				job_format_add_features( params.format_list, 
-				                         field_size, 
-				                         right_justify, 
-				                         suffix );
+				job_format_add_features( params.format_list,
+							 field_size,
+							 right_justify,
+							 suffix );
 			else if (field[0] == 'G')
 				job_format_add_group_id( params.format_list,
 							 field_size,
@@ -512,85 +561,91 @@ extern int parse_format( char* format )
 							   right_justify,
 							   suffix );
 			else if (field[0] == 'h')
-				job_format_add_shared( params.format_list, 
-				                       field_size, 
-				                       right_justify, 
-				                       suffix );
+				job_format_add_shared( params.format_list,
+						       field_size,
+						       right_justify,
+						       suffix );
 			else if (field[0] == 'H')
-				job_format_add_min_sockets( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				job_format_add_min_sockets( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
 			else if (field[0] == 'i')
-				job_format_add_job_id( params.format_list, 
-				                       field_size, 
-				                       right_justify, 
-				                       suffix );
+				job_format_add_job_id( params.format_list,
+						       field_size,
+						       right_justify,
+						       suffix );
 			else if (field[0] == 'I')
-				job_format_add_min_cores( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				job_format_add_min_cores( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
 			else if (field[0] == 'j')
-				job_format_add_name( params.format_list, 
-				                     field_size, 
-				                     right_justify, suffix );
+				job_format_add_name( params.format_list,
+						     field_size,
+						     right_justify, suffix );
 			else if (field[0] == 'J')
-				job_format_add_min_threads( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				job_format_add_min_threads( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
+			else if (field[0] == 'k')
+				job_format_add_comment( params.format_list,
+							field_size,
+							right_justify,
+							suffix );
 			else if (field[0] == 'l')
-				job_format_add_time_limit( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				job_format_add_time_limit( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
+			else if (field[0] == 'L')
+				job_format_add_time_left( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix );
 			else if (field[0] == 'm')
-				job_format_add_min_memory( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				job_format_add_min_memory( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
 			else if (field[0] == 'M')
-				job_format_add_time_used( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix );
+				job_format_add_time_used( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix );
 			else if (field[0] == 'n')
-				job_format_add_req_nodes( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix );
+				job_format_add_req_nodes( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix );
 			else if (field[0] == 'N')
-				job_format_add_nodes( params.format_list, 
-				                      field_size, 
-				                      right_justify, suffix );
-			else if (field[0] == 'o')
-				job_format_add_num_nodes( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix );
+				job_format_add_nodes( params.format_list,
+						      field_size,
+						      right_justify, suffix );
 			else if (field[0] == 'O')
-				job_format_add_contiguous( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				job_format_add_contiguous( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
 			else if (field[0] == 'p')
-				job_format_add_priority( params.format_list, 
-				                         field_size, 
-				                         right_justify, 
-				                         suffix );
+				job_format_add_priority( params.format_list,
+							 field_size,
+							 right_justify,
+							 suffix );
 			else if (field[0] == 'P')
-				job_format_add_partition( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix );
+				job_format_add_partition( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix );
 			else if (field[0] == 'q')
-				job_format_add_comment( params.format_list, 
-				                        field_size, 
-				                        right_justify, 
-				                        suffix );
+				job_format_add_qos( params.format_list,
+						    field_size,
+						    right_justify,
+						    suffix );
 			else if (field[0] == 'Q')
-				 job_format_add_priority_long( params.format_list,
+				 job_format_add_priority_long(
+							params.format_list,
 							field_size,
 							right_justify,
 							suffix );
@@ -600,73 +655,64 @@ extern int parse_format( char* format )
 							right_justify,
 							suffix );
 			else if (field[0] == 'R')
-				job_format_add_reason_list(  params.format_list,
+				job_format_add_reason_list( params.format_list,
 							field_size,
 							right_justify,
 							suffix );
 			else if (field[0] == 's')
-				job_format_add_select_jobinfo( params.format_list, 
-				                         field_size, 
-				                         right_justify, 
-				                         suffix );
+				job_format_add_select_jobinfo(
+							 params.format_list,
+							 field_size,
+							 right_justify,
+							 suffix );
 			else if (field[0] == 'S')
-				job_format_add_time_start( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				job_format_add_time_start( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
 			else if (field[0] == 't')
-				job_format_add_job_state_compact( 
-							params.format_list, 
-							field_size, 
-							right_justify, 
-				                        suffix );
+				job_format_add_job_state_compact(
+							params.format_list,
+							field_size,
+							right_justify,
+							suffix );
 			else if (field[0] == 'T')
-				job_format_add_job_state( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix );
+				job_format_add_job_state( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix );
 			else if (field[0] == 'U')
-				job_format_add_user_id( params.format_list, 
-				                        field_size, 
-				                        right_justify, 
-				                        suffix );
+				job_format_add_user_id( params.format_list,
+							field_size,
+							right_justify,
+							suffix );
 			else if (field[0] == 'u')
-				job_format_add_user_name( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix );
+				job_format_add_user_name( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix );
+			else if (field[0] == 'v')
+				job_format_add_reservation( params.format_list,
+							field_size,
+							right_justify,
+							suffix );
 			else if (field[0] == 'w')
-				job_format_add_wckey( params.format_list, 
-						      field_size, 
-						      right_justify, 
+				job_format_add_wckey( params.format_list,
+						      field_size,
+						      right_justify,
 						      suffix );
 			else if (field[0] == 'x')
-				job_format_add_exc_nodes( params.format_list, 
-				                          field_size, 
-				                          right_justify, 
-				                          suffix );
-			else if (field[0] == 'X')
-				job_format_add_num_sockets( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
-			else if (field[0] == 'Y')
-				job_format_add_num_cores( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
-			else if (field[0] == 'Z')
-				job_format_add_num_threads( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
+				job_format_add_exc_nodes( params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix );
 			else if (field[0] == 'z')
-				job_format_add_num_sct( params.format_list, 
-				                           field_size, 
-				                           right_justify, 
-				                           suffix );
-			else 
-				error( "Invalid job format specification: %c", 
+				job_format_add_num_sct( params.format_list,
+							   field_size,
+							   right_justify,
+							   suffix );
+			else
+				error( "Invalid job format specification: %c",
 				       field[0] );
 		}
 		token = strtok_r( NULL, "%", &tmp_char);
@@ -685,7 +731,7 @@ _get_prefix( char *token )
 {
 	char *pos, *prefix;
 
-	if (token == NULL) 
+	if (token == NULL)
 		return NULL;
 
 	pos = strchr(token, (int) '%');
@@ -709,7 +755,7 @@ _get_prefix( char *token )
  * OUT suffix - tring containing everthing after the field specification
  */
 static void
-_parse_token( char *token, char *field, int *field_size, bool *right_justify, 
+_parse_token( char *token, char *field, int *field_size, bool *right_justify,
 	      char **suffix)
 {
 	int i = 0;
@@ -745,7 +791,7 @@ _print_options()
 	char hostlist[8192];
 
 	if (params.nodes) {
-		hostset_ranged_string(params.nodes, sizeof(hostlist)-1, 
+		hostset_ranged_string(params.nodes, sizeof(hostlist)-1,
 				      hostlist);
 	} else
 		hostlist[0] = '\0';
@@ -760,10 +806,11 @@ _print_options()
 	printf( "nodes      = %s\n", hostlist ) ;
 	printf( "partitions = %s\n", params.partitions ) ;
 	printf( "sort       = %s\n", params.sort ) ;
+	printf( "start_flag = %d\n", params.start_flag );
 	printf( "states     = %s\n", params.states ) ;
 	printf( "step_flag  = %d\n", params.step_flag );
-	printf( "steps      = %s\n", params.steps );	
-	printf( "users      = %s\n", params.users );	
+	printf( "steps      = %s\n", params.steps );
+	printf( "users      = %s\n", params.users );
 	printf( "verbose    = %d\n", params.verbose );
 
 	if ((params.verbose > 1) && params.job_list) {
@@ -788,7 +835,7 @@ _print_options()
 		i = 0;
 		iterator = list_iterator_create( params.state_list );
 		while ( (state_id = list_next( iterator )) ) {
-			printf( "state_list[%d] = %s\n", 
+			printf( "state_list[%d] = %s\n",
 				i++, job_state_string( *state_id ));
 		}
 		list_iterator_destroy( iterator );
@@ -798,7 +845,7 @@ _print_options()
 		i = 0;
 		iterator = list_iterator_create( params.step_list );
 		while ( (job_step_id = list_next( iterator )) ) {
-			printf( "step_list[%d] = %u.%u\n", i++, 
+			printf( "step_list[%d] = %u.%u\n", i++,
 				job_step_id->job_id, job_step_id->step_id );
 		}
 		list_iterator_destroy( iterator );
@@ -822,7 +869,7 @@ _print_options()
  * IN str - comma separated list of job_ids
  * RET List of job_ids (uint32_t)
  */
-static List 
+static List
 _build_job_list( char* str )
 {
 	List my_list;
@@ -830,13 +877,12 @@ _build_job_list( char* str )
 	int i;
 	uint32_t *job_id = NULL;
 
-	if ( str == NULL)
+	if ( str == NULL )
 		return NULL;
 	my_list = list_create( NULL );
 	my_job_list = xstrdup( str );
 	job = strtok_r( my_job_list, ",", &tmp_char );
-	while (job) 
-	{
+	while (job) {
 		i = strtol( job, (char **) NULL, 10 );
 		if (i <= 0) {
 			error( "Invalid job id: %s", job );
@@ -855,7 +901,7 @@ _build_job_list( char* str )
  * IN str - comma separated list of strings
  * RET List of strings
  */
-static List 
+static List
 _build_str_list( char* str )
 {
 	List my_list;
@@ -866,7 +912,7 @@ _build_str_list( char* str )
 	my_list = list_create( NULL );
 	my_part_list = xstrdup( str );
 	part = strtok_r( my_part_list, ",", &tmp_char );
-	while (part) 
+	while (part)
 	{
 		list_append( my_list, part );
 		part = strtok_r( NULL, ",", &tmp_char );
@@ -879,7 +925,7 @@ _build_str_list( char* str )
  * IN str - comma separated list of job states
  * RET List of enum job_states values
  */
-static List 
+static List
 _build_state_list( char* str )
 {
 	List my_list;
@@ -894,9 +940,9 @@ _build_state_list( char* str )
 	my_list = list_create( NULL );
 	my_state_list = xstrdup( str );
 	state = strtok_r( my_state_list, ",", &tmp_char );
-	while (state) 
+	while (state)
 	{
-		state_id = xmalloc( sizeof( enum job_states ) );
+		state_id = xmalloc( sizeof( uint16_t ) );
 		if ( _parse_state( state, state_id ) != SLURM_SUCCESS )
 		{
 			exit( 1 );
@@ -910,23 +956,26 @@ _build_state_list( char* str )
 
 /*
  * _build_all_states_list - build a list containing all possible job states
- * RET List of enum job_states values
+ * RET List of uint16_t values
  */
-static List 
+static List
 _build_all_states_list( void )
 {
 	List my_list;
 	int i;
-	enum job_states * state_id;
+	uint16_t * state_id;
 
 	my_list = list_create( NULL );
 	for (i = 0; i<JOB_END; i++) {
-		state_id = xmalloc( sizeof( enum job_states ) );
-		*state_id = ( enum job_states ) i;
+		state_id = xmalloc( sizeof(uint16_t) );
+		*state_id = (uint16_t) i;
 		list_append( my_list, state_id );
 	}
-	state_id = xmalloc( sizeof( enum job_states ) );
-	*state_id = ( enum job_states ) JOB_COMPLETING;
+	state_id = xmalloc( sizeof(uint16_t) );
+	*state_id = (uint16_t) JOB_COMPLETING;
+	list_append( my_list, state_id );
+	state_id = xmalloc( sizeof(uint16_t) );
+	*state_id = (uint16_t) JOB_CONFIGURING;
 	list_append( my_list, state_id );
 	return my_list;
 
@@ -937,7 +986,7 @@ _build_all_states_list( void )
  * IN str - comma separated list of job_id.step_ids
  * RET List of job/step_ids (structure of uint32_t's)
  */
-static List 
+static List
 _build_step_list( char* str )
 {
 	List my_list;
@@ -951,19 +1000,19 @@ _build_step_list( char* str )
 	my_list = list_create( NULL );
 	my_step_list = xstrdup( str );
 	step = strtok_r( my_step_list, ",", &tmp_char );
-	while (step) 
+	while (step)
 	{
 		job_name = strtok_r( step, ".", &tmps_char );
 		step_name = strtok_r( NULL, ".", &tmps_char );
 		i = strtol( job_name, (char **) NULL, 10 );
 		if (step_name == NULL) {
-			error ( "Invalid job_step id: %s.??", 
-			         job_name );
+			error ( "Invalid job_step id: %s.??",
+				 job_name );
 			exit( 1 );
 		}
 		j = strtol( step_name, (char **) NULL, 10 );
 		if ((i <= 0) || (j < 0)) {
-			error( "Invalid job_step id: %s.%s", 
+			error( "Invalid job_step id: %s.%s",
 				job_name, step_name );
 			exit( 1 );
 		}
@@ -981,7 +1030,7 @@ _build_step_list( char* str )
  * IN str - comma separated list of user names
  * RET List of UIDs (uint32_t)
  */
-static List 
+static List
 _build_user_list( char* str )
 {
 	List my_list;
@@ -1015,40 +1064,42 @@ _build_user_list( char* str )
 	return my_list;
 }
 
-static void _print_version(void)
-{
-	printf("%s %s\n", PACKAGE, SLURM_VERSION);
-}
-
 static void _usage(void)
 {
-	printf("Usage: squeue [-i seconds] [-S fields] [-t states] [-p partitions]\n");
-	printf("              [-n node] [-o format] [-u user_name] [--usage] [-ahjlsv]\n");
+	printf("\
+Usage: squeue [-i seconds] [-S fields] [--start] [-t states]\n\
+	      [-p partitions] [-n node] [-o format] [-u user_name]\n\
+	      [--usage] [-ahjlsv]\n");
 }
 
 static void _help(void)
 {
 	printf("\
 Usage: squeue [OPTIONS]\n\
+  -A, --account=account(s)        comma separated list of accounts\n\
+				  to view, default is all accounts\n\
   -a, --all                       display jobs in hidden partitions\n\
   -h, --noheader                  no headers on output\n\
-  --hide                          do not display jobs in hidden partitions\n\
+      --hide                      do not display jobs in hidden partitions\n\
   -i, --iterate=seconds           specify an interation period\n\
-  -j, --jobs                      comma separated list of jobs\n\
-                                  to view, default is all\n\
+  -j, --job=job(s)                comma separated list of jobs IDs\n\
+				  to view, default is all\n\
   -l, --long                      long report\n\
   -n, --nodes=hostlist            list of nodes to view, default is \n\
-                                  all nodes\n\
+				  all nodes\n\
   -o, --format=format             format specification\n\
-  -p, --partitions=partitions     comma separated list of partitions\n\
-                                  to view, default is all partitions\n\
-  -s, --steps                     comma separated list of job steps\n\
-                                  to view, default is all\n\
-  -S, --sort=fields               comma seperated list of fields to sort on\n\
-  -t, --states=states             comma seperated list of states to view,\n\
-                                  default is pending and running,\n\
-                                  '--states=all' reports all states\n\
-  -u, --user=user_name            comma separated list of users to view\n\
+  -p, --partition=partition(s)    comma separated list of partitions\n\
+				  to view, default is all partitions\n\
+  -q, --qos=qos(s)                comma separated list of qos's\n\
+				  to view, default is all qos's\n\
+  -s, --step=step(s)              comma separated list of job steps\n\
+				  to view, default is all\n\
+  -S, --sort=fields               comma separated list of fields to sort on\n\
+      --start                     print expected start times of pending jobs\n\
+  -t, --states=states             comma separated list of states to view,\n\
+				  default is pending and running,\n\
+				  '--states=all' reports all states\n\
+  -u, --user=user_name(s)         comma separated list of users to view\n\
   -v, --verbose                   verbosity level\n\
   -V, --version                   output version information and exit\n\
 \nHelp options:\n\
