@@ -6,32 +6,33 @@
  *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -46,6 +47,7 @@ enum {
 	PRINT_CLUSTER_ACPU,
 	PRINT_CLUSTER_DCPU,
 	PRINT_CLUSTER_ICPU,
+	PRINT_CLUSTER_PDCPU,
 	PRINT_CLUSTER_OCPU,
 	PRINT_CLUSTER_RCPU,
 	PRINT_CLUSTER_TOTAL,
@@ -103,7 +105,7 @@ static int _set_wckey_cond(int *start, int argc, char *argv[],
 			}
 		}
 
-		if(!end && !strncasecmp(argv[i], "all_clusters", 
+		if(!end && !strncasecmp(argv[i], "all_clusters",
 					       MAX(command_len, 1))) {
 			local_cluster_flag = 1;
 		} else if(!end && !strncasecmp(argv[i], "withdeleted",
@@ -134,7 +136,7 @@ static int _set_wckey_cond(int *start, int argc, char *argv[],
 					 MAX(command_len, 1))) {
 			if(format_list)
 				slurm_addto_char_list(format_list, argv[i]+end);
-		} else if (!strncasecmp (argv[i], "Start", 
+		} else if (!strncasecmp (argv[i], "Start",
 					 MAX(command_len, 1))) {
 			wckey_cond->usage_start = parse_time(argv[i]+end, 1);
 			set = 1;
@@ -150,7 +152,7 @@ static int _set_wckey_cond(int *start, int argc, char *argv[],
 			exit_code=1;
 			fprintf(stderr, " Unknown condition: %s\n", argv[i]);
 		}
-	}	
+	}
 
 	(*start) = i;
 
@@ -207,17 +209,17 @@ static int _set_assoc_cond(int *start, int argc, char *argv[],
 			}
 		}
 
-		if(!end && !strncasecmp(argv[i], "all_clusters", 
+		if(!end && !strncasecmp(argv[i], "all_clusters",
 					       MAX(command_len, 1))) {
 			local_cluster_flag = 1;
 		} else if (!end && !strncasecmp (argv[i], "Tree",
 						 MAX(command_len, 4))) {
 			tree_display = 1;
 		} else if(!end
-			  || !strncasecmp (argv[i], "Users", 
+			  || !strncasecmp (argv[i], "Users",
 					   MAX(command_len, 1))) {
 			if(!assoc_cond->user_list)
-				assoc_cond->user_list = 
+				assoc_cond->user_list =
 					list_create(slurm_destroy_char);
 			slurm_addto_char_list(assoc_cond->user_list,
 					      argv[i]+end);
@@ -230,7 +232,7 @@ static int _set_assoc_cond(int *start, int argc, char *argv[],
 			slurm_addto_char_list(assoc_cond->acct_list,
 					argv[i]+end);
 			set = 1;
-		} else if (!strncasecmp (argv[i], "Clusters", 
+		} else if (!strncasecmp (argv[i], "Clusters",
 					 MAX(command_len, 1))) {
 			slurm_addto_char_list(assoc_cond->cluster_list,
 					argv[i]+end);
@@ -241,7 +243,7 @@ static int _set_assoc_cond(int *start, int argc, char *argv[],
 		} else if (!strncasecmp (argv[i], "Format",
 					 MAX(command_len, 1))) {
 			if(format_list)
-				slurm_addto_char_list(format_list, 
+				slurm_addto_char_list(format_list,
 						      argv[i]+end);
 		} else if (!strncasecmp (argv[i], "Start",
 					 MAX(command_len, 1))) {
@@ -320,7 +322,7 @@ static int _set_cluster_cond(int *start, int argc, char *argv[],
 		} else if (!strncasecmp (argv[i], "End", MAX(command_len, 1))) {
 			cluster_cond->usage_end = parse_time(argv[i]+end, 1);
 			set = 1;
-		} else if (!strncasecmp (argv[i], "Format", 
+		} else if (!strncasecmp (argv[i], "Format",
 					 MAX(command_len, 1))) {
 			if(format_list)
 				slurm_addto_char_list(format_list,
@@ -377,11 +379,11 @@ static int _setup_print_fields_list(List format_list)
 		char *tmp_char = NULL;
 		int command_len = 0;
 		int newlen = 0;
-		
+
 		if((tmp_char = strstr(object, "\%"))) {
 			newlen = atoi(tmp_char+1);
 			tmp_char[0] = '\0';
-		} 
+		}
 
 		command_len = strlen(object);
 
@@ -390,11 +392,11 @@ static int _setup_print_fields_list(List format_list)
 			field->type = PRINT_CLUSTER_ACCT;
 			field->name = xstrdup("Account");
 			if(tree_display)
-				field->len = 20;
+				field->len = -20;
 			else
 				field->len = 15;
 			field->print_routine = print_fields_str;
-		} else if(!strncasecmp("allocated", object, 
+		} else if(!strncasecmp("allocated", object,
 				       MAX(command_len, 2))) {
 			field->type = PRINT_CLUSTER_ACPU;
 			field->name = xstrdup("Allocated");
@@ -405,13 +407,13 @@ static int _setup_print_fields_list(List format_list)
 			else
 				field->len = 12;
 			field->print_routine = sreport_print_time;
-		} else if(!strncasecmp("Cluster", object, 
+		} else if(!strncasecmp("Cluster", object,
 				       MAX(command_len, 2))) {
 			field->type = PRINT_CLUSTER_NAME;
 			field->name = xstrdup("Cluster");
 			field->len = 9;
 			field->print_routine = print_fields_str;
-		} else if(!strncasecmp("cpu_count", object, 
+		} else if(!strncasecmp("cpucount", object,
 				       MAX(command_len, 2))) {
 			field->type = PRINT_CLUSTER_CPUS;
 			field->name = xstrdup("CPU count");
@@ -442,7 +444,7 @@ static int _setup_print_fields_list(List format_list)
 			field->name = xstrdup("Login");
 			field->len = 9;
 			field->print_routine = print_fields_str;
-		} else if(!strncasecmp("overcommited", object, 
+		} else if(!strncasecmp("overcommited", object,
 				       MAX(command_len, 1))) {
 			field->type = PRINT_CLUSTER_OCPU;
 			field->name = xstrdup("Over Comm");
@@ -453,7 +455,18 @@ static int _setup_print_fields_list(List format_list)
 			else
 				field->len = 9;
 			field->print_routine = sreport_print_time;
-		} else if(!strncasecmp("Proper", object, MAX(command_len, 1))) {
+		} else if(!strncasecmp("PlannedDown", object,
+				       MAX(command_len, 2))) {
+			field->type = PRINT_CLUSTER_PDCPU;
+			field->name = xstrdup("PLND Down");
+			if(time_format == SREPORT_TIME_SECS_PER
+			   || time_format == SREPORT_TIME_MINS_PER
+			   || time_format == SREPORT_TIME_HOURS_PER)
+				field->len = 18;
+			else
+				field->len = 10;
+			field->print_routine = sreport_print_time;
+		} else if(!strncasecmp("Proper", object, MAX(command_len, 2))) {
 			field->type = PRINT_CLUSTER_USER_PROPER;
 			field->name = xstrdup("Proper Name");
 			field->len = 15;
@@ -505,10 +518,10 @@ static int _setup_print_fields_list(List format_list)
 			continue;
 		}
 
-		if(newlen > 0) 
+		if(newlen)
 			field->len = newlen;
-		
-		list_append(print_fields_list, field);		
+
+		list_append(print_fields_list, field);
 	}
 	list_iterator_destroy(itr);
 
@@ -518,7 +531,7 @@ static int _setup_print_fields_list(List format_list)
 static List _get_cluster_list(int argc, char *argv[], uint32_t *total_time,
 			      char *report_name, List format_list)
 {
-	acct_cluster_cond_t *cluster_cond = 
+	acct_cluster_cond_t *cluster_cond =
 		xmalloc(sizeof(acct_cluster_cond_t));
 	int i=0;
 	List cluster_list = NULL;
@@ -527,7 +540,7 @@ static List _get_cluster_list(int argc, char *argv[], uint32_t *total_time,
 	cluster_cond->with_usage = 1;
 
 	_set_cluster_cond(&i, argc, argv, cluster_cond, format_list);
-	
+
 	cluster_list = acct_storage_g_get_clusters(db_conn, my_uid,
 						   cluster_cond);
 	if(!cluster_list) {
@@ -542,19 +555,20 @@ static List _get_cluster_list(int argc, char *argv[], uint32_t *total_time,
 		time_t my_start = cluster_cond->usage_start;
 		time_t my_end = cluster_cond->usage_end-1;
 
-		slurm_make_time_str(&my_start, 
+		slurm_make_time_str(&my_start,
 				    start_char, sizeof(start_char));
 		slurm_make_time_str(&my_end,
 				    end_char, sizeof(end_char));
 		printf("----------------------------------------"
 		       "----------------------------------------\n");
-		printf("%s %s - %s (%d*cpus secs)\n", 
-		       report_name, start_char, end_char, 
-		       (cluster_cond->usage_end - cluster_cond->usage_start));
+		printf("%s %s - %s (%d*cpus secs)\n",
+		       report_name, start_char, end_char,
+		       (int)(cluster_cond->usage_end
+			     - cluster_cond->usage_start));
 		switch(time_format) {
 		case SREPORT_TIME_PERCENT:
 			printf("Time reported in %s\n", time_format_string);
-			break; 
+			break;
 		default:
 			printf("Time reported in CPU %s\n", time_format_string);
 			break;
@@ -565,14 +579,14 @@ static List _get_cluster_list(int argc, char *argv[], uint32_t *total_time,
 	(*total_time) = cluster_cond->usage_end - cluster_cond->usage_start;
 
 	destroy_acct_cluster_cond(cluster_cond);
-	
+
 	return cluster_list;
 }
 
 extern int cluster_account_by_user(int argc, char *argv[])
 {
 	int rc = SLURM_SUCCESS;
-	acct_association_cond_t *assoc_cond = 
+	acct_association_cond_t *assoc_cond =
 		xmalloc(sizeof(acct_association_cond_t));
 	acct_cluster_cond_t cluster_cond;
 	ListIterator itr = NULL;
@@ -596,14 +610,15 @@ extern int cluster_account_by_user(int argc, char *argv[])
 
 	print_fields_list = list_create(destroy_print_field);
 
-	bzero(&cluster_cond, sizeof(acct_cluster_cond_t));
+	memset(&cluster_cond, 0, sizeof(acct_cluster_cond_t));
 
 	assoc_cond->with_sub_accts = 1;
 
 	_set_assoc_cond(&i, argc, argv, assoc_cond, format_list);
 
-	if(!list_count(format_list)) 
-		slurm_addto_char_list(format_list, "Cluster,Ac,L,P,Used");
+	if(!list_count(format_list))
+		slurm_addto_char_list(format_list,
+				      "Cluster,Ac,Login,Proper,Used");
 
 	_setup_print_fields_list(format_list);
 	list_destroy(format_list);
@@ -649,23 +664,23 @@ extern int cluster_account_by_user(int argc, char *argv[])
 		list_append(sreport_cluster_list, sreport_cluster);
 
 		sreport_cluster->name = xstrdup(cluster->name);
-		sreport_cluster->assoc_list = 
+		sreport_cluster->assoc_list =
 			list_create(destroy_sreport_assoc_rec);
 
 		/* get the amount of time and the average cpu count
 		   during the time we are looking at */
 		itr2 = list_iterator_create(cluster->accounting_list);
 		while((accting = list_next(itr2))) {
-			sreport_cluster->cpu_secs += accting->alloc_secs 
-				+ accting->down_secs + accting->idle_secs 
+			sreport_cluster->cpu_secs += accting->alloc_secs
+				+ accting->down_secs + accting->idle_secs
 				+ accting->resv_secs;
 			sreport_cluster->cpu_count += accting->cpu_count;
 		}
 		list_iterator_destroy(itr2);
 
-		sreport_cluster->cpu_count /= 
+		sreport_cluster->cpu_count /=
 			list_count(cluster->accounting_list);
-		
+
 		/* now add the associations of interest here by user */
 		while((assoc = list_next(assoc_itr))) {
 			acct_accounting_rec_t *accting2 = NULL;
@@ -676,28 +691,28 @@ extern int cluster_account_by_user(int argc, char *argv[])
 				continue;
 			}
 
-			if(strcmp(cluster->name, assoc->cluster)) 
+			if(strcmp(cluster->name, assoc->cluster))
 				continue;
 
 			sreport_assoc = xmalloc(sizeof(sreport_assoc_rec_t));
-			
-			list_append(sreport_cluster->assoc_list, 
+
+			list_append(sreport_cluster->assoc_list,
 				    sreport_assoc);
 
 			sreport_assoc->acct = xstrdup(assoc->acct);
 			sreport_assoc->parent_acct =
 				xstrdup(assoc->parent_acct);
 			sreport_assoc->user = xstrdup(assoc->user);
-				
+
 			/* get the amount of time this assoc used
 			   during the time we are looking at */
 			itr2 = list_iterator_create(assoc->accounting_list);
 			while((accting2 = list_next(itr2))) {
-				sreport_assoc->cpu_secs += 
+				sreport_assoc->cpu_secs +=
 					(uint64_t)accting2->alloc_secs;
 			}
 			list_iterator_destroy(itr2);
-			list_delete_item(assoc_itr);		
+			list_delete_item(assoc_itr);
 		}
 		list_iterator_reset(assoc_itr);
 	}
@@ -716,21 +731,21 @@ extern int cluster_account_by_user(int argc, char *argv[])
 		char end_char[20];
 		time_t my_start = assoc_cond->usage_start;
 		time_t my_end = assoc_cond->usage_end-1;
-		
+
 		slurm_make_time_str(&my_start, start_char, sizeof(start_char));
 		slurm_make_time_str(&my_end, end_char, sizeof(end_char));
 		printf("----------------------------------------"
 		       "----------------------------------------\n");
-		printf("Cluster/Account/User Utilization %s - %s (%d secs)\n", 
-		       start_char, end_char, 
-		       (assoc_cond->usage_end - assoc_cond->usage_start));
-		
+		printf("Cluster/Account/User Utilization %s - %s (%d secs)\n",
+		       start_char, end_char,
+		       (int)(assoc_cond->usage_end - assoc_cond->usage_start));
+
 		switch(time_format) {
 		case SREPORT_TIME_PERCENT:
 			printf("Time reported in %s\n", time_format_string);
-			break; 
+			break;
 		default:
-			printf("Time reported in CPU %s\n", 
+			printf("Time reported in CPU %s\n",
 			       time_format_string);
 			break;
 		}
@@ -743,16 +758,16 @@ extern int cluster_account_by_user(int argc, char *argv[])
 
 	field_count = list_count(print_fields_list);
 	list_sort(sreport_cluster_list, (ListCmpF)sort_cluster_dec);
-	
+
 	cluster_itr = list_iterator_create(sreport_cluster_list);
 	while((sreport_cluster = list_next(cluster_itr))) {
-		//list_sort(sreport_cluster->assoc_list, 
+		//list_sort(sreport_cluster->assoc_list,
 		//  (ListCmpF)sort_assoc_dec);
-		if(tree_list) 
+		if(tree_list)
 			list_flush(tree_list);
-		else 
+		else
 			tree_list = list_create(destroy_acct_print_tree);
-		
+
 		itr = list_iterator_create(sreport_cluster->assoc_list);
 		while((sreport_assoc = list_next(itr))) {
 			int curr_inx = 1;
@@ -769,7 +784,7 @@ extern int cluster_account_by_user(int argc, char *argv[])
 						if(sreport_assoc->user) {
 							local_acct =
 								xstrdup_printf(
-									"|%s", 
+									"|%s",
 									sreport_assoc->acct);
 							parent_acct =
 								sreport_assoc->acct;
@@ -782,7 +797,6 @@ extern int cluster_account_by_user(int argc, char *argv[])
 						print_acct = get_tree_acct_name(
 							local_acct,
 							parent_acct,
-							sreport_cluster->name,
 							tree_list);
 						xfree(local_acct);
 					} else {
@@ -790,10 +804,10 @@ extern int cluster_account_by_user(int argc, char *argv[])
 							sreport_assoc->acct;
 					}
 					field->print_routine(
-						field, 
+						field,
 						print_acct,
 						(curr_inx == field_count));
-					
+
 					break;
 				case PRINT_CLUSTER_NAME:
 					field->print_routine(
@@ -812,16 +826,16 @@ extern int cluster_account_by_user(int argc, char *argv[])
 						pwd = getpwnam(
 							sreport_assoc->user);
 					if(pwd) {
-						tmp_char = 
+						tmp_char =
 							strtok(pwd->pw_gecos,
 							       ",");
 						if(!tmp_char)
-							tmp_char = 
+							tmp_char =
 								pwd->pw_gecos;
 					}
 					field->print_routine(field,
 							     tmp_char,
-							     (curr_inx == 
+							     (curr_inx ==
 							      field_count));
 					break;
 				case PRINT_CLUSTER_AMOUNT_USED:
@@ -847,27 +861,27 @@ extern int cluster_account_by_user(int argc, char *argv[])
 	list_iterator_destroy(cluster_itr);
 end_it:
 	destroy_acct_association_cond(assoc_cond);
-	
+
 	if(assoc_list) {
 		list_destroy(assoc_list);
 		assoc_list = NULL;
 	}
-	
+
 	if(first_list) {
 		list_destroy(first_list);
 		first_list = NULL;
 	}
-	
+
 	if(cluster_list) {
 		list_destroy(cluster_list);
 		cluster_list = NULL;
 	}
-	
+
 	if(sreport_cluster_list) {
 		list_destroy(sreport_cluster_list);
 		sreport_cluster_list = NULL;
 	}
-	
+
 	if(print_fields_list) {
 		list_destroy(print_fields_list);
 		print_fields_list = NULL;
@@ -900,12 +914,13 @@ extern int cluster_user_by_account(int argc, char *argv[])
 
 	print_fields_list = list_create(destroy_print_field);
 
-	bzero(&cluster_cond, sizeof(acct_cluster_cond_t));
+	memset(&cluster_cond, 0, sizeof(acct_cluster_cond_t));
 
 	_set_assoc_cond(&i, argc, argv, assoc_cond, format_list);
 
-	if(!list_count(format_list)) 
-		slurm_addto_char_list(format_list, "Cluster,L,P,Ac,Used");
+	if(!list_count(format_list))
+		slurm_addto_char_list(format_list,
+				      "Cluster,Login,Proper,Ac,Used");
 
 	_setup_print_fields_list(format_list);
 	list_destroy(format_list);
@@ -948,23 +963,23 @@ extern int cluster_user_by_account(int argc, char *argv[])
 		list_append(sreport_cluster_list, sreport_cluster);
 
 		sreport_cluster->name = xstrdup(cluster->name);
-		sreport_cluster->user_list = 
+		sreport_cluster->user_list =
 			list_create(destroy_sreport_user_rec);
 
 		/* get the amount of time and the average cpu count
 		   during the time we are looking at */
 		itr2 = list_iterator_create(cluster->accounting_list);
 		while((accting = list_next(itr2))) {
-			sreport_cluster->cpu_secs += accting->alloc_secs 
-				+ accting->down_secs + accting->idle_secs 
+			sreport_cluster->cpu_secs += accting->alloc_secs
+				+ accting->down_secs + accting->idle_secs
 				+ accting->resv_secs;
 			sreport_cluster->cpu_count += accting->cpu_count;
 		}
 		list_iterator_destroy(itr2);
 
-		sreport_cluster->cpu_count /= 
+		sreport_cluster->cpu_count /=
 			list_count(cluster->accounting_list);
-		
+
 		/* now add the associations of interest here by user */
 		while((assoc = list_next(assoc_itr))) {
 			struct passwd *passwd_ptr = NULL;
@@ -979,7 +994,7 @@ extern int cluster_user_by_account(int argc, char *argv[])
 				continue;
 			}
 
-			if(strcmp(cluster->name, assoc->cluster)) 
+			if(strcmp(cluster->name, assoc->cluster))
 				continue;
 
 			/* make sure we add all associations to this
@@ -988,17 +1003,17 @@ extern int cluster_user_by_account(int argc, char *argv[])
 			   record otherwise
 			*/
 			user_itr = list_iterator_create(
-				sreport_cluster->user_list); 
+				sreport_cluster->user_list);
 			while((sreport_user = list_next(user_itr))) {
 				if(!strcmp(sreport_user->name, assoc->user)
 				   && !strcmp(sreport_user->acct, assoc->acct))
-					break;				
+					break;
 			}
 			list_iterator_destroy(user_itr);
 
 			if(!sreport_user) {
 				passwd_ptr = getpwnam(assoc->user);
-				if(passwd_ptr) 
+				if(passwd_ptr)
 					uid = passwd_ptr->pw_uid;
 				/* In this report we are using the sreport user
 				   structure to store the information we want
@@ -1010,16 +1025,16 @@ extern int cluster_user_by_account(int argc, char *argv[])
 				sreport_user->name = xstrdup(assoc->user);
 				sreport_user->uid = uid;
 				sreport_user->acct = xstrdup(assoc->acct);
-	
+
 				list_append(sreport_cluster->user_list,
 					    sreport_user);
-			} 
+			}
 
 			/* get the amount of time this assoc used
 			   during the time we are looking at */
 			itr2 = list_iterator_create(assoc->accounting_list);
 			while((accting2 = list_next(itr2))) {
-				sreport_user->cpu_secs += 
+				sreport_user->cpu_secs +=
 					(uint64_t)accting2->alloc_secs;
 			}
 			list_iterator_destroy(itr2);
@@ -1040,21 +1055,21 @@ extern int cluster_user_by_account(int argc, char *argv[])
 		char end_char[20];
 		time_t my_start = assoc_cond->usage_start;
 		time_t my_end = assoc_cond->usage_end-1;
-		
+
 		slurm_make_time_str(&my_start, start_char, sizeof(start_char));
 		slurm_make_time_str(&my_end, end_char, sizeof(end_char));
 		printf("----------------------------------------"
 		       "----------------------------------------\n");
-		printf("Cluster/User/Account Utilization %s - %s (%d secs)\n", 
-		       start_char, end_char, 
-		       (assoc_cond->usage_end - assoc_cond->usage_start));
-		
+		printf("Cluster/User/Account Utilization %s - %s (%d secs)\n",
+		       start_char, end_char,
+		       (int)(assoc_cond->usage_end - assoc_cond->usage_start));
+
 		switch(time_format) {
 		case SREPORT_TIME_PERCENT:
 			printf("Time reported in %s\n", time_format_string);
-			break; 
+			break;
 		default:
-			printf("Time reported in CPU %s\n", 
+			printf("Time reported in CPU %s\n",
 			       time_format_string);
 			break;
 		}
@@ -1069,7 +1084,7 @@ extern int cluster_user_by_account(int argc, char *argv[])
 	cluster_itr = list_iterator_create(sreport_cluster_list);
 	while((sreport_cluster = list_next(cluster_itr))) {
 		list_sort(sreport_cluster->user_list, (ListCmpF)sort_user_dec);
-	
+
 		itr = list_iterator_create(sreport_cluster->user_list);
 		while((sreport_user = list_next(itr))) {
 			int curr_inx = 1;
@@ -1103,16 +1118,16 @@ extern int cluster_user_by_account(int argc, char *argv[])
 				case PRINT_CLUSTER_USER_PROPER:
 					pwd = getpwnam(sreport_user->name);
 					if(pwd) {
-						tmp_char = 
+						tmp_char =
 							strtok(pwd->pw_gecos,
 							       ",");
 						if(!tmp_char)
-							tmp_char = 
+							tmp_char =
 								pwd->pw_gecos;
 					}
 					field->print_routine(field,
 							     tmp_char,
-							     (curr_inx == 
+							     (curr_inx ==
 							      field_count));
 					break;
 				case PRINT_CLUSTER_AMOUNT_USED:
@@ -1138,22 +1153,22 @@ extern int cluster_user_by_account(int argc, char *argv[])
 	list_iterator_destroy(cluster_itr);
 end_it:
 	destroy_acct_association_cond(assoc_cond);
-	
+
 	if(assoc_list) {
 		list_destroy(assoc_list);
 		assoc_list = NULL;
 	}
-	
+
 	if(cluster_list) {
 		list_destroy(cluster_list);
 		cluster_list = NULL;
 	}
-	
+
 	if(sreport_cluster_list) {
 		list_destroy(sreport_cluster_list);
 		sreport_cluster_list = NULL;
 	}
-	
+
 	if(print_fields_list) {
 		list_destroy(print_fields_list);
 		print_fields_list = NULL;
@@ -1186,12 +1201,13 @@ extern int cluster_user_by_wckey(int argc, char *argv[])
 
 	print_fields_list = list_create(destroy_print_field);
 
-	bzero(&cluster_cond, sizeof(acct_cluster_cond_t));
+	memset(&cluster_cond, 0, sizeof(acct_cluster_cond_t));
 
 	_set_wckey_cond(&i, argc, argv, wckey_cond, format_list);
 
-	if(!list_count(format_list)) 
-		slurm_addto_char_list(format_list, "Cluster,L,P,WCkey,Used");
+	if(!list_count(format_list))
+		slurm_addto_char_list(format_list,
+				      "Cluster,Login,Proper,WCkey,Used");
 
 	_setup_print_fields_list(format_list);
 	list_destroy(format_list);
@@ -1234,23 +1250,23 @@ extern int cluster_user_by_wckey(int argc, char *argv[])
 		list_append(sreport_cluster_list, sreport_cluster);
 
 		sreport_cluster->name = xstrdup(cluster->name);
-		sreport_cluster->user_list = 
+		sreport_cluster->user_list =
 			list_create(destroy_sreport_user_rec);
 
 		/* get the amount of time and the average cpu count
 		   during the time we are looking at */
 		itr2 = list_iterator_create(cluster->accounting_list);
 		while((accting = list_next(itr2))) {
-			sreport_cluster->cpu_secs += accting->alloc_secs 
-				+ accting->down_secs + accting->idle_secs 
+			sreport_cluster->cpu_secs += accting->alloc_secs
+				+ accting->down_secs + accting->idle_secs
 				+ accting->resv_secs;
 			sreport_cluster->cpu_count += accting->cpu_count;
 		}
 		list_iterator_destroy(itr2);
 
-		sreport_cluster->cpu_count /= 
+		sreport_cluster->cpu_count /=
 			list_count(cluster->accounting_list);
-		
+
 		/* now add the wckeys of interest here by user */
 		while((wckey = list_next(wckey_itr))) {
 			struct passwd *passwd_ptr = NULL;
@@ -1264,11 +1280,11 @@ extern int cluster_user_by_wckey(int argc, char *argv[])
 				continue;
 			}
 
-			if(strcmp(cluster->name, wckey->cluster)) 
+			if(strcmp(cluster->name, wckey->cluster))
 				continue;
 
 			passwd_ptr = getpwnam(wckey->user);
-			if(passwd_ptr) 
+			if(passwd_ptr)
 				uid = passwd_ptr->pw_uid;
 			/* In this report we are using the sreport user
 			   structure to store the information we want
@@ -1279,15 +1295,15 @@ extern int cluster_user_by_wckey(int argc, char *argv[])
 			sreport_user->name = xstrdup(wckey->user);
 			sreport_user->uid = uid;
 			sreport_user->acct = xstrdup(wckey->name);
-			
+
 			list_append(sreport_cluster->user_list,
 				    sreport_user);
-			
+
 			/* get the amount of time this wckey used
 			   during the time we are looking at */
 			itr2 = list_iterator_create(wckey->accounting_list);
 			while((accting2 = list_next(itr2))) {
-				sreport_user->cpu_secs += 
+				sreport_user->cpu_secs +=
 					(uint64_t)accting2->alloc_secs;
 			}
 			list_iterator_destroy(itr2);
@@ -1308,21 +1324,21 @@ extern int cluster_user_by_wckey(int argc, char *argv[])
 		char end_char[20];
 		time_t my_start = wckey_cond->usage_start;
 		time_t my_end = wckey_cond->usage_end-1;
-		
+
 		slurm_make_time_str(&my_start, start_char, sizeof(start_char));
 		slurm_make_time_str(&my_end, end_char, sizeof(end_char));
 		printf("----------------------------------------"
 		       "----------------------------------------\n");
-		printf("Cluster/User/WCKey Utilization %s - %s (%d secs)\n", 
-		       start_char, end_char, 
-		       (wckey_cond->usage_end - wckey_cond->usage_start));
-		
+		printf("Cluster/User/WCKey Utilization %s - %s (%d secs)\n",
+		       start_char, end_char,
+		       (int)(wckey_cond->usage_end - wckey_cond->usage_start));
+
 		switch(time_format) {
 		case SREPORT_TIME_PERCENT:
 			printf("Time reported in %s\n", time_format_string);
-			break; 
+			break;
 		default:
-			printf("Time reported in CPU %s\n", 
+			printf("Time reported in CPU %s\n",
 			       time_format_string);
 			break;
 		}
@@ -1337,7 +1353,7 @@ extern int cluster_user_by_wckey(int argc, char *argv[])
 	cluster_itr = list_iterator_create(sreport_cluster_list);
 	while((sreport_cluster = list_next(cluster_itr))) {
 		list_sort(sreport_cluster->user_list, (ListCmpF)sort_user_dec);
-	
+
 		itr = list_iterator_create(sreport_cluster->user_list);
 		while((sreport_user = list_next(itr))) {
 			int curr_inx = 1;
@@ -1371,16 +1387,16 @@ extern int cluster_user_by_wckey(int argc, char *argv[])
 				case PRINT_CLUSTER_USER_PROPER:
 					pwd = getpwnam(sreport_user->name);
 					if(pwd) {
-						tmp_char = 
+						tmp_char =
 							strtok(pwd->pw_gecos,
 							       ",");
 						if(!tmp_char)
-							tmp_char = 
+							tmp_char =
 								pwd->pw_gecos;
 					}
 					field->print_routine(field,
 							     tmp_char,
-							     (curr_inx == 
+							     (curr_inx ==
 							      field_count));
 					break;
 				case PRINT_CLUSTER_AMOUNT_USED:
@@ -1406,22 +1422,22 @@ extern int cluster_user_by_wckey(int argc, char *argv[])
 	list_iterator_destroy(cluster_itr);
 end_it:
 	destroy_acct_wckey_cond(wckey_cond);
-	
+
 	if(wckey_list) {
 		list_destroy(wckey_list);
 		wckey_list = NULL;
 	}
-	
+
 	if(cluster_list) {
 		list_destroy(cluster_list);
 		cluster_list = NULL;
 	}
-	
+
 	if(sreport_cluster_list) {
 		list_destroy(sreport_cluster_list);
 		sreport_cluster_list = NULL;
 	}
-	
+
 	if(print_fields_list) {
 		list_destroy(print_fields_list);
 		print_fields_list = NULL;
@@ -1441,7 +1457,7 @@ extern int cluster_utilization(int argc, char *argv[])
 	print_field_t *field = NULL;
 	uint32_t total_time = 0;
 
-	List cluster_list = NULL; 
+	List cluster_list = NULL;
 
 	List format_list = list_create(slurm_destroy_char);
 	int field_count = 0;
@@ -1451,11 +1467,11 @@ extern int cluster_utilization(int argc, char *argv[])
 
 	if(!(cluster_list = _get_cluster_list(argc, argv, &total_time,
 					      "Cluster Utilization",
-					      format_list))) 
+					      format_list)))
 		goto end_it;
 
-	if(!list_count(format_list)) 
-		slurm_addto_char_list(format_list, "Cl,al,d,i,res,rep");
+	if(!list_count(format_list))
+		slurm_addto_char_list(format_list, "Cl,al,d,planned,i,res,rep");
 
 	_setup_print_fields_list(format_list);
 	list_destroy(format_list);
@@ -1479,11 +1495,12 @@ extern int cluster_utilization(int argc, char *argv[])
 			continue;
 
 		memset(&total_acct, 0, sizeof(cluster_accounting_rec_t));
-		
+
 		itr3 = list_iterator_create(cluster->accounting_list);
 		while((accting = list_next(itr3))) {
 			total_acct.alloc_secs += accting->alloc_secs;
 			total_acct.down_secs += accting->down_secs;
+			total_acct.pdown_secs += accting->pdown_secs;
 			total_acct.idle_secs += accting->idle_secs;
 			total_acct.resv_secs += accting->resv_secs;
 			total_acct.over_secs += accting->over_secs;
@@ -1492,66 +1509,74 @@ extern int cluster_utilization(int argc, char *argv[])
 		list_iterator_destroy(itr3);
 
 		total_acct.cpu_count /= list_count(cluster->accounting_list);
-		
+
 		local_total_time =
 			(uint64_t)total_time * (uint64_t)total_acct.cpu_count;
-		total_reported = total_acct.alloc_secs + total_acct.down_secs 
-			+ total_acct.idle_secs + total_acct.resv_secs;
+		total_reported = total_acct.alloc_secs + total_acct.down_secs
+			+ total_acct.pdown_secs + total_acct.idle_secs
+			+ total_acct.resv_secs;
 
 		while((field = list_next(itr2))) {
 			switch(field->type) {
 			case PRINT_CLUSTER_NAME:
 				field->print_routine(field,
 						     cluster->name,
-						     (curr_inx == 
+						     (curr_inx ==
 						      field_count));
 				break;
 			case PRINT_CLUSTER_CPUS:
 				field->print_routine(field,
 						     total_acct.cpu_count,
-						     (curr_inx == 
+						     (curr_inx ==
 						      field_count));
 				break;
 			case PRINT_CLUSTER_ACPU:
 				field->print_routine(field,
 						     total_acct.alloc_secs,
 						     total_reported,
-						     (curr_inx == 
+						     (curr_inx ==
 						      field_count));
 				break;
 			case PRINT_CLUSTER_DCPU:
 				field->print_routine(field,
 						     total_acct.down_secs,
 						     total_reported,
-						     (curr_inx == 
+						     (curr_inx ==
 						      field_count));
 				break;
 			case PRINT_CLUSTER_ICPU:
 				field->print_routine(field,
 						     total_acct.idle_secs,
 						     total_reported,
-						     (curr_inx == 
+						     (curr_inx ==
 						      field_count));
 				break;
 			case PRINT_CLUSTER_RCPU:
 				field->print_routine(field,
 						     total_acct.resv_secs,
 						     total_reported,
-						     (curr_inx == 
+						     (curr_inx ==
 						      field_count));
 				break;
 			case PRINT_CLUSTER_OCPU:
 					field->print_routine(field,
 						     total_acct.over_secs,
 						     total_reported,
-						     (curr_inx == 
+						     (curr_inx ==
+						      field_count));
+				break;
+			case PRINT_CLUSTER_PDCPU:
+					field->print_routine(field,
+						     total_acct.pdown_secs,
+						     total_reported,
+						     (curr_inx ==
 						      field_count));
 				break;
 			case PRINT_CLUSTER_TOTAL:
 				field->print_routine(field,
 						     total_reported,
 						     local_total_time,
-						     (curr_inx == 
+						     (curr_inx ==
 						      field_count));
 				break;
 			default:
@@ -1574,7 +1599,7 @@ end_it:
 		list_destroy(cluster_list);
 		cluster_list = NULL;
 	}
-	
+
 	if(print_fields_list) {
 		list_destroy(print_fields_list);
 		print_fields_list = NULL;
@@ -1586,7 +1611,7 @@ end_it:
 extern int cluster_wckey_by_user(int argc, char *argv[])
 {
 	int rc = SLURM_SUCCESS;
-	acct_wckey_cond_t *wckey_cond = 
+	acct_wckey_cond_t *wckey_cond =
 		xmalloc(sizeof(acct_wckey_cond_t));
 	acct_cluster_cond_t cluster_cond;
 	ListIterator itr = NULL;
@@ -1605,7 +1630,6 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 	sreport_cluster_rec_t *sreport_cluster = NULL;
 	print_field_t *field = NULL;
 	int field_count = 0;
-	char *print_acct = NULL;
 
 	print_fields_list = list_create(destroy_print_field);
 
@@ -1613,8 +1637,9 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 
 	_set_wckey_cond(&i, argc, argv, wckey_cond, format_list);
 
-	if(!list_count(format_list)) 
-		slurm_addto_char_list(format_list, "Cluster,WCKey,L,P,Used");
+	if(!list_count(format_list))
+		slurm_addto_char_list(format_list,
+				      "Cluster,WCKey,Login,Proper,Used");
 
 	_setup_print_fields_list(format_list);
 	list_destroy(format_list);
@@ -1656,23 +1681,23 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 		list_append(sreport_cluster_list, sreport_cluster);
 
 		sreport_cluster->name = xstrdup(cluster->name);
-		sreport_cluster->assoc_list = 
+		sreport_cluster->assoc_list =
 			list_create(destroy_sreport_assoc_rec);
 
 		/* get the amount of time and the average cpu count
 		   during the time we are looking at */
 		itr2 = list_iterator_create(cluster->accounting_list);
 		while((accting = list_next(itr2))) {
-			sreport_cluster->cpu_secs += accting->alloc_secs 
-				+ accting->down_secs + accting->idle_secs 
+			sreport_cluster->cpu_secs += accting->alloc_secs
+				+ accting->down_secs + accting->idle_secs
 				+ accting->resv_secs;
 			sreport_cluster->cpu_count += accting->cpu_count;
 		}
 		list_iterator_destroy(itr2);
 
-		sreport_cluster->cpu_count /= 
+		sreport_cluster->cpu_count /=
 			list_count(cluster->accounting_list);
-		
+
 		/* now add the wckeys of interest here by user */
 		while((wckey = list_next(wckey_itr))) {
 			acct_accounting_rec_t *accting2 = NULL;
@@ -1685,14 +1710,14 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 				continue;
 			}
 
-			if(strcmp(cluster->name, wckey->cluster)) 
+			if(strcmp(cluster->name, wckey->cluster))
 				continue;
 
 			/* find the parent */
 			par_itr = list_iterator_create(
 				sreport_cluster->assoc_list);
 			while((parent_assoc = list_next(par_itr))) {
-				if(!parent_assoc->user 
+				if(!parent_assoc->user
 				   && !strcmp(parent_assoc->acct, wckey->name))
 					break;
 			}
@@ -1700,15 +1725,15 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 			if(!parent_assoc) {
 				parent_assoc = xmalloc(
 					sizeof(sreport_assoc_rec_t));
-			
-				list_append(sreport_cluster->assoc_list, 
+
+				list_append(sreport_cluster->assoc_list,
 					    parent_assoc);
 				parent_assoc->acct = xstrdup(wckey->name);
 			}
 
 			sreport_assoc = xmalloc(sizeof(sreport_assoc_rec_t));
-			
-			list_append(sreport_cluster->assoc_list, 
+
+			list_append(sreport_cluster->assoc_list,
 				    sreport_assoc);
 
 			sreport_assoc->acct = xstrdup(wckey->name);
@@ -1718,13 +1743,13 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 			   during the time we are looking at */
 			itr2 = list_iterator_create(wckey->accounting_list);
 			while((accting2 = list_next(itr2))) {
-				sreport_assoc->cpu_secs += 
+				sreport_assoc->cpu_secs +=
 					(uint64_t)accting2->alloc_secs;
-				parent_assoc->cpu_secs += 
+				parent_assoc->cpu_secs +=
 					(uint64_t)accting2->alloc_secs;
 			}
 			list_iterator_destroy(itr2);
-			list_delete_item(wckey_itr);		
+			list_delete_item(wckey_itr);
 		}
 		list_iterator_reset(wckey_itr);
 	}
@@ -1741,21 +1766,21 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 		char end_char[20];
 		time_t my_start = wckey_cond->usage_start;
 		time_t my_end = wckey_cond->usage_end-1;
-		
+
 		slurm_make_time_str(&my_start, start_char, sizeof(start_char));
 		slurm_make_time_str(&my_end, end_char, sizeof(end_char));
 		printf("----------------------------------------"
 		       "----------------------------------------\n");
-		printf("Cluster/WCKey/User Utilization %s - %s (%d secs)\n", 
-		       start_char, end_char, 
-		       (wckey_cond->usage_end - wckey_cond->usage_start));
-		
+		printf("Cluster/WCKey/User Utilization %s - %s (%d secs)\n",
+		       start_char, end_char,
+		       (int)(wckey_cond->usage_end - wckey_cond->usage_start));
+
 		switch(time_format) {
 		case SREPORT_TIME_PERCENT:
 			printf("Time reported in %s\n", time_format_string);
-			break; 
+			break;
 		default:
-			printf("Time reported in CPU %s\n", 
+			printf("Time reported in CPU %s\n",
 			       time_format_string);
 			break;
 		}
@@ -1768,16 +1793,16 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 
 	field_count = list_count(print_fields_list);
 	list_sort(sreport_cluster_list, (ListCmpF)sort_cluster_dec);
-	
+
 	cluster_itr = list_iterator_create(sreport_cluster_list);
 	while((sreport_cluster = list_next(cluster_itr))) {
-		//list_sort(sreport_cluster->wckey_list, 
+		//list_sort(sreport_cluster->wckey_list,
 		//  (ListCmpF)sort_wckey_dec);
-		if(tree_list) 
+		if(tree_list)
 			list_flush(tree_list);
-		else 
+		else
 			tree_list = list_create(destroy_acct_print_tree);
-		
+
 		itr = list_iterator_create(sreport_cluster->assoc_list);
 		while((sreport_assoc = list_next(itr))) {
 			int curr_inx = 1;
@@ -1788,37 +1813,11 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 				struct passwd *pwd = NULL;
 				switch(field->type) {
 				case PRINT_CLUSTER_WCKEY:
-					if(tree_display) {
-						char *local_acct = NULL;
-						char *parent_acct = NULL;
-						if(sreport_assoc->user) {
-							local_acct =
-								xstrdup_printf(
-									"|%s", 
-									sreport_assoc->acct);
-							parent_acct =
-								sreport_assoc->acct;
-						} else {
-							local_acct = xstrdup(
-								sreport_assoc->acct);
-							parent_acct = sreport_assoc->
-								parent_acct;
-						}
-						print_acct = get_tree_acct_name(
-							local_acct,
-							parent_acct,
-							sreport_cluster->name,
-							tree_list);
-						xfree(local_acct);
-					} else {
-						print_acct =
-							sreport_assoc->acct;
-					}
 					field->print_routine(
-						field, 
-						print_acct,
+						field,
+						sreport_assoc->acct,
 						(curr_inx == field_count));
-					
+
 					break;
 				case PRINT_CLUSTER_NAME:
 					field->print_routine(
@@ -1837,16 +1836,16 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 						pwd = getpwnam(
 							sreport_assoc->user);
 					if(pwd) {
-						tmp_char = 
+						tmp_char =
 							strtok(pwd->pw_gecos,
 							       ",");
 						if(!tmp_char)
-							tmp_char = 
+							tmp_char =
 								pwd->pw_gecos;
 					}
 					field->print_routine(field,
 							     tmp_char,
-							     (curr_inx == 
+							     (curr_inx ==
 							      field_count));
 					break;
 				case PRINT_CLUSTER_AMOUNT_USED:
@@ -1872,22 +1871,22 @@ extern int cluster_wckey_by_user(int argc, char *argv[])
 	list_iterator_destroy(cluster_itr);
 end_it:
 	destroy_acct_wckey_cond(wckey_cond);
-	
+
 	if(wckey_list) {
 		list_destroy(wckey_list);
 		wckey_list = NULL;
 	}
-	
+
 	if(cluster_list) {
 		list_destroy(cluster_list);
 		cluster_list = NULL;
 	}
-	
+
 	if(sreport_cluster_list) {
 		list_destroy(sreport_cluster_list);
 		sreport_cluster_list = NULL;
 	}
-	
+
 	if(print_fields_list) {
 		list_destroy(print_fields_list);
 		print_fields_list = NULL;
