@@ -5,32 +5,33 @@
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -40,8 +41,10 @@
 #include "src/sreport/assoc_reports.h"
 #include "src/sreport/cluster_reports.h"
 #include "src/sreport/job_reports.h"
+#include "src/sreport/resv_reports.h"
 #include "src/sreport/user_reports.h"
 #include "src/common/xsignal.h"
+#include "src/common/proc_args.h"
 
 #define OPT_LONG_HIDE   0x102
 #define BUFFER_SIZE 4096
@@ -60,6 +63,7 @@ sreport_sort_t sort_flag = SREPORT_SORT_TIME;
 
 static void	_job_rep (int argc, char *argv[]);
 static void	_user_rep (int argc, char *argv[]);
+static void	_resv_rep (int argc, char *argv[]);
 static void	_cluster_rep (int argc, char *argv[]);
 static void	_assoc_rep (int argc, char *argv[]);
 static int	_get_command (int *argc, char *argv[]);
@@ -69,8 +73,8 @@ static int      _set_time_format(char *format);
 static int      _set_sort(char *format);
 static void	_usage ();
 
-int 
-main (int argc, char *argv[]) 
+int
+main (int argc, char *argv[])
 {
 	int error_code = SLURM_SUCCESS, i, opt_char, input_field_count;
 	char **input_fields;
@@ -81,11 +85,11 @@ main (int argc, char *argv[])
 		{"all_clusters", 0, 0, 'a'},
 		{"help",     0, 0, 'h'},
 		{"immediate",0, 0, 'i'},
-		{"no_header", 0, 0, 'n'},
+		{"noheader", 0, 0, 'n'},
 		{"parsable", 0, 0, 'p'},
-		{"parsable2", 0, 0, 'P'},
-		{"quiet",    0, 0, 'q'},
-		{"sort",    0, 0, 's'},
+		{"parsable2",0, 0, 'P'},
+		{"quiet",    0, 0, 'Q'},
+		{"sort",     0, 0, 's'},
 		{"usage",    0, 0, 'h'},
 		{"verbose",  0, 0, 'v'},
 		{"version",  0, 0, 'V'},
@@ -113,7 +117,7 @@ main (int argc, char *argv[])
 	}
 	xfree(temp);
 
-	while((opt_char = getopt_long(argc, argv, "ahnpPqs:t:vV",
+	while((opt_char = getopt_long(argc, argv, "ahnpPQs:t:vV",
 			long_options, &option_index)) != -1) {
 		switch (opt_char) {
 		case (int)'?':
@@ -132,14 +136,14 @@ main (int argc, char *argv[])
 			print_fields_have_header = 0;
 			break;
 		case (int)'p':
-			print_fields_parsable_print = 
+			print_fields_parsable_print =
 			PRINT_FIELDS_PARSABLE_ENDING;
 			break;
 		case (int)'P':
 			print_fields_parsable_print =
 			PRINT_FIELDS_PARSABLE_NO_ENDING;
 			break;
-		case (int)'q':
+		case (int)'Q':
 			quiet_flag = 1;
 			break;
 		case (int)'s':
@@ -157,7 +161,7 @@ main (int argc, char *argv[])
 			break;
 		default:
 			exit_code = 1;
-			fprintf(stderr, "getopt error, returned %c\n", 
+			fprintf(stderr, "getopt error, returned %c\n",
 				opt_char);
 			exit(exit_code);
 		}
@@ -171,10 +175,14 @@ main (int argc, char *argv[])
 	if (optind < argc) {
 		for (i = optind; i < argc; i++) {
 			input_fields[input_field_count++] = argv[i];
-		}	
+		}
 	}
 
 	db_conn = acct_storage_g_get_connection(false, 0, false);
+	if(errno) {
+		error("Problem talking to the database: %m");
+		exit(1);
+	}
 	my_uid = getuid();
 
 	if (input_field_count)
@@ -182,7 +190,7 @@ main (int argc, char *argv[])
 	else
 		error_code = _get_command (&input_field_count, input_fields);
 	while (error_code == SLURM_SUCCESS) {
-		error_code = _process_command (input_field_count, 
+		error_code = _process_command (input_field_count,
 					       input_fields);
 		if (error_code || exit_flag)
 			break;
@@ -198,15 +206,16 @@ main (int argc, char *argv[])
 /*
  * Alternative to readline if readline is not available
  */
-static char *
-getline(const char *prompt)
+static char *_getline(const char *prompt)
 {
 	char buf[4096];
 	char *line;
 	int len;
 	printf("%s", prompt);
 
-	fgets(buf, 4096, stdin);
+	/* we only set this here to avoid a warning.  We throw it away
+	   later. */
+	line = fgets(buf, 4096, stdin);
 	len = strlen(buf);
 	if ((len > 0) && (buf[len-1] == '\n'))
 		buf[len-1] = '\0';
@@ -217,12 +226,12 @@ getline(const char *prompt)
 }
 #endif
 
-/* 
- * _job_rep - Reports having to do with jobs 
+/*
+ * _job_rep - Reports having to do with jobs
  * IN argc - count of arguments
  * IN argv - list of arguments
  */
-static void _job_rep (int argc, char *argv[]) 
+static void _job_rep (int argc, char *argv[])
 {
 	int error_code = SLURM_SUCCESS;
 	int command_len = strlen(argv[0]);
@@ -232,7 +241,7 @@ static void _job_rep (int argc, char *argv[])
 	if (!strncasecmp (argv[0], "SizesByAccount", MAX(command_len, 1))) {
 		error_code = job_sizes_grouped_by_top_acct(
 			(argc - 1), &argv[1]);
-	} else if (!strncasecmp (argv[0], 
+	} else if (!strncasecmp (argv[0],
 				 "SizesByWcKey", MAX(command_len, 8))) {
 		error_code = job_sizes_grouped_by_wckey(
 			(argc - 1), &argv[1]);
@@ -242,18 +251,18 @@ static void _job_rep (int argc, char *argv[])
 		fprintf(stderr, "Valid job reports are, ");
 		fprintf(stderr, "\"SizesByAccount, and SizesByWckey\"\n");
 	}
-	
+
 	if (error_code) {
 		exit_code = 1;
 	}
 }
 
-/* 
- * _user_rep - Reports having to do with jobs 
+/*
+ * _user_rep - Reports having to do with users
  * IN argc - count of arguments
  * IN argv - list of arguments
  */
-static void _user_rep (int argc, char *argv[]) 
+static void _user_rep (int argc, char *argv[])
 {
 	int error_code = SLURM_SUCCESS;
 
@@ -264,19 +273,42 @@ static void _user_rep (int argc, char *argv[])
 		fprintf(stderr, "Not valid report %s\n", argv[0]);
 		fprintf(stderr, "Valid user reports are, ");
 		fprintf(stderr, "\"Top\"\n");
-	}	
-	
+	}
+
 	if (error_code) {
 		exit_code = 1;
 	}
 }
 
-/* 
- * _cluster_rep - Reports having to do with jobs 
+/*
+ * _resv_rep - Reports having to do with reservations
  * IN argc - count of arguments
  * IN argv - list of arguments
  */
-static void _cluster_rep (int argc, char *argv[]) 
+static void _resv_rep (int argc, char *argv[])
+{
+	int error_code = SLURM_SUCCESS;
+
+	if (strncasecmp (argv[0], "Utilization", 1) == 0) {
+		error_code = resv_utilization((argc - 1), &argv[1]);
+	} else {
+		exit_code = 1;
+		fprintf(stderr, "Not valid report %s\n", argv[0]);
+		fprintf(stderr, "Valid reservation reports are, ");
+		fprintf(stderr, "\"Utilization\"\n");
+	}
+
+	if (error_code) {
+		exit_code = 1;
+	}
+}
+
+/*
+ * _cluster_rep - Reports having to do with clusters
+ * IN argc - count of arguments
+ * IN argv - list of arguments
+ */
+static void _cluster_rep (int argc, char *argv[])
 {
 	int error_code = SLURM_SUCCESS;
 
@@ -301,18 +333,18 @@ static void _cluster_rep (int argc, char *argv[])
 			"\"UserUtilizationByWckey\", \"Utilization\", "
 			"and \"WCKeyUtilizationByUser\"\n");
 	}
-	
+
 	if (error_code) {
 		exit_code = 1;
 	}
 }
 
-/* 
- * _assoc_rep - Reports having to do with jobs 
+/*
+ * _assoc_rep - Reports having to do with jobs
  * IN argc - count of arguments
  * IN argv - list of arguments
  */
-static void _assoc_rep (int argc, char *argv[]) 
+static void _assoc_rep (int argc, char *argv[])
 {
 	int error_code = SLURM_SUCCESS;
 
@@ -326,8 +358,8 @@ static void _assoc_rep (int argc, char *argv[])
  * OUT argc - location to store count of arguments
  * OUT argv - location to store the argument list
  */
-static int 
-_get_command (int *argc, char **argv) 
+static int
+_get_command (int *argc, char **argv)
 {
 	char *in_line;
 	static char *last_in_line = NULL;
@@ -339,7 +371,7 @@ _get_command (int *argc, char **argv)
 #if HAVE_READLINE
 	in_line = readline ("sreport: ");
 #else
-	in_line = getline("sreport: ");
+	in_line = _getline("sreport: ");
 #endif
 	if (in_line == NULL)
 		return 0;
@@ -370,11 +402,11 @@ _get_command (int *argc, char **argv)
 			continue;
 		if (((*argc) + 1) > MAX_INPUT_FIELDS) {	/* bogus input line */
 			exit_code = 1;
-			fprintf (stderr, 
+			fprintf (stderr,
 				 "%s: can not process over %d words\n",
 				 command_name, input_words);
 			return E2BIG;
-		}		
+		}
 		argv[(*argc)++] = &in_line[i];
 		for (i++; i < in_line_size; i++) {
 			if (in_line[i] == '\042') {
@@ -393,19 +425,19 @@ _get_command (int *argc, char **argv)
 				in_line[i] = '\0';
 				break;
 			}
-		}		
+		}
 	}
-	return 0;		
+	return 0;
 }
 
 
 static void _print_version(void)
 {
-	printf("%s %s\n", PACKAGE, SLURM_VERSION);
+	print_slurm_version ();
 	if (quiet_flag == -1) {
 		long version = slurm_api_version();
 		printf("slurm_api_version: %ld, %ld.%ld.%ld\n", version,
-			SLURM_VERSION_MAJOR(version), 
+			SLURM_VERSION_MAJOR(version),
 			SLURM_VERSION_MINOR(version),
 			SLURM_VERSION_MICRO(version));
 	}
@@ -418,7 +450,7 @@ static void _print_version(void)
  * RET 0 or errno (only for errors fatal to sreport)
  */
 static int
-_process_command (int argc, char *argv[]) 
+_process_command (int argc, char *argv[])
 {
 	int command_len = 0;
 
@@ -428,32 +460,32 @@ _process_command (int argc, char *argv[])
 			fprintf(stderr, "no input");
 		return 0;
 	}
-	
+
 	command_len = strlen(argv[0]);
 
 	if ((strncasecmp (argv[0], "association", MAX(command_len, 1)) == 0)) {
 		if (argc < 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
-				fprintf(stderr, 
-				        "too few arguments for keyword:%s\n", 
+				fprintf(stderr,
+				        "too few arguments for keyword:%s\n",
 				        argv[0]);
-		} else 
+		} else
 			_assoc_rep((argc - 1), &argv[1]);
 	} else if ((strncasecmp (argv[0], "cluster",
 				 MAX(command_len, 2)) == 0)) {
 		if (argc < 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
-				fprintf(stderr, 
-				        "too few arguments for keyword:%s\n", 
+				fprintf(stderr,
+				        "too few arguments for keyword:%s\n",
 				        argv[0]);
-		} else 
+		} else
 			_cluster_rep((argc - 1), &argv[1]);
 	} else if (strncasecmp (argv[0], "help", MAX(command_len, 2)) == 0) {
 		if (argc > 1) {
 			exit_code = 1;
-			fprintf (stderr, 
+			fprintf (stderr,
 				 "too many arguments for keyword:%s\n",
 				 argv[0]);
 		}
@@ -462,10 +494,10 @@ _process_command (int argc, char *argv[])
 		if (argc < 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
-				fprintf(stderr, 
-				        "too few arguments for keyword:%s\n", 
+				fprintf(stderr,
+				        "too few arguments for keyword:%s\n",
 				        argv[0]);
-		} else 
+		} else
 			_job_rep((argc - 1), &argv[1]);
 	} else if (strncasecmp (argv[0], "quiet", MAX(command_len, 4)) == 0) {
 		if (argc > 1) {
@@ -479,18 +511,30 @@ _process_command (int argc, char *argv[])
 		   (strncasecmp (argv[0], "quit", MAX(command_len, 4)) == 0)) {
 		if (argc > 1) {
 			exit_code = 1;
-			fprintf (stderr, 
-				 "too many arguments for keyword:%s\n", 
+			fprintf (stderr,
+				 "too many arguments for keyword:%s\n",
 				 argv[0]);
 		}
 		exit_flag = 1;
+	} else if ((strncasecmp (argv[0], "reservation",
+				 MAX(command_len, 2)) == 0)
+		   || (strncasecmp (argv[0], "resv",
+				    MAX(command_len, 2)) == 0)) {
+		if (argc < 2) {
+			exit_code = 1;
+			if (quiet_flag != 1)
+				fprintf(stderr,
+				        "too few arguments for keyword:%s\n",
+				        argv[0]);
+		} else
+			_resv_rep((argc - 1), &argv[1]);
 	} else if (strncasecmp (argv[0], "sort", MAX(command_len, 1)) == 0) {
 		if (argc < 2) {
 			exit_code = 1;
 			fprintf (stderr,
 				 "too few arguments for keyword:%s\n",
 				 argv[0]);
-		} else		
+		} else
 			_set_sort(argv[1]);
 	} else if (strncasecmp (argv[0], "time", MAX(command_len, 1)) == 0) {
 		if (argc < 2) {
@@ -498,7 +542,7 @@ _process_command (int argc, char *argv[])
 			fprintf (stderr,
 				 "too few arguments for keyword:%s\n",
 				 argv[0]);
-		} else		
+		} else
 			_set_time_format(argv[1]);
 	} else if (strncasecmp (argv[0], "verbose", MAX(command_len, 4)) == 0) {
 		if (argc > 1) {
@@ -506,7 +550,7 @@ _process_command (int argc, char *argv[])
 			fprintf (stderr,
 				 "too many arguments for %s keyword\n",
 				 argv[0]);
-		}		
+		}
 		quiet_flag = -1;
 	} else if (strncasecmp (argv[0], "version", MAX(command_len, 4)) == 0) {
 		if (argc > 1) {
@@ -514,22 +558,22 @@ _process_command (int argc, char *argv[])
 			fprintf (stderr,
 				 "too many arguments for %s keyword\n",
 				 argv[0]);
-		}		
+		}
 		_print_version();
 	} else if ((strncasecmp (argv[0], "user", MAX(command_len, 1)) == 0)) {
 		if (argc < 2) {
 			exit_code = 1;
 			if (quiet_flag != 1)
-				fprintf(stderr, 
-				        "too few arguments for keyword:%s\n", 
+				fprintf(stderr,
+				        "too few arguments for keyword:%s\n",
 				        argv[0]);
-		} else 
+		} else
 			_user_rep((argc - 1), &argv[1]);
 	} else {
 		exit_code = 1;
 		fprintf (stderr, "invalid keyword: %s\n", argv[0]);
 	}
-		
+
 	return 0;
 }
 
@@ -559,7 +603,7 @@ static int _set_time_format(char *format)
 		time_format = SREPORT_TIME_PERCENT;
 		time_format_string = "Percentage of Total";
 	} else {
-		fprintf (stderr, "unknown time format %s", format);	
+		fprintf (stderr, "unknown time format %s", format);
 		return SLURM_ERROR;
 	}
 
@@ -575,7 +619,7 @@ static int _set_sort(char *format)
 	} else if (strncasecmp (format, "Time", MAX(command_len, 6)) == 0) {
 		sort_flag = SREPORT_SORT_TIME;
 	} else {
-		fprintf (stderr, "unknown timesort format %s", format);	
+		fprintf (stderr, "unknown timesort format %s", format);
 		return SLURM_ERROR;
 	}
 
@@ -590,10 +634,10 @@ sreport [<OPTION>] [<COMMAND>]                                             \n\
     Valid <OPTION> values are:                                             \n\
      -a or --all_clusters: Use all clusters instead of current             \n\
      -h or --help: equivalent to \"help\" command                          \n\
-     -n or --no_header: equivalent to \"no_header\" command                \n\
+     -n or --noheader: equivalent to \"noheader\" command                \n\
      -p or --parsable: output will be '|' delimited with a '|' at the end  \n\
      -P or --parsable2: output will be '|' delimited without a '|' at the end\n\
-     -q or --quiet: equivalent to \"quiet\" command                        \n\
+     -Q or --quiet: equivalent to \"quiet\" command                        \n\
      -t <time_format>: Second, Minute, Hour, Percent, SecPer, MinPer, HourPer\n\
      -v or --verbose: equivalent to \"verbose\" command                    \n\
      -V or --version: equivalent to \"version\" command                    \n\
@@ -623,6 +667,8 @@ sreport [<OPTION>] [<COMMAND>]                                             \n\
      cluster - AccountUtilizationByUser, UserUtilizationByAccount,         \n\
                UserUtilizationByWckey, Utilization, WCKeyUtilizationByUser \n\
      job     - SizesByAccount, SizesByWckey                                \n\
+     reservation                                                           \n\
+             - Utilization                                                 \n\
      user    - TopUsage                                                    \n\
                                                                            \n\
   <OPTIONS> are different for each report type.                            \n\
@@ -674,6 +720,8 @@ sreport [<OPTION>] [<COMMAND>]                                             \n\
                                    1-49, 50-99, 100-149, > 150).           \n\
              - Jobs=<OPT>       - List of jobs/steps to include in report. \n\
                                   Default is all.                          \n\
+             - Nodes=<OPT>      - Only show jobs that ran on these nodes.  \n\
+                                  Default is all.                          \n\
              - Partitions=<OPT> - List of partitions jobs ran on to include\n\
                                   in report.  Default is all.              \n\
              - PrintJobCount    - When used with the any Sizes report      \n\
@@ -687,6 +735,12 @@ sreport [<OPTION>] [<COMMAND>]                                             \n\
                                   you want only certain users specify them \n\
                                   them with the Users= option.             \n\
                                                                            \n\
+     reservation                                                           \n\
+             - Names=<OPT>      - List of reservations to use for the report\n\
+                                  Default is all.                          \n\
+             - Nodes=<OPT>      - Only show reservations that used these   \n\
+                                  nodes.  Default is all.                  \n\
+                                                                           \n\
      user    - Accounts=<OPT>   - List of accounts to use for the report   \n\
                                   Default is all.                          \n\
              - Group            - Group all accounts together for each user.\n\
@@ -697,9 +751,11 @@ sreport [<OPTION>] [<COMMAND>]                                             \n\
              - Users=<OPT>      - List of users jobs to include in report. \n\
                                   Default is all.                          \n\
                                                                            \n\
-                                                                           \n\
   Below are the format options for each report.                            \n\
                                                                            \n\
+  One can get an number of characters by following the field option with   \n\
+  a %%NUMBER option.  i.e. format=name%%30 will print 30 chars of field name.\n\
+                                                                           \n\
        Cluster                                                             \n\
        - AccountUtilizationByUser                                          \n\
        - UserUtilizationByAccount                                          \n\
@@ -709,25 +765,30 @@ sreport [<OPTION>] [<COMMAND>]                                             \n\
              - Cluster, CPUCount, Login, Proper, Used, Wckey               \n\
        - Utilization                                                       \n\
              - Allocated, Cluster, CPUCount, Down, Idle, Overcommited,     \n\
-               Reported, Reserved                                          \n\
+               PlannedDown, Reported, Reserved                             \n\
                                                                            \n\
        Job                                                                 \n\
        - Sizes                                                             \n\
              - Account, Cluster                                            \n\
                                                                            \n\
+       Reservation                                                         \n\
+       - Utilization                                                       \n\
+             - Allocated, Associations, Cluster, CPUCount, CPUTime,        \n\
+               End, Idle, Name, Nodes, Start, TotalTime                    \n\
+                                                                           \n\
        User                                                                \n\
        - TopUsage                                                          \n\
              - Account, Cluster, Login, Proper, Used                       \n\
                                                                            \n\
                                                                            \n\
-                                                                           \n\
   Note, valid start/end time formats are...                                \n\
        HH:MM[:SS] [AM|PM]                                                  \n\
        MMDD[YY] or MM/DD[/YY] or MM.DD[.YY]                                \n\
        MM/DD[/YY]-HH:MM[:SS]                                               \n\
+       YYYY-MM-DD[THH:MM[:SS]]                                             \n\
                                                                            \n\
                                                                            \n\
   All commands and options are case-insensitive.                         \n\n");
-	
+
 }
 
