@@ -2,35 +2,36 @@
  *  sview.c - main for sview
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Copyright (C) 2008 Lawrence Livermore National Security.
+ *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>, et. al.
- *  LLNL-CODE-402394.
- *  
+ *  CODE-OCEC-09-009. All rights reserved.
+ *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <http://www.llnl.gov/linux/slurm/>.
- *  
+ *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
  *  SLURM is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
  *
- *  In addition, as a special exception, the copyright holders give permission 
+ *  In addition, as a special exception, the copyright holders give permission
  *  to link the code of portions of this program with the OpenSSL library under
- *  certain conditions as described in each individual source file, and 
- *  distribute linked combinations including the two. You must obey the GNU 
- *  General Public License in all respects for all of the code used other than 
- *  OpenSSL. If you modify file(s) with this exception, you may extend this 
- *  exception to your version of the file(s), but you are not obligated to do 
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
  *  so. If you do not wish to do so, delete this exception statement from your
- *  version.  If you delete this exception statement from all source files in 
+ *  version.  If you delete this exception statement from all source files in
  *  the program, then also delete it here.
- *  
+ *
  *  SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with SLURM; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -52,9 +53,15 @@ int fini = 0;
 int grid_init = 0;
 bool toggled = FALSE;
 bool force_refresh = FALSE;
-List popup_list;
+List popup_list = NULL;
+List signal_params_list = NULL;
 int page_running[PAGE_CNT];
 int global_sleep_time = 5;
+int global_x_width = 0;
+int global_horizontal = 10;
+int global_vertical = 10;
+bool global_entry_changed = 0;
+bool global_send_update_msg = 0;
 bool admin_mode = FALSE;
 GtkWidget *main_notebook = NULL;
 GtkWidget *main_statusbar = NULL;
@@ -64,37 +71,45 @@ GtkTable *main_grid_table = NULL;
 GStaticMutex sview_mutex = G_STATIC_MUTEX_INIT;
 GMutex *grid_mutex = NULL;
 GCond *grid_cond = NULL;
-GtkActionGroup *admin_action_group = NULL;
+int grid_speedup = 0;
+
+static GtkActionGroup *admin_action_group = NULL;
+static GtkActionGroup *menu_action_group = NULL;
+static bool debug_inited = 0;
 
 display_data_t main_display_data[] = {
 	{G_TYPE_NONE, JOB_PAGE, "Jobs", TRUE, -1,
 	 refresh_main, create_model_job, admin_edit_job,
-	 get_info_job, specific_info_job, 
+	 get_info_job, specific_info_job,
 	 set_menus_job, NULL},
 	{G_TYPE_NONE, STEP_PAGE, NULL, FALSE, -1,
 	 refresh_main, NULL, NULL, NULL,
 	 NULL, NULL, NULL},
-	{G_TYPE_NONE, PART_PAGE, "Partitions", TRUE, -1, 
+	{G_TYPE_NONE, PART_PAGE, "Partitions", TRUE, -1,
 	 refresh_main, create_model_part, admin_edit_part,
-	 get_info_part, specific_info_part, 
+	 get_info_part, specific_info_part,
 	 set_menus_part, NULL},
+	{G_TYPE_NONE, RESV_PAGE, "Reservations", TRUE, -1,
+	 refresh_main, create_model_resv, admin_edit_resv,
+	 get_info_resv, specific_info_resv,
+	 set_menus_resv, NULL},
 #ifdef HAVE_BG
 	{G_TYPE_NONE, BLOCK_PAGE, "BG Blocks", TRUE, -1,
 	 refresh_main, NULL, NULL,
-	 get_info_block, specific_info_block, 
+	 get_info_block, specific_info_block,
 	 set_menus_block, NULL},
 	{G_TYPE_NONE, NODE_PAGE, "Base Partitions", FALSE, -1,
 	 refresh_main, NULL, NULL,
-	 get_info_node, specific_info_node, 
+	 get_info_node, specific_info_node,
 	 set_menus_node, NULL},
 #else
 	{G_TYPE_NONE, BLOCK_PAGE, "BG Blocks", FALSE, -1,
 	 refresh_main, NULL, NULL,
-	 get_info_block, specific_info_block, 
+	 get_info_block, specific_info_block,
 	 set_menus_block, NULL},
 	{G_TYPE_NONE, NODE_PAGE, "Nodes", FALSE, -1,
 	 refresh_main, NULL, NULL,
-	 get_info_node, specific_info_node, 
+	 get_info_node, specific_info_node,
 	 set_menus_node, NULL},
 #endif
 	{G_TYPE_NONE, SUBMIT_PAGE, "Submit Job", FALSE, -1,
@@ -114,55 +129,58 @@ void *_page_thr(void *arg)
 	GtkTable *table = page->table;
 	display_data_t *display_data = &main_display_data[num];
 	static int thread_count = 0;
-
+	bool reset_highlight = true;
+	/* 	DEF_TIMERS; */
 	xfree(page);
-	
+
 	if(!grid_init) {
 		/* we need to signal any threads that are waiting */
 		g_mutex_lock(grid_mutex);
 		g_cond_signal(grid_cond);
 		g_mutex_unlock(grid_mutex);
-		
+
 		/* wait for the grid to be inited */
 		g_mutex_lock(grid_mutex);
 		g_cond_wait(grid_cond, grid_mutex);
 		g_mutex_unlock(grid_mutex);
-		
+
 		/* if the grid isn't there just return */
 		if(!grid_init)
 			return NULL;
 	}
 	gdk_threads_enter();
-	sview_reset_grid();
 	thread_count++;
 	gdk_flush();
 	gdk_threads_leave();
-	while(page_running[num]) {		
+	while(page_running[num]) {
+/* 		START_TIMER; */
 		g_static_mutex_lock(&sview_mutex);
 		gdk_threads_enter();
-		sview_init_grid();
+		sview_init_grid(reset_highlight);
+		reset_highlight=false;
 		(display_data->get_info)(table, display_data);
 		gdk_flush();
 		gdk_threads_leave();
 		g_static_mutex_unlock(&sview_mutex);
-				
+/* 		END_TIMER; */
+/* 		g_print("got for initeration: %s\n", TIME_STR); */
 		sleep(global_sleep_time);
-		
+
 		gdk_threads_enter();
 		if(thread_count > 1) {
 			gdk_flush();
-			gdk_threads_leave();	
+			gdk_threads_leave();
 			break;
 		}
 		gdk_flush();
 		gdk_threads_leave();
-	
-	}	
+
+	}
 	gdk_threads_enter();
 	thread_count--;
 	gdk_flush();
 	gdk_threads_leave();
-					
+
 	return NULL;
 }
 
@@ -174,8 +192,8 @@ void *_grid_init_thr(void *arg)
 	GtkViewport *view = NULL;
 	GtkTable *table = NULL;
 	int rc = SLURM_SUCCESS;
-		
-	while(!grid_init) {
+
+	while(!grid_init && !fini) {
 		gdk_threads_enter();
 		page = gtk_notebook_get_current_page(
 			GTK_NOTEBOOK(main_notebook));
@@ -190,17 +208,17 @@ void *_grid_init_thr(void *arg)
 		rc = get_system_stats(table);
 		gdk_flush();
 		gdk_threads_leave();
-	
+
 		if(rc != SLURM_SUCCESS)
 			sleep(global_sleep_time);
 		else
 			grid_init = 1;
-		
+
 	}
 	g_mutex_lock(grid_mutex);
 	g_cond_signal(grid_cond);
 	g_mutex_unlock(grid_mutex);
-		
+
 	return NULL;
 }
 
@@ -222,7 +240,7 @@ static void _page_switched(GtkNotebook     *notebook,
 	page_thr_t *page_thr = NULL;
 	GError *error = NULL;
 	static int started_grid_init = 0;
-	
+
 	/* make sure we aren't adding the page, and really asking for info */
 	if(adding)
 		return;
@@ -236,15 +254,15 @@ static void _page_switched(GtkNotebook     *notebook,
 		}
 		started_grid_init = 1;
 	}
-	
+
 	if(running != -1) {
 		page_running[running] = 0;
-	}	
-	
+	}
+
 	for(i=0; i<PAGE_CNT; i++) {
 		if(main_display_data[i].id == -1)
 			break;
-		else if(!main_display_data[i].show) 
+		else if(!main_display_data[i].show)
 			continue;
 		if(main_display_data[i].extra == page_num)
 			break;
@@ -253,7 +271,7 @@ static void _page_switched(GtkNotebook     *notebook,
 	if(main_display_data[i].extra != page_num) {
 		g_print("page %d not found\n", page_num);
 		return;
-	} 
+	}
 	if(main_display_data[i].get_info) {
 		running = i;
 		page_running[i] = 1;
@@ -263,13 +281,13 @@ static void _page_switched(GtkNotebook     *notebook,
 			return;
 		}
 
-		page_thr = xmalloc(sizeof(page_thr_t));		
+		page_thr = xmalloc(sizeof(page_thr_t));
 		page_thr->page_num = i;
 		page_thr->table = table;
-		
+
 		if (!g_thread_create(_page_thr, page_thr, FALSE, &error))
 		{
-			g_printerr ("Failed to create page thread: %s\n", 
+			g_printerr ("Failed to create page thread: %s\n",
 				    error->message);
 			return;
 		}
@@ -282,22 +300,22 @@ static void _set_admin_mode(GtkToggleAction *action)
 
 	if(admin_mode) {
 		admin_mode = FALSE;
-		gtk_statusbar_pop(GTK_STATUSBAR(main_statusbar), 
+		gtk_statusbar_pop(GTK_STATUSBAR(main_statusbar),
 				  STATUS_ADMIN_MODE);
 	} else {
 		admin_mode = TRUE;
-		gtk_statusbar_push(GTK_STATUSBAR(main_statusbar), 
+		gtk_statusbar_push(GTK_STATUSBAR(main_statusbar),
 				   STATUS_ADMIN_MODE,
 				   "Admin mode activated! "
 				   "Think before you alter anything.");
 	}
-	gtk_action_group_set_sensitive(admin_action_group, admin_mode); 
+	gtk_action_group_set_sensitive(admin_action_group, admin_mode);
 }
 
 static void _set_grid(GtkToggleAction *action)
 {
 	static bool open = TRUE;
-	
+
 	if(open) {
 		gtk_widget_hide(grid_window);
 		open = FALSE;
@@ -305,7 +323,7 @@ static void _set_grid(GtkToggleAction *action)
 		gtk_widget_show(grid_window);
 		open = TRUE;
 	}
-		
+
 	return;
 }
 
@@ -322,11 +340,65 @@ static void _reconfigure(GtkToggleAction *action)
 	g_free(temp);
 }
 
+static void _get_current_debug(GtkRadioAction *action)
+{
+#ifdef GTK2_USE_RADIO_SET
+	static int debug_level = 0;
+	static slurm_ctl_conf_info_msg_t  *slurm_ctl_conf_ptr = NULL;
+	static GtkAction *debug_action = NULL;
+	int err_code = get_new_info_config(&slurm_ctl_conf_ptr);
+
+	if(err_code != SLURM_ERROR)
+		debug_level = slurm_ctl_conf_ptr->slurmctld_debug;
+
+	if(!debug_action)
+		debug_action = gtk_action_group_get_action(
+			menu_action_group, "debug_quiet");
+	/* Since this is the inital value we don't signal anything
+	   changed so we need to make it happen here */
+	if(debug_level == 0)
+		debug_inited = 1;
+	gtk_radio_action_set_current_value(GTK_RADIO_ACTION(debug_action),
+					   debug_level);
+#else
+	/* Since gtk_radio_action_set_current_value wasn't added to
+	   GTK until 2.10 we have to just not set the first one here
+	   correctly unless we are running with it.
+	*/
+	debug_inited = 1;
+#endif
+}
+
+static void _set_debug(GtkRadioAction *action,
+		       GtkRadioAction *extra,
+		       GtkNotebook *notebook)
+{
+	char *temp = NULL;
+	int level;
+	/* This is here to make sure we got the correct value in the
+	   beginning.  This gets called when the value is
+	   changed. And since we don't set it at the beginning we
+	   need to check it here. */
+	if(!debug_inited) {
+		debug_inited = 1;
+		return;
+	}
+
+	level = gtk_radio_action_get_current_value(action);
+	if(!slurm_set_debug_level(level)) {
+		temp = g_strdup_printf(
+			"Slurmctld debug level is now set to %d", level);
+	} else
+		temp = g_strdup_printf("Problem with set debug level request");
+	display_edit_note(temp);
+	g_free(temp);
+}
+
 static void _tab_pos(GtkRadioAction *action,
 		     GtkRadioAction *extra,
 		     GtkNotebook *notebook)
 {
-	gtk_notebook_set_tab_pos(notebook, 
+	gtk_notebook_set_tab_pos(notebook,
 				 gtk_radio_action_get_current_value(action));
 }
 
@@ -344,9 +416,15 @@ static gboolean _delete(GtkWidget *widget,
                         GtkWidget *event,
                         gpointer data)
 {
-	gtk_main_quit ();
-	list_destroy(popup_list);
 	fini = 1;
+	gtk_main_quit();
+	ba_fini();
+	if(popup_list)
+		list_destroy(popup_list);
+	if(grid_button_list)
+		list_destroy(grid_button_list);
+	if(signal_params_list)
+		list_destroy(signal_params_list);
 	return FALSE;
 }
 
@@ -356,7 +434,6 @@ static GtkWidget *_get_menubar_menu(GtkWidget *window, GtkWidget *notebook)
 	GtkUIManager *ui_manager = NULL;
 	GtkAccelGroup *accel_group = NULL;
 	GError *error = NULL;
-	GtkActionGroup *action_group = NULL;
 
 	/* Our menu*/
 	const char *ui_description =
@@ -379,14 +456,31 @@ static GtkWidget *_get_menubar_menu(GtkWidget *window, GtkWidget *notebook)
 		"      <separator/>"
 		"        <menuitem action='node_name'/>"
 		"        <menuitem action='node_state'/>"
+		"      <separator/>"
+		"        <menuitem action='reservation_name'/>"
 		"      </menu>"
 		"      <menuitem action='refresh'/>"
 		"      <menuitem action='reconfig'/>"
+		"      <menu action='debuglevel'>"
+		"        <menuitem action='debug_quiet'/>"
+		"        <menuitem action='debug_fatal'/>"
+		"        <menuitem action='debug_error'/>"
+		"        <menuitem action='debug_info'/>"
+		"        <menuitem action='debug_verbose'/>"
+		"        <menuitem action='debug_debug'/>"
+		"        <menuitem action='debug_debug2'/>"
+		"        <menuitem action='debug_debug3'/>"
+		"        <menuitem action='debug_debug4'/>"
+		"        <menuitem action='debug_debug5'/>"
+		"      </menu>"
 		"      <separator/>"
 		"      <menuitem action='exit'/>"
 		"    </menu>"
 		"    <menu action='options'>"
 		"      <menuitem action='grid'/>"
+#ifndef HAVE_BG
+		"      <menuitem action='grid_specs'/>"
+#endif
 		"      <menuitem action='interval'/>"
 		"      <separator/>"
 		"      <menuitem action='admin'/>"
@@ -413,120 +507,147 @@ static GtkWidget *_get_menubar_menu(GtkWidget *window, GtkWidget *notebook)
 		{"options", NULL, "_Options", "<alt>o"},
 		{"displays", NULL, "_Query", "<alt>q"},
 		{"search", GTK_STOCK_FIND, "Search", ""},
-		{"jobid", NULL, "Job ID", 
-		 "", "Search for jobid", 
+		{"jobid", NULL, "Job ID",
+		 "", "Search for jobid",
 		 G_CALLBACK(create_search_popup)},
-		{"user_jobs", NULL, "Specific User's Job(s)", 
-		 "", "Search for a specific users job(s)", 
+		{"user_jobs", NULL, "Specific User's Job(s)",
+		 "", "Search for a specific users job(s)",
 		 G_CALLBACK(create_search_popup)},
-		{"state_jobs", NULL, "Job(s) in a Specific State", 
-		 "", "Search for job(s) in a specific state", 
+		{"state_jobs", NULL, "Job(s) in a Specific State",
+		 "", "Search for job(s) in a specific state",
 		 G_CALLBACK(create_search_popup)},
 #ifdef HAVE_BG
-		{"bg_block_name", NULL, "BG Block Name", 
-		 "", "Search for a specific BG Block", 
+		{"bg_block_name", NULL, "BG Block Name",
+		 "", "Search for a specific BG Block",
 		 G_CALLBACK(create_search_popup)},
-		{"bg_block_size", NULL, "BG Block Size", 
-		 "", 
-		 "Search for BG Blocks having given size in cnodes", 
+		{"bg_block_size", NULL, "BG Block Size",
+		 "",
+		 "Search for BG Blocks having given size in cnodes",
 		 G_CALLBACK(create_search_popup)},
-		{"bg_block_state", NULL, "BG Block State", 
-		 "", 
-		 "Search for BG Blocks having given state", 
+		{"bg_block_state", NULL, "BG Block State",
+		 "",
+		 "Search for BG Blocks having given state",
 		 G_CALLBACK(create_search_popup)},
 #endif
-		{"partition_name", NULL, "Slurm Partition Name", 
-		 "", "Search for a specific SLURM partition", 
+		{"partition_name", NULL, "Slurm Partition Name",
+		 "", "Search for a specific SLURM partition",
 		 G_CALLBACK(create_search_popup)},
-		{"partition_state", NULL, "Slurm Partition State", 
-		 "", "Search for SLURM partitions in a given state", 
+		{"partition_state", NULL, "Slurm Partition State",
+		 "", "Search for SLURM partitions in a given state",
 		 G_CALLBACK(create_search_popup)},
-		{"node_name", NULL, 
+		{"node_name", NULL,
 #ifdef HAVE_BG
 		 "Base Partition(s) Name",
-		 "", "Search for a specific Base Partition(s)", 
+		 "", "Search for a specific Base Partition(s)",
 #else
-		 "Node(s) Name", 
-		 "", "Search for a specific Node(s)", 
+		 "Node(s) Name",
+		 "", "Search for a specific Node(s)",
 #endif
-		 G_CALLBACK(create_search_popup)},		
-		{"node_state", NULL, 
+		 G_CALLBACK(create_search_popup)},
+		{"node_state", NULL,
 #ifdef HAVE_BG
 		 "Base Partition State",
-		 "", "Search for a Base Partition in a given state", 
+		 "", "Search for a Base Partition in a given state",
 #else
-		 "Node State", 
-		 "", "Search for a Node in a given state", 
+		 "Node State",
+		 "", "Search for a Node in a given state",
 #endif
-		 G_CALLBACK(create_search_popup)},		
+		 G_CALLBACK(create_search_popup)},
+		{"reservation_name", NULL, "Reservation Name",
+		 "", "Search for reservation",
+		 G_CALLBACK(create_search_popup)},
 		{"tab_pos", NULL, "_Tab Pos"},
-		{"interval", GTK_STOCK_REFRESH, "Set Refresh _Interval", 
-		 "<control>i", "Change Refresh Interval", 
+		{"interval", GTK_STOCK_REFRESH, "Set Refresh _Interval",
+		 "<control>i", "Change Refresh Interval",
 		 G_CALLBACK(change_refresh_popup)},
-		{"refresh", GTK_STOCK_REFRESH, "Refresh", 
+		{"refresh", GTK_STOCK_REFRESH, "Refresh",
 		 "F5", "Refreshes page", G_CALLBACK(refresh_main)},
-		{"config", GTK_STOCK_INFO, "_Config Info", 
-		 "<control>c", "Displays info from slurm.conf file", 
+		{"config", GTK_STOCK_INFO, "_Config Info",
+		 "<control>c", "Displays info from slurm.conf file",
 		 G_CALLBACK(create_config_popup)},
-		{"exit", GTK_STOCK_QUIT, "E_xit", 
+		{"exit", GTK_STOCK_QUIT, "E_xit",
 		 "<control>x", "Exits Program", G_CALLBACK(_delete)},
 		{"help", NULL, "_Help", "<alt>h"},
 		{"about", GTK_STOCK_ABOUT, "A_bout", "<control>b"},
-		{"manual", GTK_STOCK_HELP, "_Manual", "<control>m"}
+		{"manual", GTK_STOCK_HELP, "_Manual", "<control>m"},
+		{"grid_specs", GTK_STOCK_EDIT, "Set Grid _Properties",
+		 "<control>p", "Change Grid Properties",
+		 G_CALLBACK(change_grid_popup)},
 	};
 
 	GtkActionEntry admin_entries[] = {
-		{"reconfig", GTK_STOCK_REDO, "SLUR_M Reconfigure", 
-		 "<control>m", "Reconfigures System", 
+		{"reconfig", GTK_STOCK_REDO, "SLUR_M Reconfigure",
+		 "<control>m", "Reconfigures System",
 		 G_CALLBACK(_reconfigure)},
+		{"debuglevel", GTK_STOCK_DIALOG_WARNING,
+		 "Slurmctld Debug Level",
+		 "", "Set slurmctld debug level",
+		 G_CALLBACK(_get_current_debug)},
+	};
+
+	GtkRadioActionEntry debug_entries[] = {
+		{"debug_quiet", NULL, "quiet(0)", "", "Quiet level", 0},
+		{"debug_fatal", NULL, "fatal(1)", "", "Fatal level", 1},
+		{"debug_error", NULL, "error(2)", "", "Error level", 2},
+		{"debug_info", NULL, "info(3)", "", "Info level", 3},
+		{"debug_verbose", NULL, "verbose(4)", "", "Verbose level", 4},
+		{"debug_debug", NULL, "debug(5)", "", "Debug debug level", 5},
+		{"debug_debug2", NULL, "debug2(6)", "", "Debug2 level", 6},
+		{"debug_debug3", NULL, "debug3(7)", "", "Debug3 level", 7},
+		{"debug_debug4", NULL, "debug4(8)", "", "Debug4 level", 8},
+		{"debug_debug5", NULL, "debug5(9)", "", "Debug5 level", 9},
 	};
 
 	GtkRadioActionEntry radio_entries[] = {
-		{"tab_top", GTK_STOCK_GOTO_TOP, "_Top", 
+		{"tab_top", GTK_STOCK_GOTO_TOP, "_Top",
 		 "<control>T", "Move tabs to top", 2},
-		{"tab_bottom", GTK_STOCK_GOTO_BOTTOM, "_Bottom", 
+		{"tab_bottom", GTK_STOCK_GOTO_BOTTOM, "_Bottom",
 		 "<control>B", "Move tabs to the bottom", 3},
-		{"tab_left", GTK_STOCK_GOTO_FIRST, "_Left", 
+		{"tab_left", GTK_STOCK_GOTO_FIRST, "_Left",
 		 "<control>L", "Move tabs to the Left", 4},
-		{"tab_right", GTK_STOCK_GOTO_LAST, "_Right", 
+		{"tab_right", GTK_STOCK_GOTO_LAST, "_Right",
 		 "<control>R", "Move tabs to the Right", 1}
 	};
 
 	GtkToggleActionEntry toggle_entries[] = {
 		{"grid", GTK_STOCK_SELECT_COLOR, "Show _Grid",
-		 "<control>g", "Visual display of cluster", 
+		 "<control>g", "Visual display of cluster",
 		 G_CALLBACK(_set_grid), TRUE},
-		{"admin", GTK_STOCK_PREFERENCES,          
-		 "_Admin Mode", "<control>a", 
-		 "Allows user to change or update information", 
-		 G_CALLBACK(_set_admin_mode), 
-		 FALSE} 
+		{"admin", GTK_STOCK_PREFERENCES,
+		 "_Admin Mode", "<control>a",
+		 "Allows user to change or update information",
+		 G_CALLBACK(_set_admin_mode),
+		 FALSE}
 	};
-		
+
 	/* Make an accelerator group (shortcut keys) */
-	action_group = gtk_action_group_new ("MenuActions");
-	gtk_action_group_add_actions(action_group, entries, 
+	menu_action_group = gtk_action_group_new ("MenuActions");
+	gtk_action_group_add_actions(menu_action_group, entries,
 				     G_N_ELEMENTS(entries), window);
-	gtk_action_group_add_radio_actions(action_group, radio_entries, 
-					   G_N_ELEMENTS(radio_entries), 
+	gtk_action_group_add_radio_actions(menu_action_group, radio_entries,
+					   G_N_ELEMENTS(radio_entries),
 					   0, G_CALLBACK(_tab_pos), notebook);
-	gtk_action_group_add_toggle_actions(action_group, toggle_entries, 
-					   G_N_ELEMENTS(toggle_entries), 
-					   NULL);
-	admin_action_group = gtk_action_group_new ("MenuActions");
-	gtk_action_group_add_actions(admin_action_group, admin_entries, 
+	gtk_action_group_add_radio_actions(menu_action_group, debug_entries,
+					   G_N_ELEMENTS(debug_entries),
+					   -1, G_CALLBACK(_set_debug),
+					   notebook);
+	gtk_action_group_add_toggle_actions(menu_action_group, toggle_entries,
+					    G_N_ELEMENTS(toggle_entries),
+					    NULL);
+	admin_action_group = gtk_action_group_new ("MenuAdminActions");
+	gtk_action_group_add_actions(admin_action_group, admin_entries,
 				     G_N_ELEMENTS(admin_entries),
 				     window);
-	gtk_action_group_set_sensitive(admin_action_group, FALSE); 
-	
+	gtk_action_group_set_sensitive(admin_action_group, FALSE);
+
 	ui_manager = gtk_ui_manager_new();
-	gtk_ui_manager_insert_action_group(ui_manager, action_group, 0);
+	gtk_ui_manager_insert_action_group(ui_manager, menu_action_group, 0);
 	gtk_ui_manager_insert_action_group(ui_manager, admin_action_group, 1);
-	
+
 	accel_group = gtk_ui_manager_get_accel_group(ui_manager);
 	gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
 
-	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, 
+	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description,
 						-1, &error))
 	{
 		g_error("building menus failed: %s", error->message);
@@ -537,9 +658,10 @@ static GtkWidget *_get_menubar_menu(GtkWidget *window, GtkWidget *notebook)
 	/* Finally, return the actual menu bar created by the item factory. */
 	return gtk_ui_manager_get_widget (ui_manager, "/main");
 }
+
 void *_popup_thr_main(void *arg)
 {
-	popup_thr(arg);		
+	popup_thr(arg);
 	return NULL;
 }
 
@@ -552,16 +674,20 @@ extern void refresh_main(GtkAction *action, gpointer user_data)
 	_page_switched(GTK_NOTEBOOK(main_notebook), NULL, page, NULL);
 }
 
-extern void tab_pressed(GtkWidget *widget, GdkEventButton *event, 
-			const display_data_t *display_data)
+extern void tab_pressed(GtkWidget *widget, GdkEventButton *event,
+			display_data_t *display_data)
 {
+	signal_params_t signal_params;
+	signal_params.display_data = display_data;
+	signal_params.button_list = &grid_button_list;
+
 	/* single click with the right mouse button? */
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(main_notebook),
 				      display_data->extra);
 	if(event->button == 3) {
-		right_button_pressed(NULL, NULL, event, 
-				     display_data, TAB_CLICKED);
-	} 
+		right_button_pressed(NULL, NULL, event,
+				     &signal_params, TAB_CLICKED);
+	}
 }
 
 int main(int argc, char *argv[])
@@ -572,7 +698,10 @@ int main(int argc, char *argv[])
 	GtkBin *bin = NULL;
 	GtkViewport *view = NULL;
 	int i=0;
-	
+
+	if(getenv("SVIEW_GRID_SPEEDUP"))
+		grid_speedup = 1;
+
 	_init_pages();
 	g_thread_init(NULL);
 	gdk_threads_init();
@@ -591,7 +720,7 @@ int main(int argc, char *argv[])
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(grid_window),
 				       GTK_POLICY_NEVER,
 				       GTK_POLICY_AUTOMATIC);
-	
+
 /* #ifdef HAVE_BG */
 /* 	gtk_widget_set_size_request(grid_window, 164, -1); */
 /* #endif */
@@ -600,9 +729,9 @@ int main(int argc, char *argv[])
 	main_window = gtk_dialog_new();
 	g_signal_connect(G_OBJECT(main_window), "delete_event",
 			 G_CALLBACK(_delete), NULL);
-	
+
 	gtk_window_set_title(GTK_WINDOW(main_window), "Sview");
-	gtk_window_set_default_size(GTK_WINDOW(main_window), 700, 450);
+	gtk_window_set_default_size(GTK_WINDOW(main_window), 1000, 450);
 	gtk_container_set_border_width(
 		GTK_CONTAINER(GTK_DIALOG(main_window)->vbox), 1);
 	/* Create the main notebook, place the position of the tabs */
@@ -612,7 +741,7 @@ int main(int argc, char *argv[])
 			 NULL);
 	table = gtk_table_new(1, 2, FALSE);
 	gtk_table_set_homogeneous(GTK_TABLE(table), FALSE);
-	gtk_container_set_border_width(GTK_CONTAINER(table), 1);	
+	gtk_container_set_border_width(GTK_CONTAINER(table), 1);
 	/* Create a menu */
 	menubar = _get_menubar_menu(main_window, main_notebook);
 	gtk_table_attach_defaults(GTK_TABLE(table), menubar, 0, 1, 0, 1);
@@ -620,9 +749,9 @@ int main(int argc, char *argv[])
 	gtk_notebook_popup_enable(GTK_NOTEBOOK(main_notebook));
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(main_notebook), TRUE);
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(main_notebook), GTK_POS_TOP);
-	
+
 	main_statusbar = gtk_statusbar_new();
-	gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(main_statusbar), 
+	gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(main_statusbar),
 					  FALSE);
 	/* Pack it all together */
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(main_window)->vbox),
@@ -633,24 +762,25 @@ int main(int argc, char *argv[])
 			 GTK_SHRINK, GTK_EXPAND | GTK_FILL,
 			 0, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), main_notebook, 1, 2, 0, 1);
-	
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(main_window)->vbox), 
-			   table, TRUE, TRUE, 0);	
+
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(main_window)->vbox),
-			   main_statusbar, FALSE, FALSE, 0);	
-	
+			   table, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(main_window)->vbox),
+			   main_statusbar, FALSE, FALSE, 0);
+
 	for(i=0; i<PAGE_CNT; i++) {
 		if(main_display_data[i].id == -1)
 			break;
-		else if(!main_display_data[i].show) 
+		else if(!main_display_data[i].show)
 			continue;
-		create_page(GTK_NOTEBOOK(main_notebook), 
+		create_page(GTK_NOTEBOOK(main_notebook),
 			    &main_display_data[i]);
 	}
 
 	/* tell signal we are done adding */
 	adding = 0;
 	popup_list = list_create(destroy_popup_info);
+	signal_params_list = list_create(destroy_signal_params);
 	gtk_widget_show_all(main_window);
 
 	/* Finished! */
