@@ -3,21 +3,22 @@
 # Copyright (C) 2006 The Regents of the University of California.
 # Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
 # Written by Christopher J. Morrone <morrone2@llnl.gov>
-# LLNL-CODE-402394.
-# 
+# CODE-OCEC-09-009. All rights reserved.
+#
 # This file is part of SLURM, a resource management program.
-# For details, see <http://www.llnl.gov/linux/slurm/>.
-#  
+# For details, see <https://computing.llnl.gov/linux/slurm/>.
+ # Please also read the supplied file: DISCLAIMER.
+#
 # SLURM is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
 # Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
-# 
+#
 # SLURM is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
-# 
+#
 # You should have received a copy of the GNU General Public License along
 # with SLURM; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
@@ -53,10 +54,10 @@ def main(argv=None):
                       dest='time_individual', default=False)
     parser.add_option('-e', '--exclude', type='string', dest='exclude_tests',
                       action='callback', callback=test_parser,
-                      help='comma or space seperated string of tests to skip')
+                      help='comma or space separated string of tests to skip')
     parser.add_option('-i', '--include', type='string', dest='include_tests',
                       action='callback', callback=test_parser,
-                      help='comma or space seperated string of tests to include')
+                      help='comma or space separated string of tests to include')
     parser.add_option('-k', '--keep-logs', action='store_true', default=False)
     (options, args) = parser.parse_args(args=argv)
 
